

;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

AY_Init:
                    ;------------------------------------
                    ; AY init
                    ;------------------------------------
                    ; - in:
                    ; ESI - AY structure address
                    ; - out:
                    ; none
                    ;------------------------------------

                    pushad
                    push    es
                    mov     eax, ds
                    mov     es, eax
                    mov     al, 0FFh
                    mov     edi, esi
                    mov     ecx, aydata_size
                    rep stosb
                    mov     [esi].AYEnvelopePtr, offset AYEnvelopeTable
                    mov     [esi].AYWhiteNoiseLFSR, 0FFFFh
                    mov     ah, 0FFh
                    call    AY_WriteReg7

                    pop     es
                    popad
                    ret

;----------------------------------------------------------------------------;

AY_Process:
                    ;------------------------------------
                    ; AY process
                    ;------------------------------------
                    ; - in:
                    ; ESI - AY structure address
                    ; - out:
                    ; EAX, ECX used
                    ;------------------------------------

                    mov     ecx, CFG_AYGenClkDivider
                    ; Channel A tone Generator
                    sub     [esi].AYToneACnr, ecx
                    jns     AYproc_010
                    ; >-- restarting counter
                    mov     eax, [esi].AYToneAConst
                    not     [esi].AYToneStateA
                    add     [esi].AYToneACnr, eax
                    jns     AYproc_010
                    mov     [esi].AYToneACnr, eax
AYproc_010:
                    ; Channel B tone Generator
                    sub     [esi].AYToneBCnr, ecx
                    jns     AYproc_020
                    ; >-- restarting counter
                    mov     eax, [esi].AYToneBConst
                    not     [esi].AYToneStateB
                    add     [esi].AYToneBCnr, eax
                    jns     AYproc_020
                    mov     [esi].AYToneBCnr, eax
AYproc_020:
                    ; Channel C tone Generator
                    sub     [esi].AYToneCCnr, ecx
                    jns     AYproc_030
                    ; >-- restarting counter
                    mov     eax, [esi].AYToneCConst
                    not     [esi].AYToneStateC
                    add     [esi].AYToneCCnr, eax
                    jns     AYproc_030
                    mov     [esi].AYToneCCnr, eax
AYproc_030:
                    ; White Noise Generator
                    sub     [esi].AYWhiteNoiseCnr, ecx
                    jns     AYproc_040
                    ; >-- 17-bit LFSR
                    mov     eax, [esi].AYWhiteNoiseLFSR
                    test    eax, 000010000h
                    setz    cl
                    test    eax, 000002000h
                    setz    ch
                    xor     ch, cl
                    shl     eax, 1
                    or      al, ch
                    mov     [esi].AYWhiteNoiseLFSR, eax
                    not     [esi].AYWhiteNoiseState
                    ; >-- restarting counter
                    mov     eax, [esi].AYWhiteNoiseConst
                    shl     eax, cl
                    add     [esi].AYWhiteNoiseCnr, eax
                    jns     AYproc_040
                    mov     [esi].AYWhiteNoiseCnr, eax
AYproc_040:
                    mov     ecx, CFG_AYGenClkDivider
                    ; Envelope Generator
                    sub     [esi].AYEnvelopeCnr, ecx
                    jns     AYproc_050
                    ; >-- getting next envelope volume
                    mov     ecx, [esi].AYEnvelopePtr
                    movsx   eax, byte ptr [ecx + 1]
                    add     eax, ecx
                    mov     [esi].AYEnvelopePtr, eax
                    movzx   eax, byte ptr [eax]
                    mov     eax, [eax*4 + AYVolumeTable]
                    mov     [esi].AYEnvelopeVolume, eax
                    ; >-- restarting counter
                    mov     eax, [esi].AYEnvelopeConst
                    add     [esi].AYEnvelopeCnr, eax
                    jns     AYproc_050
                    mov     [esi].AYEnvelopeCnr, eax
AYproc_050:
                    ret

;----------------------------------------------------------------------------;

AY_Mix:
                    ;------------------------------------
                    ; AY mix
                    ;------------------------------------
                    ; - in:
                    ; ESI - AY structure address
                    ; - out:
                    ; EAX, ECX used
                    ;------------------------------------

                    ; Mixing Channel A (tone,envelope,noise)
                    mov     eax, [esi].AYWhiteNoiseState
                    mov     ecx, [esi].AYToneStateA
                    or      eax, [esi].AYWhiteNoiseEnableA
                    or      ecx, [esi].AYToneEnableA
                    and     eax, ecx
                    mov     ecx, [esi].AYEnvelopeVolume
                    and     ecx, [esi].AYEnvelopeMaskA
                    or      ecx, [esi].AYChannelVolumeA
                    and     ecx, eax
                    mov     [esi].AYChannelLevelA, ecx
                    ; Mixing Channel B (tone,envelope,noise)
                    mov     eax, [esi].AYWhiteNoiseState
                    mov     ecx, [esi].AYToneStateB
                    or      eax, [esi].AYWhiteNoiseEnableB
                    or      ecx, [esi].AYToneEnableB
                    and     eax, ecx
                    mov     ecx, [esi].AYEnvelopeVolume
                    and     ecx, [esi].AYEnvelopeMaskB
                    or      ecx, [esi].AYChannelVolumeB
                    and     ecx, eax
                    mov     [esi].AYChannelLevelB, ecx
                    ; Mixing Channel C (tone,envelope,noise)
                    mov     eax, [esi].AYWhiteNoiseState
                    mov     ecx, [esi].AYToneStateC
                    or      eax, [esi].AYWhiteNoiseEnableC
                    or      ecx, [esi].AYToneEnableC
                    and     eax, ecx
                    mov     ecx, [esi].AYEnvelopeVolume
                    and     ecx, [esi].AYEnvelopeMaskC
                    or      ecx, [esi].AYChannelVolumeC
                    and     ecx, eax
                    mov     [esi].AYChannelLevelC, ecx
                    ret

;----------------------------------------------------------------------------;

AY_SelectRegister:
                    mov     [esi].AYSelectedReg, ah
                    ret
AY_ReadRegister:
                    mov     cl, [esi].AYSelectedReg
                    and     ecx, 00Fh
                    mov     ah, [ecx + [esi].AYRegister0]
                    ret
AY_WriteRegister:
                    mov     cl, [esi].AYSelectedReg
                    and     ecx, 00Fh
                    jmp     [ecx*4 + AYRegistersTable]

;----------------------------------------------------------------------------;
; Channel A tone pair write
AY_WriteReg0:
                    mov     [esi].AYRegister0, ah
AYwr0r1_1:
                    movzx   eax, word ptr [esi].AYRegister0
                    cmp     eax, 1
                    adc     al, 0
                    mov     [esi].AYToneAConst, eax
                    ret
AY_WriteReg1:
                    mov     [esi].AYRegister1, ah
                    jmp     AYwr0r1_1

;----------------------------------------------------------------------------;
; Channel B tone pair write
AY_WriteReg2:
                    mov     [esi].AYRegister2, ah
AYwr2r3_1:
                    movzx   eax, word ptr [esi].AYRegister2
                    cmp     eax, 1
                    adc     al, 0
                    mov     [esi].AYToneBConst, eax
                    ret

AY_WriteReg3:
                    mov     [esi].AYRegister3, ah
                    jmp     AYwr2r3_1

;----------------------------------------------------------------------------;
; Channel C tone pair write
AY_WriteReg4:
                    mov     [esi].AYRegister4, ah
AYwr4r5_1:
                    movzx   eax, word ptr [esi].AYRegister4
                    cmp     eax, 1
                    adc     al, 0
                    mov     [esi].AYToneCConst, eax
                    ret

AY_WriteReg5:
                    mov     [esi].AYRegister5, ah
                    jmp     AYwr4r5_1

;----------------------------------------------------------------------------;
AY_WriteReg6:
                    mov     [esi].AYRegister6, ah
                    and     ah, 01Fh
                    setz    al
                    add     al, ah
                    movzx   eax, al
                    mov     [esi].AYWhiteNoiseConst, eax
                    ret

;----------------------------------------------------------------------------;
AY_WriteReg7:
                    mov     [esi].AYRegister7, ah
                    ror     ah, 1
                    sbb     ecx, ecx
                    mov     [esi].AYToneEnableA, ecx
                    ror     ah, 1
                    sbb     ecx, ecx
                    mov     [esi].AYToneEnableB, ecx
                    ror     ah, 1
                    sbb     ecx, ecx
                    mov     [esi].AYToneEnableC, ecx
                    ror     ah, 1
                    sbb     ecx, ecx
                    mov     [esi].AYWhiteNoiseEnableA, ecx
                    ror     ah, 1
                    sbb     ecx, ecx
                    mov     [esi].AYWhiteNoiseEnableB, ecx
                    ror     ah, 1
                    sbb     ecx, ecx
                    mov     [esi].AYWhiteNoiseEnableC, ecx
                    ret

;----------------------------------------------------------------------------;
AY_WriteReg8:
                    mov     [esi].AYRegister8, ah
                    mov     al, ah
                    and     eax, 01Fh
                    xor     ecx, ecx
                    test    al, 010h
                    setz    cl
                    and     al, 00Fh
                    dec     ecx
                    mov     eax, [eax*4 + offset AYVolumeTable]
                    mov     [esi].AYEnvelopeMaskA, ecx
                    not     ecx
                    and     eax, ecx
                    mov     [esi].AYChannelVolumeA, eax
                    ret

AY_WriteReg9:
                    mov     [esi].AYRegister9, ah
                    mov     al, ah
                    and     eax, 01Fh
                    xor     ecx, ecx
                    test    al, 010h
                    setz    cl
                    and     al, 00Fh
                    dec     ecx
                    mov     eax, [eax*4 + offset AYVolumeTable]
                    mov     [esi].AYEnvelopeMaskB, ecx
                    not     ecx
                    and     eax, ecx
                    mov     [esi].AYChannelVolumeB, eax
                    ret

AY_WriteRegA:
                    mov     [esi].AYRegisterA, ah
                    mov     al, ah
                    and     eax, 01Fh
                    xor     ecx, ecx
                    test    al, 010h
                    setz    cl
                    and     al, 00Fh
                    dec     ecx
                    mov     eax, [eax*4 + offset AYVolumeTable]
                    mov     [esi].AYEnvelopeMaskC, ecx
                    not     ecx
                    and     eax, ecx
                    mov     [esi].AYChannelVolumeC, eax
                    ret

;----------------------------------------------------------------------------;
AY_WriteRegB:
                    mov     [esi].AYRegisterB, ah
                    movzx   eax, word ptr [esi].AYRegisterB
                    cmp     eax, 1
                    adc     al, 0
                    add     eax, eax
                    mov     [esi].AYEnvelopeConst, eax
                    ret

AY_WriteRegC:
                    mov     [esi].AYRegisterC, ah
                    movzx   eax, word ptr [esi].AYRegisterB
                    cmp     eax, 1
                    adc     al, 0
                    add     eax, eax
                    mov     [esi].AYEnvelopeConst, eax
                    ret

AY_WriteRegD:
                    mov     [esi].AYRegisterD, ah
                    and     ah, 00Fh
                    movzx   ecx, ah
                    shl     ecx, 6
                    lea     ecx, [ecx + offset AYEnvelopeTable]
                    mov     [esi].AYEnvelopePtr, ecx
                    ret

AY_WriteRegE:
                    mov     [esi].AYRegisterE, ah
                    ret

AY_WriteRegF:
                    mov     [esi].AYRegisterF, ah
                    ret

;----------------------------------------------------------------------------;


;****************************************************************************;

.DATA

AYEnvelopeTable     label   byte
_ENVELOPE_0         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  0
                    db      32 dup(0)
_ENVELOPE_1         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  0
                    db      32 dup(0)
_ENVELOPE_2         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  0
                    db      32 dup(0)
_ENVELOPE_3         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  0
                    db      32 dup(0)
_ENVELOPE_4         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 0
                    db      32 dup(0)
_ENVELOPE_5         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 0
                    db      32 dup(0)
_ENVELOPE_6         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 0
                    db      32 dup(0)
_ENVELOPE_7         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 0
                    db      32 dup(0)
_ENVELOPE_8         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  -30
                    db      32 dup(0)
_ENVELOPE_9         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  0
                    db      32 dup(0)
_ENVELOPE_A         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  2
                    db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, -62
_ENVELOPE_B         db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  2
                    db      15,  0
                    db      30 dup(0)
_ENVELOPE_C         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, -30
                    db      32 dup(0)
_ENVELOPE_D         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 0
                    db      32 dup(0)
_ENVELOPE_E         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 2
                    db      15, 2,  14, 2,  13, 2,  12, 2,  11, 2,  10, 2,  9,  2,  8,  2
                    db      7,  2,  6,  2,  5,  2,  4,  2,  3,  2,  2,  2,  1,  2,  0,  -62
_ENVELOPE_F         db      0,  2,  1,  2,  2,  2,  3,  2,  4,  2,  5,  2,  6,  2,  7,  2
                    db      8,  2,  9,  2,  10, 2,  11, 2,  12, 2,  13, 2,  14, 2,  15, 2
                    db      0,  0
                    db      30 dup(0)
AYVolumeTable       label   dword
                    dd      0
                    dd      259
                    dd      555
                    dd      779
                    dd      1079
                    dd      1427
                    dd      1843
                    dd      2375
                    dd      3055
                    dd      3911
                    dd      5027
                    dd      6443
                    dd      8191
                    dd      10404
                    dd      13124
                    dd      16368
AYRegistersTable    label   dword
                    dd      AY_WriteReg0, AY_WriteReg1, AY_WriteReg2, AY_WriteReg3
                    dd      AY_WriteReg4, AY_WriteReg5, AY_WriteReg6, AY_WriteReg7
                    dd      AY_WriteReg8, AY_WriteReg9, AY_WriteRegA, AY_WriteRegB
                    dd      AY_WriteRegC, AY_WriteRegD, AY_WriteRegE, AY_WriteRegF

;****************************************************************************;

