
include defines.inc
include macro.inc


;****************************************************************************;

;----------------------------------------------------------------------------
                    ; Total tacts for Operation/Prefix
TOTAL               macro   tacts
                    ; dummy in this core
;                    lea     ebp, [ebp + tacts]
;                    add     [ebx].VideoCounter, tacts
;                    push    eax
;                    UPDSCR
;                    pop     eax
                endm
;----------------------------------------------------------------------------
                    ; Tacts for Port Write
T_PWR               macro   tacts
                    lea     ebp, [ebp + tacts]
                    add     [ebx].VideoCounter, tacts
                    endm
;----------------------------------------------------------------------------
                    ; Tacts for Port Read
T_PRD               macro   tacts
                    lea     ebp, [ebp + tacts]
                    add     [ebx].VideoCounter, tacts
                    endm
;----------------------------------------------------------------------------
                    ; Tacts for Internal Operations
T_IOP               macro   tacts
                    lea     ebp, [ebp + tacts]
                    add     [ebx].VideoCounter, tacts
                    endm
;----------------------------------------------------------------------------
                    ; Tacts for Memory Write till next memory access or IOP cycle
T_MWR               macro   tacts
                    lea     ebp, [ebp + tacts]
                    add     [ebx].VideoCounter, tacts
;                    lea     ebp, [ebp + 3]
;                    add     [ebx].VideoCounter, 3
;                    if      tacts gt 3
;                        T_IOP   tacts-3
;                    endif
                    endm
;----------------------------------------------------------------------------
                    ; Tacts for Memory Read
T_MRD               macro   tacts
                    lea     ebp, [ebp + tacts]
                    add     [ebx].VideoCounter, tacts
;                    lea     ebp, [ebp + 3]
;                    add     [ebx].VideoCounter, 3
;                    if      tacts gt 3
;                        T_IOP   tacts-3
;                    endif
                    endm
;----------------------------------------------------------------------------
                    ; Tacts for M1 (followed by IOP if exists)
T_M1R               macro   tacts, total
                    ifnb    <total>
                        lea     ebp, [ebp + total]
                        add     [ebx].VideoCounter, total
                    else
                        lea     ebp, [ebp + tacts]
                        add     [ebx].VideoCounter, tacts
                    endif
                    endm
;----------------------------------------------------------------------------
                    ; M1 Cycle
M1R                 macro
                    ROMSW   TRDOS, esi
                    mov     [ebx].LastAdrOnBus, edi
                    READ    di
                    endm
;----------------------------------------------------------------------------
                    ; Memory Read Cycle
MRD                 macro   reg
                    mov     word ptr [ebx].LastAdrOnBus, reg
                    READ    reg
                    endm
;----------------------------------------------------------------------------
                    ; Memory Write Cycle
MWR                 macro   reg
                    local   mwr_1
                    mov     word ptr [ebx].LastAdrOnBus, reg
                    bt      [ebx].VideoCounter, 31
                    jc      mwr_1
                    test    reg, 04000h
                    jz      mwr_1
                    push    eax
                    UPDSCR
                    pop     eax
mwr_1:
                    WRITE   reg
                    endm
;----------------------------------------------------------------------------

include cpu.asm
include ports.asm

;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

ZXCommandMonitor:
                    ; init machine config and z80 registers
                    xor     eax, eax
                    mov     dword ptr KbdHalfrow0, eax
                    movzx   edi, [ebx].rPC
                    mov     dword ptr KbdHalfrow3, eax
                    mov     dword ptr KeyboardF0, eax
                    mov     edx, [ebx].rAFxAF
                    mov     dword ptr KeyboardF3, eax
                    mov     ah, [ebx].SystemZX128
                    mov     [ebx].BusIntVector, 0FFh
                    call    ZX128k_SetConfig
                    ; init video and sync
                    mov     [ebx].Z80DividePoint, 0
                    mov     VideoFrameCount, 0
                    mov     VideoCurItem, 10
                    and     [ebx].FlagsA, not(FA_INT_ + FA_ONESHOTDI)
                    mov     eax, CFG_ULATactsInFrame
                    mov     [ebx].FrameTactsCnr, eax
                    sub     eax, CFG_Z80TactsAsOne
                    mov     [ebx].PreFrameTactsConst, eax
                    xor     eax, eax
                    mov     [ebx].VideoCounter, eax
                    mov     [ebx].Z80MinTactsAs1, eax
                    mov     [ebx].Z80InterptCount, eax
                    mov     [ebx].Z80TactsCnr, eax
                    mov     Z80SynchroCount, eax
                    mov     SyncEventsCnr, eax
                    mov     LastSyncEvents, eax
                    call    VGA_WaitVerticalRetrace
                    mov     VideoScheme, offset TMUpdate_SBorder
                    test    CFG_VideoModeMain, VM_BUFFERED
                    jz      ZCmdmon_1
                    mov     VideoGfxBaseAdr, offset VideoBuffer
ZCmdmon_1:
                    mov     eax, VideoScheme
                    mov     [ebx].VideoRefreshRoutine, eax
                    call    TMR_ReadCounter
                    mov     TMRLastSyncLow, eax

                    mov     ebp, 4
;                    jmp     ZX_Interrupt_Execution

; -------------------------------------------------------
; -=* W-A-I-T---F-O-R---S-Y-N-C-H-R-O-N-I-Z-A-T-I-O-N *=-
; -------------------------------------------------------
PC_Wait_4_Sync:
                    sub     [ebx].Z80TactsCnr, ebp
                    jc      PC_Sync
PW4sync_End:

; ---------------------------------------------------------
; -=* C-A-L-C-U-L-A-T-E---I-N-T-E-R-R-U-P-T---E-V-E-N-T *=-
; ---------------------------------------------------------
ZX_Interrupt_EventCalc:
                    ; emulated vertical retrace
                    mov     eax, [ebx].FrameTactsCnr
                    cmp     eax, [ebx].PreFrameTactsConst
                    jc      ZIeventcalc_2
                    mov     [ebx].Z80MinTactsAs1, 0
                    cmp     eax, CFG_ULATactsInFrame
                    jc      ZIeventcalc_2
                    mov     [ebx].LastFrameTacts, eax
                    mov     eax, CFG_ULATactsInFrame
                    sub     [ebx].FrameTactsCnr, eax
                    and     [ebx].FlagsA, not(FA_INT_)
                    mov     al, CFG_ULAFlashFreq
                    add     [ebx].FlashTimer, al
                    sbb     al, al
                    and     al, 080h
                    xor     byte ptr VideoFlashTag, al
                    UPDSCR
;                    mov     [ebx].VideoCounter, 0
                    test    CFG_VideoModeMain, VM_BUFFERED
                    jz      ZIeventcalc_1
                    call    [VideoInstUpdRoutine]
ZIeventcalc_1:
                    call    Display_DebugInfo
ZIeventcalc_2:

; -------------------------------------------------
; -=* E-X-E-C-U-T-E---Z-X---C-O-M-M-A-N-D-(-S-) *=-
; -------------------------------------------------
ZX_Commmand_Execution:
                    xor     ebp, ebp
ZCexec_1:
                    ; loading and executing command
                    M1R
                    inc     [ebx].regR
                    xor     eax, eax
                    inc     edi
                    mov     al, [esi]
                    push    offset ZCexec_2
                    mov     eax, [eax*4 + offset Z80_CMDs_General]
                    and     [ebx].FlagsA, not(FA_CMDPREF + FA_ONESHOTDI)
                    jmp     eax
ZCexec_2:
                    ; running several commands as one
                    cmp     ebp, [ebx].Z80MinTactsAs1
                    jc      ZCexec_1
ZCexec_5:

                    add     [ebx].FrameTactsCnr, ebp
                    add     [ebx].TotalTactsCnr, ebp

; ---------------------------------------------
; -=* I-N-T-E-R-R-U-P-T---E-X-E-C-U-T-I-O-N *=-
; ---------------------------------------------
ZX_Interrupt_Execution:
                    test    [ebx].FlagsA, FA_INT_
                    jz      ZX_Interrupt
ZIexec_End:

; ---------------------------------------
; -=* K-E-Y-P-R-E-S-S---S-E-R-V-I-C-E *=-
; ---------------------------------------
PC_Hotkeys_Service:
                    test    dword ptr KeyboardF0, 01F1F1F1Fh
                    jnz     PC_Hotkeys
Phkeys_End:

; -----------------------------------
; -=* G-E-N-E-R-A-T-E---S-O-U-N-D *=-
; -----------------------------------
                    shl     ebp, FREQ_RAISE+16
                    mov     eax, ebp
                    sub     SoundGenTactsCnr, eax
                    sub     SamplerTactsCnr, eax
                    neg     ebp

ZX_Sound_Generate:
                    mov     eax, SoundGenTactsCnr
                    cmp     eax, ebp
                    jnc     ZX_Sound
ZSgen_End:

; -------------------------------
; -=* S-O-U-N-D---O-U-T-P-U-T *=-
; -------------------------------
PC_Sound_Output:
                    ; dividing Z80 and Sampler frequencies
                    mov     eax, SamplerTactsCnr
                    cmp     eax, ebp
                    jnc     PC_Sound
PSout_End:
                    neg     ebp
                    shr     ebp, FREQ_RAISE+16
ZX_Goto_Sync:
                    jmp     PC_Wait_4_Sync

;----------------------------------------------------------------------------;

                    ;------------------------------
                    ; ZX audio signal output to PC
                    ;------------------------------
PC_Sound:
                    add     eax, SamplerTactsConst
                    mov     SamplerTactsCnr, eax
                    test    InitResources, IR_DMA
                    jz      PSout_End

OutSBMono8:
                    ; generate mono
                    lea     esi, [ebx].AY1st
                    call    AY_Mix
                    mov     eax, [esi].AYChannelLevelA
                    add     eax, [esi].AYChannelLevelB
                    add     eax, [esi].AYChannelLevelC
                    add     eax, [ebx].BeeperLevel
                    add     eax, [ebx].TapeLevel
                    ; simple hard limiting filter
                    mov     ecx, 0FFFF0000h
                    cmp     ecx, eax
                    sbb     ecx, ecx
                    or      eax, ecx
;                    add     ah, byte ptr CurrentTotalLevels
;                    rcr     ah, 1
                    mov     al, ah
                    mov     cx, ax
                    shl     eax, 16
                    mov     ax, cx
                    mov     CurrentTotalLevels, eax

                    ; skip if DMA region is full - updated by SB interrupt
                    mov     esi, SoundSamplesCnr
                    inc     SoundSamplesCnr
                    cmp     esi, CFG_SBBufferSize
                    jnc     PSout_End
                    add     esi, SBStartSampleOffs
                    mov     eax, CurrentTotalLevels
                    cmp     SBTotalBufferEnd, esi
                    sbb     ecx, ecx
                    and     ecx, CFG_SBBufferSize
                    sub     esi, ecx
                    mov     [esi], eax
                    mov     [esi + 4], eax
                    jmp     PSout_End;PC_Sound_Output

;----------------------------------------------------------------------------;

                    ;--------------------------
                    ; ZX audio signal building
                    ;--------------------------
ZX_Sound:
                    add     eax, SoundGenTactsConst
                    mov     SoundGenTactsCnr, eax
;                    push    edi edx
Zsnd_1:
                    lea     esi, [ebx].AY1st
                    call    AY_Process
                    mov     eax, SoundGenTactsCnr
                    test    eax, eax
                    jns     Zsnd_End
                    add     eax, SoundGenTactsConst
                    mov     SoundGenTactsCnr, eax
                    jmp     Zsnd_1
Zsnd_End:
;                    mov     [ebx].AY0LeftLevel, edi
;                    mov     [ebx].AY0MiddleLevel, ecx
;                    mov     [ebx].AY0RightLevel, edx
;                    pop     edx edi

                    jmp     ZSgen_End

OutPCSpeaker:
                    mov     eax, [ebx].BeeperLevel
                    add     eax, [ebx].TapeLevel
                    lea     esi, [ebx].AY1st
                    add     eax, [ebx].AY0LeftLevel
                    add     eax, [ebx].AY0MiddleLevel
                    add     eax, [ebx].AY0RightLevel
                    in      al, 061h
                    or      al, 003h
                    out     061h, al
                    mov     al, CBEEPM
                    out     043h, al
                    shr     eax, 10
                    out     042h, al
                    jmp     ZSgen_End

;----------------------------------------------------------------------------;

Sound_MonoMixer:
                    ;---------------------------------
                    ; Mono Mixer
                    ;---------------------------------
                    ; - in:
                    ; EDX - MIXDATA structure address
                    ; - out:
                    ; EAX - Output Level
                    ;---------------------------------
                    push    eax
                    ; mix Beeper
                    mov     eax, BeeperLevel
                    mov     cl, [edx].MixBeeper
                    shr     eax, cl
                    mov     [esp], eax
                    ; mix TapeOut
                    mov     eax, [ebx].TapeLevel
                    mov     cl, [edx].MixTapeOut
                    shr     eax, cl
                    add     [esp], eax
                    ; mix AY0
                    lea     esi, [ebx].AY1st
                    mov     eax, [esi].AY0LeftLevel
                    add     eax, [esi].AY0MiddleLevel
                    mov     cl, [edx].MixAY0
                    add     eax, [esi].AY0RightLevel
                    shr     eax, cl
                    add     [esp], eax

;                    ; mix Covox
;                    mov     cl, [edx].MixCovox
;                    mov     eax, CovoxLevelLeft
;                    add     eax, CovoxLevelRight
;                    shr     eax, cl
;                    add     [esp], eax
;                    ; mix AY1
;                    lea     esi, AY2nd
;                    mov     eax, [ebx].AYChannelLevelA
;                    add     eax, [ebx].AYChannelLevelB
;                    mov     cl, [edx].MixAY1
;                    add     eax, [ebx].AYChannelLevelC
;                    shr     eax, cl
;                    add     [esp], eax
;                    ; mix GS
;                    mov     cl, [edx].MixGS
;                    mov     eax, GSLevelA
;                    add     eax, GSLevelB
;                    add     eax, GSLevelC
;                    add     eax, GSLevelD
;                    shr     eax, cl
;                    add     [esp], eax
;                    ; mix AY2
;                    lea     esi, AY3rd
;                    mov     eax, [ebx].AYChannelLevelA
;                    add     eax, [ebx].AYChannelLevelB
;                    mov     cl, [edx].MixAY2
;                    add     eax, [ebx].AYChannelLevelC
;                    shr     eax, cl
;                    add     [esp], eax
;                    ; mix AY3
;                    lea     esi, AY4th
;                    mov     eax, [ebx].AYChannelLevelA
;                    add     eax, [ebx].AYChannelLevelB
;                    mov     cl, [edx].MixAY3
;                    add     eax, [ebx].AYChannelLevelC
;                    shr     eax, cl
;                    add     [esp], eax
;                    ; mix AY4
;                    lea     esi, AY5th
;                    mov     eax, [esi].AYChannelLevelA
;                    add     eax, [esi].AYChannelLevelB
;                    mov     cl, [edx].MixAY4
;                    add     eax, [esi].AYChannelLevelC
;                    shr     eax, cl
;                    add     [esp], eax
;                    ; mix AY5
;                    lea     esi, AY6th
;                    mov     eax, [esi].AYChannelLevelA
;                    add     eax, [esi].AYChannelLevelB
;                    mov     cl, [edx].MixAY5
;                    add     eax, [esi].AYChannelLevelC
;                    shr     eax, cl
;                    add     [esp], eax
                    pop     eax
                    ret

;----------------------------------------------------------------------------;

Sound_StereoMixer:
                    ;---------------------------------
                    ; Stereo Mixer
                    ;---------------------------------
                    ; - in:
                    ; EDX - MIXDATA structure address
                    ; - out:
                    ; EAX - Output Level
                    ;---------------------------------
                    ; ...
                    ret

;----------------------------------------------------------------------------;

                    ;----------------
                    ; PC synchronize
                    ;----------------
PC_Sync:
;                    call    Kbd_Process
                    call    TMR_ReadCounter
                    sub     eax, TMRLastSyncLow
                    neg     eax
                    cmp     eax, TMRPeriodSyncLow
                    jc      Psyn_1
                    mov     eax, TMRPeriodSyncLow
                    sub     TMRLastSyncLow, eax
                    call    Sync_ZX
                    jmp     PC_Sync
Psyn_1:
                    cmp     Z80SynchroCount, 0
                    jz      PC_Sync
                    dec     Z80SynchroCount
                    mov     eax, [ebx].Z80DividePoint
                    mov     ecx, [eax + [ebx].Z80DivideTable + 4]
                    mov     eax, [eax + [ebx].Z80DivideTable]
                    add     [ebx].Z80DividePoint, ecx
                    add     [ebx].Z80TactsCnr, eax
                    js      PC_Sync
                    jmp     PW4sync_End

;----------------------------------------------------------------------------;

                    ;----------------------
                    ; ZX interrupt service
                    ;----------------------
ZX_Interrupt:
                    cmp     [ebx].Z80InterptCount, 0
                    jnz     Zint_1
                    sub     VideoFrameCount, 1
                    jnc     Zint_1
                    mov     eax, VideoScheme
                    mov     [ebx].VideoRefreshRoutine, eax
Zint_1:
                    sub     BottomMessageTimer, 1
                    adc     BottomMessageTimer, 0
                    mov     ecx, [ebx].Z80InterptCount
                    add     ecx, ebp
                    cmp     ecx, CFG_ULAInterruptLen
                    sbb     eax, eax
                    and     ecx, eax
                    mov     [ebx].Z80InterptCount, ecx
                    jnz     Zint_15
                    mov     eax, CFG_Z80TactsAsOne
                    mov     [ebx].Z80MinTactsAs1, eax
Zint_15:
                    not     al
                    and     al, FA_INT_
                    or      [ebx].FlagsA, al
                    mov     ah, [ebx].FlagsA
                    test    ah, (FA_INT_ + FA_ONESHOTDI + FA_IFF1_)
                    jnz     Zint_3
                    test    ah, FA_HALT
                    setz    al
                    sub     al, 1
                    adc     edi, 0
                    and     ah, not(FA_HALT)
                    or      ah, (FA_IFF1_ + FA_IFF2_)
                    mov     [ebx].FlagsA, ah

                    mov     eax, edi
                    mov     di, [ebx].rSP
                    dec     edi
                    WRITE   di
                    mov     [esi], ah
                    dec     edi
                    WRITE   di
                    mov     [esi], al
                    mov     [ebx].rSP, di
                    mov     edi, 56
                    mov     [ebx].rMP, di

                    mov     al, [ebx].FlagsA
                    and     al, (FA_INTMODE0 + FA_INTMODE1)
                    jz      Zintmode_0
                    dec     al
                    jz      Zintmode_1
Zintmode_2:
                    mov     al, [ebx].BusIntVector
                    mov     ah, [ebx].regI
                    READ    ax
                    inc     eax
                    mov     cl, [esi]
                    READ    ax
                    mov     ch, [esi]
                    mov     edi, ecx
                    mov     [ebx].rMP, di
                    mov     eax, 19
                    add     ebp, eax
                    add     [ebx].Z80InterptCount, eax
Zint_3:
                    jmp     ZIexec_End
Zintmode_0:
                    mov     eax, 12
                    add     ebp, eax
                    add     [ebx].Z80InterptCount, eax
                    jmp     ZIexec_End
Zintmode_1:
                    mov     eax, 13
                    add     ebp, eax
                    add     [ebx].Z80InterptCount, eax
                    jmp     ZIexec_End

;----------------------------------------------------------------------------;

                    ;--------------------
                    ; PC hotkeys service
                    ;--------------------
PC_Hotkeys:
                    mov     [ebx].rAFxAF, edx
                    mov     [ebx].rPC, di
                    test    KeyboardF1, 001h
                    jz      Phkeys_1
                    ; [F1] - Reinit Video Hardware
                    mov     al, 1
                    call    VGA_InitMode
                    and     KeyboardF1, not(001h)
                    jmp     Phkeys_End
Phkeys_1:
                    test    KeyboardF1, 002h
                    jz      Phkeys_2
                    ; [F2] - Save Snapshot
                    mov     al, 1
                    call    LoadSaveSnapshot
                    jnc     ZXCommandMonitor
Phkeys_2:
                    test    KeyboardF1, 004h
                    jz      Phkeys_3
                    ; [F3] - Load Snapshot
                    ; reset FDC
                    lea     esi, [ebx].FDCtrd
                    call    FDC_Init
                    ; reset AY
                    lea     esi, [ebx].AY1st
                    call    AY_Init
                    xor     al, al
                    call    LoadSaveSnapshot
                    jmp     ZXCommandMonitor
Phkeys_3:
                    test    KeyboardF2, 002h
                    jz      Phkeys_4
                    ; [F7] - AY Turn ON/OFF
                    and     KeyboardF2, 0FDh
                    xor     CFG_MixAudioEnable, 002h
                    jmp     ZXCommandMonitor
Phkeys_4:
                    test    KeyboardF2, 004h
                    jz      Phkeys_5
                    ; [F8] - Beeper Turn ON/OFF
                    and     KeyboardF2, 0FBh
                    xor     CFG_MixAudioEnable, 001h
                    jmp     ZXCommandMonitor
Phkeys_5:
                    test    KeyboardF2, 001h
                    jz      Phkeys_6
                    ; [F6] - Reset to 48k
                    call    Std48k_Init
                    jmp     Phkeys_8
Phkeys_6:
                    test    KeyboardF1, 008h
                    jz      Phkeys_7
                    ; [F4] - Reset to 128k TR-DOS
                    call    Ext128k_Init
                    jmp     Phkeys_8
Phkeys_7:
                    test    KeyboardF1, 010h
                    jz      Phkeys_9
                    ; [F5] - Reset to 128k
                    call    Std128k_Init
Phkeys_8:
                    ; reset
                    and     [ebx].FlagsA, FA_INT_
                    or      [ebx].FlagsA, (FA_IFF1_ + FA_IFF2_)
                    xor     edi, edi
                    mov     [ebx].rIR, di
                    ; reset FDC
                    lea     esi, [ebx].FDCtrd
                    call    FDC_Init
                    ; reset AY
                    lea     esi, [ebx].AY1st
                    call    AY_Init
                    add     ebp, 3              ; RESET signal minimum length
                    jmp     Phkeys_End
Phkeys_9:
                    ; [F9] - Decrease Top Border Area Size (multicolor X/Y-position)
                    test    KeyboardF2, 008h
                    jz      Phkeys_B
                    and     KeyboardF2, not(008h)
                    mov     al, KeyboardF4
                    or      al, KeyboardF5
                    test    al, 004h
                    jnz     Phkeys_A
                    mov     eax, CFG_ULAResolutionX
                    sub     VideoGPause, eax
                    jnc     Phkeys_End
                    add     VideoGPause, eax
                    jmp     Phkeys_End
Phkeys_A:
                    sub     VideoGPause, 1
                    jnc     Phkeys_End
                    inc     VideoGPause
                    jmp     Phkeys_End
Phkeys_B:
                    ; [F10] - Increase Top Border Area Size (multicolor X/Y-position)
                    test    KeyboardF2, 010h
                    jz      Phkeys_D
                    and     KeyboardF2, not(010h)
                    mov     al, KeyboardF4
                    or      al, KeyboardF5
                    test    al, 004h
                    jnz     Phkeys_C
                    mov     eax, CFG_ULAResolutionX
                    add     VideoGPause, eax
                    jnz     Phkeys_End
                    sub     VideoGPause, eax
                    jmp     Phkeys_End
Phkeys_C:
                    add     VideoGPause, 1
                    jnc     Phkeys_End
                    dec     VideoGPause
                    jmp     Phkeys_End
Phkeys_D:
                    ; [F11] - Generate NMI
                    test    KeyboardF3, 001h
                    jz      Phkeys_E
                    and     KeyboardF3, not(001h)
                    and     [ebx].FlagsA, not(FA_IFF1_)
                    or      [ebx].FlagsA, al

                    lea     ebp, [ebp + 11]
                    mov     eax, edi
                    mov     di, [ebx].rSP
                    dec     edi
                    WRITE   di
                    mov     [esi], ah
                    dec     edi
                    WRITE   di
                    mov     [esi], al
                    mov     [ebx].rSP, di
                    mov     edi, 066h
                    mov     [ebx].rPC, di
                    mov     [ebx].rMP, di

                    mov     eax, [ebx].RomBank3Adr    ; select TR-DOS
                    SR16K0  eax
                    jmp     Phkeys_End
Phkeys_E:
                    jmp     Phkeys_End

;----------------------------------------------------------------------------;

Display_DebugInfo:
                    ; verify for RTC seconds change
                    xor     eax, eax
                    call    Get_CurrentSecond
                    cmp     al, LastPCSecond
                    jz      Ddinfo_1
                    mov     LastPCSecond, al
                    xor     eax, eax
                    xchg    eax, SyncEventsCnr
                    mov     LastSyncEvents, eax
Ddinfo_1:
                    mov     eax, CFG_VideoYVisible
                    shr     eax, 3
                    dec     al
                    mov     SymbolYPosition, al
                    mov     SymbolXPosition, 0
                    cmp     BottomMessageTimer, 0
                    jz      Ddinfo_2
                    PSTR    BottomMessageAdr
                    ret
Ddinfo_2:
                    ; if no current message, display counter(s)
                    PCHR    " "
                    PDECD   LastSyncEvents
                    PCHR    " "
                    mov     eax, LastSoundSamples
                    test    eax, eax
                    jz      Ddinfo_3
                    PDECD   LastSoundSamples
                    PCHR    "/"
                    PDECD   CFG_SBBufferSize
                    PCHR    " "
Ddinfo_3:
                    PDECD   [ebx].LastFrameTacts
                    PCHR    "/"
                    PDECD   CFG_ULATactsInFrame
                    mov     eax, Read8kBank0Adr
                    cmp     eax, [ebx].RomBank_Menu128k
                    mov     cl, 0
                    jz      Ddinfo_4
                    cmp     eax, [ebx].RomBank_Basic48k
                    mov     cl, 1
                    jz      Ddinfo_4
                    cmp     eax, [ebx].RomBank_Monitor
                    mov     cl, 2
                    jz      Ddinfo_4
                    cmp     eax, [ebx].RomBank_TRDOS
                    mov     cl, 3
                    jnz     Ddinfo_5
Ddinfo_4:
                    PCHR    " "
                    PCHR    "R"
                    PCHR    "O"
                    PCHR    "M"
                    PDECB   cl
Ddinfo_5:
                    PCHR    " "
                    PCHR    "R"
                    PCHR    "A"
                    PCHR    "M"
                    mov     al, [ebx].SystemZX128
                    and     al, 007h
                    PDECB   al
                    PCHR    " "
;                    mov     eax, [ebx].VideoCounter
;                    PHEXB   al
;                    PDECD   Z80SynchroCount
;                    PCHR    " "
                    call    TMR_ReadCounter
                    PHEXD   eax
                    PCHR    " "
                    PCHR    " "
                    ret

;----------------------------------------------------------------------------;

                    ;-----------------------------------
                    ; Text-mode Update with SOFT Border
                    ;-----------------------------------
TMUpdate_SBorder:
                    ; "general" pause area
                    mov     eax, VideoGPause
                    mov     VideoCurItem, 0
                    mov     [ebx].VideoWordCount, eax
                    mov     al, CFG_VideoFrameSkip
                    mov     [ebx].VideoTablePoint, offset ZXScrnAdrTable
                    mov     [ebx].SystemZXSync, -1
                    mov     VideoFrameCount, al
                    mov     eax, VideoGfxBaseAdr
                    mov     [ebx].Temp_rPC, eax
                    VPAUSE
                    ; top border area
                    mov     eax, CFG_VideoUBVisible
                    test    eax, eax
                    jz      TUS_lborder_start
                    mov     VideoLineCount, al
TUS_tborder_1:
                    movzx   eax, CFG_VideoXVisible
                    mov     VideoCurItem, 1
                    mov     VideoBytesCount, eax
                    TBORDER
                    ; "upper-center" pause
                    movzx   eax, VideoLPause
                    inc     VideoCurItem
                    mov     VideoBytesCount, eax
                    HPAUSE
                    dec     VideoLineCount
                    jnz     TUS_tborder_1

TUS_lborder_start:
                    ; left border area
                    mov     VideoLineCount, 0C0h
TUS_lborder_1:
                    movzx   eax, CFG_VideoLBVisible
                    test    al, al
                    jz      TUS_mainscr_start
                    mov     VideoCurItem, 3
                    mov     VideoBytesCount, eax
                    TBORDER
TUS_mainscr_start:
                    ; main screen area
                    mov     esi, [ebx].VideoTablePoint
                    mov     eax, [esi]
                    inc     VideoCurItem
                    mov     [ebx].VideoCurLineAdr, eax
                    mov     VideoBytesCount, 0
                    mov     eax, [esi + 004h]
                    mov     [ebx].VideoCurAttrAdr, eax
                    lea     esi, [esi + 008h]
                    mov     [ebx].VideoTablePoint, esi
                    TSCREEN
                    ; right border area
                    movzx   eax, VideoRBorder
                    test    al, al
                    jz      TUS_cpause_start
                    inc     VideoCurItem
                    mov     VideoBytesCount, eax
                    mov     [ebx].SystemZXSync, -1
                    TBORDER
TUS_cpause_start:
                    ; "center" pause
                    movzx   eax, VideoLPause
                    inc     VideoCurItem
                    mov     VideoBytesCount, eax
                    HPAUSE
                    dec     VideoLineCount
                    jnz     TUS_lborder_1

TUS_bborder_start:
                    ; bottom border area
                    mov     eax, CFG_VideoYVisible
                    sub     eax, 0C0h
                    jz      TUS_end
                    sub     eax, CFG_VideoUBVisible
                    jz      TUS_end
                    mov     VideoLineCount, al
TUS_bborder_1:
                    movzx   eax, CFG_VideoXVisible
                    mov     VideoCurItem, 7
                    mov     VideoBytesCount, eax
                    TBORDER
                    ; "lower-center" pause
                    movzx   eax, VideoLPause
                    inc     VideoCurItem
                    mov     VideoBytesCount, eax
                    HPAUSE
                    dec     VideoLineCount
                    jnz     TUS_bborder_1
TUS_end:
                    ; stop
                    mov     VideoCurItem, 9
                    USSTOP

;----------------------------------------------------------------------------;

Std48k_Init:
                    push    eax esi

                    ; recall previously loaded ROMs
                    call    ROMs_Recall
                    ; set memory configuration and disable #7FFD
                    mov     ah, 030h
                    call    ZX128k_SetConfig
                    ; bank #0 (ROM) select
                    mov     eax, [ebx].RomPrimary
                    SR16K0  eax
                    ; bank #1 (screen) select
                    mov     eax, [ebx].RamBank5Adr
                    SRW16K1 eax
                    ; bank #2 select
                    mov     eax, [ebx].RamBank2Adr
                    SRW16K2 eax
                    ; reset FDC
                    lea     esi, [ebx].FDCtrd
                    call    FDC_Init
                    ; reset AY
                    lea     esi, [ebx].AY1st
                    call    AY_Init

                    pop     esi eax
                    ret

;----------------------------------------------------------------------------;

Ext48k_Init:
                    push    eax esi

                    ; recall previously loaded ROMs
                    call    ROMs_Recall
                    ; set memory configuration and disable #7FFD
                    mov     ah, 030h
                    call    ZX128k_SetConfig
                    ; bank #0 (ROM) select
                    mov     eax, [ebx].RomSecondary
                    SR16K0  eax
                    ; bank #1 (screen) select
                    mov     eax, [ebx].RamBank5Adr
                    SRW16K1 eax
                    ; bank #2 select
                    mov     eax, [ebx].RamBank2Adr
                    SRW16K2 eax
                    ; reset FDC
                    lea     esi, [ebx].FDCtrd
                    call    FDC_Init
                    ; reset AY
                    lea     esi, [ebx].AY1st
                    call    AY_Init

                    pop     esi eax
                    ret

;----------------------------------------------------------------------------;

Std128k_Init:
                    push    eax esi

                    ; recall previously loaded ROMs
                    call    ROMs_Recall
                    ; set memory configuration
                    xor     ah, ah
                    call    ZX128k_SetConfig
                    ; bank #0 (ROM) select
                    mov     eax, [ebx].RomPrimary
                    SR16K0  eax
                    ; bank #1 (screen) select
                    mov     eax, [ebx].RamBank5Adr
                    SRW16K1 eax
                    ; bank #2 select
                    mov     eax, [ebx].RamBank2Adr
                    SRW16K2 eax
                    ; reset FDC
                    lea     esi, [ebx].FDCtrd
                    call    FDC_Init
                    ; reset AY
                    lea     esi, [ebx].AY1st
                    call    AY_Init

                    pop     esi eax
                    ret

;----------------------------------------------------------------------------;

Ext128k_Init:
                    push    eax esi

                    ; recall previously loaded ROMs
                    call    ROMs_Recall
                    ; set memory configuration
                    mov     ah, 010h
                    call    ZX128k_SetConfig
                    ; bank #0 (ROM) select
                    mov     eax, [ebx].RomSecondary
                    SR16K0  eax
                    ; bank #1 (screen) select
                    mov     eax, [ebx].RamBank5Adr
                    SRW16K1 eax
                    ; bank #2 select
                    mov     eax, [ebx].RamBank2Adr
                    SRW16K2 eax
                    ; reset FDC
                    lea     esi, [ebx].FDCtrd
                    call    FDC_Init
                    ; reset AY
                    lea     esi, [ebx].AY1st
                    call    AY_Init

                    pop     esi eax
                    ret

;----------------------------------------------------------------------------;

ROMs_Recall:
                    push    eax esi

                    mov     eax, [ebx].RomBank_Menu128k
                    mov     esi, [ebx].RomBank_Basic48k
                    mov     [ebx].RomBank0Adr, eax
                    mov     [ebx].RomBank1Adr, esi
                    mov     eax, [ebx].RomBank_Monitor
                    mov     esi, [ebx].RomBank_TRDOS
                    mov     [ebx].RomBank2Adr, eax
                    mov     [ebx].RomBank3Adr, esi

                    pop     esi eax
                    ret

;****************************************************************************;

.DATA

; Tacts Counters
SoundGenTactsCnr    dd      0           ;   Z80      ( ..  AY)
SoundGenTactsConst  dd      0           ;   SoundGenTactsCnr
SamplerTactsCnr     dd      0           ;   Z80      DMA
SamplerTactsConst   dd      0           ;   SamplerTactsCnr
LastSoundSamples    dd      0
SoundSamplesCnr     dd      0


ULADelaysTable      label   byte
                    db      128 dup(0)

CurrentTotalLevels  dd      0
CurrentTotalLevelL  dd      0
CurrentTotalLevelR  dd      0

BottomMessageAdr    dd      offset MsgDummy
BottomMessageTimer  db      0

LastPCSecond        db      0
LastPCScrnRetraces  db      0
LastSyncEvents      dd      0
SyncEventsCnr       dd      0
TMRLastSyncLow      dd      0

VideoBytesCount     dd      0
VideoBytesSkip      dd      0
VideoLineCount      db      0
AYGenClockFreq      dd      0

VideoFlashTag       dd      07Fh
VideoFrameCount     db      0
VideoRBorder        db      0
VideoCurItem        db      0

Z80SynchroCount     dd      0
Z80SynchroAdd       dd      0



ZXCore              ZXDATA  <>

MsgDummy            db      " ", 0
MsgReset48kTRDOS    db      "Resetting to 48K TR-DOS mode.", 0
MsgReset128kTRDOS   db      "Resetting to 128K TR-DOS mode.", 0
MsgReset48kBASIC    db      "Resetting to 48K BASIC mode.", 0
MsgReset128kBASIC   db      "Resetting to 128K BASIC mode.", 0
MsgSnapLoadingNow   db      "Loading snapshot: ", 0
MsgSnapLoadOK       db      " snapshot loaded successfully.", 0
MsgSnapLoadErr      db      " snapshot loading error.", 0
MsgSnapSavingNow    db      "Saving snapshot: ", 0
MsgSnapSaveOK       db      " snapshot saved successfully.", 0
MsgSnapSaveErr      db      " snapshot saving error.", 0
MsgSnapUnsupported  db      "Unsupported snapshot file format.", 0
MsgFileOpenErr      db      "Error opening file: ", 0
MsgFileCreateErr    db      "Error creating file: ", 0
MsgFileCloseErr     db      "Warning: couldn't close file: ", 0
MsgEnterSnapName    db      "Enter snapshot filename: ", 0
MsgAYEnabled        db      "AY sound enabled.", 0
MsgAYDisabled       db      "AY sound disabled.", 0
MsgBeeperEnabled    db      "Beeper sound enabled.", 0
MsgBeeperDisabled   db      "Beeper sound disabled.", 0

;****************************************************************************;

                end