
include defines.inc
include macro.inc


;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

; -----------------------------------------------------------------
; -=* S-C-A-N---I-N-I-T-I-A-L-I-Z-A-T-I-O-N---T-E-X-T---F-I-L-E *=-
; -----------------------------------------------------------------
INI_Load:
                pushad

                PSTR    TxtLoadINI
                lea     edx, INIFileName
                ; open ini file
                mov     al, FILE_OPEN_READONLY
                call    File_Open
                jc      LifMAIN_14
                mov     CurErrorCounter, 0
                ; store file handler
                mov     CurFileHandler, eax
                push    ds
                pop     es
                ; store stack
                mov     CurStackPointer, esp
                mov     ecx, 1                  ; ECX = line numbers
                ; read first part
                lea     ebp, TempBuffer_A
                add     ebp, tmpbufa_size
                mov     esi, ebp
                call    LifMAIN_8
LifMAIN_1:
                ; find matched keyword
                lea     ebx, IniKeysTable
                call    LifGetKeywordID
                jnc     LifMAIN_3
LifMAIN_2:
                ; unknown keyword - syntax error
                call    LifMsgSyntaxError
                jmp     LifMAIN_1
LifMAIN_3:
                ; skip "equal" symbol
                or      dl, 1
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                jz      LifMAIN_5
LifMAIN_4:
                ; no parameter till end of line - missing parameter
                call    LifMsgParameterMissing
                jmp     LifMAIN_1
LifMAIN_5:
                mov     ah, [edi]
                cmp     ah, "="
                jnz     LifMAIN_2
LifMAIN_6:
                ; call keyword service routine
                call    LifMAIN_7
                or      dl, 1
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                jnz     LifMAIN_4
                call    ebx
                jmp     LifMAIN_1

; << SYNC_FREQUENCY >> --------------------------------------------
Lif_SyncFrequency:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      Lif_SF_2
                cmp     eax, 596591
                ja      Lif_SF_1
                mov     CFG_SynchroFreq, eax
                ret
Lif_SF_1:
                PSTR    TxtNumberTooBig
                mov     ax, cx
                call    Print_DecWord
                PENT
                ret
Lif_SF_2:
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_VRETRACE
                jnz     LifMsgParameterInvalid
                mov     CFG_SynchroFreq, 0
                ret

; << AY_VOLUME >> -------------------------------------------------
Lif_AYVolume:
                ret

; << SB_ADDRESS >> ------------------------------------------------
Lif_SBAddress:
                mov     al, 010h                ; hex port number as default
                call    LifReadNumber
                jc      Lif_SBA_1
                test    eax, 0FFFFF00Fh
                jnz     LifMsgParameterInvalid
                cmp     ah, 2
                jnz     LifMsgParameterInvalid
                mov     CFG_SBAddress, ax
                ret
Lif_SBA_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     CFG_SBAddress, 0
                ret

; << SB_VOLUME >> -------------------------------------------------
Lif_SBVolume:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jnc     Lif_SBV_1
                cmp     al, 1
                jnz     LifMessage
                mov     eax, 3
                xor     ebx, ebx
                call    LifMinMax
                jnc     Lif_SBV_1
                ret
Lif_SBV_1:
                cmp     eax, 4
                jnc     Lif_SF_1
                mov     CFG_SBMasterVolume, al
                ret

; << SB_STEREO >> -------------------------------------------------
Lif_SBStereo:
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      Lif_SBS_12
                cmp     ebx, ID_INICMN_ON
                jnz     Lif_SBS_01
                or      CFG_MixStereo, 080h
                jmp     Lif_SBS_10
Lif_SBS_01:
                cmp     ebx, ID_INICMN_OFF
                jnz     Lif_SBS_10
                and     CFG_MixStereo, not(080h)
Lif_SBS_10:
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                jz      Lif_SBS_11
                ret
Lif_SBS_11:
                cmp     byte ptr [edi], ","
                jnz     LifMsgSyntaxError
                call    LifMAIN_7
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                jnz     LifMsgParameterMissing
                jmp     Lif_SBStereo
Lif_SBS_12:
                lea     ebx, CFG_MixStereoLeftA
                mov     al, [edi]
                cmp     al, "L"
                jz      Lif_SBS_13
                cmp     al, "l"
                jnz     Lif_SBS_14
Lif_SBS_13:
                call    LifMAIN_7
                cmp     byte ptr [edi], "<"
                jnz     LifMsgSyntaxError
                mov     byte ptr [ebx], 0
                ; AY channel A ("A", "a" or "1")
                call    LifMAIN_7
                cmp     byte ptr [edi], "A"
                setz    ah
                cmp     byte ptr [edi], "a"
                setz    al
                or      al, ah
                cmp     byte ptr [edi], "1"
                setz    ah
                or      al, ah
                shl     al, 1
                or      [ebx], al
                ; AY channel B ("B", "b" or "2")
                ;call    LifMAIN_7
                cmp     byte ptr [edi], "B"
                setz    ah
                cmp     byte ptr [edi], "b"
                setz    al
                or      al, ah
                cmp     byte ptr [edi], "2"
                setz    ah
                or      al, ah
                shl     al, 2
                or      [ebx], al
                ; AY channel C ("C", "c" or "3")
                ;call    LifMAIN_7
                cmp     byte ptr [edi], "C"
                setz    ah
                cmp     byte ptr [edi], "c"
                setz    al
                or      al, ah
                cmp     byte ptr [edi], "3"
                setz    ah
                or      al, ah
                shl     al, 3
                or      [ebx], al
                ; Beeper ("D", "d", "0" or "4")
                ;call    LifMAIN_7
                cmp     byte ptr [edi], "D"
                setz    ah
                cmp     byte ptr [edi], "d"
                setz    al
                or      al, ah
                cmp     byte ptr [edi], "0"
                setz    ah
                or      al, ah
                cmp     byte ptr [edi], "4"
                setz    ah
                or      al, ah
                or      [ebx], al
                ; Tape output ("T", "t" or "5")
                ;call    LifMAIN_7
                cmp     byte ptr [edi], "T"
                setz    ah
                cmp     byte ptr [edi], "t"
                setz    al
                or      al, ah
                cmp     byte ptr [edi], "5"
                setz    ah
                or      al, ah
                shl     al, 3
                or      [ebx], al

                cmp     byte ptr [edi], ">"
                jnz     LifMsgSyntaxError
                call    LifMAIN_7
                jmp     Lif_SBS_10
Lif_SBS_14:
                lea     ebx, CFG_MixStereoRightA
                cmp     al, "R"
                jz      Lif_SBS_13
                cmp     al, "r"
                jz      Lif_SBS_13
                jmp     LifMsgSyntaxError

; << Z80_SPEED >> -------------------------------------------------
Lif_Z80Speed:
                mov     al, 08Ah                ; decimal constant only
                call    LifReadNumber
                jc      Lif_Z80S_4
                cmp     eax, 000010000h
                jnc     Lif_SF_1
                mov     ebx, 10
                push    edx
                mul     ebx
                pop     edx
                mov     ebx, eax
                mov     al, [edi]
                cmp     al, "."
                jz      Lif_Z80S_1
                cmp     al, ","
                jnz     Lif_Z80S_2
Lif_Z80S_1:
                call    LifMAIN_7
                movzx   eax, byte ptr [edi]
                cmp     al, "0"
                jc      LifMsgParameterInvalid
                cmp     al, "9"
                ja      LifMsgParameterInvalid
                call    LifMAIN_7
                sub     al, "0"
                add     ebx, eax
                jc      Lif_SF_1
Lif_Z80S_2:
                mov     CFG_Z80Speed, ebx
                ret
Lif_Z80S_3: 
                PSTR    TxtDecimalOnly
                mov     ax, cx
                call    Print_DecWord
                PENT
                ret
Lif_Z80S_4: 
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     CFG_Z80Speed, 0
                ret

; << SAMPLE_RATE >> -----------------------------------------------
Lif_SampleRate:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      LifMessage
                test    eax, eax
                jz      LifMsgParameterInvalid
                cmp     eax, 96000
                ja      Lif_SF_1
                cmp     eax, 3892
                jc      Lif_SF_1
                mov     CFG_SBSampleRate, eax
                ret
Lif_SR_1:
                PSTR    TxtNumTooSmall
                mov     ax, cx
                call    Print_DecWord
                PENT
                ret

; << PRELIMINARY_VIDEOMODE >> -------------------------------------
Lif_PreliminaryVideoMode:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      LifMessage
                cmp     eax, 00100h
                jnc     Lif_SF_1
                mov     CFG_VideoModePre, ax
                ret

; << MAIN_VIDEOMODE >> --------------------------------------------
Lif_MainVideoMode:
                xor     ebx, ebx
Lif_MVM_1:
                cmp     bh, 1
                jnc     LifMsgParameterInvalid
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      Lif_MVM_3
                cmp     eax, 080h
                jnc     Lif_SF_1
                cmp     eax, vmtbl_size
                jnc     Lif_SF_1
                and     ax, not(VM_FLAGS_MASK)
                mov     CFG_VideoModeMain, ax
                inc     bl
                xor     bh, bh
Lif_MVM_2:
                push    eax
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                pop     eax
                jnz     Lif_MVM_9
                cmp     byte ptr [edi], ","
                jnz     LifMsgSyntaxError
                push    eax
                call    LifMAIN_7
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                pop     eax
                jnz     LifMsgParameterMissing
Lif_MVM_3:
                inc     bh
                push    ebx
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                mov     eax, ebx
                pop     ebx
                jc      Lif_MVM_1
                cmp     eax, ID_INICMN_BW
                jnz     Lif_MVM_4
                ; black-and-white
                or      CFG_VideoModeMain, VM_BLACKANDWHITE
                xor     bh, bh
                jmp     Lif_MVM_2
Lif_MVM_4:
                cmp     eax, ID_INICMN_COLOR
                jnz     Lif_MVM_5
                ; color
                and     CFG_VideoModeMain, not(VM_BLACKANDWHITE)
                xor     bh, bh
                jmp     Lif_MVM_2
Lif_MVM_5:
                cmp     eax, ID_INICMN_FORCEVGA
                jnz     Lif_MVM_6
                ; force vga
                or      CFG_VideoModeMain, VM_FORCEVGA
                xor     bh, bh
                jmp     Lif_MVM_2
Lif_MVM_6:
                cmp     eax, ID_INICMN_NOVBE
                jnz     Lif_MVM_7
                ; no vbe
                or      CFG_VideoModeMain, VM_DONOTUSEVBE
                xor     bh, bh
                jmp     Lif_MVM_2
Lif_MVM_7:
                cmp     eax, ID_INICMN_BUFFERED
                jnz     Lif_MVM_8
                ; buffered
                or      CFG_VideoModeMain, VM_BUFFERED
                xor     bh, bh
                jmp     Lif_MVM_2
Lif_MVM_8:
                cmp     eax, ID_INICMN_INTERLACE
                jnz     LifMsgParameterInvalid
                ; buffered
                or      CFG_VideoModeMain, VM_INTERLACED
                xor     bh, bh
                jmp     Lif_MVM_2
Lif_MVM_9:
                test    bl, bl
                jz      LifMsgParameterMissing
                ret

; << Z80_CLOCK_MUL >> ---------------------------------------------
Lif_Z80ClockMul:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      LifMessage
                test    eax, eax
                jz      LifMsgParameterInvalid
                cmp     eax, 17
                jnc     Lif_SF_1
                mov     CFG_Z80ClockMul, eax
                ret

; << SB_DMA >> ----------------------------------------------------
Lif_SBDMAChannel:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      Lif_SBDC_1
                cmp     eax, 8
                jnc     Lif_SF_1
                mov     CFG_SBDMAChannel, al
                ret
Lif_SBDC_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     CFG_SBDMAChannel, 8
                ret

; << FRAME_SKIP >> ------------------------------------------------
Lif_FrameSkip:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      LifMessage
                cmp     eax, 00100h
                jnc     Lif_SF_1
                mov     CFG_VideoFrameSkip, al
                ret

; << INTERRUPT_LENGTH >> ------------------------------------------
Lif_InterruptLength:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      LifMessage
                test    eax, eax
                jz      LifMsgParameterInvalid
                cmp     eax, 000010000h
                jnc     Lif_SF_1
                mov     CFG_ULAInterruptLen, eax
                ret

; << FLASH_SPEED >> -----------------------------------------------
Lif_FlashSpeed:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jc      Lif_FS_1
                cmp     eax, 00100h
                jnc     Lif_SF_1
                mov     CFG_ULAFlashFreq, al
                ret
Lif_FS_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_OFF
                jnz     LifMsgParameterInvalid
                mov     CFG_ULAFlashFreq, 0
                ret

; << BEEPER_VOLUME >> ---------------------------------------------
Lif_ZXBeeperVolume:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jnc     Lif_ZXBV_1
                cmp     al, 1
                jnz     LifMessage
                mov     eax, 03Fh
                xor     ebx, ebx
                call    LifMinMax
                jnc     Lif_ZXBV_1
                ret
Lif_ZXBV_1:
                cmp     eax, 040h
                jnc     Lif_SF_1
                shl     eax, 8
                xor     eax, 03F80h
                mov     CFG_MixBeeperVolume, eax
                ret

; << BEEPER_DEVICE >> ---------------------------------------------
Lif_ZXBeeperDevice:
                mov     eax, 001h               ; SoundBlaster and/or PCSpeaker.
Lif_SDev_1:
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_ON
                jz      Lif_SDev_2
                cmp     ebx, ID_INICMN_OFF
                jz      Lif_SDev_3
                cmp     ebx, ID_INICMN_DEVSB
                jz      Lif_SDev_4
                cmp     ebx, ID_INICMN_DEVSPK
                jnz     LifMsgParameterInvalid
                ; PC Speaker
                or      CFG_MixAudioMapA, eax
                jmp     Lif_SDev_5
Lif_SDev_2:
                ; ON
                or      CFG_MixAudioEnable, eax
                jmp     Lif_SDev_5
Lif_SDev_3:
                ; OFF
                mov     ebx, eax
                not     ebx
                and     CFG_MixAudioEnable, ebx
                jmp     Lif_SDev_5
Lif_SDev_4:
                ; Sound Blaster
                or      CFG_MixAudioMapB, eax
Lif_SDev_5:
                call    LifSkipComments
                cmp     byte ptr [edi], ","
                jnz     Lif_SDev_6
                call    LifMAIN_7
                push    eax
                mov     ax, cx
                call    LifSkipComments
                cmp     ax, cx
                pop     eax
                jnz     LifMsgParameterMissing
                jmp     Lif_SDev_1
Lif_SDev_6:
                ret

; << AY_DEVICE >> -------------------------------------------------
Lif_AYDevice:
                mov     eax, 002h               ; SoundBlaster and/or PCSpeaker.
                jmp     Lif_SDev_1

; << REAL_TAPE >> -------------------------------------------------
Lif_RealTape:
                lea     ebx, IniParsTable
                call    LifGetKeywordID      ; ON or OFF.
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_ON
                jnz     Lif_RT_1
                or      CFG_MixAudioEnable, 080000000h
                ret
Lif_RT_1:
                cmp     ebx, ID_INICMN_OFF
                jnz     LifMsgParameterInvalid
                and     CFG_MixAudioEnable, not(080000000h)
                ret

; << STEREO_SEPARATION >> -----------------------------------------
Lif_StereoSeparation:
                mov     al, 00Ah
                call    LifReadNumber           ; values 0-7 for 8-bit sound and
                jc      Lif_SS_1                ; 0-15 for 16-bit sound.
                and     eax, eax
                jz      LifMsgParameterInvalid
                cmp     eax, 010h
                jnc     Lif_SF_1
                mov     CFG_MixStereoSepar, al
                ret
Lif_SS_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_OFF
                jnz     LifMsgParameterInvalid
                mov     CFG_MixStereoSepar, -1
                ret

; << STEREO_REVERSE >> --------------------------------------------
Lif_ReverseStereo:
                lea     ebx, IniParsTable
                call    LifGetKeywordID         ; ON or OFF.
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_ON
                jnz     Lif_RS_1
                or      CFG_MixStereo, 040h
                ret
Lif_RS_1:
                cmp     ebx, ID_INICMN_OFF
                jnz     LifMsgParameterInvalid
                and     CFG_MixStereo, not(040h)
                ret

; << ZX_RESOLUTION_X >> -------------------------------------------
Lif_ZXResolutionX:
                mov     al, 00Ah
                call    LifReadNumber           ; values 32-64.
                jc      LifMessage
                cmp     eax, 041h
                jnc     Lif_SF_1
                cmp     eax, 020h
                jc      Lif_SR_1
                mov     CFG_ULAResolutionX, eax
                ret

; << ZX_RESOLUTION_Y >> -------------------------------------------
Lif_ZXResolutionY:
                mov     al, 00Ah
                call    LifReadNumber           ; values 256-384.
                jc      LifMessage
                cmp     eax, 00181h
                jnc     Lif_SF_1
                cmp     eax, 00100h
                jc      Lif_SR_1
                mov     CFG_ULAResolutionY, eax
                ret

; << UPPER_BORDER_SIZE >> -----------------------------------------
Lif_ZXUpperBorderSize:
                mov     al, 00Ah
                call    LifReadNumber           ; values 0-128.
                jc      LifMessage
                cmp     eax, 081h
                jnc     Lif_SF_1
                mov     CFG_ULAUpperBorder, al
                ret

; << LEFT_BORDER_SIZE >> ------------------------------------------
Lif_ZXLeftBorderSize:
                mov     al, 00Ah
                call    LifReadNumber           ; values 0-32.
                jc      LifMessage
                cmp     eax, 020h
                ja      Lif_SF_1
                mov     CFG_ULALeftBorder, al
                ret

; << X_VISIBLE >> -------------------------------------------------
Lif_XVisible:
                mov     al, 00Ah
                call    LifReadNumber           ; values 32-56.
                jc      LifMessage
                cmp     eax, 56
                ja      Lif_XV_1
                cmp     eax, 32
                jc      Lif_SR_1
                mov     CFG_VideoXVisible, al
                ret
Lif_XV_1:
                cmp     eax, 256
                jc      Lif_SR_1
                cmp     eax, 448
                ja      Lif_SF_1
                shr     eax, 3
                mov     CFG_VideoXVisible, al
                ret

; << Y_VISIBLE >> -------------------------------------------------
Lif_YVisible:
                mov     al, 00Ah
                call    LifReadNumber           ; values 192-320.
                jc      LifMessage
                cmp     eax, 320
                ja      Lif_SF_1
                cmp     eax, 192
                jc      Lif_SR_1
                mov     CFG_VideoYVisible, eax
                ret

; << UPPER_BORDER_VISIBLE >> --------------------------------------
Lif_UBorderVisible:
                mov     al, 00Ah
                call    LifReadNumber           ; values 0-128.
                jc      Lif_UBV_1
                cmp     eax, 080h
                ja      Lif_SF_1
                mov     CFG_VideoUBVisible, eax
                ret
Lif_UBV_1:
                ; "auto"
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     CFG_VideoUBVisible, 0
                ret

; << LEFT_BORDER_VISIBLE >> ---------------------------------------
Lif_LBorderVisible:
                mov     al, 00Ah
                call    LifReadNumber           ; values 0-32.
                jc      Lif_LBV_1
                cmp     eax, 020h
                ja      Lif_SF_1
                mov     CFG_VideoLBVisible, al
                ret
Lif_LBV_1:
                ; "auto"
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     CFG_VideoLBVisible, 0
                ret

; << AY_FILTER >> -------------------------------------------------
Lif_AYFilter:
                lea     ebx, IniParsTable
                call    LifGetKeywordID         ; values 0/1 or ON/OFF
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_ON
                jnz     Lif_AYF_1
                ; ON
                and     CFG_FilterSelect, 0FCh
                or      CFG_FilterSelect, 001h
                ret
Lif_AYF_1:
                cmp     ebx, ID_INICMN_OFF
                jnz     LifMsgParameterInvalid
                ; OFF
                and     CFG_FilterSelect, 0FCh
                ret

; << SB_MODE >> ---------------------------------------------------
Lif_SBMode:
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_DMA16
                jnz     Lif_SBM_1
                ; 16-bit DMA mode selected
                and     CFG_AudioProperties, not(SP_SBMODE1 + SP_SBMODE2)
                ret
Lif_SBM_1:
                cmp     ebx, ID_INICMN_DMA
                jnz     Lif_SBM_5
                ; 8-bit DMA mode selected
                and     CFG_AudioProperties, not(SP_SBMODE2)
                or      CFG_AudioProperties, SP_SBMODE1
                ret
Lif_SBM_5:
                cmp     ebx, ID_INICMN_FDAC
                jnz     Lif_SBM_6
                ; fast DAC mode selected
                and     CFG_AudioProperties, not(SP_SBMODE1)
                or      CFG_AudioProperties, SP_SBMODE2
                ret
Lif_SBM_6:
                cmp     ebx, ID_INICMN_SDAC
                jnz     LifMsgParameterInvalid
                ; slow DAC mode selected
                or      CFG_AudioProperties, SP_SBMODE1 + SP_SBMODE2
Lif_SBM_7:
                ret

; << TACTS_PER_FRAME >> -------------------------------------------
Lif_TactsPerFrame:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      Lif_TPF_1
                mov     CFG_ULATactsInFrame, eax
                ret
Lif_TPF_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     CFG_ULATactsInFrame, 0
                ret

; << Z80_CLOCK_FREQUENCY >> ---------------------------------------
Lif_Z80ClockFreq:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      Lif_ZCF_1
                cmp     eax, 1000000
                jc      LifMsgParameterInvalid
                cmp     eax, 10000000
                ja      LifMsgParameterInvalid
                mov     CFG_Z80ClockFreq, eax
                ret
Lif_ZCF_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_DEFAULT
                jnz     LifMsgParameterInvalid
                mov     CFG_Z80ClockFreq, 3500000
                ret

; << AY_CLOCK_FREQUENCY >> ----------------------------------------
Lif_AYClockFreq:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      Lif_AYCF_1
                cmp     eax, 500000
                jc      LifMsgParameterInvalid
                cmp     eax, 5000000
                ja      LifMsgParameterInvalid
                mov     CFG_AYClockFreq, eax
                ret
Lif_AYCF_1:
                cmp     al, 1
                jnz     LifMessage
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_DEFAULT
                jnz     LifMsgParameterInvalid
                mov     CFG_AYClockFreq, 0
                ret

; << SB_BUFFER_SIZE >> --------------------------------------------
Lif_SBBufferSize:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      LifMsgParameterInvalid
                cmp     eax, 16
                jc      LifMsgParameterInvalid
                cmp     eax, 65536
                ja      LifMsgParameterInvalid
                mov     CFG_SBBufferSize, eax
                ret

; << AY_GENERATOR_CLOCK_DIVIDER >> --------------------------------
Lif_AYGenClockDiv:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      LifMsgParameterInvalid
                test    eax, eax
                jz      LifMsgParameterInvalid
                cmp     eax, 16
                jnc     LifMsgParameterInvalid
                mov     CFG_AYGenClkDivider, eax
                ret

; << SB_TYPE >> ---------------------------------------------------
Lif_SBType:
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_SB1
                jnz     Lif_SBT_1
                ; SB1
                mov     SBVersion, 00101h
                ret
Lif_SBT_1:
                cmp     ebx, ID_INICMN_SB2
                jnz     Lif_SBT_2
                ; SB2
                mov     SBVersion, 00201h
                ret
Lif_SBT_2:
                cmp     ebx, ID_INICMN_SBPRO1
                jnz     Lif_SBT_3
                ; SBPro1
                mov     SBVersion, 00302h
                ret
Lif_SBT_3:
                cmp     ebx, ID_INICMN_SBPRO2
                jnz     Lif_SBT_4
                ; SBPro2
                mov     SBVersion, 00305h
                ret
Lif_SBT_4:
                cmp     ebx, ID_INICMN_SB16
                jnz     Lif_SBT_5
                ; SB16
                mov     SBVersion, 00400h
                ret
Lif_SBT_5:
                ; "auto"
                cmp     ebx, ID_INICMN_AUTO
                jnz     LifMsgParameterInvalid
                mov     SBVersion, 0
                ret

; << STARTUP_MODE >> ----------------------------------------------
Lif_StartupMode:
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jc      LifMsgSyntaxError
                cmp     ebx, ID_INICMN_48K
                jnz     Lif_SM_1
                mov     CFG_ZXStartupMode, 0
                ret
Lif_SM_1:
                cmp     ebx, ID_INICMN_B48K
                jnz     Lif_SM_2
                mov     CFG_ZXStartupMode, 1
                ret
Lif_SM_2:
                cmp     ebx, ID_INICMN_128K
                jnz     Lif_SM_3
                mov     CFG_ZXStartupMode, 2
                ret
Lif_SM_3:
                cmp     ebx, ID_INICMN_B128K
                jnz     LifMsgParameterInvalid
                mov     CFG_ZXStartupMode, 3
                ret

; << MIN_TACTS_AS_ONE >> ------------------------------------------
Lif_Z80MinTactsAs1:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      LifMsgParameterInvalid
                cmp     eax, 4
                jc      LifMsgParameterInvalid
                cmp     eax, 100
                ja      LifMsgParameterInvalid
                mov     CFG_Z80TactsAsOne, eax
                ret

; << TAPE_VOLUME >> -----------------------------------------------
Lif_ZXTapeVolume:
                mov     al, 00Ah                ; decimal constant as default
                call    LifReadNumber
                jnc     Lif_ZXTV_1
                cmp     al, 1
                jnz     LifMessage
                mov     eax, 03Fh
                xor     ebx, ebx
                call    LifMinMax
                jnc     Lif_ZXTV_1
                ret
Lif_ZXTV_1:
                cmp     eax, 040h
                jnc     Lif_SF_1
                shl     eax, 8
                xor     eax, 03F80h
                mov     CFG_MixTapeVolume, eax
                ret

; << TAPE_DEVICE >> -----------------------------------------------
Lif_ZXTapeDevice:
                mov     eax, 004h
                jmp     Lif_SDev_1


; << GENERAL_SOUND_DEVICE >> --------------------------------------
Lif_GeneralSoundDevice:
                mov     eax, 008h
                jmp     Lif_SDev_1

; << TMR_READ_METHOD >> -------------------------------------------
Lif_TMRReadMethod:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      LifMsgParameterInvalid
                cmp     eax, 8
                jnc     LifMsgParameterInvalid
                mov     CFG_TimerMode, al
                ret

; << CMOS_READ_METHOD >> -------------------------------------------
Lif_CMOSReadMethod:
                mov     al, 00Ah                ; decimal value as default
                call    LifReadNumber
                jc      LifMsgParameterInvalid
                cmp     eax, 2
                jnc     LifMsgParameterInvalid
                ror     al, 1
                and     CFG_TimerMode, 07Fh
                or      CFG_TimerMode, al
                ret

; << ZX_DEVICE >> -------------------------------------------------
Lif_ZXAddDevice:
                lea     ebx, TempBuffer_A

                mov     al, 010h                ; hex value as default
                call    LifReadNumber
                ret

; -------------------------------------------------------
; -=* M-I-N-/-M-A-X---O-R---E-R-R-O-R---M-E-S-S-A-G-E *=-
; -------------------------------------------------------
LifMinMax:
                ;-----------------------
                ; IN:   EAX - equal MIN
                ;       EBX - equal MAX
                ; OUT:  EAX
                ;-----------------------
                push    ebx ebx
                lea     ebx, IniParsTable
                call    LifGetKeywordID
                jnc     LifMM_1
                pop     ebx
                call    LifMsgSyntaxError
                stc
                jmp     LifMM_end
LifMM_1:
                cmp     ebx, ID_INICMN_MIN
                jnz     LifMM_2
                ; keyword "MIN"
                pop     ebx
                clc
                jmp     LifMM_end
LifMM_2:
                cmp     ebx, ID_INICMN_MAX
                jnz     LifMM_3
                ; keyword "MAX"
                pop     ebx
                mov     eax, ebx
                clc
                jmp     LifMM_end
LifMM_3:
                pop     ebx
                call    LifMsgParameterInvalid
                stc
LifMM_end:
                pop     ebx
                ret

; -----------------------------------------------
; -=* S-Y-N-T-A-X---E-R-R-O-R---M-E-S-S-A-G-E *=-
; -----------------------------------------------
LifMsgSyntaxError:
                PSTR    TxtSyntaxError
LifMSE_1:
                mov     ax, cx
                call    Print_DecWord
                PENT
LifMSE_2:
                inc     CurErrorCounter
                ; skip all till next line
                mov     ah, [edi]
                call    LifMAIN_7
                cmp     ah, 00Dh
                jnz     LifMSE_2
                inc     cx
                cmp     byte ptr [edi], 00Ah
                jz      LifMAIN_7
                ret

; -----------------------------------------------------
; -=* P-A-R-A-M-E-T-E-R---M-I-S-S-I-N-G---E-R-R-O-R *=-
; -----------------------------------------------------
LifMsgParameterMissing:
                ; in: AX - line number
                PSTR    TxtParamMissing
                call    Print_DecWord
                PENT
                inc     CurErrorCounter
                ret
LifMsgParameterInvalid:
                PSTR    TxtBadParameter
                mov     eax, ecx
                jmp     LifMSE_1
LifMessage:
                cmp     al, 1
                jz      LifMsgParameterInvalid
                cmp     al, 2
                jz      LifMsgParameterInvalid
                cmp     al, 3
                jz      LifMsgSyntaxError
                PSTR    TxtBadNumber
                mov     ax, cx
                call    Print_DecWord
                PENT
                jmp     LifMSE_2

; ---------------------------------------------------------------
; -=* R-E-A-D---N-U-M-B-E-R---O-R---E-R-R-O-R---M-E-S-S-A-G-E *=-
; ---------------------------------------------------------------
LifReadNumber:
                ;-----------------------------------------------------------
                ; IN:   AL  - radix by default:
                ;             D0-D6 - default radix value
                ;             D7    - 1=strict number radix to default only
                ; OUT:  C   - error
                ;       and AL - error code:
                ;                1 - value not present
                ;                2 - invalid radix
                ;                3 - overflow
                ;       NC  - number read OK
                ;       and EAX - number value
                ;-----------------------------------------------------------
                push    ebx ecx edx esi
                xor     ecx, ecx
                push    eax
                ; radix specified by default
                xor     ebx, ebx
                and     al, 07Fh
                mov     bl, al
                mov     bh, al

                xor     ah, ah
                xor     dl, dl
                cmp     byte ptr [edi], "&"
                jnz     Lif_RN_45
                call    LifMAIN_7

                mov     al, [edi]
                and     al, not(020h)

                ; check for hex radix
                cmp     al, "H"
                jnz     Lif_RN_1
                ; hex radix specified
                mov     ebx, 01010h
                or      dl, 001h
                call    LifMAIN_7
                jmp     Lif_RN_5
Lif_RN_1:
                ; check for octal radix
                cmp     al, "O"
                jz      Lif_RN_2
                cmp     al, "Q"
                jnz     Lif_RN_3
Lif_RN_2:
                ; octal radix specified
                mov     ebx, 00808h
                or      dl, 001h
                call    LifMAIN_7
                jmp     Lif_RN_5
Lif_RN_3:
                cmp     al, "B"
                jz      Lif_RN_4
                mov     al, 2                   ; invalid radix
                jmp     Lif_RN_16
Lif_RN_4:
                ; binary radix specified
                mov     ebx, 00202h
                or      dl, 001h
                call    LifMAIN_7
                jmp     Lif_RN_5
Lif_RN_45:
                cmp     byte ptr [edi], "0"
                jnz     Lif_RN_5
                call    LifMAIN_7
                mov     al, [edi]
                and     al, not(020h)
                cmp     al, "X"
                jnz     Lif_RN_55
                mov     ebx, 01010h
                or      dl, 001h
                call    LifMAIN_7
Lif_RN_5:
                mov     al, [edi]
                cmp     al, "0"
                jc      Lif_RN_9
                cmp     al, "9"
                ja      Lif_RN_7
                sub     al, "0"
                jnz     Lif_RN_6
                ; skipping leading zeroes
                test    ecx, ecx
                jnz     Lif_RN_6
                call    LifMAIN_7
Lif_RN_55:
                or      dl, 002h
                jmp     Lif_RN_5
Lif_RN_6:
                shl     ecx, 4
                or      cl, al
                call    LifMAIN_7
                inc     ah
                cmp     ah, 9
                jc      Lif_RN_5
                mov     al, 3                   ; overflow
                jmp     Lif_RN_16
Lif_RN_7:
                and     al, not(020h)
                cmp     al, "A"
                jc      Lif_RN_9
                cmp     al, "F"
                ja      Lif_RN_8
                mov     bh, 010h
                sub     al, "7"
                test    dl, dl
                jnz     Lif_RN_6
                mov     al, 1                   ; value not present
                jmp     Lif_RN_16
Lif_RN_8:
                cmp     al, "H"
                jnz     Lif_RN_9
                mov     al, ah
                or      al, dl
                mov     al, 1                   ; value not present
                jz      Lif_RN_16
                call    LifMAIN_7
                test    dl, 001h
                mov     al, 2                   ; invalid radix
                jnz     Lif_RN_16
                mov     ebx, 01010h
Lif_RN_9:
                test    ah, ah
                jnz     Lif_RN_10
                test    dl, 2
                mov     al, 1                   ; value not present
                jz      Lif_RN_16
                mov     ah, al
Lif_RN_10:
                cmp     bl, bh
                mov     al, 2                   ; invalid radix
                stc
                jnz     Lif_RN_16
                xor     bh, bh
                push    ebx ebp edi
                xor     edi, edi
                mov     ebp, ebx
                mov     bl, ah
                mov     esi, 1
Lif_RN_11:
                mov     eax, ecx
                and     eax, 00Fh
                shr     ecx, 4
                mul     esi
                add     edi, eax
                mov     eax, esi
                mul     ebp
                mov     esi, eax
                test    edx, edx
                mov     al, 3                   ; overflow
                jnz     Lif_RN_15
                dec     bl
                jnz     Lif_RN_11
                mov     eax, edi
Lif_RN_12:
                pop     edi ebp ebx
Lif_RN_13:
                pop     ecx
                test    cl, 080h
                jz      Lif_RN_14
                and     cl, 07Fh
                cmp     bl, cl
                jz      Lif_RN_14
                mov     al, 3
                jmp     Lif_RN_17
Lif_RN_14:
                clc
                jmp     Lif_RN_18
Lif_RN_15:
                pop     edi ebp ebx
Lif_RN_16:
                pop     ecx
Lif_RN_17:
                stc
Lif_RN_18:
                pop     esi edx ecx ebx
                ret

; ---------------------------------------------------------
; -=* R-E-A-D---K-E-Y---O-R---E-R-R-O-R---M-E-S-S-A-G-E *=-
; ---------------------------------------------------------
LifGetKeywordID:
                ;------------------------------------------
                ; IN:   EBX - table address
                ; OUT:  NC  - keyword found
                ;       and EBX - keyword ID
                ;       C   - keyword not found
                ;------------------------------------------
                ; GKID Workspace
GKIDWS          struc
gkidws_begin = $
GKIDTmpregESI   dd      ?
GKIDTmpregEBX   dd      ?
GKIDSymbolCnr   dd      ?
GKIDKeywordID   dd      ?
GKIDKeywordLen  dd      ?
GKIDSymbolPtr   dd      ?
GKIDTemplatePtr dd      ?
GKIDTemplates   dd      00100h - (GKIDTemplates - gkidws_begin) dup(?)
gkidws_tmplts_size = $ - GKIDTemplates
gkidws_size = $ - GKIDTmpregESI
                ends
                ;------------------------------------------
                ; GKID Subroutine
                push    eax
                and     dl, 0FEh
                call    LifSkipComments
                ; clean up and init GKIDWS
                mov     eax, ecx
                mov     ecx, gkidws_size / 4
LifGKID_005:
                push    dword ptr 0
                loopd   LifGKID_005
                mov     ecx, eax
                mov     [esp].GKIDTmpregESI, esi
                ; starting
                mov     esi, ebx
LifGKID_010:
                mov     al, [esi]
                inc     esi
                mov     ah, [edi]
                cmp     al, 0FFh
                jz      LifGKID_120
                cmp     al, 080h
                jz      LifGKID_090
                cmp     al, 0F0h
                jnz     LifGKID_020
                ; control code 0F0h - set current keyword ID
                lea     esi, [esi + 4]
                mov     eax, [esi - 4]
                mov     [esp].GKIDTmpregEBX, eax
                jmp     LifGKID_010
LifGKID_020:
                cmp     al, 0F1h
                jnz     LifGKID_022
                ; control code 0F1h - set template
                lea     esi, [esi + 8]
                mov     ebx, [esp].GKIDTemplatePtr
                cmp     ebx, gkidws_tmplts_size / 8 - 1
                jnc     LifGKID_Err2
                mov     eax, [esi - 4]
                mov     [[esp].GKIDTemplates + ebx*8], eax
                mov     eax, [esi - 8]
                mov     [[esp].GKIDTemplates + ebx*8 + 004h], eax
                inc     ebx
                mov     [esp].GKIDTemplatePtr, ebx
                jmp     LifGKID_010
LifGKID_022:
                cmp     al, "#"
                jnz     LifGKID_028
                ; control symbol "#" - use template if next symbol 0..9 or A..Z
                mov     al, [esi]
                inc     esi
                cmp     al, "0"
                jc      LifGKID_028
                cmp     al, "Z"
                ja      LifGKID_028
                cmp     al, "9"
                ja      LifGKID_025
                sub     al, "0"
                jmp     LifGKID_026
LifGKID_025:
                cmp     al, "A"
                jc      LifGKID_028
                sub     al, "A" - 10
LifGKID_026:
                movzx   ebx, al
                mov     eax, [[esp].GKIDTemplates + ebx*8 + 004h]
                mov     ebx, [[esp].GKIDTemplates + ebx*8]
                test    ebx, ebx
                jz      LifGKID_Err3
                call    LifGetKeywordID
                cmp     eax, ebx
                jnz     LifGKID_070
                jmp     LifGKID_010
LifGKID_028:
                cmp     al, "%"
                jnz     LifGKID_050
                ; control symbol "%" - means " ", "_" or nothing
                mov     al, [esi]
                cmp     al, "%"
                jnz     LifGKID_030
                ; double control symbol "%%" means single "%"
                inc     esi
                jmp     LifGKID_060
LifGKID_030:
                cmp     ah, "_"
                jz      LifGKID_040
                cmp     ah, " "
                jnz     LifGKID_010
LifGKID_040:
                xchg    esi, [esp].GKIDTmpregESI
                call    LifMAIN_7               ; EDI++
                xchg    esi, [esp].GKIDTmpregESI
                jmp     LifGKID_010
LifGKID_050:
                xchg    esi, [esp].GKIDTmpregESI
                call    LifMAIN_7
                xchg    esi, [esp].GKIDTmpregESI
                cmp     ah, "A"
                jc      LifGKID_060
                cmp     ah, "Z"
                ja      LifGKID_060
                or      ah, 020h
LifGKID_060:
                cmp     al, ah
                jnz     LifGKID_070
                inc     [esp].GKIDSymbolCnr
                jmp     LifGKID_010
LifGKID_070:
                ; skip to next keyword
                mov     al, [esi]
                inc     esi
                cmp     al, 0FFh
                jz      LifGKID_120
                cmp     al, 080h
                jnz     LifGKID_070
LifGKID_080:
                mov     [esp].GKIDSymbolCnr, 0
                mov     edi, [esp].GKIDTmpregESI
                jmp     LifGKID_010
LifGKID_090:
                mov     eax, [esp].GKIDSymbolCnr
                cmp     eax, [esp].GKIDKeywordLen
                jc      LifGKID_080
                jz      LifGKID_Err1
LifGKID_100:
                mov     [esp].GKIDKeywordLen, eax
                mov     eax, [esp].GKIDTmpregEBX
                mov     [esp].GKIDKeywordID, eax
                mov     [esp].GKIDSymbolPtr, edi
                jmp     LifGKID_080
LifGKID_120:
                cmp     [esp].GKIDSymbolPtr, 0;[esp].GKIDKeywordLen, 0;
                stc
                jz      LifGKID_130
                mov     edi, [esp].GKIDSymbolPtr
                mov     ebx, [esp].GKIDKeywordID
                mov     eax, [esp].GKIDKeywordLen
                clc
LifGKID_130:
                mov     esi, [esp].GKIDTmpregESI
                lea     esp, [esp + gkidws_size]
                pop     eax
                ret
LifGKID_Err1:
                PSTR    TxtIniDuplicateKey
                mov     eax, [esp].GKIDTmpregEBX
                PHEXD   eax
                PCHR    ","
                PCHR    " "
                mov     eax, [esp].GKIDKeywordID
                PHEXD   eax
                PENT
                QUIT
LifGKID_Err2:
                PSTR    TxtIniNoMoreTempls
                QUIT
LifGKID_Err3:
                PSTR    TxtIniNoTemplate
                QUIT

; -------------------------------------------------------
; -=* S-K-I-P---C-O-M-M-E-N-T-S---A-N-D---S-P-A-C-E-S *=-
; -------------------------------------------------------
LifSkipComments:
                push    eax cx cx
                pop     ecx
                xor     al, al
LifSC_1:
                mov     ah, [edi]
                cmp     ah, 020h
                ja      LifSC_2
                call    LifSC_9
                add     al, 1
                sbb     al, 0
                cmp     ah, 00Dh
                jnz     LifSC_1
                inc     cx
                cmp     byte ptr [edi], 00Ah
                jnz     LifSC_1
                call    LifSC_9
                jmp     LifSC_1
LifSC_2:
                cmp     ah, ";"
                jnz     LifSC_4
LifSC_3:
                mov     ah, [edi]
                call    LifSC_9
                cmp     ah, 00Dh
                jnz     LifSC_3
                inc     cx
                cmp     byte ptr [edi], 00Ah
                jnz     LifSC_1
                call    LifSC_9
                jmp     LifSC_1
LifSC_4:
                cmp     ah, "%"
                jz      LifSC_3
                cmp     ah, "'"
                jz      LifSC_3
                cmp     ah, "/"
                jnz     LifSC_8
                call    LifSC_9
                mov     ah, [edi]
                cmp     ah, "/"
                jz      LifSC_3
                cmp     ah, "*"
                jz      LifSC_5
                call    LifMsgSyntaxError
                jmp     LifSC_1
LifSC_5:
                call    LifSC_9
LifSC_6:
                mov     ah, [edi]
                call    LifSC_9
                cmp     ah, 00Dh
                jnz     LifSC_7
                inc     cx
                jmp     LifSC_6
LifSC_7:
                cmp     ah, "*"
                jnz     LifSC_6
                mov     ah, [edi]
                call    LifSC_9
                cmp     ah, "/"
                jnz     LifSC_6
                jmp     LifSC_1
LifSC_8:
                mov     esi, edi
                pop     eax
                ret
LifSC_9:
                ; same as LifMAIN_7 but also updates ESI
                ;  and aborts procedure <LifSkipComments> if BP=1
                push    eax
                lea     eax, TempBuffer_A
                inc     eax
                cmp     eax, ebp
                pop     eax
                jz      LifSC_10
                inc     edi
                mov     esi, edi
                cmp     edi, ebp
                jnc     LifMAIN_8
                ret
LifSC_10:
                pop     ax
                jmp     LifSC_8

LifMAIN_7:
                ; increments text pointer and loads next part
                ;  of the INI file if overflow
                inc     edi
                cmp     edi, ebp
                jnc     LifMAIN_8
                ret
LifMAIN_8:
                push    eax
                lea     eax, TempBuffer_A
                add     eax, 2
                cmp     ebp, eax
                jz      LifMAIN_13
LifMAIN_9:
                push    ecx edx
                mov     ecx, ebp
                sub     ecx, esi
                lea     edi, TempBuffer_A
                jz      LifMAIN_10
                cld
                rep     movs byte ptr es:[esi], ds:[edi]
LifMAIN_10:
                mov     ecx, ebp
                sub     ecx, edi
                mov     edx, edi
                call    File_Read
                jc      LifMAIN_12
                lea     esi, TempBuffer_A
                lea     edx, [eax + edx]
                sub     ebp, edx
                cmp     ebp, 2
                jc      LifMAIN_11
                mov     dword ptr [edx], 00A0Dh
                lea     edx, [edx + 2]
LifMAIN_11:
                mov     ebp, edx
                pop     edx ecx eax
                ret
LifMAIN_12:
                PSTR    TxtErrorINIFile
LifMAIN_13:
                mov     esp, CurStackPointer
                call    File_Close
LifMAIN_14:
                mov     eax, CurErrorCounter
                test    eax, eax
                jz      LifMAIN_15
                PSTR    TxtPressAnyKey
                call    Kbd_WaitKeypress
LifMAIN_15:
                popad
                ret

;----------------------------------------------------------------------------;


;****************************************************************************;

.DATA

CurFileHandler      dd      0    ; file handler
CurStackPointer     dd      0    ; ESP
CurErrorCounter     dd      0


;****************************************************************************
;* MESSAGES *****************************************************************
;****************************************************************************

TxtLoadINI          db      " Loading initialization file...", 00Dh, 00Ah, 0
TxtErrorINIFile     db      "Error loading INI file.", 00Dh, 00Ah, 0
TxtSyntaxError      db      "Syntax error at line ", 0
TxtBadParameter     db      "Invalid parameter, line ", 0
TxtNumberTooBig     db      "Parameter too large, line ", 0
TxtNumTooSmall      db      "Parameter too small, line ", 0
TxtBadNumber        db      "Number too big, line ", 0
TxtDecimalOnly      db      "Only decimal values possible for this key, line ", 0
TxtParamMissing     db      "Parameter missing in line ", 0
TxtIniDuplicateKey  db      "*** GKID: Duplicate keywords detected, keys ", 0
TxtIniNoMoreTempls  db      "*** GKID: Not enough space for templates.", 00Dh, 00Ah, 0
TxtIniNoTemplate    db      "*** GKID: Template not found.", 00Dh, 00Ah, 0


;****************************************************************************
;* KEYWORDS FOR .INI FILE-BASED CONFIGURATION *******************************
;****************************************************************************

IniKeysTable        label   byte
                    ; Synchronization Frequency
                    db      0F0h
                    dd      Lif_SyncFrequency
                    db      "synchronization%frequency", 080h, "sync%frequency", 080h, "sync%freq", 080h
                    ; AY Volume
                    db      0F0h
                    dd      Lif_AYVolume
                    db      "ay%volume", 080h
                    ; Sound Blaster Address
                    db      0F0h
                    dd      Lif_SBAddress
                    db      "sound%blaster%address", 080h, "sb%address", 080h
                    ; Sound Blaster Volume
                    db      0F0h
                    dd      Lif_SBVolume
                    db      "sound%blaster%volume", 080h, "sb%volume", 080h
                    ; Z80 Speed
                    db      0F0h
                    dd      Lif_Z80Speed
                    db      "z80%speed", 080h, "speed", 080h
                    ; Beeper Volume
                    db      0F0h
                    dd      Lif_ZXBeeperVolume
                    db      "zx%beeper%volume", 080h, "beeper%volume", 080h
                    ; Sample Rate
                    db      0F0h
                    dd      Lif_SampleRate
                    db      "sb%sample%rate", 080h, "sample%rate", 080h
                    ; Preliminary Video Mode
                    db      0F0h
                    dd      Lif_PreliminaryVideoMode
                    db      "preliminary%video%mode", 080h, "video%mode%preliminary", 080h, "pre%video%mode", 080h
                    ; Main Video Mode
                    db      0F0h
                    dd      Lif_MainVideoMode
                    db      "main%video%mode", 080h, "video%mode%main", 080h, "video%mode", 080h
                    ; Z80 Clock Multiplier
                    db      0F0h
                    dd      Lif_Z80ClockMul
                    db      "z80%clock%multiplier", 080h, "z80%clock%mul", 080h
                    ; Sound Blaster DMA
                    db      0F0h
                    dd      Lif_SBDMAChannel
                    db      "sound%blaster%dma", 080h, "sb%dma", 080h
                    ; Frame Skip
                    db      0F0h
                    dd      Lif_FrameSkip
                    db      "frame%skip", 080h, "frame", 080h
                    ; Interrupt Length
                    db      0F0h
                    dd      Lif_InterruptLength
                    db      "zx%interrupt%length%in%tacts", 080h, "interrupt%length", 080h
                    ; Beeper Device
                    db      0F0h
                    dd      Lif_ZXBeeperDevice
                    db      "zx%beeper%device", 080h, "zx%beeper", 080h, "beeper", 080h, "beeper%device", 080h
                    ; Flash Speed
                    db      0F0h
                    dd      Lif_FlashSpeed
                    db      "flash%speed", 080h, "flash%rate", 080h
                    ; AY Device
                    db      0F0h
                    dd      Lif_AYDevice
                    db      "ay8912%device", 080h, "ay%device", 080h, "ay", 080h
                    ; Sound Blaster ADC
                    db      0F0h
                    dd      Lif_RealTape
                    db      "sound%blaster%adc", 080h, "sb%adc", 080h, "real%tape", 080h
                    ; Stereo Separation
                    db      0F0h
                    dd      Lif_StereoSeparation
                    db      "stereo%separation", 080h
                    ; Stereo Reverse
                    db      0F0h
                    dd      Lif_ReverseStereo
                    db      "stereo%reverse", 080h, "reverse%stereo", 080h, "swap%speakers", 080h
                    ; Sound Stereo
                    db      0F0h
                    dd      Lif_SBStereo
                    db      "stereo%layout", 080h, "stereo%mode", 080h, "stereo%type", 080h, "stereo", 080h
                    ; ZX Resolution By X
                    db      0F0h
                    dd      Lif_ZXResolutionX
                    db      "zx%resolution%x", 080h, "zx%screen%width", 080h, "screen%width", 080h
                    ; ZX Resolution By Y
                    db      0F0h
                    dd      Lif_ZXResolutionY
                    db      "zx%resolution%y", 080h, "zx%screen%height", 080h, "screen%height", 080h
                    ; Video Visible By X
                    db      0F0h
                    dd      Lif_XVisible
                    db      "video%x%visible", 080h, "visible%x", 080h
                    ; Video Visible By Y
                    db      0F0h
                    dd      Lif_YVisible
                    db      "video%y%visible", 080h, "visible%y", 080h
                    ; Left Border Size
                    db      0F0h
                    dd      Lif_ZXLeftBorderSize
                    db      "left%border%size", 080h, "left%border", 080h
                    ; Upper Border Size
                    db      0F0h
                    dd      Lif_ZXUpperBorderSize
                    db      "upper%border%size", 080h, "upper%border", 080h, "top%border%size", 080h, "top%border", 080h
                    ; Left Border Visible
                    db      0F0h
                    dd      Lif_LBorderVisible
                    db      "left%border%visible", 080h, "left%border", 080h
                    ; Upper Border Visible
                    db      0F0h
                    dd      Lif_UBorderVisible
                    db      "upper%border%visible", 080h, "upper%border", 080h, "top%border%visible", 080h, "top%border", 080h
                    ; AY Filter
                    db      0F0h
                    dd      Lif_AYFilter
                    db      "ay%filter", 080h
                    ; Sound Blaster Mode
                    db      0F0h
                    dd      Lif_SBMode
                    db      "sound%blaster%mode", 080h, "sb%mode", 080h
                    ; Tacts Per Frame
                    db      0F0h
                    dd      Lif_TactsPerFrame
                    db      "tacts%per%frame", 080h, "tacts%in%frame", 080h, "tpf", 080h
                    ; Z80 Clock Frequency
                    db      0F0h
                    dd      Lif_Z80ClockFreq
                    db      "z80%clock%frequency", 080h, "z80%clock%freq", 080h
                    ; AY Clock Frequency
                    db      0F0h
                    dd      Lif_AYClockFreq
                    db      "ay8912%clock%frequency", 080h, "ay%clock%frequency", 080h, "ay8912%clock%freq", 080h, "ay%clock%freq", 080h
                    ; SB Buffer Size
                    db      0F0h
                    dd      Lif_SBBufferSize
                    db      "sound%blaster%buffer%size", 080h, "sb%buffer%size", 080h
                    ; AY Generator Clock Divider
                    db      0F0h
                    dd      Lif_AYGenClockDiv
                    db      "ay8912%generator%clock%divider", 080h, "ay%generator%clock%divider", 080h, "ay%generator%clock%div", 080h
                    ; Sound Blaster ADC
                    db      0F0h
                    dd      Lif_SBType
                    db      "sound%blaster%type", 080h, "sb%type", 080h
                    ; Startup Mode
                    db      0F0h
                    dd      Lif_StartupMode
                    db      "start%up%mode", 080h, "spectrum%mode", 080h
                    ; Minimum Tacts as One Instruction
                    db      0F0h
                    dd      Lif_Z80MinTactsAs1
                    db      "min%tacts%as%one%instruction", 080h, "min%tacts%as%one", 080h, "minimum%tacts%as%one%instruction", 080h
                    ; ZX Tape Volume
                    db      0F0h
                    dd      Lif_ZXTapeVolume
                    db      "zx%tape%volume", 080h, "tape%volume", 080h
                    ; ZX Tape Device
                    db      0F0h
                    dd      Lif_ZXTapeDevice
                    db      "zx%tape%device", 080h, "tape%device", 080h, "tape", 080h
                    ; General Sound Device
                    db      0F0h
                    dd      Lif_GeneralSoundDevice
                    db      "general%sound%device", 080h, "general%sound", 080h, "gs%device", 080h
                    ; Timer Read Method
                    db      0F0h
                    dd      Lif_TMRReadMethod
                    db      "timer%read%method", 080h, "tmr%read%method", 080h
                    ; CMOS Read Method
                    db      0F0h
                    dd      Lif_CMOSReadMethod
                    db      "cmos%read%method", 080h, "seconds%read%method", 080h
                    ; Add Device
                    db      0F0h
                    dd      Lif_ZXAddDevice
                    db      "add%device", 080h, "device", 080h
                    ; END OF TABLE
                    db      0FFh


;****************************************************************************
;* PARAMETERS FOR .INI FILE-BASED CONFIGURATION *****************************
;****************************************************************************

ID_INICMN_ON                equ     000h
ID_INICMN_OFF               equ     001h
ID_INICMN_MIN               equ     002h
ID_INICMN_MAX               equ     003h
ID_INICMN_AUTO              equ     004h
ID_INICMN_DEFAULT           equ     005h
ID_INICMN_DEVSB             equ     007h
ID_INICMN_DEVSPK            equ     008h
ID_INICMN_DMA16             equ     012h
ID_INICMN_DMA               equ     013h
ID_INICMN_BUFFERED          equ     016h
ID_INICMN_FDAC              equ     018h
ID_INICMN_SDAC              equ     019h
ID_INICMN_48K               equ     01Bh
ID_INICMN_B48K              equ     01Ch
ID_INICMN_128K              equ     01Dh
ID_INICMN_B128K             equ     01Eh
ID_INICMN_SB1               equ     01Fh
ID_INICMN_SB2               equ     020h
ID_INICMN_SBPRO1            equ     021h
ID_INICMN_SBPRO2            equ     022h
ID_INICMN_SB16              equ     023h
ID_INICMN_BW                equ     024h
ID_INICMN_COLOR             equ     025h
ID_INICMN_VRETRACE          equ     026h
ID_INICMN_NOVBE             equ     027h
ID_INICMN_FORCEVGA          equ     02Ch
ID_INICMN_INTERLACE         equ     02Dh
ID_INICMN_DEVPORT           equ     030h
ID_INICMN_DEVPORTMASK       equ     031h
ID_INICMN_DEVSPACE          equ     032h
ID_INICMN_DEVPRIO           equ     033h
ID_INICMN_READFROM          equ     034h or ID_DEVFLAG_WR
ID_INICMN_WRITETO           equ     034h or ID_DEVFLAG_RD

IniParsTable        label   byte
                    ; On / Enable / Yes
                    db      0F0h
                    dd      ID_INICMN_ON
                    db      "on", 080h, "enable", 080h, "yes", 080h
                    ; Off / Disable / None / No
                    db      0F0h
                    dd      ID_INICMN_OFF
                    db      "off", 080h, "disable", 080h, "none", 080h, "no", 080h
                    ; Minimum / Min
                    db      0F0h
                    dd      ID_INICMN_MIN
                    db      "minimum", 080h, "min", 080h
                    ; Maximum / Max
                    db      0F0h
                    dd      ID_INICMN_MAX
                    db      "maximum", 080h, "max", 080h
                    ; Auto
                    db      0F0h
                    dd      ID_INICMN_AUTO
                    db      "auto", 080h
                    ; Default
                    db      0F0h
                    dd      ID_INICMN_DEFAULT
                    db      "default", 080h
                    ; Sound Blaster / Blaster / SB
                    db      0F0h
                    dd      ID_INICMN_DEVSB
                    db      "sound%blaster", 080h, "blaster", 080h, "sb", 080h
                    ; PC Speaker / Speaker
                    db      0F0h
                    dd      ID_INICMN_DEVSPK
                    db      "pc%speaker", 080h, "speaker", 080h
                    ; DMA16
                    db      0F0h
                    dd      ID_INICMN_DMA16
                    db      "dma16", 080h
                    ; DMA
                    db      0F0h
                    dd      ID_INICMN_DMA
                    db      "dma", 080h
                    ; Buffered
                    db      0F0h
                    dd      ID_INICMN_BUFFERED
                    db      "buffered", 080h
                    ; Fast DAC
                    db      0F0h
                    dd      ID_INICMN_FDAC
                    db      "fast%dac", 080h
                    ; Slow DAC
                    db      0F0h
                    dd      ID_INICMN_SDAC
                    db      "slow%dac", 080h
                    ; 48k Basic / Basic 48k / 48k
                    db      0F0h
                    dd      ID_INICMN_48K
                    db      "48k%basic", 080h, "basic%48k", 080h, "48k", 080h
                    ; 48k TRDOS / 48k TR-DOS / TRDOS 48k / TR-DOS 48k
                    db      0F0h
                    dd      ID_INICMN_B48K
                    db      "48k%trdos", 080h, "48k%tr-dos", 080h, "trdos%48k",  080h, "tr-dos%48k", 080h
                    ; 128k Basic / Basic 128k / 128k
                    db      0F0h
                    dd      ID_INICMN_128K
                    db      "128k%basic", 080h, "basic%128k", 080h, "128k", 080h
                    ; 128k TRDOS / 128k TR-DOS / TRDOS 128k / TR-DOS 128k
                    db      0F0h
                    dd      ID_INICMN_B128K
                    db      "128k%trdos", 080h, "128k%tr-dos", 080h, "trdos%128k",  080h, "tr-dos%128k", 080h
                    ; SB1
                    db      0F0h
                    dd      ID_INICMN_SB1
                    db      "sb1", 080h
                    ; SB2
                    db      0F0h
                    dd      ID_INICMN_SB2
                    db      "sb2", 080h
                    ; SBPro1
                    db      0F0h
                    dd      ID_INICMN_SBPRO1
                    db      "sbpro1", 080h
                    ; SBPro2
                    db      0F0h
                    dd      ID_INICMN_SBPRO2
                    db      "sbpro2", 080h
                    ; SB16
                    db      0F0h
                    dd      ID_INICMN_SB16
                    db      "sb16", 080h
                    ; Black and White / B&W / BW
                    db      0F0h
                    dd      ID_INICMN_BW
                    db      "black%and%white", 080h, "b&w", 080h, "bw", 080h
                    ; Color
                    db      0F0h
                    dd      ID_INICMN_COLOR
                    db      "color", 080h
                    ; Vertical Retrace
                    db      0F0h
                    dd      ID_INICMN_VRETRACE
                    db      "vertical%retrace", 080h
                    ; No VESA / No VBE
                    db      0F0h
                    dd      ID_INICMN_NOVBE
                    db      "no%vesa", 080h, "no%vbe", 080h
                    ; Force VGA
                    db      0F0h
                    dd      ID_INICMN_FORCEVGA
                    db      "force%vga", 080h
                    ; Interlaced / Interlace
                    db      0F0h
                    dd      ID_INICMN_INTERLACE
                    db      "interlaced", 080h, "interlace", 080h
                    ; Port
                    db      0F0h
                    dd      ID_INICMN_DEVPORT
                    db      "port", 080
                    ; Mask
                    db      0F0h
                    dd      ID_INICMN_DEVPORTMASK
                    db      "mask", 080h
                    ; IO Space / Space
                    db      0F0h
                    dd      ID_INICMN_DEVSPACE
                    db      "io%space", 080h, "space", 080h
                    ; Priority / Prio
                    db      0F0h
                    dd      ID_INICMN_DEVPRIO
                    db      "priority", 080h, "prio", 080h
                    ; Read From / Read / RD
                    db      0F0h
                    dd      ID_INICMN_READFROM
                    db      "read%from", 080, "read", 080h, "rd", 080h
                    ; Write To / Write / WR
                    db      0F0h
                    dd      ID_INICMN_WRITETO
                    db      "write%to", 080, "write", 080h, "wr", 080h
                    ; END OF TABLE
                    db      0FFh


;****************************************************************************
;* PARAMETERS FOR "DEVICE" KEYWORD IN .INI FILE-BASED CONFIGURATION *********
;****************************************************************************

IniDeviceTable      label   byte
                    ; TEMPLATE #0: ID_INICMN_WRITETO
                    db      0F1h
                    dd      IniParsTable, ID_INICMN_WRITETO
                    ; TEMPLATE #1: ID_INICMN_READFROM
                    db      0F1h
                    dd      IniParsTable, ID_INICMN_READFROM
                    ; Read Keyboard
                    db      0F0h
                    dd      ID_DEVICE_RD_KBD
                    db      "#1%keyboard", 080h
                    ; Write ULA
                    db      0F0h
                    dd      ID_DEVICE_WR_ULA
                    db      "#0%ula", 080h, "#0%ula%48k", 080h, "#0%ula%128k", 080h
                    ; Read Bus State
                    db      0F0h
                    dd      ID_DEVICE_RD_BUS
                    db      "#1%read%bus%state", 080h
                    ; Write 128k System
                    db      0F0h
                    dd      ID_DEVICE_WR_MMU128K
                    db      "#0%128k%system", 080h, "#0%128k%mmu", 080h, "#0%128k%config", 080h
                    ; Read AY0 Register
                    db      0F0h
                    dd      ID_DEVICE_RD_AY0REG
                    db      "#1%ay0%register", 080h, "#1%ay0%reg", 080h
                    ; Write AY0 Register Selection
                    db      0F0h
                    dd      ID_DEVICE_WR_AY0SEL
                    db      "#0%ay0%select%register", 080h, "#0%ay0%select", 080h, "#0%ay0%sel", 080h
                    ; Write AY0 Register
                    db      0F0h
                    dd      ID_DEVICE_WR_AY0REG
                    db      "#0%ay0%register", 080h, "#0%ay0%reg", 080h
                    ; Read Kempston Joystick
                    db      0F0h
                    dd      ID_DEVICE_RD_KJOYSTICK
                    db      "#1%kempston%joystick", 080h, "#1%kjoystick", 080h
                    ; Read Kempston Mouse X Coordinate
                    db      0F0h
                    dd      ID_DEVICE_RD_KMOUSX
                    db      "#1%kempston%mouse%x%coordinate", 080h, "#1%kempston%mouse%x%coord", 080h, "#1%kmouse%x", 080h
                    ; Read Kempston Mouse Y Coordinate
                    db      0F0h
                    dd      ID_DEVICE_RD_KMOUSY
                    db      "#1%kempston%mouse%y%coordinate", 080h, "#1%kempston%mouse%y%coord", 080h, "#1read%kmouse%y", 080h
                    ; Read Kempston Mouse Buttons
                    db      0F0h
                    dd      ID_DEVICE_RD_KMOUSBTNS
                    db      "#1%kempston%mouse%buttons", 080h, "#1%kmouse%buttons", 080h
                    ; Read BDI Status
                    db      0F0h
                    dd      ID_DEVICE_RD_BDISTAT
                    db      "#1%bdi%status", 080h, "#1%bdi%stat", 080h
                    ; Write BDI Command
                    db      0F0h
                    dd      ID_DEVICE_WR_BDICMD
                    db      "#0%bdi%command", 080h, "#0%bdi%cmd", 080h
                    ; Read BDI Track
                    db      0F0h
                    dd      ID_DEVICE_RD_BDITRK
                    db      "#1%bdi%track", 080h, "#1%bdi%trk", 080h
                    ; Write BDI Track
                    db      0F0h
                    dd      ID_DEVICE_WR_BDITRK
                    db      "#0%bdi%track", 080h, "#0%bdi%trk", 080h
                    ; Read BDI Sector
                    db      0F0h
                    dd      ID_DEVICE_RD_BDISECT
                    db      "#1%bdi%sector", 080h, "#1%bdi%sec", 080h
                    ; Write BDI Sector
                    db      0F0h
                    dd      ID_DEVICE_WR_BDISECT
                    db      "#0%bdi%sector", 080h, "#0%bdi%sec", 080h
                    ; Read BDI Data
                    db      0F0h
                    dd      ID_DEVICE_RD_BDIDATA
                    db      "#1%bdi%data", 080h
                    ; Write BDI Data
                    db      0F0h
                    dd      ID_DEVICE_WR_BDIDATA
                    db      "#0%bdi%data", 080h
                    ; Read BDI System
                    db      0F0h
                    dd      ID_DEVICE_RD_BDISYS
                    db      "#1%bdi%system", 080h, "#1%bdi%sys", 080h
                    ; Write BDI System
                    db      0F0h
                    dd      ID_DEVICE_WR_BDISYS
                    db      "#0%bdi%system", 080h, "#0%bdi%sys", 080h
                    ; END OF TABLE
                    db      0FFh


;****************************************************************************;



                end
