
;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

Ports_Init:

;----------------------------------------------------------------------------;

ZX128k_SetConfig:
                push    eax
                mov     [ebx].SystemZX128, ah
                call    Ps128_3
                pop     eax
                ret

;----------------------------------------------------------------------------;

SetZXBorder:
                push    eax
                mov     [ebx].SystemZX48, ah
                call    Ps48_1
                pop     eax
                ret

;----------------------------------------------------------------------------;

ZXPort_Out:
                ; CX = port address, AH = data
                test    [ebx].flagIO, 001h
                jnz     dos_out
                mov     al, ch
                movzx   ecx, cl
                jmp     [ecx*4 + offset STDPortOutTable]
dos_out:
                mov     al, cl
                and     al, 0E0h
                ror     al, 3
                and     cl, 002h
                or      cl, al
                mov     al, ch
                movzx   ecx, cl
                jmp     [ecx*2 + offset DOSPortOutTable]

;----------------------------------------------------------------------------;

ZXPort_In:
                ; CX = port address
                test    [ebx].flagIO, 001h
                jnz     dos_in
                mov     al, ch
                movzx   ecx, cl
                jmp     [ecx*4 + offset STDPortInTable]
dos_in:
                mov     al, cl
                and     al, 0E0h
                ror     al, 3
                and     cl, 002h
                or      cl, al
                mov     al, ch
                movzx   ecx, cl
                jmp     [ecx*2 + offset DOSPortInTable]

;----------------------------------------------------------------------------;

; Ports Service Procedures


;****************************************************************************
;* AY8912 registers selection, reading and writing **************************
;****************************************************************************
Port_AY0_ReadReg:
                lea     esi, [ebx].AY1st
                jmp     AY_ReadRegister

Port_AY0_Write:
                cmp     al, 0C0h
                jnz     Pway_1
Port_AY0_SelectReg:
                lea     esi, [ebx].AY1st
                jmp     AY_SelectRegister
Pway_1:
                cmp     al, 080h
                jnz     Ps128_1
Port_AY0_WriteReg:
                lea     esi, [ebx].AY1st
                jmp     AY_WriteRegister
                align   4

;****************************************************************************
;* System ports *************************************************************
;****************************************************************************

Port_128_Write:
                and     al, 0C0h
                cmp     al, 040h
                jnz     Port_AY0_Write
Ps128_1:
                test    [ebx].SystemZX128, 020h
                jnz     Ps128_End
Ps128_2:
                mov     [ebx].SystemZX128, ah
                UPDSCR
Ps128_3:
                ; Select normal (0) or shadow (1) screen
                mov     eax, [ebx].RamBank5Adr
                test    [ebx].SystemZX128, 008h
                jz      Ps128_4
                mov     eax, [ebx].RamBank7Adr
Ps128_4:
                mov     [ebx].ScreenBankAdr, eax
                ; ROM select. ROM 0 is the 128k editor and menu system; ROM 1 contains 48k BASIC
                test    [ebx].SystemZX128, 010h
                jz      Ps128_5
                mov     eax, [ebx].RomBank1Adr
                mov     [ebx].RomPrimary, eax
                mov     eax, [ebx].RomBank3Adr
                mov     [ebx].RomSecondary, eax
                jmp     Ps128_6
Ps128_5:
                mov     eax, [ebx].RomBank0Adr
                mov     [ebx].RomPrimary, eax
                mov     [ebx].RomSecondary, eax
Ps128_6:
                GR16K0  eax
                cmp     eax, [ebx].RomBank3Adr
                jz      Ps128_7
                mov     eax, [ebx].RomPrimary
                SR16K0  eax
Ps128_7:
                mov     eax, [ebx].BlankBankAdr
                SW16K0  eax
                ; RAM page (0-7) to map into memory at 0C000h
                mov     ah, [ebx].SystemZX128
                and     ah, 007h
                movzx   eax, ah
                mov     eax, [eax*4 + [ebx].RamBank0Adr]
                SRW16K3 eax
Ps128_End:
                ret
                align   4

Port_ULA_ReadKbd:
                push    edx
                mov     ecx, dword ptr KbdHalfrow0
                not     al
                ror     al, 1
                cbw
                and     cl, ah
                ror     al, 1
                cbw
                and     ch, ah
                ror     ecx, 16
                ror     al, 1
                cbw
                and     cl, ah
                ror     al, 1
                cbw
                and     ch, ah
                mov     edx, dword ptr KbdHalfrow4
                ror     al, 1
                cbw
                and     dl, ah
                ror     al, 1
                cbw
                and     dh, ah
                ror     al, 1
                cbw
                and     ah, KbdHalfrow6
                ror     al, 1
                sbb     al, al
                and     al, KbdHalfrow7
                or      ah, al
                or      ah, dl
                or      ah, dh
                or      ah, cl
                or      ah, ch
                ror     ecx, 16
                or      ah, cl
                or      ah, ch
                xor     ah, 01Fh
;                test    CFG_MixAudioEnable, 080000000h
;                jz      Pk_2
;                mov     dx, CFG_SBAddress
;                cmp     dx, 0FFFFh
;                jz      Pk_1
;                add     dl, 00Ch
;                test    SBFlagsIO, 001h
;                jnz     Pk_01
;                in      al, dx
;                test    al, 080h
;                jnz     Pk_1
;                mov     al, SBDSP_DIRECT_8_ADC
;                out     dx, al
;                or      SBFlagsIO, 001h
;                jmp     Pk_1
;Pk_01:
;                sub     dl, 2
;                in      al, dx
;                and     SBFlagsIO, not(001h)
;                mov     ADCState, al
;Pk_1:
;                mov     al, ADCState
;                cmp     al, ADCSense
;                setnc   al
;                ror     al, 2
;                or      ah, al
Pk_2:
                pop     edx
                ret
                align   4
Port_ULA_Write:
                mov     al, [ebx].SystemZX48
                xor     al, ah
                mov     [ebx].SystemZX48, ah
                test    al, 007h
                jz      Ps48_2
                UPDSCR
Ps48_1:
                mov     eax, [ebx].VideoCounter
                and     eax, 03Fh
                add     eax, VideoShiftPoint
                mov     al, [eax]
                mov     [ebx].VideoBorderGfxL, al
                mov     al, [ebx].SystemZX48
                and     eax, 007h
                shr     [ebx].VideoBorderGfxH, 4
                mov     ah, [eax + ZXPalleteTableB]
                and     ah, 0F0h
                or      [ebx].VideoBorderGfxH, ah
Ps48_2:
                mov     al, [ebx].SystemZX48
                shl     al, 4
                sbb     eax, eax
                and     eax, CFG_MixBeeperVolume
                mov     [ebx].BeeperLevel, eax
                mov     al, [ebx].SystemZX48
                shl     al, 5
                sbb     eax, eax
                not     eax
                and     eax, CFG_MixTapeVolume
                mov     [ebx].TapeLevel, eax
                ret
                align   4
Port_CFG_SamRam:
                ret
                align   4
Port_NONE:
                ret
                align   4
Port_KJoystick_Read:
                mov     ah, Kempston
                ret
                align   4
Port_KMouse_Read:
                cmp     MousePresense, 0DFh
                mov     ah, 0FFh
                jnz     Pkm9
                cmp     al, 0FBh
                jnz     Pkm1
Port_KMouse_ReadXCoord:
                push    ebx edx
                mov     ax, 3
                INTX    33h
                mov     MouseCoordX, cl
                mov     ah, MouseCoordX
                jmp     Pkm8
Pkm1:
                cmp     al, 0FFh
                jnz     Pkm2
Port_KMouse_ReadYCoord:
                push    ebx edx
                mov     ax, 3
                INTX    33h
                not     dl
                mov     MouseCoordY, dl
                mov     ah, MouseCoordY
                jmp     Pkm8
Pkm2:
                cmp     al, 0FAh
                jnz     Pkm9
Port_KMouse_ReadButtons:
                push    ebx edx
                mov     eax, 5
                xor     ebx, ebx
                INTX    33h
                not     al
                mov     MouseKeys, al
                mov     ah, al
Pkm8:
                pop     edx ebx
Pkm9:
                ret
                align   4
Port_ULA_ReadBus:
                mov     ah, [ebx].SystemZXSync
                ret


Port_BDI_WriteCmd:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_WriteCommandReg

Port_BDI_ReadStatus:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_ReadStatusReg

Port_BDI_WriteTrack:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_WriteTrackReg

Port_BDI_ReadTrack:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_ReadTrackReg

Port_BDI_WriteSector:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_WriteSectorReg

Port_BDI_ReadSector:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_ReadSectorReg

Port_BDI_WriteData:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_WriteDataReg

Port_BDI_ReadData:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_ReadDataReg

Port_BDI_WriteSys:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_WriteSystemReg

Port_BDI_ReadSys:
                lea     esi, [ebx].FDCtrd
                jmp     FDC_ReadSystemReg


;****************************************************************************;

.DATA

;                            port    mask    space prio  function
DevicesSet_48k      label   word
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_RD_KBD>
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_WR_ULA>
                    DEVDESC <000FFh, 00000h, 001h, 0FFh, ID_DEVICE_RD_BUS>
                    DEVDESC <>
DevicesSet_48kBDI   label   word
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_RD_KBD>
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_WR_ULA>
                    DEVDESC <000FFh, 00000h, 001h, 0FFh, ID_DEVICE_RD_BUS>
                    DEVDESC <0001Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDICMD>
                    DEVDESC <0001Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDISTAT>
                    DEVDESC <0003Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDITRK>
                    DEVDESC <0003Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDITRK>
                    DEVDESC <0005Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDISECT>
                    DEVDESC <0005Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDISECT>
                    DEVDESC <0007Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDIDATA>
                    DEVDESC <0007Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDIDATA>
                    DEVDESC <000FFh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDISYS>
                    DEVDESC <000FFh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDISYS>
                    DEVDESC <>
DevicesSet_128k     label   word
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_RD_KBD>
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_WR_ULA>
                    DEVDESC <000FFh, 00000h, 001h, 0FFh, ID_DEVICE_RD_BUS>
                    DEVDESC <0FFFDh, 0C003h, 001h, 1,    ID_DEVICE_WR_AY0SEL>
                    DEVDESC <0FFFDh, 0C003h, 001h, 1,    ID_DEVICE_RD_AY0REG>
                    DEVDESC <0BFFDh, 0C003h, 001h, 1,    ID_DEVICE_WR_AY0REG>
                    DEVDESC <07FFDh, 0C003h, 001h, 1,    ID_DEVICE_WR_MMU128K>
                    DEVDESC <>
DevicesSet_128kBDI  label   word
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_RD_KBD>
                    DEVDESC <000FEh, 00001h, 001h, 0,    ID_DEVICE_WR_ULA>
                    DEVDESC <000FFh, 00000h, 001h, 0FFh, ID_DEVICE_RD_BUS>
                    DEVDESC <0FFFDh, 0C003h, 001h, 1,    ID_DEVICE_WR_AY0SEL>
                    DEVDESC <0FFFDh, 0C003h, 001h, 1,    ID_DEVICE_RD_AY0REG>
                    DEVDESC <0BFFDh, 0C003h, 001h, 1,    ID_DEVICE_WR_AY0REG>
                    DEVDESC <07FFDh, 0C003h, 003h, 1,    ID_DEVICE_WR_MMU128K>
                    DEVDESC <0001Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDICMD>
                    DEVDESC <0001Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDISTAT>
                    DEVDESC <0003Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDITRK>
                    DEVDESC <0003Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDITRK>
                    DEVDESC <0005Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDISECT>
                    DEVDESC <0005Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDISECT>
                    DEVDESC <0007Fh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDIDATA>
                    DEVDESC <0007Fh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDIDATA>
                    DEVDESC <000FFh, 000F2h, 002h, 1,    ID_DEVICE_WR_BDISYS>
                    DEVDESC <000FFh, 000F2h, 002h, 1,    ID_DEVICE_RD_BDISYS>
                    DEVDESC <>
DevicesList         label   dword
                    dd      ID_DEVICE_RD_KBD,       Port_ULA_ReadKbd
                    dd      ID_DEVICE_WR_ULA,       Port_ULA_Write
                    dd      ID_DEVICE_RD_BUS,       Port_ULA_ReadBus
                    dd      ID_DEVICE_WR_MMU128K,   Port_128_Write
                    dd      ID_DEVICE_RD_AY0REG,    Port_AY0_ReadReg
                    dd      ID_DEVICE_WR_AY0SEL,    Port_AY0_SelectReg
                    dd      ID_DEVICE_WR_AY0REG,    Port_AY0_WriteReg
                    dd      ID_DEVICE_RD_KJOYSTICK, Port_KJoystick_Read
                    dd      ID_DEVICE_RD_KMOUSX,    Port_KMouse_ReadXCoord
                    dd      ID_DEVICE_RD_KMOUSY,    Port_KMouse_ReadYCoord
                    dd      ID_DEVICE_RD_KMOUSBTNS, Port_KMouse_ReadButtons
                    dd      ID_DEVICE_RD_BDISTAT,   Port_BDI_ReadStatus
                    dd      ID_DEVICE_WR_BDICMD,    Port_BDI_WriteCmd
                    dd      ID_DEVICE_RD_BDITRK,    Port_BDI_ReadTrack
                    dd      ID_DEVICE_WR_BDITRK,    Port_BDI_WriteTrack
                    dd      ID_DEVICE_RD_BDISECT,   Port_BDI_ReadSector
                    dd      ID_DEVICE_WR_BDISECT,   Port_BDI_WriteSector
                    dd      ID_DEVICE_RD_BDIDATA,   Port_BDI_ReadData
                    dd      ID_DEVICE_WR_BDIDATA,   Port_BDI_WriteData
                    dd      ID_DEVICE_RD_BDISYS,    Port_BDI_ReadSys
                    dd      ID_DEVICE_WR_BDISYS,    Port_BDI_WriteSys
                    dd      0FFFFh,                 Port_NONE


DOSPortInTable      label   dword
                    dd      Port_AY0_ReadReg, Port_BDI_ReadStatus,  Port_AY0_ReadReg, Port_BDI_ReadTrack
                    dd      Port_AY0_ReadReg, Port_BDI_ReadSector,  Port_AY0_ReadReg, Port_BDI_ReadData
                    dd      Port_AY0_ReadReg, Port_BDI_ReadSys,     Port_AY0_ReadReg, Port_BDI_ReadSys
                    dd      Port_AY0_ReadReg, Port_BDI_ReadSys,     Port_AY0_ReadReg, Port_BDI_ReadSys
DOSPortOutTable     label   dword
                    dd      Port_128_Write,  Port_BDI_WriteCmd,     Port_128_Write,   Port_BDI_WriteTrack
                    dd      Port_128_Write,  Port_BDI_WriteSector,  Port_128_Write,   Port_BDI_WriteData
                    dd      Port_128_Write,  Port_BDI_WriteSys,     Port_128_Write,   Port_BDI_WriteSys
                    dd      Port_128_Write,  Port_BDI_WriteSys,     Port_128_Write,   Port_BDI_WriteSys
STDPortInTable      label   dword
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 000h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 004h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 008h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 00Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 010h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 014h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 018h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 01Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 020h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 024h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 028h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 02Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 030h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 034h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 038h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 03Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 040h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 044h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 048h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 04Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 050h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 054h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 058h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 05Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 060h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 064h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 068h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 06Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 070h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 074h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 078h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 07Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 080h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 084h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 088h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 08Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 090h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 094h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 098h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KJoystick_Read        ; 09Ch
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0A0h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0A4h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0A8h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0ACh
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0B0h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0B4h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0B8h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0BCh
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0C0h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0C4h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0C8h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0CCh
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0D0h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0D4h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0D8h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_KMouse_Read       ; 0DCh
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0E0h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0E4h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0E8h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0ECh
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0F0h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0F4h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0F8h
                    dd      Port_ULA_ReadKbd,   Port_AY0_ReadReg, Port_ULA_ReadKbd,   Port_ULA_ReadBus       ; 0FCh
STDPortOutTable     label   dword
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;00h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;04h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;08h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;0Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;10h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;14h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;18h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_CFG_SamRam     ;1Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;20h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;24h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;28h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;2Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;30h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;34h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;38h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;3Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;40h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;44h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;48h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;4Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;50h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;54h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;58h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;5Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;60h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;64h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;68h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;6Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;70h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;74h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;78h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;7Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;80h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;84h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;88h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;8Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;90h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;94h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;98h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;9Ch
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;A0h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;A4h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;A8h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;ACh
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;B0h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;B4h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;B8h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;BCh
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;C0h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;C4h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;C8h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;CCh
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;D0h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;D4h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;D8h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;DCh
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;E0h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;E4h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;E8h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;ECh
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;F0h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;F4h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;F8h
                    dd      Port_ULA_Write,Port_128_Write,Port_ULA_Write,Port_NONE       ;FCh


ADCSense        db 080h

MouseCoordX     db 0
MouseCoordY     db 0
MouseKeys       db 0



;****************************************************************************;

