
include defines.inc
include macro.inc


;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

; -=* R-E-S-T-O-R-E---A-L-L---U-S-E-D---R-E-S-O-U-R-C-E-S *=-

ResourceRestore:
                    pushad
                    mov     ds, cs:sel_Data
RRscreen:
                    ; set standart video mode
                    test    InitResources, IR_SCREEN1
                    jz      RRscr_1
                    mov     eax, 013h
                    int     10h
                    mov     eax, 3
                    int     10h
                    and     InitResources, not(IR_SCREEN1)
RRscr_1:

RRtimerdevice:
                    ; free timer
                    xor     eax, eax
                    call    PIT_WriteLatch

RRkeyboard:
                    ; reset keyboard
                    mov     al, 0F4h
                    out     060h, al

RRdmadevice:
                    ; stop DMA
                    test    CriticalRuntime, CR_DMASTOP
                    jnz     RRdd_1
                    mov     al, CFG_SBDMAChannel
                    call    DMA_Stop
RRdd_1:

RRsoundblaster:
                    ; reset Sound Blaster
                    test    CriticalRuntime, CR_SBRESET
                    jnz     RRsb_1
                    call    SB_Reset
RRsb_1:

RRpathlocation:
                    ; restore path location before running VZX
                    PCHR    "*"
                    test    InitResources, IR_VZXDIR
                    jz      RRfreememory
                    PCHR    "*"
                    ; set current path
                    lea     edx, DOSCurrentDir
                    call    File_SetPath
                    jc      RRfreememory
                    and     InitResources, not(IR_VZXDIR)

RRfreememory:
                    mov     eax, 00101h
                    mov     dx, sel_DataDOS
                    int     31h

RRinterrupthandlers:
                    ; set old interrupt handlers
                    test    CriticalRuntime, CR_GETSETVECTOR
                    jnz     RRih_3
                    mov     ecx, 16
                    lea     esi, InterruptsOld
RRih_1:
                    push    ecx
                    mov     ax, 00B10h ;08B10h
                    sub     al, cl
                    mov     ecx, [esi + 4]
                    jcxz    RRih_2
                    mov     edx, [esi]
                    call    GetSet_VectorHandler
RRih_2:
                    lea     esi, [esi + 6]
                    pop     ecx
                    loop    RRih_1
RRih_3:

RRpicdevice:
                    ; restore PIC device
                    test    InitResources, IR_PIC
                    jz      RRpd_1
                    test    CriticalRuntime, CR_PICRESTORE
                    jnz     RRpd_1
                    mov     ax, DOSIntMasks
                    call    PICs_Restore
                    mov     al, 020h
                    out     020h, al
                    out     0A0h, al
RRpd_1:

RRmouse:
                    ; reset mouse
                    EINTS
                    xor     eax, eax
                    int     33h

RRfiles:
                    ; close all opened files
                    call    File_CloseAll

RRend:
                    popad
                    ret

;----------------------------------------------------------------------------;

TMR_Init:
                    ;------------------------------------------
                    ; IN:
                    ; OUT:
                    ;------------------------------------------
                    ; TMRI Workspace
TMRIWS              struc
tmriws_begin = $
TMRICurClkkHz       dd      0
TMRICurPITVal       dd      0
TMRINumOfMeasures   dd      0
TMRIRdtscBufferPtr  dd      0
TMRIRdtscBuffer     dd      00100h - (TMRIRdtscBuffer - tmriws_begin) dup(0)
tmriws_rdtscbuf_size = $ - TMRIRdtscBuffer
tmriws_size = $ - tmriws_begin
                    ends
                    ;------------------------------------------
                    push    eax

                    mov     eax, 0100h
                    mov     TMRPeriodKbdLow, eax
                    mov     eax, 00010000h
                    mov     TMRPeriodInt8Low, eax
                    mov     eax, 0001234DDh
                    mov     TMRFrequencyLow, eax
                    xor     eax, eax
                    mov     TMRPeriodKbdHigh, eax
                    mov     TMRPeriodInt8High, eax
                    mov     TMRFrequencyHigh, eax
                    mov     PITReadFlag, al
                    mov     TMRInitializedFlag, al
                    call    PIT_WriteLatch
                    test    CFG_TimerMode, 004h
                    jz      TMRinit_110
                    pushad
                    mov     TMRDegradation, 0

                    ; clean up and init TMRIWS
                    ;--------------------------
                    mov     ecx, tmriws_size / 4
TMRinit_010:
                    push    dword ptr 0
                    loopd   TMRinit_010
                    PSTR    TxtCPUSpeed
TMRinit_020:
                    mov     [esp].TMRINumOfMeasures, 3
TMRinit_030:
                    ; measuring CPU tacts
                    ;---------------------
                    mov     ecx, tmriws_rdtscbuf_size / 8
                    lea     esi, [esp].TMRIRdtscBuffer
                    mov     [esp].TMRIRdtscBufferPtr, esi
TMRinit_040:
                    xor     edi, edi
                    call    TMR_ReadCounter
                    mov     edx, eax
                    call    PIT_ReadCounter
                    mov     ebx, eax
                    mov     [esp].TMRICurPITVal, eax
TMRinit_050:
                    call    TMR_ReadCounter
                    mov     esi, eax
                    call    PIT_ReadCounter
                    sub     ebx, eax
                    and     ebx, 0FFFFh
                    xchg    eax, ebx
                    cmp     edi, eax
                    jnc     TMRinit_060
                    mov     edi, eax
TMRinit_060:
                    mov     eax, [esp].TMRICurPITVal
                    sub     eax, ebx
                    and     eax, 0FFFFh
                    cmp     eax, 1000
                    jc      TMRinit_050
                    cmp     eax, 20000
                    jnc     TMRinit_040
                    mov     ebx, eax
                    sub     edx, esi
                    mov     eax, edx
                    xor     edx, edx
                    div     ebx
                    mov     esi, [esp].TMRIRdtscBufferPtr
                    ; average CPU tacts in one PIT tact
                    mov     [esi], eax
                    ; maximum delay in PIT tacts was occurred while measuring CPU tacts
                    mov     [esi + 4], edi
                    lea     esi, [esi + 8]
                    mov     [esp].TMRIRdtscBufferPtr, esi
                    loopd   TMRinit_040

                    lea     esi, [esp].TMRIRdtscBuffer
                    mov     ecx, tmriws_rdtscbuf_size / 8
                    call    Data_Average

                    mov     ebx, 0001234DDh
                    mul     ebx
                    mov     TMRFrequencyLow, eax
                    mov     TMRFrequencyHigh, edx
                    call    Print_SearchSym
                    mov     ebx, 1000
                    div     ebx
                    cmp     eax, [esp].TMRICurClkkHz
                    mov     [esp].TMRICurClkkHz, eax
                    jnz     TMRinit_020
                    dec     [esp].TMRINumOfMeasures
                    jnz     TMRinit_030
                    mov     eax, TMRFrequencyLow
                    mov     edx, TMRFrequencyHigh
                    mov     ebx, 1000000 ; displaying in MHz
                    div     ebx
                    call    Print_SearchEnd
                    PDECD   eax
                    PSTR    TxtCPUMHz
                    mov     eax, TMRFrequencyLow
                    mov     edx, TMRFrequencyHigh
                    mov     ebx, 0001234DDh
                    div     ebx
                    mov     ecx, eax
                    mov     edx, TMRPeriodKbdLow
                    mul     edx
                    mov     TMRPeriodKbdLow, eax
                    mov     TMRPeriodKbdHigh, edx
                    mov     eax, ecx
                    mov     edx, TMRPeriodInt8Low
                    mul     edx
                    mov     TMRPeriodInt8Low, eax
                    mov     TMRPeriodInt8High, edx
                    lea     esp, [esp + tmriws_size]
                    popad
TMRinit_110:
                    mov     TMRInitializedFlag, 1
                    call    TMR_ReadCounter
                    pop     eax
                    ret

;----------------------------------------------------------------------------;

Data_Average:
                    push    ebx edx edi ebp esi ecx
                    lea     esi, [esi + 4]
                    mov     edi, 2
                    call    Math_QuadraticMean
                    mov     ebx, eax

                    ; sorting values
                    ;----------------
                    dec     ecx
                    lea     edi, [esi - 4]
TMRinit_070:
                    push    ecx
                    lea     esi, [edi + 8]
                    mov     eax, [edi + 4]
TMRinit_080:
                    cmp     eax, [esi + 4]
                    jna     TMRinit_090
                    xchg    eax, [esi + 4]
                    mov     [edi + 4], eax
                    push    eax
                    mov     eax, [edi]
                    xchg    eax, [esi]
                    mov     [edi], eax
                    pop     eax
TMRinit_090:
                    lea     esi, [esi + 8]
                    loopd   TMRinit_080
                    pop     ecx
                    cmp     ebx, [edi + 4]
                    jc      TMRinit_100
                    lea     edi, [edi + 8]
                    loopd   TMRinit_070
TMRinit_100:
;                    xor     eax, eax
;                    mov     [edi], eax
;                    mov     [edi + 4], eax
                    pop     eax esi
                    dec     eax
                    sub     ecx, eax
                    neg     ecx
                    mov     edi, 2
                    call    Math_QuadraticMean
                    pop     ebp edi edx ebx
                    ret

;----------------------------------------------------------------------------;

Math_SquareRoot:
                    ;--------------------------------------------------------;
                    ; Get Square Root
                    ;--------------------------------------------------------;
                    ; - in:
                    ; EDX:EAX - 64-bit source
                    ; - out:
                    ; CF      - NC: EDX:EAX = SQRT(EDX:EAX)
                    ;--------------------------------------------------------;
                    push  ebx ecx esi edi ebp
                    mov   edi, edx
                    mov   esi, eax
                    mov   ebx, 080000000h
                    xor   ebp, ebp
Msqrt_010:
                    mov   eax, ebp
                    mul   eax
                    cmp   edx, edi
                    ja    Msqrt_030
                    jc    Msqrt_020
                    cmp   eax, esi
                    ja    Msqrt_030
                    jz    Msqrt_090
Msqrt_020:
                    ; EDX:EAX < EDI:ESI
                    add   ebp, ebx
                    jz    Msqrt_110
                    jmp   Msqrt_040
Msqrt_030:
                    ; EDX:EAX > EDI:ESI
                    sub   ebp, ebx
Msqrt_040:
                    shr   ebx, 1
                    test  ebx, ebx
                    jnz   Msqrt_010

                    mov   eax, ebp
                    mul   eax
                    cmp   edx, edi
                    ja    Msqrt_060
                    jc    Msqrt_050
                    cmp   eax, esi
                    ja    Msqrt_060
                    jz    Msqrt_090
Msqrt_050:
                    ; EDX:EAX < EDI:ESI
                    lea   ecx, [ebp + 1]
                    test  ecx, ecx
                    jz    Msqrt_110
                    xchg  ecx, ebp
                    jmp   Msqrt_070
Msqrt_060:
                    ; EDX:EAX > EDI:ESI
                    lea   ecx, [ebp - 1]
Msqrt_070:
                    ; ECX < EBP
                    push  esi edi
                    mov   eax, ebp
                    mul   eax
                    mov   edi, edx
                    mov   esi, eax
                    mov   eax, ecx
                    mul   eax
                    add   eax, esi
                    adc   edx, edi
                    rcr   edx, 1
                    rcr   eax, 1
                    pop   edi esi
                    cmp   edx, edi
                    jc    Msqrt_090
                    ja    Msqrt_080
                    cmp   eax, esi
                    jc    Msqrt_090
Msqrt_080:
                    ; EDX:EAX > EDI:ESI
                    mov   ebp, ecx
Msqrt_090:
                    xor   edx, edx
                    mov   eax, ebp
Msqrt_100:
                    clc
                    pop   ebp edi esi ecx ebx
                    ret
Msqrt_110:
                    mov   edx, 1
                    xor   eax, eax
                    jmp   Msqrt_100

;----------------------------------------------------------------------------;

Math_QuadraticMean:
                    ;--------------------------------------------------------;
                    ; Get Quadratic Mean
                    ;--------------------------------------------------------;
                    ; - in:
                    ; ESI - pointer to source array of 32-bit values
                    ; EDI - increment step in 32-bit units
                    ; ECX - number of elements
                    ; - out:
                    ; CF  - NC: EAX = result
                    ;--------------------------------------------------------;
                    push    ebx ecx edx esi edi ebp
                    push    ecx
                    xor     ebp, ebp
                    xor     ebx, ebx
Mqmean_050:
                    mov     eax, [esi]
                    mul     eax
                    add     ebx, eax
                    adc     ebp, edx
                    lea     esi, [esi + edi*4]
                    loopd   Mqmean_050
                    pop     ecx
                    mov     edx, ebp
                    mov     eax, ebx
;                    call    Math_SquareRoot
;                    push    eax
;                    mov     eax, ecx
;                    xor     edx, edx
;                    call    Math_SquareRoot
;                    mov     ecx, eax
;                    pop     eax
                    div     ecx
                    xor     edx, edx
                    call    Math_SquareRoot
                    pop     ebp edi esi edx ecx ebx
                    ret

;----------------------------------------------------------------------------;

PIT_WaitTicks:
                    push    eax ecx esi
                    mov     ecx, eax
                    call    PIT_ReadCounter32
                    mov     esi, eax
PITwt_1:
                    call    PIT_ReadCounter32
                    sub     eax, esi
                    neg     eax
                    cmp     eax, 1193;000010000h
                    jc      PITwt_1
                    add     esi, 1193;000010000h
                    loopd   PITwt_1

                    pop     esi ecx eax
                    ret

;----------------------------------------------------------------------------;

PIT_ReadCounter:
                    cmp     PITReadFlag, 0
                    jnz     Rpitcnr_6
                    push    edx
                    mov     al, CFG_TimerMode
                    and     al, 003h
                    jz      Rpitcnr_4
                    cmp     al, 2
                    jnz     Rpitcnr_1
                    ; 2 = read back command
                    mov     al, 002h
                    mov     ah, cl
                    mov     cl, CFG_PITChannel
                    shl     al, cl
                    or      al, 0D0h
                    mov     cl, ah
                    mov     dx, 043h
                    out     dx, al
                    jmp     Rpitcnr_2
Rpitcnr_1:
                    cmp     al, 1
                    jnz     Rpitcnr_2
                    ; 1 = latch command
                    mov     al, (TIMER_MODE shl 1)
                    mov     dx, 043h
                    mov     ah, CFG_PITChannel
                    shl     ah, 6
                    or      al, ah
                    out     dx, al
                    jmp     $+2
Rpitcnr_2:
                    ; 3 = simply read counter
                    mov     dx, 040h
                    xor     eax, eax
                    add     dl, CFG_PITChannel
                    inc     PITReadFlag
                    in      al, dx
                    jmp     $+2
                    mov     ah, al
                    in      al, dx
                    dec     PITReadFlag
Rpitcnr_3:
                    xchg    al, ah
                    mov     PITLastReadValue, eax
                    pop     edx
                    ret
Rpitcnr_4:
                    ; 0 = read counter twice
                    mov     dx, 040h
                    xor     eax, eax
                    add     dl, CFG_PITChannel
                    inc     PITReadFlag
                    in      al, dx
                    jmp     $+2
                    mov     ah, al
                    in      al, dx
                    dec     PITReadFlag
                    push    ecx
                    mov     ecx, eax
                    inc     PITReadFlag
                    in      al, dx
                    jmp     $+2
                    mov     ah, al
                    in      al, dx
                    dec     PITReadFlag
                    cmp     cl, al
                    jz      Rpitcnr_5
                    mov     eax, ecx
Rpitcnr_5:
                    pop     ecx
                    jmp     Rpitcnr_3
Rpitcnr_6:
                    mov     eax, PITLastReadValue
                    ret

;----------------------------------------------------------------------------;

TMR_ReadCounter:
                    test    CFG_TimerMode, 004h
                    jnz     TMRrdcnr_5
PIT_ReadCounter32:
                    push    ecx edx
                    mov     ecx, PITLastReadValue
                    call    PIT_ReadCounter
                    sub     ecx, eax
                    and     ecx, 0FFFFh
                    mov     eax, TMRLastReadLow
                    add     eax, ecx
                    mov     TMRLastReadLow, eax
                    adc     TMRLastReadHigh, 0
                    mov     edx, TMRLastReadHigh
TMRrdcnr_0:
                    cmp     TMRInitializedFlag, 0
                    jz      TMRrdcnr_1
                    sub     eax, TMRLastKbdLow
                    sbb     edx, TMRLastKbdHigh
                    sub     eax, TMRPeriodKbdLow
                    sbb     edx, TMRPeriodKbdHigh
                    jnc     TMRrdcnr_2
TMRrdcnr_1:
                    mov     eax, TMRLastReadLow
                    neg     eax
                    pop     edx ecx
                    ret
TMRrdcnr_2:
                    ; polling keyboard
                    ;------------------
                    mov     eax, TMRLastKbdLow
                    or      eax, TMRLastKbdHigh
                    jz      TMRrdcnr_6
TMRrdcnr_3:
                    call    Kbd_Process
                    mov     eax, TMRPeriodKbdLow
                    mov     edx, TMRPeriodKbdHigh
                    add     TMRLastKbdLow, eax
                    adc     TMRLastKbdHigh, edx

                    mov     eax, TMRLastReadLow
                    mov     edx, TMRLastReadHigh
                    sub     eax, TMRLastInt8Low
                    sbb     edx, TMRLastInt8High
                    sub     eax, TMRPeriodInt8Low
                    sbb     edx, TMRPeriodInt8High
                    jc      TMRrdcnr_4

                    ; executing system timer
                    ;------------------------
                    INTX    08h
                    mov     eax, TMRPeriodInt8Low
                    mov     edx, TMRPeriodInt8High
                    add     TMRLastInt8Low, eax
                    adc     TMRLastInt8High, edx
TMRrdcnr_4:
                    mov     eax, TMRLastReadLow
                    mov     edx, TMRLastReadHigh
                    sub     eax, TMRLastSecondLow
                    sbb     edx, TMRLastSecondHigh
                    sub     eax, TMRFrequencyLow
                    sbb     edx, TMRFrequencyHigh
                    jc      TMRrdcnr_1

                    ; updating current second
                    ;-------------------------
                    inc     CMOSCurrentSecond
                    mov     eax, TMRFrequencyLow
                    mov     edx, TMRFrequencyHigh
                    add     TMRLastSecondLow, eax
                    adc     TMRLastSecondHigh, edx
                    jmp     TMRrdcnr_1
TMRrdcnr_5:
                    push    ecx edx
                    db      00Fh, 031h          ; RDTSC
                    mov     cl, TMRDegradation
                    shrd    eax, edx, cl
                    shr     edx, cl
                    mov     TMRLastReadLow, eax
                    mov     TMRLastReadHigh, edx
                    jmp     TMRrdcnr_0
TMRrdcnr_6:
                    ; first call (probably) - (re)init
                    ;----------------------------------
                    mov     eax, TMRLastReadLow
                    mov     edx, TMRLastReadHigh
                    mov     TMRLastKbdLow, eax
                    mov     TMRLastKbdHigh, edx
                    mov     TMRLastInt8Low, eax
                    mov     TMRLastInt8High, edx
                    mov     TMRLastSecondLow, eax
                    mov     TMRLastSecondHigh, edx
                    jmp     TMRrdcnr_3

;----------------------------------------------------------------------------;

PIT_WriteLatch:
                    push    ecx edx
                    mov     ecx, eax
                    mov     al, (TIMER_MODE shl 1) or 030h
                    mov     dx, 00043h
                    mov     ah, CFG_PITChannel
                    shl     ah, 6
                    or      al, ah
                    out     dx, al
                    jmp     $+2
                    mov     dl, 040h
                    mov     al, cl
                    add     dl, CFG_PITChannel
                    out     dx, al
                    jmp     $+2
                    mov     al, ch
                    out     dx, al
                    jmp     $+2
                    mov     eax, ecx
                    mov     PITLatchValue, eax
                    pop     edx ecx
                    ret

;----------------------------------------------------------------------------;

DMA_Start:
                    ;--------------------------------------------------------;
                    ; Start DMA channel
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL   - channel,
                    ; AH   - mode,
                    ; EBX  - physical address,
                    ; ECX  - counter in bytes
                    ; - out:
                    ; CF   - NC: DMA successfully initialized
                    ;         C: invalid channel or address/counter
                    ;--------------------------------------------------------;
                    or      CriticalRuntime, CR_DMASTART
                    pushad
                    mov     esi, ebx
                    mov     ebx, eax
                    cmp     bl, 4
                    jnc     DMAstart_1
                    cmp     esi, 01000000h
                    jnc     DMAstart_3
                    test    ecx, ecx
                    jz      DMAstart_3
                    ; DMA 8-bit channel initialization
                    and     bx, 0FC03h
                    movzx   edi, bl
                    call    DMA_Pause
                    xor     al, al
                    mov     dx, 00Ch
                    out     dx, al      ; stop all current DMA processes
                    mov     al, bl
                    mov     dl, 00Bh
                    or      al, bh
                    out     dx, al      ; mode register
                    lea     edx, [edi*2]
                    mov     eax, esi
                    out     dx, al      ; base address register [A0...A15]
                    mov     al, ah
                    out     dx, al
                    ror     eax, 16
                    mov     dl, [edi + DMAPagePorts]
                    out     dx, al      ; page register [A16...A23]
                    lea     edx, [edi*2 + 1]
                    mov     eax, ecx
                    dec     eax
                    out     dx, al      ; base count register [D0...D15]
                    mov     al, ah
                    out     dx, al
                    mov     al, bl
                    call    DMA_Continue
;                    mov     dx, 9
;                    out     dx, al      ; request register (???)
                    jmp     DMAstart_2
DMAstart_1:
                    cmp     bl, 8
                    jnc     DMAstart_3
                    cmp     esi, 02000000h
                    jnc     DMAstart_3
                    shr     ecx, 1
                    jna     DMAstart_3      ; CF=1 & ZF=1
                    ; DMA 16-bit channel initialization
                    and     bx, 0FC07h
                    movzx   edi, bl
                    call    DMA_Pause
                    xor     al, al
                    mov     dx, 0D8h
                    out     dx, al      ; stop all current DMA processes
                    mov     al, bl
                    and     al, 003h
                    mov     dl, 0D6h
                    or      al, bh
                    out     dx, al      ; mode register
                    lea     edx, [edi*4 + 0C0h - 010h]
                    mov     eax, esi
                    shr     eax, 1
                    out     dx, al      ; base address register [A1...A16], A0=0
                    mov     al, ah
                    out     dx, al
                    ror     eax, 16
                    mov     dl, [edi + DMAPagePorts]
                    out     dx, al      ; page register [A17...A24]
                    lea     edx, [edi*4 + 0C2h - 010h]
                    mov     eax, ecx
                    dec     eax
                    out     dx, al      ; base count register [D1...D16], D0=0
                    mov     al, ah
                    out     dx, al
                    mov     al, bl
                    call    DMA_Continue
;                    mov     dx, 0D2h
;                    out     dx, al      ; request register (???)
DMAstart_2:
                    popad
                    and     CriticalRuntime, 0FFh - CR_DMASTART
                    clc
                    ret
DMAstart_3:
                    popad
                    and     CriticalRuntime, 0FFh - CR_DMASTART
                    stc
                    ret

;----------------------------------------------------------------------------;

DMA_Stop:
                    ;--------------------------------------------------------;
                    ; Stop DMA channel
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - channel,
                    ; - out:
                    ; CF  - NC: OK
                    ;        C: invalid channel
                    ;--------------------------------------------------------;
                    cmp     al, 8
                    jc      DMAstop_0
                    stc
                    ret
DMAstop_0:
                    or      CriticalRuntime, CR_DMASTOP
                    push    eax edx ecx
                    mov     cx, 00Ch
                    mov     dx, 009h
                    cmp     al, 4
                    jc      DMAstop_2
DMAstop_1:
                    mov     cl, 0D8h
                    mov     dl, 0D2h
DMAstop_2:
                    ; mask channel
                    call    DMA_Pause
                    ; clear request
                    and     al, 003h
                    out     dx, al
                    ; send clear command
                    mov     dx, cx
                    xor     al, al
                    out     dx, al
                    ; unmask channel
                    ;call    DMA_Continue

                    pop     ecx edx eax
                    and     CriticalRuntime, not(CR_DMASTOP)
                    clc
                    ret

;----------------------------------------------------------------------------;

DMA_Continue:
                    ;--------------------------------------------------------;
                    ; Continue (unpause) DMA channel
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - channel,
                    ; - out:
                    ; CF  - NC: OK
                    ;        C: invalid channel
                    ;--------------------------------------------------------;
                    push    eax edx
                    mov     dx, 00Ah
                    cmp     al, 4
                    jc      DMAcont_2
DMAcont_1:
                    cmp     al, 8
                    jnc     DMAcont_3
                    mov     dl, 0D4h
DMAcont_2:
                    ; unmask channel
                    and     al, 003h
                    out     dx, al

                    pop     edx eax
                    clc
                    ret
DMAcont_3:
                    pop     edx eax
                    stc
                    ret

;----------------------------------------------------------------------------;

DMA_Pause:
                    ;--------------------------------------------------------;
                    ; Pause DMA channel
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - channel,
                    ; - out:
                    ; CF  - NC: OK
                    ;        C: invalid channel
                    ;--------------------------------------------------------;
                    push    eax edx
                    mov     dx, 00Ah
                    cmp     al, 4
                    jc      DMApau_2
DMApau_1:
                    cmp     al, 8
                    jnc     DMApau_3
                    mov     dl, 0D4h
DMApau_2:
                    ; mask channel
                    or      al, 004h
                    out     dx, al

                    pop     edx eax
                    clc
                    ret
DMApau_3:
                    pop     edx eax
                    stc
                    ret

;----------------------------------------------------------------------------;

DMA_GetRequests:
                    ;--------------------------------------------------------;
                    ; Get DMA requests
                    ;--------------------------------------------------------;
                    ; - in:
                    ; none
                    ; - out:
                    ; AL  - D0..D7 - DMA requests
                    ;--------------------------------------------------------;
                    push    ebx edx
                    mov     dx, 8
                    in      al, dx
                    shr     al, 4
                    mov     bl, al
                    mov     dx, 0D0h
                    in      al, dx
                    and     al, 0F0h
                    or      al, bl
                    pop     edx ebx
                    ret

;----------------------------------------------------------------------------;

DMA_GetCount:
                    ;--------------------------------------------------------;
                    ; Get DMA channel count register
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - channel,
                    ; - out:
                    ; CF  - NC: EAX - count
                    ;        C: invalid channel or other error
                    ;--------------------------------------------------------;
                    push    edx
                    xor     eax, eax
                    cmp     al, 4
                    jc      DMAgetcnt_1
                    cmp     al, 8
                    jnc     DMAgetcnt_3
                    sub     al, 4
                    movzx   edx, al
                    lea     edx, [edx*4 + 0C2h]
                    jmp     DMAgetcnt_2
DMAgetcnt_1:
                    movzx   edx, al
                    lea     edx, [edx*2 + 001h]
DMAgetcnt_2:
                    in      al, dx
                    mov     ah, al
                    in      al, dx
                    xchg    al, ah
                    clc
                    pop     edx
                    ret
DMAgetcnt_3:
                    stc
                    pop     edx
                    ret

;----------------------------------------------------------------------------;

DMA_GetAddr:
                    ;--------------------------------------------------------;
                    ; Returns DMA channel address register
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - channel,
                    ; - out:
                    ; CF  - NC: EAX - address
                    ;        C: invalid channel or other error
                    ;--------------------------------------------------------;
                    push    edx
                    xor     eax, eax
                    cmp     al, 4
                    jc      DMAgetadr_1
                    cmp     al, 8
                    jnc     DMAgetadr_3
                    sub     al, 4
                    movzx   edx, al
                    lea     edx, [edx*4 + 0C0h]
                    jmp     DMAgetadr_2
DMAgetadr_1:
                    movzx   edx, al
                    lea     edx, [edx*2]
DMAgetadr_2:
                    in      al, dx
                    mov     ah, al
                    in      al, dx
                    xchg    al, ah
                    clc
                    pop     edx
                    ret
DMAgetadr_3:
                    stc
                    pop     edx
                    ret

;----------------------------------------------------------------------------;

PICs_Init:
                    push    eax ecx
                    mov     ecx, eax

                    ; master PIC
                    mov     al, cl
                    out     021h, al        ; OCW1 - interrupt mask
                    mov     al, 00Ah
                    out     020h, al        ; read IRR by default

                    ; slave PIC
                    mov     al, ch
                    out     0A1h, al        ; OCW1
                    mov     al, 00Ah
                    out     020h, al        ; read IRR by default

                    pop     ecx eax
                    ret

;----------------------------------------------------------------------------;

PICs_Restore:
                    or      CriticalRuntime, CR_PICRESTORE
                    push    eax ecx
                    mov     ecx, eax

                    ; master PIC
                    mov     al, cl
                    out     021h, al        ; OCW1 - interrupt mask
                    mov     al, 00Ah
                    out     020h, al        ; read IRR by default

                    ; slave PIC
                    mov     al, ch
                    out     0A1h, al        ; OCW1
                    mov     al, 00Ah
                    out     020h, al        ; read IRR by default

                    pop     ecx eax
                    and     CriticalRuntime, not(CR_PICRESTORE)
                    ret

;----------------------------------------------------------------------------;

Get_CurrentSecond:
                    test    CFG_TimerMode, 080h
                    jnz     Gcursec_3
                    mov     al, CMOSCurrentSecond
                    cmp     al, CMOSLastSecond
                    mov     CMOSLastSecond, al
                    ret
Gcursec_3:
                    xor     al, al
                    out     070h, al
                    jmp     $+2
                    in      al, 071h
                    mov     CMOSCurrentSecond, al
                    cmp     al, CMOSLastSecond
                    mov     CMOSLastSecond, al
                    ret

;----------------------------------------------------------------------------;

Print_String:
                    ; DS:ESI
                    push    edx esi
Pstr_1:
                    mov     dl, [esi]
                    test    dl, dl
                    jz      Pstr_End
                    PCHR    dl
                    inc     esi
                    jmp     Pstr_1
Pstr_End:
                    pop     esi edx
                    ret

;----------------------------------------------------------------------------;

Print_CRLF:
                    PCHR    00Dh
                    PCHR    00Ah
                    ret

;----------------------------------------------------------------------------;

Print_Symbol:
                    test    dl, 0FFh
                    jz      Print_CRLF
                    test    InitResources, IR_SCREEN1
                    jnz     Draw_Symbol
                    push    eax
                    mov     ah, 2
                    int     21h                     ; DOS - WRITE CHARACTER
                    pop     eax
                    ret

;----------------------------------------------------------------------------;

Draw_Symbol:
                    pushad
                    cmp     dl, 00Dh
                    jnz     Dsym_01
                    ; Carriage Return
                    mov     SymbolXPosition, 0
                    jmp     Dsym_90
Dsym_01:
                    cmp     dl, 00Ah
                    jnz     Dsym_02
                    ; Line Feed
                    mov     eax, CFG_VideoYVisible
                    shr     eax, 3
                    mov     ah, SymbolYPosition
                    inc     ah
                    cmp     al, ah
                    jna     Dsym_90
                    mov     SymbolYPosition, ah
                    jmp     Dsym_90
Dsym_02:
                    cmp     dl, 008h
                    jnz     Dsym_05
                    ; Backspace
                    cmp     SymbolXPosition, 0
                    jnz     Dsym_04
                    cmp     SymbolYPosition, 0
                    jz      Dsym_03
                    dec     SymbolYPosition
Dsym_03:
                    mov     al, CFG_VideoXVisible
                    inc     al
                    mov     SymbolXPosition, al
Dsym_04:
                    dec     SymbolXPosition
                    jmp     Dsym_90
Dsym_05:
                    mov     ax, not(VM_FLAGS_MASK)
                    and     ax, CFG_VideoModeMain
                    jnz     Dsym_20
                    ;-------------------------------------
                    ; Draw symbol in pseudo-graphics mode
                    ;-------------------------------------
                    movzx   eax, SymbolYPosition
                    movzx   edi, dl
                    lea     edi, [edi*8 + MSGSystemFont]
                    movzx   ebx, CFG_VideoXVisible
                    shl     ebx, 1
                    add     ebx, VideoBytesSkip
                    mul     ebx
                    shl     eax, 3
                    movzx   ecx, SymbolXPosition
                    shl     ecx, 1
                    add     eax, ecx
                    mov     esi, eax
                    add     esi, VideoAreaBaseAdr
                    mov     ecx, 8
Dsym_10:
                    mov     al, [edi]
                    mov     [esi], al
                    inc     edi
                    mov     al, [esi + 1]
                    and     al, 0F0h
                    cmp     al, 040h
                    jnc     Dsym_11
                    or      al, 007h
Dsym_11:
                    mov     [esi + 1], al
                    lea     esi, [esi + ebx]
                    loop    Dsym_10
                    jmp     Dsym_50
Dsym_20:
                    ;-------------------------------------------------------
                    ; Draw symbol in graphics mode (320x200, 320x240, etc.)
                    ;-------------------------------------------------------
                    xor     eax, eax
                    mov     al, SymbolYPosition
                    mov     ah, CFG_VideoXVisible
                    mul     ah
                    shl     eax, 3
                    add     al, SymbolXPosition
                    adc     ah, 0
                    movzx   ebp, CFG_VideoXVisible
                    shl     ebp, 3
                    mov     esi, eax
                    mov     word ptr [esi*2 + VideoBuffer], 000FFh
                    shl     esi, 3
                    add     esi, VideoAreaBaseAdr
                    movzx   edi, dl
                    lea     edi, [edi*8 + MSGSystemFont]
                    mov     ecx, 8
Dsym_21:
                    mov     ah, [edi]
                    movzx   ebx, byte ptr ZXPalleteTableA + 007h
                    shl     ebx, 4
                    shr     bl, 4
                    STPOINT dh
                    STPOINT dl
                    ror     edx, 16
                    STPOINT dh
                    STPOINT dl
                    xor     [esi + 4], edx
                    STPOINT dh
                    STPOINT dl
                    ror     edx, 16
                    STPOINT dh
                    STPOINT dl
                    xor     [esi], edx
                    inc     edi
                    lea     esi, [esi + ebp]
                    dec     ecx
                    jnz     Dsym_21
Dsym_50:
                    inc     SymbolXPosition
                    mov     al, CFG_VideoXVisible
                    cmp     al, SymbolXPosition
                    ja      Dsym_90
                    mov     SymbolXPosition, 0
                    inc     SymbolYPosition
;                    mov     eax, CFG_VideoYVisible
;                    shr     eax, 3
;                    cmp     al, SymbolYPosition
;                    ja      Dsym_90
;                    dec     SymbolYPosition
Dsym_90:
                    popad
                    ret

;----------------------------------------------------------------------------;

Print_DecDword_Fix:
                    push    eax ebx ecx edx
                    mov     dl, 11
                    sub     dl, ch
                    mov     ch, dl
                    xor     cl, cl
                    jmp     Pddword_1
Print_DecDword:
                    push    eax ebx ecx edx
                    xor     ecx, ecx
Pddword_1:
                    mov     ebx, 1000000000
                    call    Print_DecDigit
                    mov     ebx, 100000000
                    call    Print_DecDigit
                    mov     ebx, 10000000
                    call    Print_DecDigit
                    mov     ebx, 1000000
                    call    Print_DecDigit
                    mov     ebx, 100000
                    call    Print_DecDigit
Pddword_2:
                    mov     ebx, 10000
                    call    Print_DecDigit
                    mov     ebx, 1000
                    call    Print_DecDigit
Pddword_3:
                    mov     ebx, 100
                    call    Print_DecDigit
                    mov     ebx, 10
                    call    Print_DecDigit
                    add     al, "0"
                    PCHR
                    pop     edx ecx ebx eax
                    ret

;----------------------------------------------------------------------------;

Print_DecWord_Fix:
                    push    eax ebx ecx edx
                    mov     dl, 6
                    sub     dl, ch
                    mov     ch, dl
                    xor     cl, cl
                    and     eax, 0FFFFh
                    jmp     Pddword_2
Print_DecWord:
                    push    eax ebx ecx edx
                    and     eax, 0FFFFh
                    xor     ecx, ecx
                    jmp     Pddword_2

;----------------------------------------------------------------------------;

Print_DecByte_Fix:
                    push    eax ebx ecx edx
                    mov     dl, 4
                    sub     dl, ch
                    mov     ch, dl
                    xor     cl, cl
                    and     eax, 0FFh
                    jmp     Pddword_3
Print_DecByte:
                    push    eax ebx ecx edx
                    and     eax, 0FFh
                    xor     ecx, ecx
                    jmp     Pddword_3

;----------------------------------------------------------------------------;

Print_DecDigit:
                    ; EBX - divider
                    ; EAX - number
                    xor     edx, edx
                    div     ebx
                    dec     ch
                    jz      Pddigit_1
                    or      cl, al
                    jz      Pddigit_2
Pddigit_1:
                    xor     ch, ch
                    add     al, "0"
                    inc     cl
                    PCHR
Pddigit_2:
                    mov     eax, edx
                    ret

;----------------------------------------------------------------------------;
Print_HexDword:
                    rol     eax, 16
                    call    Print_HexWord
                    rol     eax, 16
;----------------------------------------------------------------------------;
Print_HexWord:
                    xchg    al, ah
                    call    Print_HexByte
                    xchg    al, ah
;----------------------------------------------------------------------------;
Print_HexByte:
                    push    eax
                    mov     ah, al

                    ror     al, 4
                    call    Print_HexNibble
                    mov     al, ah
                    call    Print_HexNibble

                    pop     eax
                    ret
;----------------------------------------------------------------------------;
Print_HexNibble:
                    and     al, 00Fh
                    cmp     al, 10
                    jc      Phn_1
                    add     al, 7
Phn_1:
                    add     al, "0"
                    PCHR
                    ret
;----------------------------------------------------------------------------;
Print_BinDword:
                    rol     eax, 16
                    call    Print_BinWord
                    rol     eax, 16
;----------------------------------------------------------------------------;
Print_BinWord:
                    xchg    al, ah
                    call    Print_BinByte
                    xchg    al, ah
;----------------------------------------------------------------------------;
Print_BinByte:
                    push    eax ecx
                    mov     ecx, 8
Pbin1:
                    xor     ah, ah
                    rol     al, 1
                    adc     ah, "0"
                    xchg    al, ah
                    PCHR
                    mov     al, ah

                    loopd   Pbin1

                    pop     ecx eax
                    ret

;----------------------------------------------------------------------------;

Print_SearchSym:
                    pushad
                    mov     esi, SearchingSymPtr
                    cmp     esi, searchsyms_size
                    jnc     Prnss_1
                    mov     al, [esi + offset SearchingSymbol]
                    test    al, al
                    jnz     Prnss_2
Prnss_1:
                    xor     esi, esi
                    mov     al, [esi + offset SearchingSymbol]
Prnss_2:
                    PCHR    al
                    PCHR    8
                    inc     esi
                    mov     SearchingSymPtr, esi
                    popad
                    ret

;----------------------------------------------------------------------------;

Print_SearchEnd:
                    PCHR    " "
                    PCHR    8
                    ret

;----------------------------------------------------------------------------;

File_OperatePathName:
                    ;--------------------------------------------------------;
                    ; Get text ptr
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - options: 0=get ptr to end of pathname, 1=get
                    ;                ptr to extension, 2=get ptr to filename,
                    ;                3=get ptr to pathname except drive,
                    ;                4=get drive number,
                    ;                8=convert pathname to canonical with LFN
                    ; EDX - address of pathname
                    ; - out:
                    ; CF  - NC: EDX - ptr or drive number
                    ;        C: some error was occured
                    ;--------------------------------------------------------;
                    push    eax ecx
                    test    al, al
                    jnz     Fileopn_020
                    ; find end of pathname
                    mov     ecx, 0200h
                    xor     al, al
Fileopn_010:
                    cmp     al, [edx]
                    jz      Fileopn_End
                    inc     edx
                    loopd   Fileopn_010
                    jmp     Fileopn_ERR             ; end of full path name not found
Fileopn_020:
                    cmp     al, 1
                    jnz     Fileopn_050
                    ; find file extension
                    xor     al, al
                    call    File_OperatePathName
                    jc      Fileopn_End
                    mov     ecx, 0100h
                    mov     al, "."
                    mov     ah, "\"
Fileopn_030:
                    dec     edx
                    cmp     ah, [edx]
                    jz      Fileopn_ERR             ; directory delimiter found first
                    cmp     al, [edx]
                    jz      Fileopn_040
                    loopd   Fileopn_030
                    jmp     Fileopn_ERR             ; file extension not found
Fileopn_040:
                    inc     edx
                    xor     al, al
                    cmp     al, [edx]
                    jz      Fileopn_ERR
                    clc
                    jmp     Fileopn_End
Fileopn_050:
                    cmp     al, 2
                    jnz     Fileopn_059
                    ; get ptr to filename
                    mov     al, 1
                    call    File_OperatePathName
                    jc      Fileopn_ERR
                    mov     ecx, 0100h
                    mov     al, "\"
Fileopn_051:
                    dec     edx
                    cmp     al, [edx]
                    jz      Fileopn_055
                    loopd   Fileopn_051
                    jmp     Fileopn_ERR
Fileopn_055:        inc     edx
                    xor     al, al
                    cmp     al, [edx]
                    jz      Fileopn_ERR
                    jmp     Fileopn_End
Fileopn_059:
                    cmp     al, 3
                    jnz     Fileopn_070
                    ; get ptr to pathname except drive
                    mov     al, '"'
                    cmp     al, [edx]
                    jnz     Fileopn_060
                    inc     edx
Fileopn_060:
                    mov     al, ":"
                    cmp     al, [edx + 1]
                    jnz     Fileopn_End
                    lea     edx, [edx + 2]
                    xor     al, al
                    cmp     al, [edx]
                    jz      Fileopn_ERR
                    jmp     Fileopn_End
Fileopn_070:
                    cmp     al, 4
                    jnz     Fileopn_090
                    ; get drive number
                    mov     al, '"'
                    cmp     al, [edx]
                    jnz     Fileopn_080
                    inc     edx
Fileopn_080:
                    mov     al, ":"
                    cmp     al, [edx + 1]
                    jnz     Fileopn_ERR
                    mov     al, [edx]
                    and     al, not(020h)
                    sub     al, "A"
                    jc      Fileopn_ERR
                    cmp     al, "Z" - "A"
                    ja      Fileopn_ERR
                    mov     dl, al
                    jmp     Fileopn_End
Fileopn_090:
                    cmp     al, 8
                    jnz     Fileopn_110
                    ; convert pathname to canonical with LFN
                    push    es esi edi
                    mov     eax, ds
                    mov     es, eax
                    ; - clear buffer
                    lea     edi, PathNameBuf
                    mov     ecx, fakebuf_size
                    xor     al, al
                    cld
                    rep     stosb
                    ; - try to get canonical pathname
                    mov     ecx, 2
                    mov     esi, edx
                    lea     edi, PathNameBuf
                    mov     eax, 07160h
                    int     21h
                    jc      Fileopn_100
                    ; - copy canonical pathname
                    mov     ecx, fakebuf_size
                    xor     al, al
                    push    edi
                    repnz   scasb
                    mov     ecx, edi
                    pop     edi
                    stc
                    jnz     Fileopn_100
                    sub     ecx, edi
                    xchg    esi, edi
                    rep     movsb
                    clc
Fileopn_100:
                    pop     edi esi es
                    jmp     Fileopn_End
Fileopn_110:
                    stc
Fileopn_End:
                    pop     ecx eax
                    ret
Fileopn_ERR:
                    stc
                    jmp     Fileopn_End

;----------------------------------------------------------------------------;

File_GetPath:
                    ;--------------------------------------------------------;
                    ; Get current path
                    ;--------------------------------------------------------;
                    ; - in:
                    ; EDX - address of buffer
                    ; - out:
                    ; CF  - NC: OK
                    ;        C: some error was occured
                    ;--------------------------------------------------------;
                    push    eax edx esi
                    mov     esi, edx
                    ; get current drive
                    mov     ax, 019FFh
                    ;INTX    21h                     ; DOS - GET DRIVE
                    int     21h
                    mov     dl, al
                    add     al, "A"
                    mov     [esi], al
                    mov     dword ptr [esi + 1], "\:"
                    ; get current dir via LFN API
                    inc     dl
                    lea     esi, [esi + 3]
                    mov     eax, 07147h
                    int     21h                     ; DOS/LFN - GET DIR
                    jc      Filegp_050
                    lea     edx, [esi - 3]
                    mov     al, 8
                    ; try to convert to canonical pathname with LFN
                    call    File_OperatePathName
                    clc
                    jmp     Filegp_End
Filegp_050:
                    ; get current dir via DOS
                    mov     ah, 047h
                    int     21h                     ; DOS - GET DIR
                    jc      Filegp_End
Filegp_End:
                    pop     esi edx eax
                    ret
    
;----------------------------------------------------------------------------;

File_SetPath:
                    ;--------------------------------------------------------;
                    ; Set current path
                    ;--------------------------------------------------------;
                    ; - in:
                    ; EDX - address of buffer with path
                    ; - out:
                    ; CF  - NC: OK
                    ;        C: some error was occured
                    ;--------------------------------------------------------;
                    push    eax edx esi
                    mov     esi, edx

                    ; try to get canonical pathname
                    push    es edi ecx
                    mov     eax, ds
                    mov     es, eax
                    mov     ecx, 2
                    mov     esi, edx
                    lea     edi, PathNameBuf
                    mov     eax, 07160h
                    int     21h                     ; DOS/LFN - CANONICALIZE FILENAME OR PATH
                    pop     ecx edi es
                    jc      Filesp_020
                    lea     edx, PathNameBuf
Filesp_020:
                    ; set default drive
                    mov     al, [esi]
                    or      al, 020h
                    sub     al, "a"
                    jc      Filesp_050
                    mov     dl, al
                    mov     ah, 00Eh
                    ;INTX    21h                     ; DOS - SET DRIVE
                    int     21h
                    ; set current dir via LFN API
                    mov     edx, esi
                    mov     eax, 0713Bh
                    int     21h                     ; DOS/LFN - SET DIR
                    jnc     Filesp_050
                    ; set current dir via DOS
                    mov     ah, 03Bh
                    int     21h                     ; DOS - SET DIR
Filesp_050:
                    pop     esi edx eax
                    ret

;----------------------------------------------------------------------------;

File_Open:
                    ;--------------------------------------------------------;
                    ; Create or open existing file
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - options: 0=read, 1=write, 2=read/write,
                    ;                3=create new
                    ; EDX - address to filename string
                    ; - out:
                    ; CF  - NC: OK
                    ;        C: some error was occured :)
                    ;--------------------------------------------------------;
                    cmp     FileHandlersPtr, filehndlrs_size / 4
                    jnc     File_ErrNoHandler
                    pushad
                    mov     esi, edx
                    xor     edi, edi
                    xor     ecx, ecx
                    mov     edx, 001h
                    and     al, 003h
                    movzx   ebx, al
                    or      bx, 02040h
                    cmp     al, 3
                    jnz     Fileop_010
                    mov     edx, 012h
                    mov     bl, 2
Fileop_010:
                    mov     eax, 0716Ch
                    int     21h
                    jnc     Fileop_050
                    popad
                    pushad
                    and     al, 003h
                    cmp     al, 3
                    jnz     Fileop_020
                    xor     ecx, ecx
                    mov     ah, 03Ch
                    int     21h                     ; DOS - CREATE FILE
                    jmp     Fileop_040
Fileop_020:
                    or      al, 040h
                    mov     ah, 03Dh
                    int     21h                     ; DOS - OPEN FILE
Fileop_040:
                    jc      Fileop_End
Fileop_050:
                    mov     esi, FileHandlersPtr
                    mov     [esi*4 + offset FileHandlers], eax
                    inc     FileHandlersPtr
                    clc
Fileop_End:
                    popad
                    ret

;----------------------------------------------------------------------------;

File_Close:
                    ;--------------------------------------------------------;
                    ; Close file
                    ;--------------------------------------------------------;
                    cmp     FileHandlersPtr, 0
                    jz      Filecl_End
                    dec     FileHandlersPtr
                    pushad
                    mov     esi, FileHandlersPtr
                    mov     ebx, [esi*4 + offset FileHandlers]
                    mov     ah, 03Eh
                    INTX    21h                     ; DOS - CLOSE FILE
                    popad
Filecl_End:
                    ret

;----------------------------------------------------------------------------;

File_CloseAll:
                    ;--------------------------------------------------------;
                    ; Close all opened files
                    ;--------------------------------------------------------;
                    pushad
Filecla_1:
                    cmp     FileHandlersPtr, 0
                    jz      Filecla_End
                    call    File_Close
                    jmp     Filecla_1
Filecla_End:
                    popad
                    ret

;----------------------------------------------------------------------------;

File_SetPosition:
                    ;--------------------------------------------------------;
                    ; Set position in currently opened file
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - options: 0=from start, 1=from current, 2=from end
                    ; (E)CX:(E)DX - position offset
                    ; - out:
                    ; CF  - NC: OK
                    ;           (E)DX:(E)AX - new file position from start
                    ;        C: error was occured
                    ;           EAX - error code
                    ;--------------------------------------------------------;
                    cmp     FileHandlersPtr, 0
                    jz      File_ErrNoFiles
                    push    ebx esi
                    mov     esi, FileHandlersPtr
                    mov     ebx, [esi*4 + FileHandlers - 4]
                    pop     esi
                    mov     ah, 042h
                    and     al, 003h
                    int     21h                     ; DOS - SET FILE POSITION
                    pop     ebx
                    ret

;----------------------------------------------------------------------------;

File_Read:
                    ;--------------------------------------------------------;
                    ; Read from currently opened file
                    ;--------------------------------------------------------;
                    ; - in:
                    ; EDX - address of buffer to read to
                    ; ECX - number of bytes to read
                    ; - out:
                    ; CF  - NC: OK
                    ;           EAX - number of bytes actually read (can be 0)
                    ;        C: error was occured
                    ;           EAX - error code
                    ;--------------------------------------------------------;
                    cmp     FileHandlersPtr, 0
                    jz      File_ErrNoFiles
                    push    ebx esi
                    mov     esi, FileHandlersPtr
                    mov     ebx, [esi*4 + FileHandlers - 4]
                    pop     esi
                    mov     ah, 03Fh
                    int     21h                     ; DOS - READ FROM FILE
                    pop     ebx
                    ret

;----------------------------------------------------------------------------;

File_Write:
                    ;--------------------------------------------------------;
                    ; Write to currently opened file
                    ;--------------------------------------------------------;
                    ; - in:
                    ; EDX - address of buffer to write from
                    ; ECX - number of bytes to write
                    ; - out:
                    ; CF  - NC: OK
                    ;           EAX - number of bytes actually written
                    ;        C: error was occured
                    ;           EAX - error code
                    ;--------------------------------------------------------;
                    cmp     FileHandlersPtr, 0
                    jz      File_ErrNoFiles
                    push    ebx esi
                    mov     esi, FileHandlersPtr
                    mov     ebx, [esi*4 + FileHandlers - 4]
                    pop     esi
                    mov     ah, 040h
                    int     21h                     ; DOS - WRITE TO FILE
                    pop     ebx
                    ret

File_ErrNoFiles:
                    QUIT    TxtNoFilesOpened
File_ErrNoHandler:
                    QUIT    TxtNoRoomForFHndlr

;----------------------------------------------------------------------------;

File_QueryLfnApi:
                    ;--------------------------------------------------------;
                    ; Query LFN API
                    ;--------------------------------------------------------;
                    ; - in:
                    ; AL  - drive number
                    ; - out:
                    ; CF  - NC: OK, LFN present
                    ;        C: error, no suitable LFN API present
                    ;--------------------------------------------------------;
                    push    eax edx esi

                    mov     dl, al
                    lea     esi, PathNameBuf
                    mov     eax, 07147h
                    stc
                    int     21h
                    jnc     Fileqlfn_010
                    PHEXW   ax
                    PCHR    ","
                    PCHR    " "
                    stc
Fileqlfn_010:
                    pop     esi edx eax
                    ret

;----------------------------------------------------------------------------;

; -------------------------------------------------------------------
; -=* H-I-G-H---L-E-V-E-L---R-E-A-L---M-O-D-E---I-N-T-E-R-R-U-P-T *=-
; -------------------------------------------------------------------
RealMode_Interrupt:
                    push    es ds
                    mov     ds, cs:sel_Data

                    mov     V86regs._EAX, eax
                    mov     V86regs._EBX, ebx
                    mov     V86regs._ECX, ecx
                    mov     V86regs._EDX, edx
                    mov     V86regs._ESI, esi
                    mov     V86regs._EDI, edi
                    mov     V86regs._EBP, ebp

                    mov     ax, ds
                    mov     es, ax
                    mov     ax, 00300h
                    mov     bx, V86regs._INT
                    xor     ecx, ecx
                    mov     V86regs._SP, cx
                    mov     V86regs._SS, cx
                    mov     V86regs._INT, cx
                    lea     edi, V86regs
                    int     31h                     ; DPMI - SIMULATE REAL MODE INTERRUPT
                    jnc     RMint_1
                    QUIT    TxtRMIError, eax
RMint_1:
                    inc     V86calls
                    mov     ebx, V86regs._EBX
                    mov     ecx, V86regs._ECX
                    mov     edx, V86regs._EDX
                    mov     esi, V86regs._ESI
                    mov     edi, V86regs._EDI
                    mov     ebp, V86regs._EBP
                    mov     ah, V86regs._FLL
                    sahf
                    mov     eax, V86regs._EAX

                    pop     ds es
                    ret

;----------------------------------------------------------------------------;

; -----------------------------------------------------------------------------
; -=* H-I-G-H---L-E-V-E-L---R-E-A-L---M-O-D-E---P-R-O-C-E-D-U-R-E---C-A-L-L *=-
; -----------------------------------------------------------------------------
RealMode_Call:
                    push    es ds
                    mov     ds, cs:sel_Data

                    mov     V86regs._EAX, eax
                    mov     V86regs._EBX, ebx
                    mov     V86regs._ECX, ecx
                    mov     V86regs._EDX, edx
                    mov     V86regs._ESI, esi
                    mov     V86regs._EDI, edi
                    mov     V86regs._EBP, ebp

                    mov     ax, ds
                    mov     es, ax
                    mov     ax, 00301h
                    xor     bh, bh
                    xor     ecx, ecx
                    mov     V86regs._SP, cx
                    mov     V86regs._SS, cx
                    lea     edi, V86regs
                    int     31h                     ; DPMI - SIMULATE REAL MODE FAR CALL
                    jnc     RMcall_1
                    QUIT    TxtRMCError, eax
RMcall_1:
                    mov     ebx, V86regs._EBX
                    mov     ecx, V86regs._ECX
                    mov     edx, V86regs._EDX
                    mov     esi, V86regs._ESI
                    mov     edi, V86regs._EDI
                    mov     ebp, V86regs._EBP
                    mov     ah, V86regs._FLL
                    sahf
                    mov     eax, V86regs._EAX

                    pop     ds es
                    ret


;****************************************************************************;

.DATA

DMAPagePorts        db      087h, 083h, 081h, 082h, 08Fh, 08Bh, 089h, 08Ah
SymbolPosition      label   dword
SymbolXPosition     db      0
SymbolYPosition     db      0
                    dw      0
PITLastReadValue    dd      0
TMRLastRead         label   qword
TMRLastReadLow      dd      0
TMRLastReadHigh     dd      0
TMRFrequency        label   qword
TMRFrequencyLow     dd      0
TMRFrequencyHigh    dd      0
TMRLastSecondLow    dd      0
TMRLastSecondHigh   dd      0
TMRPeriodInt8       label   qword
TMRPeriodInt8Low    dd      0
TMRPeriodInt8High   dd      0
TMRLastInt8Low      dd      0
TMRLastInt8High     dd      0
TMRPeriodKbd        label   qword
TMRPeriodKbdLow     dd      0
TMRPeriodKbdHigh    dd      0
TMRLastKbdLow       dd      0
TMRLastKbdHigh      dd      0
TMRPeriodSyncLow    dd      0
TMRRetracePeriod    dd      0
PITLatchValue       dd      0
PITReadFlag         db      0
CMOSLastSecond      db      0
CMOSCurrentSecond   db      0
TMRDegradation      db      0
TMRInitializedFlag  db      0

SearchingSymbol     label   byte
                    db      "-", "\", "|", "/", 0
searchsyms_size=$-SearchingSymbol
SearchingSymPtr     dd      0

PathNameBuf         label   byte
                    db      512 dup(0)
fakebuf_size=$-PathNameBuf
FileHandlers        label   dword
                    dd      64 dup(0)
filehndlrs_size=$-FileHandlers
FileHandlersPtr     dd      0


;     V86
V86regs             INTRM   <>
V86calls            dd      0

TxtRMIError         db      "*** RMI: Error #", 0
TxtRMCError         db      "*** RMC: Error #", 0
TxtNoRoomForFHndlr  db      "*** FILE: No room for file handler.", 00Dh, 00Ah, 0
TxtNoFilesOpened    db      "*** FILE: No opened files.", 00Dh, 00Ah, 0
TxtCPUSpeed         db      " CPU running at ", 0
TxtCPUMHz           db      " MHz", 00Dh, 00Ah, 0

;****************************************************************************;



                    end