
include defines.inc
include macro.inc


;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;
LoadSaveSnapshot:
                ; FUNCTION:  Load/Save Snapshot file
                ; Call With: AL - 0/1=read/write
                ;            SnapshotToLoad & SnapshotToSave
                ;            Key <ALT> - on=SnapshotToLoad<>SnapshotToSave
                pushad
                push    es
                and     al, 001h
                movzx   edi, al

                ; get current path
                lea     edx, VZXCurrentDir
                call    File_GetPath
                pushfd

                ; set current path
                lea     edx, DOSCurrentDir
                call    File_SetPath
;                jc      LSsnap_100

                lea     esi, SnapshotToSave
                lea     edx, SnapshotToLoad
                mov     al, KeyboardF4
                or      al, KeyboardF5
                shr     al, 2
                xor     eax, edi
                test    al, 001h
                jz      LSsnap_020
                xchg    edx, esi
LSsnap_020:
                test    byte ptr [edx], -1
                jnz     LSsnap_030
                xchg    edx, esi
LSsnap_030:
                mov     ebp, edx

                ; find file extension
                mov     al, 1
                call    File_OperatePathName
                jc      LSsnap_100

                ; verify it
                lea     esi, SnapsDescript
LSsnap_050:
                mov     ah, [esi]
                cmp     ah, -1
                stc
                jz      LSsnap_100
                inc     esi

                push    edx
                mov     ecx, 3
LSsnap_060:
                mov     al, [esi]
                cmp     al, [edx]
                jz      LSsnap_090
                cmp     al, " "
                jnz     LSsnap_070
                cmp     byte ptr [edx], 0
                jz      LSsnap_090
                cmp     byte ptr [edx], 00Dh
                jz      LSsnap_090
LSsnap_070:
                cmp     al, "A"
                jc      LSsnap_080
                cmp     al, "Z"
                ja      LSsnap_080
                or      al, 020h
                cmp     al, [edx]
                jz      LSsnap_090
LSsnap_080:
                or      ch, cl                  ; CH is nonzero now
LSsnap_090:
                inc     esi
                inc     edx
                dec     cl
                jnz     LSsnap_060
                pop     edx

                test    ch, ch
                jnz     LSsnap_050
                add     ah, ah
                movzx   eax, ah
                or      edi, eax
                mov     eax, edi
                mov     edi, [edi*4 + SnapsPrograms]

                ; create (write) or open (read) file
                mov     edx, ebp

                test    al, 001h
                jz      LSsnap_094
                ; create file for writing
                mov     al, FILE_CREATE_READWRITE
                call    File_Open
                jmp     LSsnap_095
LSsnap_094:
                ; open file for reading
                mov     al, FILE_OPEN_READONLY
                call    File_Open
LSsnap_095:
                jc      LSsnap_100

                ; operate the snapshot
                mov     RWSnapshotError, 0
                mov     BufferOffset, offset TempBuffer_A
                mov     BufferSize, tmpbufa_size
                call    edi

                ; close file
                call    File_Close
LSsnap_100:
                pop     eax
                test    al, 001h
                jnz     LSsnap_120

                ; set current path
                lea     edx, VZXCurrentDir
                call    File_SetPath
LSsnap_120:
                pop     es
                popad
                ret
;----------------------------------------------------------------------------;
LoadSnapshotSNA:
                ; read SNA header
                lea     edx, TempBuffer_A
                mov     ecx, 27
                call    File_Read
                jc      LsfEnd
                ; load Z80 registers
                mov     al, [edx]
                mov     [ebx].regI, al
                mov     eax, [edx + 1]
                mov     [ebx].rHLx, ax
                ror     eax, 16
                mov     [ebx].rDEx, ax
                mov     eax, [edx + 5]
                mov     [ebx].rBCx, ax
                ror     eax, 16
                test    al, 004h
                setnz   [ebx].flagPVx
                and     al, 0FBh
                mov     [ebx].rAFx, ax
                mov     eax, [edx + 9]
                mov     [ebx].rHL, ax
                ror     eax, 16
                mov     [ebx].rDE, ax
                mov     eax, [edx + 13]
                mov     [ebx].rBC, ax
                ror     eax, 16
                mov     [ebx].rIY, ax
                mov     eax, [edx + 17]
                mov     [ebx].rIX, ax
                ror     eax, 16
                and     [ebx].FlagsA, 0F0h
                not     al
                and     al, 004h
                or      [ebx].FlagsA, al
                rol     al, 1
                or      [ebx].FlagsA, al
                mov     [ebx].regR, ah
                test    ah, 080h
                setnz   [ebx].regRbit7
                mov     eax, [edx + 21]
                test    al, 004h
                setnz   [ebx].flagPV
                and     al, 0FBh
                mov     [ebx].rAF, ax
                ror     eax, 16
                mov     [ebx].rSP, ax
                mov     ax, [edx + 25]
                and     al, 003h
                or      [ebx].FlagsA, al
                ; initialize 48K model by default
                call    SetZXBorder
                call    Std48k_Init
                mov     ecx, 04000h
                ; load bank 5 (standart screen)
                GW16K1  edx
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 020h
                ; load bank 2
                GW16K2  edx
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 004h
                ; load bank 0
                GW16K3  edx
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 001h
                ; read model-specific info
                lea     edx, TempBuffer_A
                mov     ecx, 4
                call    File_Read
                jc      Lsf01
                cmp     ax, 4
                jz      Lsf1
Lsf01:
                ; 48K model was loaded
                or      RWSnapshotError, 0DBh
                mov     dx, [ebx].rSP
                READ    dx
                mov     al, [esi]
                WRITE   dx
                mov     byte ptr [esi], 0
                inc     edx
                READ    dx
                mov     ah, [esi]
                WRITE   dx
                mov     byte ptr [esi], 0
                inc     edx
                mov     [ebx].rSP, dx
                mov     [ebx].rPC, ax
                jmp     LsfEnd
Lsf1:
                ; initialize 128K model
                GW16K3  esi
                call    Std128k_Init
                mov     ax, [edx]
                mov     [ebx].rPC, ax
                mov     al, [edx + 3]
                test    al, al
                jz      Lsf2
                call    Ext128k_Init
;                or      [ebx].FlagsA, FA_INT_
Lsf2:
                mov     ah, [edx + 2]
                mov     cl, ah
                call    ZX128k_SetConfig
                GW16K3  edi
                cmp     esi, edi
                jz      Lsf35
                and     RWSnapshotError, 0FEh
                and     cl, 007h
                mov     al, 001h
                rol     al, cl
                or      RWSnapshotError, al
                mov     eax, ds
                mov     es, eax
                mov     ecx, 01000h
                cld
                rep     movs dword ptr es:[edi], ds:[esi]
Lsf35:
                mov     ecx, 04000h
                ; load bank 0
                test    RWSnapshotError, 001h
                jnz     Lsf4
                mov     edx, [ebx].RamBank0Adr
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 001h
Lsf4:
                ; load bank 1
                test    RWSnapshotError, 002h
                jnz     Lsf5
                mov     edx, [ebx].RamBank1Adr
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 002h
Lsf5:
                ; load bank 3
                test    RWSnapshotError, 008h
                jnz     Lsf6
                mov     edx, [ebx].RamBank3Adr
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 008h
Lsf6:
                ; load bank 4
                test    RWSnapshotError, 010h
                jnz     Lsf7
                mov     edx, [ebx].RamBank4Adr
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 010h
Lsf7:
                ; load bank 6
                test    RWSnapshotError, 040h
                jnz     Lsf8
                mov     edx, [ebx].RamBank6Adr
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 040h
Lsf8:
                ; load bank 7
                test    RWSnapshotError, 080h
                jnz     Lsf9
                mov     edx, [ebx].RamBank7Adr
                call    File_Read
                jc      Lsf9
                or      RWSnapshotError, 080h
Lsf9:
LsfEnd:
                ret
;----------------------------------------------------------------------------;
SaveSnapshotSNA:
                ; if 48K model - push PC on stack
                test    [ebx].SystemZX128, 020h
                jz      SsSNA_1
                mov     ax, [ebx].rPC
                mov     dx, [ebx].rSP
                dec     edx
                READ    dx
                mov     [esi], ah
                dec     edx
                READ    dx
                mov     [esi], al
                mov     [ebx].rSP, dx
SsSNA_1:
                ; prepare SNA header to write
                lea     edx, TempBuffer_A
                mov     al, [ebx].regI
                mov     [edx], al
                mov     ax, [ebx].rDEx
                shl     eax, 16
                mov     ax, [ebx].rHLx
                mov     [edx + 1], eax
                mov     ax, [ebx].rAFx
                mov     cl, [ebx].flagPVx
                rol     cl, 2
                and     al, not(004h)
                or      al, cl
                shl     eax, 16
                mov     ax, [ebx].rBCx
                mov     [edx + 5], eax
                mov     ax, [ebx].rDE
                shl     eax, 16
                mov     ax, [ebx].rHL
                mov     [edx + 9], eax
                mov     ax, [ebx].rIY
                shl     eax, 16
                mov     ax, [ebx].rBC
                mov     [edx + 13], eax
                test    [ebx].regRbit7, 0FFh
                setnz   al
                ror     al, 1
                mov     ah, [ebx].regR
                and     ah, 07Fh
                or      ah, al
                mov     al, [ebx].FlagsA
                not     al
                and     al, FA_IFF2_
                shl     eax, 16
                mov     ax, [ebx].rIX
                mov     [edx + 17], eax
                mov     ax, [ebx].rSP
                shl     eax, 16
                mov     ax, [ebx].rAF
                mov     cl, [ebx].flagPV
                rol     cl, 2
                and     al, not(004h)
                or      al, cl
                mov     [edx + 21], eax
                mov     al, [ebx].FlagsA
                and     al, (FA_INTMODE0 + FA_INTMODE1)
                mov     ah, [ebx].SystemZX48
                mov     [edx + 25], ax
                ; write header
                mov     ecx, 27
                call    File_Write
                jc      SsSNA_End
                mov     ecx, 04000h
                ; write bank 5
                GW16K1  edx
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 020h
                ; write bank 2
                GW16K2  edx
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 004h
                ; write bank 3 and exit if 48K model
                GW16K3  edx
                call    File_Write
                jc      SsSNA_End
                test    [ebx].SystemZX128, 020h
                jz      SsSNA_2
                or      RWSnapshotError, 0DBh
                jmp     SsSNA_End
SsSNA_2:
                ; write 128K model-specific info
                lea     edx, TempBuffer_A
                mov     ax, [ebx].rPC
                mov     [edx], ax
                mov     al, [ebx].SystemZX128
                mov     [edx + 2],al
                and     al, 007h
                mov     cl, al
                mov     al, 1
                rol     al, cl
                or      RWSnapshotError, al
                GR16K0  eax
                cmp     eax, [ebx].RomPrimary
                setnz   al
                mov     [edx + 3], al
                mov     ecx, 4
                call    File_Write
                jc      SsSNA_End
                mov     ecx, 04000h
                ; write bank 0
                test    RWSnapshotError, 001h
                jnz     SsSNA_3
                mov     edx, [ebx].RamBank0Adr
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 001h
SsSNA_3:
                ; write bank 1
                test    RWSnapshotError, 002h
                jnz     SsSNA_5
                mov     edx, [ebx].RamBank1Adr
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 002h
SsSNA_5:
                ; write bank 3
                test    RWSnapshotError, 008h
                jnz     SsSNA_6
                mov     edx, [ebx].RamBank3Adr
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 008h
SsSNA_6:
                ; write bank 4
                test    RWSnapshotError, 010h
                jnz     SsSNA_7
                mov     edx, [ebx].RamBank4Adr
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 010h
SsSNA_7:
                ; write bank 6
                test    RWSnapshotError, 040h
                jnz     SsSNA_8
                mov     edx, [ebx].RamBank6Adr
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 040h
SsSNA_8:
                ; write bank 7
                test    RWSnapshotError, 080h
                jnz     SsSNA_End
                mov     edx, [ebx].RamBank7Adr
                call    File_Write
                jc      SsSNA_End
                or      RWSnapshotError, 080h
SsSNA_End:
                ret

;----------------------------------------------------------------------------;

SP_SNAPSHOT_OLDHEADERSIZE   equ     32
SP_SNAPSHOT_OLDFILESIZE_16k equ     16384 + SP_SNAPSHOT_OLDHEADERSIZE
SP_SNAPSHOT_OLDFILESIZE_48k equ     49152 + SP_SNAPSHOT_OLDHEADERSIZE
SP_SNAPSHOT_HEADERSIZE      equ     38

LoadSnapshotSP:
                ; read SP header
                lea     edx, TempBuffer_A
                mov     ecx, SP_SNAPSHOT_HEADERSIZE
                call    File_Read
                jc      LsSP_End
                cmp     ax, SP_SNAPSHOT_HEADERSIZE
                jnz     LsSP_End
                mov     eax, [edx]
                cmp     ax, "PS"
                jnz     LsSP_End
                shr     eax, 16
                test    eax, eax
                jz      LsSP_End
                mov     ebp, eax                ; EBP = size of RAM to load
                dec     eax
                mov     cx, [edx + 4]
                add     eax, ecx
                jc      LsSP_End
                mov     edi, ecx                ; EDI = offset in RAM
                cmp     word ptr [edx + 32], 0
                jnz     LsSP_End
                cmp     byte ptr [edx + 35], 0
                jnz     LsSP_End
                ; load Z80 registers
                mov     eax, [edx + 6]
                mov     [ebx].rBC, ax
                ror     eax, 16
                mov     [ebx].rDE, ax
                mov     eax, [edx + 10]
                mov     [ebx].rHL, ax
                ror     eax, 16
                test    al, 004h
                setnz   [ebx].flagPV
;                and     al, not(004h)
                mov     [ebx].rAF, ax
                mov     eax, [edx + 14]
                mov     [ebx].rIX, ax
                ror     eax, 16
                mov     [ebx].rIY, ax
                mov     eax, [edx + 18]
                mov     [ebx].rBCx, ax
                ror     eax, 16
                mov     [ebx].rDEx, ax
                mov     eax, [edx + 22]
                mov     [ebx].rHLx, ax
                ror     eax, 16
                test    al, 004h
                setnz   [ebx].flagPVx
;                and     al, not(004h)
                mov     [ebx].rAFx, ax
                mov     eax, [edx + 26]
                test    al, 080h
                setnz   [ebx].regRbit7
                mov     [ebx].rIR, ax
                ror     eax, 16
                mov     [ebx].rSP, ax
                mov     eax, [edx + 30]
                mov     [ebx].rPC, ax
                ; setup z80 interrupt mode
                and     [ebx].FlagsA, not(FA_INTMODE0 + FA_INTMODE1)
                mov     al, [edx + 36]
                mov     ah, al
;                test    al, 008h
;                setnz   ch
;                dec     ch
                test    al, 002h
                setnz   cl
                inc     cl
;                and     cl, ch
                mov     al, cl
                rcr     ah, 1
                sbb     cl, cl
                rcr     ah, 2
                sbb     ch, ch
                not     ecx
                and     cl, FA_IFF1_
                and     ch, FA_IFF2_
                or      al, cl
                or      al, ch
                mov     [ebx].FlagsA, al

                ; initialize 48K mode
                mov     al, [edx + 34]
                call    SetZXBorder
                call    Std48k_Init

                ; load/combine ROM data
                cmp     edi, 04000h
                jnc     LsSP_7
                mov     edx, 04000h
                sub     edx, edi
                mov     ecx, edi
                mov     edi, [ebx].RomBank0Tmp
                jecxz   LsSP_6
                mov     esi, [ebx].RomBank_Basic48k
                push    es
                mov     eax, ds
                mov     es, eax
                cld
                rep     movs byte ptr es:[edi], ds:[esi]
                pop     es
LsSP_6:
                mov     ecx, edx
                sub     ebp, ecx
                jnc     LsSP_65
                add     ecx, ebp
                xor     ebp, ebp
LsSP_65:
                mov     edx, edi
                call    File_Read
                jc      LsSP_End
                mov     eax, [ebx].RomBank0Tmp
                mov     [ebx].RomBank1Adr, eax
                mov     edi, 04000h
                test    ebp, ebp
                jnz     LsSP_7
                mov     RWSnapshotError, 0FFh
                jmp     LsSP_End
LsSP_7:
                ; load bank 5 (write bank 1, usually standart screen)
                cmp     edi, 08000h
                jnc     LsSP_8
                mov     ecx, 08000h
                sub     ecx, edi
                sub     ebp, ecx
                jnc     LsSP_75
                add     ecx, ebp
                xor     ebp, ebp
LsSP_75:
                WRITE   di
                mov     edx, esi
                call    File_Read
                jc      LsSP_End
                mov     edi, 08000h
                test    ebp, ebp
                jnz     LsSP_8
                mov     RWSnapshotError, 0FFh
                jmp     LsSP_End

LsSP_8:
                or      RWSnapshotError, 020h
                ; load bank 2 (write bank 2)
                cmp     edi, 0C000h
                jnc     LsSP_9
                mov     ecx, 0C000h
                sub     ecx, edi
                sub     ebp, ecx
                jnc     LsSP_85
                add     ecx, ebp
                xor     ebp, ebp
LsSP_85:
                WRITE   di
                mov     edx, esi
                call    File_Read
                jc      LsSP_End
                mov     edi, 0C000h
                test    ebp, ebp
                jnz     LsSP_9
                mov     RWSnapshotError, 0FFh
                jmp     LsSP_End
LsSP_9:
                or      RWSnapshotError, 004h
                ; load bank 0 (write bank 3)
                mov     ecx, 000010000h
                sub     ecx, edi
                sub     ebp, ecx
                jnc     LsSP_95
                add     ecx, ebp
                xor     ebp, ebp
LsSP_95:
                WRITE   di
                mov     edx, esi
                call    File_Read
                jc      LsSP_End
                ; 48K model was loaded successfully
                mov     RWSnapshotError, 0FFh
LsSP_End:
                ret
;----------------------------------------------------------------------------;
SaveSnapshotSP:
                ret

;----------------------------------------------------------------------------;

ACH_SNAPSHOT_FILESIZE       equ     65792
ACH_SNAPSHOT_HEADERSIZE     equ     256

LoadSnapshotACH:
                ; verify ACH file size
                xor     ecx, ecx
                dec     ecx
                mov     edx, ecx
                mov     al, FILE_SETPOS_FROMSTART
                call    File_SetPosition
                jc      LsACH_End
                test    edx, edx
                jnz     LsACH_End
                cmp     eax, ACH_SNAPSHOT_FILESIZE
                jnz     LsACH_End
                ; read ACH header
                xor     ecx, ecx
                xor     edx, edx
                mov     al, FILE_SETPOS_FROMSTART
                call    File_SetPosition
                jc      LsACH_End
                lea     edx, TempBuffer_A
                mov     ecx, ACH_SNAPSHOT_HEADERSIZE
                call    File_Read
                jc      LsACH_End
                cmp     ax, ACH_SNAPSHOT_HEADERSIZE
                jnz     LsACH_End
                mov     eax, [edx]
                mov     [ebx].regA, al
                mov     eax, [edx + 004h]
                mov     [ebx].regF, al
                mov     eax, [edx + 008h]
                mov     [ebx].regB, al
                mov     eax, [edx + 00Ch]
                mov     [ebx].regC, al
                mov     eax, [edx + 010h]
                mov     [ebx].regD, al
                mov     eax, [edx + 014h]
                mov     [ebx].regE, al
                mov     eax, [edx + 018h]
                mov     [ebx].regH, al
                mov     eax, [edx + 01Ch]
                mov     [ebx].regL, al
                mov     eax, [edx + 020h]
                mov     [ebx].rPC, ax
                mov     eax, [edx + 028h]
                mov     [ebx].rSP, ax
                mov     al, [edx + 094h]
                mov     [ebx].regR, al
                test    al, 080h
                setnz   [ebx].regRbit7
                mov     al, [edx + 09Ch]
                call    SetZXBorder
                mov     al, [edx + 0A4h]
                and     [ebx].FlagsA, not(FA_INTMODE0 + FA_INTMODE1 + FA_IFF1_ + FA_IFF2_)
                or      [ebx].FlagsA, al
                mov     ax, [edx + 0BEh]
                mov     [ebx].regI, al
                test    ah, ah
                setnz   al
                dec     al
                and     al, FA_IFF1_ + FA_IFF2_
                or      [ebx].FlagsA, al
                mov     eax, [edx + 0ECh]
                xchg    al, ah
                mov     [ebx].rAFx, ax
                mov     eax, [edx + 0F0h]
                xchg    al, ah
                mov     [ebx].rBCx, ax
                mov     eax, [edx + 0F4h]
                xchg    al, ah
                mov     [ebx].rDEx, ax
                shr     eax, 16
                xchg    al, ah
                mov     [ebx].rHLx, ax
                mov     eax, [edx + 0F8h]
                mov     [ebx].rIX, ax
                mov     eax, [edx + 0FCh]
                mov     [ebx].rIY, ax
                ; initialize 48k mode
                call    Std48k_Init
                ; load ROM to temp (replace existing ROM till next RESET)
                mov     edx, [ebx].RomBank0Tmp
                mov     ecx, 04000h
                call    File_Read
                jc      LsACH_End
                ; load bank 5 (write bank 1, usually standart screen)
                GW16K1  edx
                mov     ecx, 04000h
                call    File_Read
                jc      LsACH_End
                or      RWSnapshotError, 020h
                ; load bank 2 (write bank 2)
                GW16K2  edx
                call    File_Read
                jc      LsACH_End
                or      RWSnapshotError, 004h
                ; load bank 0 (write bank 3)
                GW16K3  edx
                call    File_Read
                jc      LsACH_End
                ; 48K model was loaded successfully
                mov     RWSnapshotError, 0FFh
                mov     eax, [ebx].RomBank0Tmp
                mov     [ebx].RomBank1Adr, eax
LsACH_End:
                ret
;----------------------------------------------------------------------------;
SaveSnapshotACH:
                ; prepare ACH header to write
                lea     edx, TempBuffer_A
                mov     edi, edx
                push    es
                mov     eax, ds
                mov     es, eax
                xor     eax, eax
                mov     ecx, ACH_SNAPSHOT_HEADERSIZE / 4
                cld
                rep     stos dword ptr es:[edi]
                pop     es
                mov     al, [ebx].regA
                mov     [edx], al
                mov     al, [ebx].regF
                mov     ah, [ebx].flagPV
                rol     ah, 2
                and     al, not(004h)
                or      al, ah
                mov     [edx + 004h], al
                mov     al, [ebx].regB
                mov     [edx + 008h], al
                mov     al, [ebx].regC
                mov     [edx + 00Ch], al
                mov     al, [ebx].regD
                mov     [edx + 010h], al
                mov     al, [ebx].regE
                mov     [edx + 014h], al
                mov     al, [ebx].regH
                mov     [edx + 018h], al
                mov     al, [ebx].regL
                mov     [edx + 01Ch], al
                mov     ax, [ebx].rPC
                mov     [edx + 020h], ax
                mov     ax, [ebx].rSP
                mov     [edx + 028h], ax
                mov     al, [ebx].regR
                test    [ebx].regRbit7, 0FFh
                setnz   ah
                ror     ah, 1
                and     al, 07Fh
                or      al, ah
                mov     [edx + 094h], al
                mov     al, [ebx].SystemZX48
                mov     [edx + 09Ch], al
                mov     al, [ebx].FlagsA
                mov     ah, al
                and     al, FA_INTMODE0 + FA_INTMODE1
                mov     [edx + 0A4h], al
                mov     al, [ebx].regI
                test    ah, FA_IFF1_
                setz    ah
                mov     [edx + 0BEh], ax
                mov     al, [ebx].regFx
                mov     ah, [ebx].flagPV
                rol     ah, 2
                and     al, not(004h)
                or      ah, al
                mov     al, [ebx].regAx
                mov     [edx + 0ECh], ax
                mov     ax, [ebx].rBCx
                xchg    al, ah
                mov     [edx + 0F0h], ax
                mov     ax, [ebx].rHLx
                xchg    al, ah
                rol     eax, 16
                mov     ax, [ebx].rDEx
                xchg    al, ah
                mov     [edx + 0F4h], eax
                mov     ax, [ebx].rIX
                mov     [edx + 0F8h], ax
                mov     ax, [ebx].rIY
                mov     [edx + 0FCh], ax
                ; write header
                mov     ecx, ACH_SNAPSHOT_HEADERSIZE
                call    File_Write
                jc      SsACH_End
                mov     ecx, 04000h
                ; write current ROM paged in bank 0
                GR16K0  edx
                call    File_Write
                jc      SsACH_End
                or      RWSnapshotError, 001h
                ; write bank 1 (page 5)
                GW16K1  edx
                call    File_Write
                jc      SsACH_End
                or      RWSnapshotError, 020h
                ; write bank 2 (page 2)
                GW16K2  edx
                call    File_Write
                jc      SsACH_End
                or      RWSnapshotError, 004h
                ; write bank 3 (current page in bank 3)
                GW16K3  edx
                call    File_Write
                jc      SsACH_End
                or      RWSnapshotError, 0DAh
SsACH_End:
                ret
;----------------------------------------------------------------------------;
LoadSnapshotZ80:
                ; read Z80 header
                ;-----------------
                mov     edx, BufferOffset
                mov     HeaderOffset, edx
                mov     ecx, 30
                call    File_Read
                jc      LsZ80_End
                and     eax, 0FFFFh
                cmp     eax, 30
                jnz     LsZ80_End
                add     BufferOffset, eax
                sub     BufferSize, eax
                jc      LsZ80_End
                mov     ax, [edx + 6]
                test    ax, ax
                jz      LsZ80_v2
LszZ80_v1:
                ; version 1.45 snapshot 48K
                ;---------------------------
                call    Std48k_Init
;                mov     [ebx].rPC, ax
                test    cl, 010h
                jnz     LsZ80_v1packed
                ; non-packed screen
                mov     edx, [ebx].RamBank5Adr
                mov     ecx, 04000h
                call    File_Read
                jc      LsZ80_End
                ; non-packed bank 2
                mov     edx, [ebx].RamBank2Adr
                call    File_Read
                jc      LsZ80_LoadRegisters
                ; non-packed bank 3
                mov     edx, [ebx].RamBank0Adr
                call    File_Read
                jmp     LsZ80_LoadRegisters
LsZ80_v1packed:
                ; packed 48K memory dump
                mov     ebp, 04000h
                mov     ecx, 000040000h
                mov     esi, 0FFFFh
                call    LsZ80_Unpack
                jmp     LsZ80_LoadRegisters
LsZ80_v2:
                ; version 2.01+ snapshots
                ;-------------------------
                mov     edx, BufferOffset
                mov     ecx, 2
                call    File_Read
                jc      LsZ80_End
                and     eax, 0FFFFh
                cmp     eax, 2
                jnz     LsZ80_End
                add     BufferOffset, eax
                sub     BufferSize, eax
                jc      LsZ80_End
                mov     ecx, 23
                cmp     cx, [edx]
                jz      LsZ80v2_01
                mov     cl, 54
                cmp     cx, [edx]
                jz      LsZ80v2_01
                inc     cl
                cmp     cx, [edx]
                stc
                jnz     LsZ80_End
LsZ80v2_01:
                ; read Z80 extra header
                mov     edx, BufferOffset
                call    File_Read
                jc      LsZ80_End
                and     eax, 0FFFFh
                cmp     eax, ecx
                jnz     LsZ80_End
                add     BufferOffset, eax
                sub     BufferSize, eax
                jc      LsZ80_End
                push    ecx edi
                mov     esi, HeaderOffset
                mov     ax, [edx]
                mov     [esi + 6], ax
LsZ80ayregs:
                ; load AY registers
                lea     esi, [ebx].AY1st
                call    AY_Init
                mov     al, [edx + 6]
                and     al, 00Fh
                mov     [esi].AYSelectedReg, al
                mov     ecx, 010h
LsZ80ar_1:
                mov     edi, [ecx*4 + AYRegistersTable - 4]
                mov     ah, [edx + ecx + 6]
                push    ecx
                call    edi
                pop     ecx
                loopd   LsZ80ar_1
                pop     edi ecx

                cmp     byte ptr [edx + 2], 0
                jnz     LsZ80v2_02
LsZ80v2_zx48k:
                ; ZX Spectrum 48k
                ;-----------------
                call    Std48k_Init
                xor     edi, edi                ; EDI - flags
LsZ80v2zx48_1:
                mov     edx, BufferOffset
                mov     ecx, 3
                call    File_Read
                jc      LsZ80v2zx48_end
                and     eax, 0FFFFh
                cmp     eax, ecx
                jnz     LsZ80v2zx48_end
LsZ80v2zx48_2:
                inc     edi
                movzx   ecx, word ptr [edx]
                mov     esi, 03FFFh
                or      ecx, 080000000h
                mov     al, [edx + 2]
                test    al, al
                jnz     LsZ80v2zx48_3
                ; 48k ROM
                mov     ebp, [ebx].RomBank0Tmp
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jc      LsZ80v2zx48_1
                or      edi, 080000000h
                jmp     LsZ80v2zx48_1
LsZ80v2zx48_3:
                cmp     al, 1
                jnz     LsZ80v2zx48_4
                ; If.1 of M.G.T. ROM
                ;...unsupported so far - skip
;                jmp     LsZ80v2zx48_1
LsZ80v2zx48_4:
                cmp     al, 4
                jnz     LsZ80v2zx48_5
                ; bank 2 RAM (RAM2)
                mov     ebp, [ebx].RamBank2Adr
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jmp     LsZ80v2zx48_1
LsZ80v2zx48_5:
                cmp     al, 5
                jnz     LsZ80v2zx48_6
                ; bank 3 RAM (RAM0)
                mov     ebp, [ebx].RamBank0Adr
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jmp     LsZ80v2zx48_1
LsZ80v2zx48_6:
                cmp     al, 8
                jnz     LsZ80v2zx48_7
                ; bank 1 (screen) RAM (RAM5)
                mov     ebp, [ebx].RamBank5Adr
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jmp     LsZ80v2zx48_1
LsZ80v2zx48_7:
                cmp     al, 11
                jnz     LsZ80v2zx48_8
                ; Multiface ROM
                ;...unsupported so far - skip
;                jmp     LsZ80v2zx48_1
LsZ80v2zx48_8:
                ; unknown block type - skip
                mov     ebp, [ebx].BlankBankAdr
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jmp     LsZ80v2zx48_1
LsZ80v2zx48_end:
                test    edi, edi
                jz      LsZ80_End
                test    edi, 080000000h
                jz      LsZ80v2zx48_end1
                ; 48k ROM was loaded - replace existing (till next RESET)
                mov     eax, [ebx].RomBank0Tmp
                mov     [ebx].RomBank1Adr, eax
LsZ80v2zx48_end1:
                jmp     LsZ80_LoadRegisters
LsZ80v2_02:
                cmp     byte ptr [edx + 2], 1
                jnz     LsZ80v2_03
                ; ZX Spectrum 48k + Interface 1
                ;-------------------------------
                ;...
                jmp     LsZ80_LoadRegisters
LsZ80v2_03:
                cmp     byte ptr [edx + 2], 2
                jnz     LsZ80v2_04
                ; ZX Spectrum + SamRam
                ;----------------------
                ;...
                jmp     LsZ80_LoadRegisters
LsZ80v2_04:
                cmp     byte ptr [edx + 2], 3
                jnz     LsZ80v2_05
                cmp     ecx, 23
                jz      LsZ80v2_128k
                ; ZX Spectrum 48k + M.G.T.
                ;--------------------------
                ;...
                jmp     LsZ80_LoadRegisters
LsZ80v2_05:
                cmp     byte ptr [edx + 2], 4
                jnz     LsZ80v2_06
                cmp     ecx, 23
                jz      LsZ80v2_128kIf1
LsZ80v2_128k:
                ; ZX Spectrum 128k
                ;------------------
                call    Std128k_Init
                xor     edi, edi                ; EDI - flags
LsZ80v2zx128_1:
                mov     edx, BufferOffset
                mov     ecx, 3
                call    File_Read
                jc      LsZ80v2zx128_end
                and     eax, 0FFFFh
                cmp     eax, ecx
                jnz     LsZ80v2zx128_end
LsZ80v2zx128_2:
                inc     edi
                movzx   ecx, word ptr [edx]
                mov     esi, 03FFFh
                or      ecx, 080000000h
                mov     al, [edx + 2]
                test    al, al
                jnz     LsZ80v2zx128_3
                ; 48k ROM
                mov     ebp, [ebx].RomBank0Tmp
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jc      LsZ80v2zx128_1
                or      edi, 080000000h
                jmp     LsZ80v2zx128_1
LsZ80v2zx128_3:
                cmp     al, 1
                jnz     LsZ80v2zx128_4
                ; If.1 of M.G.T. ROM
                ;...unsupported so far - skip
;                jmp     LsZ80v2zx128_1
LsZ80v2zx128_4:
                cmp     al, 2
                jnz     LsZ80v2zx128_5
                ; 128k ROM
                mov     ebp, [ebx].RomBank1Tmp
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jc      LsZ80v2zx128_1
                or      edi, 040000000h
                jmp     LsZ80v2zx128_1
LsZ80v2zx128_5:
                cmp     al, 3
                jc      LsZ80v2zx128_6
                cmp     al, 11
                jnc     LsZ80v2zx128_6
                ; RAM0 - RAM7
                sub     al, 3
                movzx   eax, al
                mov     ebp, [eax*4 + [ebx].RamBank0Adr]
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jmp     LsZ80v2zx128_1
LsZ80v2zx128_6:
                cmp     al, 11
                jnz     LsZ80v2zx128_8
                ; Multiface ROM
                ;...unsupported so far - skip
;                jmp     LsZ80v2zx128_1
LsZ80v2zx128_8:
                ; unknown block type - skip
                mov     ebp, [ebx].BlankBankAdr
                lea     esi, [ebp + esi]
                call    LsZ80_Unpack
                jmp     LsZ80v2zx128_1
LsZ80v2zx128_end:
                test    edi, edi
                jz      LsZ80_End
                test    edi, 080000000h
                jz      LsZ80v2zx128_e1
                ; 48k ROM was loaded - replace existing (till next RESET)
                mov     eax, [ebx].RomBank0Tmp
                mov     [ebx].RomBank1Adr, eax
LsZ80v2zx128_e1:
                test    edi, 040000000h
                jz      LsZ80v2zx128_e2
                ; 128k ROM was loaded - replace existing (till next RESET)
                mov     eax, [ebx].RomBank1Tmp
                mov     [ebx].RomBank0Adr, eax
LsZ80v2zx128_e2:
                mov     esi, HeaderOffset
                mov     ah, [esi + 35]
                call    ZX128k_SetConfig
                jmp     LsZ80_LoadRegisters
LsZ80v2_06:
                cmp     byte ptr [edx + 2], 5
                jnz     LsZ80v2_07
LsZ80v2_128kIf1:
                ; ZX Spectrum 128k + Interface 1
                ;--------------------------------
                ;...
;                jmp     LsZ80_LoadRegisters
LsZ80v2_07:
                cmp     byte ptr [edx + 2], 6
                jnz     LsZ80v2_08
                ; ZX Spectrum 128k + M.G.T.
                ;---------------------------
                ;...
;                jmp     LsZ80_LoadRegisters
LsZ80v2_08:
                cmp     byte ptr [edx + 2], 7
                jnz     LsZ80v2_09
LsZ80v2_plus3:
                ; ZX Spectrum 128k +3
                ;---------------------
                ;...
;                jmp     LsZ80_LoadRegisters
LsZ80v2_09:
                cmp     byte ptr [edx + 2], 8
                jz      LsZ80v2_plus3
                cmp     byte ptr [edx + 2], 9
                jnz     LsZ80v2_10
                ; Pentagon 128k
                ;---------------
                ;...
                jmp     LsZ80v2_128k;LsZ80_LoadRegisters
LsZ80v2_10:
                cmp     byte ptr [edx + 2], 10
                jnz     LsZ80v2_11
                ; Scorpion 256k
                ;---------------
                ;...
;                jmp     LsZ80_LoadRegisters
LsZ80v2_11:
                cmp     byte ptr [edx + 2], 11
                jnz     LsZ80v2_12
                ; Didaktik-Kompakt
                ;------------------
                ;...
;                jmp     LsZ80_LoadRegisters
LsZ80v2_12:
                cmp     byte ptr [edx + 2], 12
                jnz     LsZ80v2_13
                ; ZX Spectrum 128k +2
                ;---------------------
                ;...
                jmp     LsZ80v2_128k;LsZ80_LoadRegisters
LsZ80v2_13:
                cmp     byte ptr [edx + 2], 3
                jnz     LsZ80v2_14
                ; ZX Spectrum 128k +2A
                ;----------------------
                ;...
;                jmp     LsZ80_LoadRegisters
LsZ80v2_14:
                ; Other models are unsupported so far
                jmp     LsZ80_End
LsZ80_LoadRegisters:
                ; load registers
                mov     edx, HeaderOffset
                mov     ax, [edx]
                xchg    al, ah
                test    al, 004h
                setnz   [ebx].flagPV
                and     al, 0FBh
                mov     [ebx].rAF, ax
                mov     ax, [edx + 2]
                mov     [ebx].rBC, ax
                mov     ax, [edx + 4]
                mov     [ebx].rHL, ax
                mov     ax, [edx + 8]
                mov     [ebx].rSP, ax
                mov     ax, [edx + 10]
                xchg    al, ah
                mov     [ebx].rIR, ax
                mov     cl, [edx + 12]
                cmp     cl, -1
                jnz     LsZ80loadregs_1
                neg     cl
LsZ80loadregs_1:
                shr     cl, 1
                setc    [ebx].regRbit7
                mov     ah, cl
                and     ah, 007h
                call    SetZXBorder
                mov     ax, [edx + 13]
                mov     [ebx].rDE, ax
                mov     ax, [edx + 15]
                mov     [ebx].rBCx, ax
                mov     ax, [edx + 17]
                mov     [ebx].rDEx, ax
                mov     ax, [edx + 19]
                mov     [ebx].rHLx, ax
                mov     ax, [edx + 21]
                xchg    al, ah
                test    al, 004h
                setnz   [ebx].flagPVx
                and     al, 0FBh
                mov     [ebx].rAFx, ax
                mov     ax, [edx + 23]
                mov     [ebx].rIY, ax
                mov     ax, [edx + 25]
                mov     [ebx].rIX, ax
                mov     al, [edx + 29]
                and     al, 003h
                test    byte ptr [edx + 28], -1
                setz    ah
                shl     ah, 2
                or      al, ah
                test    byte ptr [edx + 27], -1
                setz    ah
                shl     ah, 3
                or      al, ah
                mov     [ebx].FlagsA, al
                mov     ax, [edx + 6]
                mov     [ebx].rPC, ax
                clc
LsZ80_End:
                ret


LsZ80_Unpack:
                ;-----------------------------------------------------
                ; Unpack Z80 compressed block to Spectrum ROM/RAM
                ;-----------------------------------------------------
                ; - in:
                ; EBP - pointer in Spectrum memory or Spectrum bank
                ;       starting address in emulator memory (see ECX)
                ; ECX - size of (un)packed block to read,
                ;       0FFFFh=read non-packed block of 16384 bytes
                ;       D31=0: EBP/ESI addresses Spectrum memory
                ;       D31=1: EBP/ESI addresses PC memory
                ; ESI - last address of Spectrum or emulator memory
                ;       to unpack to (see ECX)
                ; BufferOffset - buffer start address
                ; BufferSize   - buffer size
                ; - out:
                ; CF  - NC: OK
                ;       C:  file or internal error
                ; EBP - pointer to the next byte after unpacked block
                ; ECX - bytes left to read from a file
                ;-----------------------------------------------------
                push    eax edx edi
                and     ecx, 08FFFFFFFh
                cmp     cx, 0FFFFh
                jnz     LsZ80unp_0
                mov     cx, 04000h
                or      ecx, 040000000h
LsZ80unp_0:
                xor     edi, edi
LsZ80unp_1:
                mov     eax, BufferSize
                sub     eax, edi
                mov     edx, ecx
                and     edx, 03FFFFFFFh
                cmp     eax, edx
                jna     LsZ80unp_2
                test    edx, edx
                jz      LsZ80unp_9              ; end of block - OK
                mov     eax, edx
LsZ80unp_2:
                sub     edx, eax
                cmp     edx, 000040000h
                jnc     LsZ80unp_7              ; internal error - ERR
                and     ecx, 0C0000000h
                or      ecx, edx
                mov     edx, BufferOffset
                lea     edx, [edx + edi]
                push    ecx
                movzx   ecx, ax                 ; ECX - partial packed bank length to read
                call    File_Read
                pop     ecx
                jc      LsZ80unp_7              ; read file error - ERR
                test    eax, eax
                jz      LsZ80unp_9              ; end of file - OK
                add     edi, eax
                cmp     edi, BufferSize
                ja      LsZ80unp_7              ; buffer overrun - ERR
                mov     edx, BufferOffset
                jz      LsZ80unp_3
                mov     byte ptr [edx + edi], 0FFh
LsZ80unp_3:
                test    ecx, 040000000h         ; D30=1: block is not packed
                jnz     LsZ80unp_6
                cmp     byte ptr [edx], 0EDh
                jnz     LsZ80unp_6
                cmp     edi, 4
                jnc     LsZ80unp_4
                mov     eax, [edx]
                mov     edx, BufferOffset
                mov     [edx], eax
                jmp     LsZ80unp_1
LsZ80unp_4:
                cmp     byte ptr [edx + 1], 0EDh
                jnz     LsZ80unp_6
                movzx   eax, word ptr [edx + 2]
;                test    eax, eax
;                jz      LsZ80unp_7
                lea     edx, [edx + 4]
LsZ80unp_5:
                ; repeat AH byte AL times in Spectrum bank/memory
                test    ecx, 080000000h
                jnz     LsZ80unp_55
                push    esi
                WRITE   bp
                mov     [esi], ah
                pop     esi
LsZ80unp_51:
                cmp     ebp, esi
                jnc     LsZ80unp_9              ; end of memory - OK
                inc     ebp
                dec     al
                jnz     LsZ80unp_5
                sub     edi, 4
                jnz     LsZ80unp_3
                jmp     LsZ80unp_1
LsZ80unp_55:
                mov     ds:[ebp], ah
                jmp     LsZ80unp_51
LsZ80unp_6:
                ; copy byte from buffer to Spectrum bank/memory
                mov     al, [edx]
                test    ecx, 080000000h
                jnz     LsZ80unp_65
                push    esi
                WRITE   bp
                mov     [esi], al
                pop     esi
LsZ80unp_61:
                cmp     ebp, esi
                jnc     LsZ80unp_9              ; end of memory - OK
                inc     ebp
                inc     edx
                dec     edi
                jnz     LsZ80unp_3
                jmp     LsZ80unp_1
LsZ80unp_65:
                mov     ds:[ebp], al
                jmp     LsZ80unp_61
LsZ80unp_7:
                stc
                jmp     LsZ80unp_End
LsZ80unp_8:

LsZ80unp_9:
                clc
LsZ80unp_End:
                pop     edi edx eax
                ret

;----------------------------------------------------------------------------;
SaveSnapshotZ80:
                ret

;----------------------------------------------------------------------------;

;****************************************************************************;

.DATA

HeaderOffset        dd      0
BufferOffset        dd      0
BufferSize          dd      0
SnapsPrograms       label   dword
                    dd      LoadSnapshotSNA, SaveSnapshotSNA
                    dd      LoadSnapshotSP,  SaveSnapshotSP
                    dd      LoadSnapshotACH, SaveSnapshotACH
                    dd      LoadSnapshotZ80, SaveSnapshotZ80
;                    dd      LoadSnapshotSLT, SaveSnapshotSLT
;                    dd      LoadSnapshotZX, SaveSnapshotZX
;                    dd      LoadSnapshotZXS, SaveSnapshotZXS
;                    dd      LoadSnapshotSNP, SaveSnapshotSNP
SnapsDescript       label   dword
                    db      0, "SNA", 1, "SP ", 2, "ACH", 3, "Z80", -1
                    db      4, "SLT", 5, "ZX ", 6, "ZXS", 7, "SNP"
RWSnapshotError     db      0

;****************************************************************************;


                end
