
include defines.inc
include macro.inc


;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

VGA_InitMode:
                ;-----------------------------------------
                ; Initialize Virtual ZX main video mode
                ;-----------------------------------------
                ; - in:
                ; AL  - 0=initialize all, 1=hardware only
                ;-----------------------------------------

                pushad
                push    eax

                ;------------------------------
                ; Verify for VGA compatibility
                ;------------------------------
                ; if work with VESA may corrupt program execution, skip VESA calls
                call    VGA_DetectCompatibility
                jnc     Zsisvm_2
                test    InitResources, IR_VESA
                jnz     Zsisvm_1
                ; VESA/VBE not present
                test    CFG_VideoModeMain, VM_FORCEVGA
                jnz     Zsisvm_3
                and     CFG_VideoModeMain, VM_FLAGS_MASK
                or      CFG_VideoModeMain, 001h
                jmp     Zsisvm_3
Zsisvm_1:
                ; VESA/VBE is present but no VGA compatibility
                and     CFG_VideoModeMain, VM_FLAGS_MASK
                or      CFG_VideoModeMain, 001h
Zsisvm_2:
                test    InitResources, IR_VESA
                jz      Zsisvm_3

                ;------------------
                ; Setup video mode
                ;------------------
                mov     eax, 04F02h
                mov     bx, CFG_VideoModePre
                INTX    10h
Zsisvm_3:
                ; set main video mode
                movzx   eax, CFG_VideoModeMain
                and     eax, not(VM_FLAGS_MASK)
                mov     eax, [eax*4 + offset VMInitPrograms]
                call    eax

                ; set ZX Spectrum-specific colors
                lea     esi, DACRegisters
                call    VGA_InitDAC

                ; clear screen
                push    es
                mov     eax, ds
                mov     es, eax
                mov     edi, VideoAreaBaseAdr
                mov     ecx, VideoAreaTotalSize
                add     ecx, 3
                shr     ecx, 2
                xor     eax, eax
                cld
                rep     stos dword ptr es:[edi]
                pop     es
                mov     SymbolXPosition, 0
                mov     SymbolYPosition, 0

                ;-----------------
                ; Setup variables
                ;-----------------
                or      InitResources, IR_SCREEN1
                ; generate ZX->PC pallete table
                lea     esi, ZXPalleteTableA
                xor     al, al
Zsisvars_6:
                and     al, 0F0h
Zsisvars_7:
                mov     [esi], al
                xor     al, 088h
                mov     [esi + 040h], al
                ror     al, 4
                mov     [esi + 0C0h], al
                xor     al, 088h
                mov     [esi + 080h], al
                ror     al, 4
                inc     esi
                inc     al
                test    al, 007h
                jnz     Zsisvars_7
                add     al, 010h
                cmp     al, 088h
                jnz     Zsisvars_6

                pop     eax
                cmp     al, 1
                jz      Zsisvm_End

                ; calculate left border area
                mov     al, CFG_ULALeftBorder
                mov     bl, CFG_VideoLBVisible
                test    bl, bl
                jnz     Zsisvars_1
                ; left border area auto size
                mov     bl, CFG_VideoXVisible
                sub     bl, 32
                shr     bl, 1
                mov     CFG_VideoLBVisible, bl
                jmp     Zsisvars_2
Zsisvars_1:
                ; verify if left border area is too large
                mov     cl, CFG_VideoXVisible
                sub     cl, 32
                cmp     bl, cl
                jc      Zsisvars_2
                mov     bl, cl
                mov     CFG_VideoLBVisible, bl
Zsisvars_2:
                mov     al, CFG_VideoXVisible
                sub     al, 32
                sub     al, bl
                mov     VideoRBorder, al
                ; calculate top border area
                movzx   eax, CFG_ULAUpperBorder
                mov     ebx, CFG_VideoUBVisible
                test    ebx, ebx
                jnz     Zsisvars_3
                ; top border area auto size
                mov     ebx, CFG_VideoYVisible
                sub     ebx, 192
                shr     ebx, 1
                mov     CFG_VideoUBVisible, ebx
                jmp     Zsisvars_4
Zsisvars_3:
                ; verify if top border area is too large
                mov     ecx, CFG_VideoYVisible
                sub     ecx, 192
                cmp     ebx, ecx
                jc      Zsisvars_4
                mov     ebx, ecx
                mov     CFG_VideoUBVisible, ebx
Zsisvars_4:
                ; calculate vertical and horizontal pauses
                sub     eax, ebx
                sbb     ecx, ecx
                not     ecx
                and     eax, ecx
                mov     ecx, CFG_ULAResolutionX
                mul     ecx
                mov     ebx, eax
                movzx   eax, CFG_ULALeftBorder
                sub     al, CFG_VideoLBVisible
                add     eax, ebx
                mov     VideoGPause, eax
                mov     eax, CFG_ULAResolutionX
                sub     al, CFG_VideoXVisible
                mov     VideoLPause, al
                ; video scheme corresponding to z80 clock multiplier
                mov     ebx, CFG_Z80ClockMul
                dec     ebx
                and     ebx, 00Fh
                mov     eax, [ebx*8 + VideoSTables]
                mov     VideoShiftPoint, eax
                mov     eax, [ebx*8 + VideoSTables + 004h]
                mov     VideoDivider, eax
                ; build table of zx screen addresses
                lea     esi, ZXScrnAdrTable
                xor     dl, dl
Zsisvars_5:
                mov     ah, dl
                mov     bh, ah
                shr     bh, 3
                shr     bh, 3
                add     bh, 018h
                mov     dh, bh
                mov     bl, ah
                shr     ah, 3
                xor     ah, bl
                and     ah, 0F8h
                xor     ah, bl
                rol     bl, 2
                mov     al, bl
                and     eax, 0FF1Fh
                xor     al, bl
                mov     [esi], eax
                sub     dh, ah
                mov     ah, dh
                xor     al, al
                mov     [esi + 4], eax
                inc     dl
                lea     esi, [esi + 8]
                cmp     dl, 192
                jc      Zsisvars_5

                popad
                ret

;----------------------------------------------------------------------------;

VGA_DetectVR:
                ;--------------------------------
                ; Detect vertical retrace signal
                ;--------------------------------
                pushad
                PSTR    TxtDVideoSignal
                mov     VRetraceFreqRaised, 0
Zsidvr_010:
                ; collecting vertical retrace ticks data
                mov     edx, TMRFrequencyHigh
                mov     eax, TMRFrequencyLow
                mov     ebx, 40         ; frame rates not lower than 40 Hz
                div     ebx
                mov     esi, eax
                lea     edi, VGARetraceDetectBuf
                mov     ecx, vgardbuf_size / 4
                mov     edx, ecx
Zsidvr_020:
                call    VGA_WaitVerticalRetrace
                jc      Zsidvr_6
                call    TMR_ReadCounter
Zsidvr_021:
                mov     ebx, eax
                call    VGA_WaitVerticalRetrace
                jc      Zsidvr_6
                mov     ebp, eax
;                cmp     eax, ebp
;                jnc     Zsidvr_020
                call    TMR_ReadCounter
                sub     ebx, eax
                cmp     ebx, esi;0FFFFh
                ja      Zsidvr_022
                mov     [edi], ebx
                lea     edi, [edi + 4]
                dec     edx
                call    Print_SearchSym
Zsidvr_022:
                loopd   Zsidvr_021
                test    edx, edx
                jz      Zsidvr_030
                push    es
                mov     eax, ds
                mov     es, eax
                mov     ecx, edx
                xor     eax, eax
                cld
                rep     stos dword ptr es:[edi]
                pop     es
Zsidvr_030:
                ; sorting values
                call    Print_SearchEnd
                mov     ecx, (vgardbuf_size / 4) - 1
                lea     edi, VGARetraceDetectBuf
Zsidvr_031:
                push    ecx
                lea     esi, [edi + 4]
                mov     eax, [edi]
Zsidvr_032:
                cmp     eax, [esi]
                jna     Zsidvr_033
                xchg    eax, [esi]
                mov     [edi], eax
Zsidvr_033:
                lea     esi, [esi + 4]
                loopd   Zsidvr_032
                pop     ecx
                lea     edi, [edi + 4]
                loopd   Zsidvr_031

                ; filtering
                mov     ebp, 10
Zsidvr_034:
                xor     eax, eax
                mov     VGAMaxNumOfTPFHits, eax
                mov     VGANumOfTPFHits, eax
                mov     VGATicksPerFrame, eax
                mov     VGACurTicksPerFrame, eax
                mov     ecx, vgardbuf_size / 4
                lea     edi, VGARetraceDetectBuf
Zsidvr_042:
                mov     ebx, [edi]
                cmp     ebx, VGACurTicksPerFrame
                jna     Zsidvr_045
                test    ebx, ebx
                jz      Zsidvr_045
                push    ecx
                mov     edx, ebx
                xor     eax, eax
                mov     VGACurTicksPerFrame, ebx
                mov     VGACurNumOfTPFHits, eax
                sub     ebx, ebp
                add     edx, ebp
                lea     esi, VGARetraceDetectBuf
                mov     ecx, vgardbuf_size / 4
Zsidvr_043:
                cmp     ebx, [esi]
                setc    al
                cmp     edx, [esi]
                seta    ah
                and     al, ah
                xor     ah, ah
                add     VGACurNumOfTPFHits, eax
                lea     esi, [esi + 4]
                loopd   Zsidvr_043
                pop     ecx
                mov     eax, VGACurNumOfTPFHits
                cmp     eax, VGANumOfTPFHits
                jc      Zsidvr_045
                jnz     Zsidvr_044
                inc     VGAMaxNumOfTPFHits
                jmp     Zsidvr_045
Zsidvr_044:
                mov     ebx, VGACurTicksPerFrame
                mov     VGATicksPerFrame, ebx
                mov     VGANumOfTPFHits, eax
                mov     VGAMaxNumOfTPFHits, 0
Zsidvr_045:
                lea     edi, [edi + 4]
                loopd   Zsidvr_042

                mov     ebx, VGATicksPerFrame
                shl     ebx, 1
                jnz     Zsidvr_047
Zsidvr_046:
                dec     ebp
                jnz     Zsidvr_034
                jmp     Zsidvr_6
Zsidvr_047:
                cmp     VGAMaxNumOfTPFHits, 0
                jz      Zsidvr_048
                dec     ebp
                jnz     Zsidvr_034
Zsidvr_048:
                mov     edx, TMRFrequencyHigh
                mov     eax, TMRFrequencyLow
                shld    edx, eax, FREQ_RAISE + 1
                shl     eax, FREQ_RAISE + 1
                div     ebx
                shr     ebx, 1
                mov     TMRRetracePeriod, ebx
                mov     VRetraceFreqRaised, eax
                mov     esi, eax
                xor     edx, edx
                mov     eax, esi
                mov     ebx, 1 shl FREQ_RAISE
                div     ebx
                test    eax, eax
                jnz     Zsidvr_7
Zsidvr_6:
                mov     edx, TMRFrequencyHigh
                mov     eax, TMRFrequencyLow
                mov     ebx, 50
                div     ebx
                shl     ebx, FREQ_RAISE
                mov     VRetraceFreqRaised, ebx
                mov     TMRRetracePeriod, eax
                jmp     Zsidvr_8
Zsidvr_7:
                PCHR    " "
                PDECD   eax
                mov     eax, edx
                xor     edx, edx
                mov     ecx, 1000
                mul     ecx
                div     ebx
                PCHR    "."
                PDECD   eax, 3
                PCHR    " "
Zsidvr_8:
;                jmp     Zsidvr_010

                mov     eax, CFG_SynchroFreq
                shl     eax, FREQ_RAISE
                test    eax, eax
                jnz     Zsidvr_9
                mov     eax, VRetraceFreqRaised
Zsidvr_9:
                mov     SynchroFreqRaised, eax
                PENT
Zsisvm_End:
                popad
                ret

;----------------------------------------------------------------------------;

VGA_WaitVerticalRetrace:
                push    ebx ecx edx esi edi ebp
                xor     edi, edi
                mov     dx, 003DAh
                call    Get_CurrentSecond
                mov     bh, al
                call    TMR_ReadCounter
                jmp     VGAwvr_2
VGAwvr_1:
                ; waiting for "0"
                call    TMR_ReadCounter
                sub     esi, eax
                cmp     edi, esi
                ja      VGAwvr_2
                mov     edi, esi
VGAwvr_2:
                mov     esi, eax
                in      al, dx
                and     al, 008h
                jz      VGAwvr_4
                call    Get_CurrentSecond
                cmp     al, bh          ; verify RTC seconds left
                mov     bh, al
                setnz   cl
                add     ch, cl
                cmp     ch, 2
                jnc     VGAwvr_ERR
                jmp     VGAwvr_1
VGAwvr_4:
                ; waiting for "1"
                call    TMR_ReadCounter
                sub     esi, eax
                cmp     edi, esi
                ja      VGAwvr_5
                mov     edi, esi
VGAwvr_5:
                mov     esi, eax
                in      al, dx
                and     al, 008h
                jnz     VGAwvr_7
                call    Get_CurrentSecond
                cmp     al, bh          ; verify RTC seconds left
                mov     bh, al
                setnz   cl
                add     ch, cl
                cmp     ch, 2
                jnc     VGAwvr_ERR
                jmp     VGAwvr_4
VGAwvr_7:
                clc
VGAwvr_End:
                mov     eax, edi
                and     eax, 0FFFFh
                pop     ebp edi esi edx ecx ebx
                ret
VGAwvr_ERR:
                ; error - no vertical retrace in 1-2 seconds
                xor     eax, eax
                stc
                jmp     VGAwvr_End

;----------------------------------------------------------------------------;

VGA_BuildPseudographicsFont:
                push    eax ebx ecx edx edi es

                mov     es, sel_DataDOS
                ; Generate font
                mov     bh, al
                sub     bh, ah
                xor     ah, ah
                xor     edi, edi
                mov     ecx, 00100h
VGAbpf_1:
                mov     bl, al
VGAbpf_2:
                cmp     bl, bh
                mov     byte ptr es:[edi], 0
                jna     VGAbpf_3
                mov     es:[edi], ah
VGAbpf_3:
                inc     edi
                dec     bl
                jnz     VGAbpf_2
                inc     ah
                loopd   VGAbpf_1
                ; Load font via BIOS
                shl     eax, 8
                mov     bx, ax
                mov     cx, 00100h
                xor     edx, edx
                xor     ebp, ebp
                mov     ax, rmbase_DataDOS
                mov     V86regs._ES, ax
                mov     ax, 01100h
                INTX    10h
                ; Set current character table
                mov     ax, 01103h
                xor     bl, bl
                INTX    10h

                pop     es edi edx ecx ebx eax
                ret

;----------------------------------------------------------------------------;

VGA_Mode_UpdateText40x:
                pushad
                mov     eax, ds
                mov     es, eax
                mov     ecx, VideoAreaTotalSize
                add     ecx, 3
                shr     ecx, 2
                mov     edi, VideoAreaBaseAdr
                mov     esi, VideoGfxBaseAdr
                cld
                rep     movsd
                popad
                ret

VGA_Mode_InitText40x:
                pushad

                and     CFG_VideoModeMain, VM_FLAGS_MASK
                mov     VideoScheme, offset TMUpdate_SBorder
                mov     VideoInstUpdRoutine, offset VGA_Mode_UpdateText40x
                mov     VideoGfxBaseAdr, 0000B8000h
                mov     VideoAreaBaseAdr, 0000B8000h
                mov     VGABytesPer8Pixels, 2
                and     InitResources, not(IR_VBEVMODE)

                test    InitResources, IR_VESA
                jnz     VGAmit40x_1
                ; If no VESA, try to switch into VGA compatibility mode
                mov     ax, CFG_VideoModePre
                INTX    10h
VGAmit40x_1:
                ; Set 40x25 text mode
                mov     eax, 001h
                INTX    10h
                ; Generate special symbols set for graphics illusion
                mov     eax, 02020h
                call    VGA_BuildPseudographicsFont
                ; Disable blink - enable bright
                mov     eax, 01003h
                xor     bl, bl
                INTX    10h
VGAmit40x_2:
                mov     al, 006h        ; read Vertical Total
                call    VGACRTC_Input
                cmp     ax, 400
                setnc   cl
                mov     al, 009h        ; read Maximum Scan Line
                call    VGACRTC_Input
                and     al, 0E0h
                mov     ah, 1
                shl     ah, cl
                dec     ah
                or      al, ah
                mov     VideoBytesSkip, 0
                test    CFG_VideoModeMain, VM_INTERLACED
                jz      VGAmit40x_3
                movzx   ebx, CFG_VideoXVisible
                shl     ebx, 1
                mov     VideoBytesSkip, ebx
                dec     al
VGAmit40x_3:
                shl     eax, 8
                mov     al, 009h
                call    VGACRTC_Output  ; write Maximum Scan Line
                ; try to set 8-dot mode
                mov     dx, 003C4h
                mov     al, 001h
                out     dx, al
                inc     dl
                in      al, dx
                or      al, 001h
                out     dx, al

                test    CFG_VideoModeMain, VM_FORCEVGA
                jnz     VGAmit40x_4

                ; check for VGA compatibility via VBE
                mov     ecx, 001h
                call    VGA_GetVBEModeInfo
                jc      VGAmit320x_5
                test    ModeAttributes, 020h
                jnz     VGAmit320x_5
VGAmit40x_4:
                jmp     VGAmit320x_3

;----------------------------------------------------------------------------;

VGA_Mode_InitText80x:
                ;...
                jmp     VGA_Mode_InitText40x

;----------------------------------------------------------------------------;

VGA_Mode_UpdateGraph320x:
                pushad
                mov     ecx, VideoAreaTotalSize
                add     ecx, 7
                shr     ecx, 3
                mov     edi, VideoAreaBaseAdr
                mov     esi, VideoGfxBaseAdr
VGAmug320x_1:
                mov     ax, [esi]
                movzx   ebx, ah
                mov     ah, al
                shl     ebx, 4
                shr     bl, 4
                STPOINT dh
                STPOINT dl
                ror     edx, 16
                STPOINT dh
                STPOINT dl
                mov     [edi + 4], edx
                STPOINT dh
                STPOINT dl
                ror     edx, 16
                STPOINT dh
                STPOINT dl
                mov     [edi], edx
                lea     esi, [esi + 2]
                lea     edi, [edi + 8]
                dec     ecx
                jnz     VGAmug320x_1
                popad
                ret

VGA_Mode_InitGraph320x:
                pushad

                mov     VideoScheme, offset TMUpdate_SBorder
                mov     VideoInstUpdRoutine, offset VGA_Mode_UpdateGraph320x
                mov     VideoGfxBaseAdr, 0000A0000h
                mov     VideoAreaBaseAdr, 0000A0000h
                mov     VGABytesPer8Pixels, 8
                and     InitResources, not(IR_VBEVMODE)
                ; always buffered
                or      CFG_VideoModeMain, VM_BUFFERED
                ; no interlace
                and     CFG_VideoModeMain, not(VM_INTERLACED)
                mov     VideoBytesSkip, 0

                test    InitResources, IR_VESA
                jz      VGAmit320x_1
                ; search for XxYx256 graf mode
                mov     dx, 010h
                mov     ebp, edx
                movzx   esi, CFG_VideoXVisible
                shl     esi, 3
                mov     edi, CFG_VideoYVisible
                mov     al, 8
                call    VGA_SearchMode
                jc      VGAmit320x_1
                test    ModeAttributes, 080h
                jz      VGAmit320x_1
                cmp     PhysBasePtr, 0
                jz      VGAmit320x_1
                mov     bx, cx
                or      bx, 04000h
                mov     ax, 04F02h
                INTX    10h
                cmp     ax, 04Fh
                jnz     VGAmit320x_1
                or      InitResources, IR_VBEVMODE
                mov     eax, PhysBasePtr
                mov     VideoGfxBaseAdr, eax
                mov     VideoAreaBaseAdr, eax
                jmp     VGAmit320x_6
VGAmit320x_1:
                ; Set 320x200x256 graf mode
                mov     eax, 013h
                INTX    10h
                test    CFG_VideoModeMain, VM_FORCEVGA
                jnz     VGAmit320x_2
                ; check for VGA compatibility via VBE
                mov     cx, 013h
                call    VGA_GetVBEModeInfo
                jc      VGAmit320x_5
                test    ModeAttributes, 020h
                jnz     VGAmit320x_5
VGAmit320x_2:
                ; select extended memory mode
                mov     dx, 003C4h
                mov     al, 004h
                out     dx, al
                inc     dl
                in      al, dx
                or      al, 002h
                out     dx, al
VGAmit320x_3:
                ;---------------------------------
                ; read horizontal characteristics
                ;---------------------------------
                mov     al, 000h        ; read Horizontal Total
                call    VGACRTC_Input
                mov     VGACurHTotal, al
                mov     al, 001h        ; read End Horizontal Display
                call    VGACRTC_Input
                mov     VGACurEndHDisplay, al
                mov     al, 002h        ; read Start Horizontal Blanking
                call    VGACRTC_Input
                mov     VGACurStartHBlank, al
                mov     al, 004h        ; read Start Horizontal Retrace
                call    VGACRTC_Input
                mov     VGACurStartHRetrace, al
                mov     al, 013h        ; read Offset
                call    VGACRTC_Input
                mov     VGACurOffset, al
                cmp     VGACurStartHRetrace, 80
                setnc   cl
                mov     al, CFG_VideoXTotalAdd
                mov     bl, CFG_VideoXVisible
                sub     bl, 40          ; = 320 / 8
                shl     al, cl
                shl     bl, cl
                add     VGACurHTotal, al
                add     VGACurEndHDisplay, bl
                add     VGACurStartHBlank, bl
                sar     bl, 1
                add     al, bl
                add     VGACurStartHRetrace, al
                cmp     VGACurOffset, 40
                setc    cl
                mov     al, CFG_VideoXVisible
                and     al, 0FEh
                mov     CFG_VideoXVisible, al
                shr     al, cl
                mov     VGACurOffset, al
                ;-----------------------------------
                ; modify horizontal characteristics
                ;-----------------------------------
                mov     al, 000h        ; write Horizontal Total
                mov     ah, VGACurHTotal
                call    VGACRTC_Output
                mov     al, 001h        ; write End Horizontal Display
                mov     ah, VGACurEndHDisplay
                call    VGACRTC_Output
                mov     al, 002h        ; write Start Horizontal Blanking
                mov     ah, VGACurStartHBlank
                call    VGACRTC_Output
                mov     al, 004h        ; write Start Horizontal Retrace
                mov     ah, VGACurStartHRetrace
                call    VGACRTC_Output
                mov     al, 013h        ; write Offset
                mov     ah, VGACurOffset
                call    VGACRTC_Output
VGAmit320x_4:
                ;-------------------------------
                ; read vertical characteristics
                ;-------------------------------
                mov     al, 006h        ; read Vertical Total
                call    VGACRTC_Input
                mov     VGACurVTotal, eax
                mov     al, 010h        ; read Vertical Retrace Start
                call    VGACRTC_Input
                mov     VGACurVRetraceStart, eax
                mov     al, 012h        ; read Vertical Display End
                call    VGACRTC_Input
                mov     VGACurVDisplayEnd, eax
                mov     al, 015h        ; read Start Vertical Blanking
                call    VGACRTC_Input
                mov     VGACurStartVBlank, eax
                mov     al, 016h        ; read End Vertical Blanking
                call    VGACRTC_Input
                mov     VGACurEndVBlank, eax
                cmp     VGACurVTotal, 400
                setnc   cl
                movzx   eax, CFG_VideoYTotalAdd
                mov     ebx, CFG_VideoYVisible
                sub     ebx, 200
                shl     eax, cl
                shl     ebx, cl
                add     VGACurVTotal, ebx
                add     VGACurVTotal, eax
                add     VGACurVRetraceStart, ebx
                add     VGACurVDisplayEnd, ebx
                add     VGACurStartVBlank, ebx
                add     VGACurEndVBlank, ebx
                ;---------------------------------
                ; modify vertical characteristics
                ;---------------------------------
                mov     eax, VGACurVTotal
                shl     eax, 8
                mov     al, 006h        ; write Vertical Total
                call    VGACRTC_Output
                mov     eax, VGACurVRetraceStart
                shl     eax, 8
                mov     al, 010h        ; write Vertical Retrace Start
                call    VGACRTC_Output
                mov     eax, VGACurVDisplayEnd
                shl     eax, 8
                mov     al, 012h        ; write Vertical Display End
                call    VGACRTC_Output
                mov     eax, VGACurStartVBlank
                shl     eax, 8
                mov     al, 015h        ; write Start Vertical Blanking
                call    VGACRTC_Output
                mov     eax, VGACurEndVBlank
                shl     eax, 8
                mov     al, 016h        ; write End Vertical Blanking
                call    VGACRTC_Output
                ; try to enable IRQ
                mov     al, 011h
                call    VGACRTC_Input
                and     al, 0CFh
                mov     ah, al
                mov     al, 011h
                call    VGACRTC_Output

                ; set misc output
                mov     dx, 003CCh
                in      al, dx
                mov     dl, 0C2h
                or      al, 001h ; 0C1h
                and     al, 0F3h
                out     dx, al
                jmp     VGAmit320x_6

VGAmit320x_5:
                mov     CFG_VideoXVisible, 40   ; = 320 / 8
                mov     CFG_VideoYVisible, 200
                mov     VideoBytesSkip, 0
VGAmit320x_6:
                movzx   eax, CFG_VideoXVisible
                movzx   ebx, VGABytesPer8Pixels
                mul     ebx
                add     eax, VideoBytesSkip
                mov     VGABytesPerLine, eax
                mov     ebx, CFG_VideoYVisible
                mul     ebx
;                test    edx, edx
;                jnz     VGAmit320x_5
                mov     VideoAreaTotalSize, eax
                cmp     eax, 08000h
                jc      VGAmit320x_7
                test    InitResources, IR_VBEVMODE
                jnz     VGAmit320x_7
                ; change memory map to 0A0000h-0BFFFFh
                mov     dx, 003CEh
                mov     al, 006h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, 0F3h
                out     dx, al
                mov     VideoGfxBaseAdr, 0000A0000h
                mov     VideoAreaBaseAdr, 0000A0000h
                ; verify >64K accessibility
                mov     esi, VideoGfxBaseAdr
                mov     ecx, 000010000h
                mov     bl, [esi]
                mov     bh, [esi + ecx]
                mov     al, 0FFh
                mov     [esi + ecx], al
                cmp     al, [esi]
                jnz     VGAmit320x_69
                xor     al, al
                mov     [esi + ecx], al
                cmp     al, [esi]
                jnz     VGAmit320x_69
                ; >64K is not accessible, cut screen Y-resolution
                xor     edx, edx
                mov     eax, ecx
                mov     edi, VGABytesPerLine
                div     edi
                mov     CFG_VideoYVisible, eax
                mov     VideoAreaTotalSize, ecx
VGAmit320x_69:
                xor     al, al
                mov     [esi + ecx], al
VGAmit320x_7:
                popad
                ret

;----------------------------------------------------------------------------;

VGA_Mode_UpdateGraph640x:
                pushad
                popad
                ret

VGA_Mode_InitGraph640x:
                push    eax esi
                
                mov     CFG_VideoXVisible, 40   ; = (640/8)/2
                mov     CFG_VideoYVisible, 240  ; = (480  )/2
                mov     VideoScheme, offset TMUpdate_SBorder
                mov     VideoInstUpdRoutine, offset VGA_Mode_UpdateGraph320x
                mov     VideoGfxBaseAdr, 0000A0000h
                mov     VideoAreaBaseAdr, 0000A0000h
                mov     VideoAreaTotalSize, 307200

                mov     eax, 012h
                INTX    10h

                pop     esi eax
                ret

;----------------------------------------------------------------------------;

VGA_Mode_InitGraph800x:
                jmp     VGA_Mode_InitGraph640x

;----------------------------------------------------------------------------;

VGA_Mode_InitGraph1024x:
                jmp     VGA_Mode_InitGraph640x

;----------------------------------------------------------------------------;

VGA_InitDAC:
                pushad
                push    es

                mov     es, sel_DataDOS
                mov     ebx, 010h
                xor     edi, edi
                cld
VGAidac_1:
                mov     ecx, 00Ch
                rep     movs dword ptr es:[edi], ds:[esi]
                sub     esi, 030h
                dec     bl
                jnz     VGAidac_1
                mov     cx, 00100h
                mov     ax, rmbase_DataDOS
                mov     V86regs._ES, ax
                xor     edx, edx
                mov     ax, 01012h
                INTX    10h
                test    CFG_VideoModeMain, VM_BLACKANDWHITE
                jz      VGAidac_2
                ; black-and-white pallete
                mov     al, 01Bh
                INTX    10h
VGAidac_2:
                pop     es
                popad
                ret

;----------------------------------------------------------------------------;

VGA_DetectCompatibility:
                pushad
                and     InitResources, not(IR_VESA)
                test    CFG_VideoModeMain, VM_DONOTUSEVBE
                jnz     VGAdc_1

                mov     eax, ds
                mov     es, eax

                xor     edi, edi
                mov     ax, rmbase_DataDOS
                mov     V86regs._ES, ax
                mov     ax, 04F00h
                INTX    10h
                cmp     ax, 04Fh
                jz      VGAdc_2

                PSTR    TxtVBENotFound
VGAdc_1:
                ; VESA not present, check system for overall VGA-compatibility
                mov     eax, 01A00h
                INTX    10h
                cmp     al, 01Ah
                jnz     VGAdc_ERR
                ; check that primary display is VGA-compatible...
                cmp     bl, 7
                setz    al
                cmp     bl, 8
                setz    ah
                or      al, ah
                jnz     VGAdc_5
                ; ... or check that secondary display is VGA-compatible...
                cmp     bh, 7
                setz    al
                cmp     bh, 8
                setz    ah
                or      al, ah
                jz      VGAdc_ERR
                ; ... and set VGA-compatible display active
                mov     bl, bh
                mov     ax, 01A01h
                INTX    10h
                cmp     al, 01Ah
                jnz     VGAdc_ERR
                ; again check that primary display is VGA-compatible...
                cmp     bl, 7
                setz    al
                cmp     bl, 8
                setz    ah
                or      al, ah
                jnz     VGAdc_5
                ; ...and if not, go home
                mov     CFG_VideoModePre, 0FFFFh
                PSTR    TxtNoVGA
                jmp     VGAdc_ERR
VGAdc_2:
                mov     esi, pmoffs_DataDOS
                lea     edi, VBEInfoBlock
                cld
                mov     ecx, vbeinfo_size/4
                rep     movs dword ptr es:[edi], ds:[esi]

                or      InitResources, IR_VESA
                PSTR    TxtVBEFound
                mov     ax, VBEVersion
                PDECB   ah
                PCHR    "."
                PDECB   al
                PCHR    "."
                PENT
                ; search for VGA-compatible mode in VESA modes list
                xor     ebp, ebp
                mov     dx, 020h
                xor     esi, esi
                xor     edi, edi
                xor     al, al
                call    VGA_SearchMode
                jc      VGAdc_ERR
                mov     CFG_VideoModePre, cx
VGAdc_5:
                clc
VGAdc_End:
                popad
                ret
VGAdc_ERR:
                stc
                jmp     VGAdc_End

;----------------------------------------------------------------------------;

VGA_SearchMode:
                ;-----------------------------------------
                ; Search for Video Mode
                ;-----------------------------------------
                ; - in:
                ; AL  - BitsPerPixel, 0=any
                ; DX  - ModeAttributes mask
                ; BP  - ModeAttributes value
                ; SI  - XResolution, 0=any
                ; DI  - YResolution, 0=any
                ; - out:
                ; CF  - NC: CX = video mode number
                ;        C: no VBE or
                ;           VBE version lower than 1.2 or
                ;           no such video mode
                ;-----------------------------------------
                test    InitResources, IR_VESA
                jnz     VGAsm_1
                stc
                ret
VGAsm_1:
                push    ebx eax ebp
                movzx   ebx, word ptr VideoModePtr + 2
                shl     ebx, 4
                movzx   eax, word ptr VideoModePtr
                add     ebx, eax
                xor     ebp, ebp
VGAsm_2:
                mov     ecx, ebp
                inc     ebp
                cmp     ebp, 00100h
                jc      VGAsm_3
                mov     cx, [ebx]
                lea     ebx, [ebx + 2]
                cmp     cx, 0FFFFh
                jz      VGAsm_ERR
VGAsm_3:
                call    VGA_GetVBEModeInfo
                jc      VGAsm_2
                mov     ax, dx
                and     ax, ModeAttributes
                cmp     ax, bp
                jnz     VGAsm_2
                cmp     VBEVersion, 00102h
                jnc     VGAsm_4
                mov     ax, si
                or      ax, di
                jnz     VGAsm_ERR
                jmp     VGAsm_7
VGAsm_4:
                test    si, si
                jz      VGAsm_5
                cmp     si, XResolution
                jnz     VGAsm_2
VGAsm_5:
                test    di, di
                jz      VGAsm_7
VGAsm_6:
                cmp     di, YResolution
                jnz     VGAsm_2
VGAsm_7:
                mov     al, [esp]
                test    al, al
                jz      VGAsm_End
                cmp     al, BitsPerPixel
                jnz     VGAsm_2
                cmp     MemoryModel, 4
                jnz     VGAsm_2
                clc
VGAsm_End:
                pop     ebp eax ebx
                ret
VGAsm_ERR:
                stc
                jmp     VGAsm_End

;----------------------------------------------------------------------------;

VGA_GetVBEModeInfo:
                test    InitResources, IR_VESA
                jnz     VGAgvbemi_1
                stc
                ret
VGAgvbemi_1:
                push    es eax ebx edx esi edi ecx
                ; clear ModeInfoBlock area
                mov     es, cs:sel_Data
                mov     edi, pmoffs_DataDOS
                xor     eax, eax
                mov     ecx, vmodeinfo_size/4
                cld
                rep     stos dword ptr es:[edi]
                ; get VBE Mode info
                mov     cx, [esp]
                xor     edi, edi
                mov     ax, rmbase_DataDOS
                mov     V86regs._ES, ax
                mov     ax, 04F01h
                INTX    10h
                cmp     ax, 04Fh
                jnz     VGAgvbemi_ERR
                ; copy VBE Mode info to ModeInfoBlock area
                mov     esi, pmoffs_DataDOS
                lea     edi, ModeInfoBlock
                mov     ecx, vmodeinfo_size/4
                cld
                rep     movs dword ptr es:[edi], ds:[esi]
                clc
VGAgvbemi_End:
                pop     ecx edi esi edx ebx eax es
                ret
VGAgvbemi_ERR:
                stc
                jmp     VGAgvbemi_End

;----------------------------------------------------------------------------;

VGACRTC_Input:
                push    ebx ecx edx
                mov     bl, al
                ; store CRTC index register
                mov     dx, 003D4h
                in      al, dx
                mov     cl, al
                ; select CRTC index register
                mov     al, bl
                cmp     al, 006h        ; Vertical Total
                jz      Vci_VT
                cmp     al, 00Ch        ; Start Address
                jz      Vci_SA
                cmp     al, 00Dh        ; Start Address
                jz      Vci_SA
                cmp     al, 00Eh        ; Cursor Location
                jz      Vci_CL
                cmp     al, 00Fh        ; Cursor Location
                jz      Vci_CL
                cmp     al, 010h        ; Vertical Retrace Start
                jz      Vci_VRS
                cmp     al, 011h        ; Vertical Retrace End
                jz      Vci_VRE
                cmp     al, 012h        ; Vertical Display End
                jz      Vci_VDE
                cmp     al, 015h        ; Start Vertical Blanking
                jz      Vci_SVB
                cmp     al, 018h        ; Line Compare
                jz      Vci_LC
                ; read a common CRTC register
                out     dx, al
                inc     dl
                in      al, dx
Vci_1:
                ; restore CRTC index register
                mov     ebx, eax
                mov     al, cl
                dec     dl
                out     dx, al
                mov     eax, ebx
                and     eax, 0FFFFh

                pop     edx ecx ebx
                ret
Vci_VT:
                ; Vertical Total
                ; - read bits 0-7 of Vertical Total
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bits 8-9 of Vertical Total
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bl, al
                and     al, 001h
                and     bl, 020h
                shr     bl, 4
                or      al, bl
                xchg    al, ah
                jmp     Vci_1
Vci_VRS:
                ; Vertical Retrace Start
                ; - store crtc registers protect
                mov     al, 011h
                out     dx, al
                inc     dl
                in      al, dx
                mov     ch, al
                ; - disable crtc registers protect
                and     al, 07Fh
                out     dx, al
                dec     dl
                ; - store vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bh, al
                ; - enable vertical retrace access
                or      al, 080h
                out     dx, al
                dec     dl
                ; - read bits 0-7 of Vertical Retrace Start
                mov     al, 010h
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bits 8-9 of Vertical Retrace Start
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bl, al
                and     al, 004h
                and     bl, 080h
                rol     bl, 2
                shr     al, 2
                or      bl, al
                dec     dl
                ; - restore vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                mov     al, bh
                out     dx, al
                dec     dl
                ; - restore crtc registers protect
                mov     al, 011h
                out     dx, al
                inc     dl
                mov     al, ch
                out     dx, al
                mov     al, ah
                mov     ah, bl
                jmp     Vci_1
Vci_VRE:
                ; Vertical Retrace End
                ; - store crtc registers protect
                mov     al, 011h
                out     dx, al
                inc     dl
                in      al, dx
                mov     ch, al
                ; - disable crtc registers protect
                and     al, 07Fh
                out     dx, al
                dec     dl
                ; - store vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bh, al
                ; - enable vertical retrace access
                or      al, 080h
                out     dx, al
                dec     dl
                ; - read Vertical Retrace End
                mov     al, 011h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bl, al
                dec     dl
                ; - restore vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                mov     al, bh
                out     dx, al
                dec     dl
                ; - restore crtc registers protect
                mov     al, 011h
                out     dx, al
                inc     dl
                mov     al, ch
                out     dx, al
                mov     al, bl
                jmp     Vci_1
Vci_VDE:
                ; Vertical Display End
                ; - read bits 0-7 of Vertical Display End
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bits 8-9 of Vertical Display End
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bl, al
                and     al, 002h
                shr     al, 1
                and     bl, 040h
                shr     bl, 5
                or      al, bl
                xchg    al, ah
                jmp     Vci_1
Vci_SVB:
                ; Start Vertical Blanking
                ; - read bits 0-7 of Start Vertical Blanking
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bit 8 of Start Vertical Blanking
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bl, al
                and     al, 008h
                shr     al, 3
                mov     bl, al
                dec     dl
                ; - read bit 9 of Start Vertical Blanking
                mov     al, 009h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, 020h
                shr     al, 4
                or      al, bl
                xchg    al, ah
                jmp     Vci_1
Vci_LC:
                ; Line Compare
                ; - read bits 0-7 of Line Compare
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bit 8 of Line Compare
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bl, al
                and     al, 010h
                shr     al, 4
                mov     bl, al
                dec     dl
                ; - read bit 9 of Line Compare
                mov     al, 009h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, 040h
                shr     al, 5
                or      al, bl
                xchg    al, ah
                jmp     Vci_1
Vci_SA:
                ; Start Address
                ; - read bits 0-7 of Start Address
                mov     al, 00Ch
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bits 8-15 of Start Address
                mov     al, 00Dh
                out     dx, al
                inc     dl
                in      al, dx
                jmp     Vci_1
Vci_CL:
                ; Cursor Location
                ; - read bits 0-7 of Cursor Location
                mov     al, 00Eh
                out     dx, al
                inc     dl
                in      al, dx
                mov     ah, al
                dec     dl
                ; - read bits 8-15 of Cursor Location
                mov     al, 00Fh
                out     dx, al
                inc     dl
                in      al, dx
                jmp     Vci_1

;----------------------------------------------------------------------------;

VGACRTC_Output:
                pushad
                mov     esi, eax
                ; store CRTC index register
                mov     dx, 003D4h
                in      al, dx
                mov     cl, al
                ;; select CRTC index register
                ;mov     eax, esi
                ;cmp     al, 008h
                ;jnc     Vco_0
                ; - store crtc registers protect
                mov     al, 011h
                out     dx, al
                inc     dl
                in      al, dx
                mov     ch, al
                and     ch, 080h
                ; - disable crtc registers protect
                and     al, 07Fh
                out     dx, al
                dec     dl
                mov     eax, esi
                cmp     al, 006h        ; Vertical Total
                jz      Vco_VT
Vco_0:
                cmp     al, 00Ch        ; Start Address
                jz      Vco_SA
                cmp     al, 00Dh        ; Start Address
                jz      Vco_SA
                cmp     al, 00Eh        ; Cursor Location
                jz      Vco_CL
                cmp     al, 00Fh        ; Cursor Location
                jz      Vco_CL
                cmp     al, 010h        ; Vertical Retrace Start
                jz      Vco_VRS
                cmp     al, 011h        ; Vertical Retrace End
                jz      Vco_VRE
                cmp     al, 012h        ; Vertical Display End
                jz      Vco_VDE
                cmp     al, 015h        ; Start Vertical Blanking
                jz      Vco_SVB
                cmp     al, 018h        ; Line Compare
                jz      Vco_LC
                ; write a common CRTC register
                out     dx, al
                inc     dl
                mov     al, ah
                out     dx, al
Vco_1:
                ;mov     eax, esi
                ;cmp     al, 008h
                ;jnc     Vco_2
                ; - restore crtc registers protect
                dec     dl
                mov     al, 011h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, 07Fh
                or      al, ch
                out     dx, al
Vco_2:
                ; restore CRTC index register
                mov     al, cl
                dec     dl
                out     dx, al

                popad
                ret
Vco_VT:
                ; Vertical Total
                ; - write bits 0-7 of Vertical Total
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                dec     dl
                ; - write bits 8-9 of Vertical Total
                shr     eax, 8
                mov     bl, al
                and     al, 001h
                and     bl, 002h
                shl     bl, 4
                or      bl, al
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(021h)
                or      al, bl
                out     dx, al
                jmp     Vco_1
Vco_VRS:
                ; Vertical Retrace Start
                ; - store crtc registers protect
                mov     al, 011h
                out     dx, al
                inc     dl
                in      al, dx
                mov     ch, al
                and     ch, 080h
                ; - disable crtc registers protect
                and     al, 07Fh
                out     dx, al
                dec     dl
                ; - store vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bh, al
                ; - enable vertical retrace access
                or      al, 080h
                out     dx, al
                dec     dl
                ; - write bits 0-7 of Vertical Retrace Start
                mov     al, 010h
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                dec     dl
                ; - write bits 8-9 of Vertical Retrace Start
                shr     eax, 8
                mov     bl, al
                and     al, 001h
                and     bl, 002h
                shl     al, 2
                shl     bl, 6
                or      bl, al
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(084h)
                or      al, bl
                out     dx, al
                dec     dl
                ; - restore vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                mov     al, bh
                out     dx, al
                jmp     Vco_1
Vco_VRE:
                ; Vertical Retrace End
                ; - store vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                in      al, dx
                mov     bh, al
                ; - enable vertical retrace access
                or      al, 080h
                out     dx, al
                dec     dl
                ; - write Vertical Retrace End
                mov     al, 011h
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                mov     ch, al
                and     ch, 080h
                dec     dl
                ; - restore vertical retrace access
                mov     al, 003h
                out     dx, al
                inc     dl
                mov     al, bh
                out     dx, al
                jmp     Vco_1
Vco_VDE:
                ; Vertical Display End
                ; - write bits 0-7 of Vertical Display End
                mov     al, 012h
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                dec     dl
                ; - write bits 8-9 of Vertical Display End
                shr     eax, 8
                mov     bl, al
                and     al, 001h
                and     bl, 002h
                shl     al, 1
                shl     bl, 5
                or      bl, al
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(042h)
                or      al, bl
                out     dx, al
                jmp     Vco_1
Vco_SVB:
                ; Start Vertical Blanking
                ; - write bits 0-7 of Start Vertical Blanking
                mov     al, 015h
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                dec     dl
                ; - write bit 8 of Start Vertical Blanking
                shr     eax, 8
                mov     bl, al
                mov     ah, al
                and     bl, 001h
                and     ah, 002h
                shl     bl, 3
                shl     ah, 4
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(008h)
                or      al, bl
                out     dx, al
                dec     dl
                ; - write bit 9 of Start Vertical Blanking
                mov     al, 009h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(020h)
                or      al, ah
                out     dx, al
                jmp     Vco_1
Vco_LC:
                ; Line Compare
                ; - write bits 0-7 of Line Compare
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                ; - write bit 8 of Line Compare
                shr     eax, 8
                mov     bl, al
                mov     bh, al
                and     bl, 001h
                and     bh, 002h
                shl     bl, 4
                shl     bh, 5
                mov     al, 007h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(010h)
                or      al, bl
                out     dx, al
                dec     dl
                ; - write bit 9 of Line Compare
                mov     al, 009h
                out     dx, al
                inc     dl
                in      al, dx
                and     al, not(040h)
                or      al, bh
                out     dx, al
                jmp     Vco_1
Vco_SA:
                ; Start Address
                ; - write bits 0-7 of Start Address
                mov     al, 00Dh
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                dec     dl
                ; - write bits 8-15 of Start Address
                mov     al, 00Ch
                out     dx, al
                inc     dl
                shr     eax, 8
                out     dx, al
                jmp     Vco_1
Vco_CL:
                ; Cursor Location
                ; - write bits 0-7 of Cursor Location
                mov     al, 00Fh
                out     dx, al
                inc     dl
                mov     eax, esi
                shr     eax, 8
                out     dx, al
                dec     dl
                ; - write bits 8-15 of Cursor Location
                mov     al, 00Eh
                out     dx, al
                inc     dl
                shr     eax, 8
                out     dx, al
                jmp     Vco_1

;****************************************************************************;

.DATA

VideoBuffer         label   word
                    dd      16384 dup(0)
videobuf_size = $-VideoBuffer
VideoBufferAdr      dd      0

VGACurHTotal        db      0
VGACurEndHDisplay   db      0
VGACurStartHBlank   db      0
VGACurStartHRetrace db      0
VGACurOffset        db      0
VGACurMaxScanLine   db      0
VGABytesPer8Pixels  db      0
VGABytesPerLine     dd      0

VGACurVTotal        dd      0
VGACurVRetraceStart dd      0
VGACurVDisplayEnd   dd      0
VGACurStartVBlank   dd      0
VGACurEndVBlank     dd      0

;****************************************************************************
;* MESSAGES *****************************************************************
;****************************************************************************

TxtDVideoSignal     db      " Detecting frame rate.", 0
TxtVBEFound         db      " VESA/VBE found version ", 0
TxtVBENotFound      db      " VESA/VBE not found.", 00Dh, 00Ah, 0
TxtNoVGA            db      "Could not obtain desired VGA compatibility.", 00Dh, 00Ah, 0

DACRegisters        label   byte
                    ; Black
                    db      000h, 000h, 000h
                    ; Blue
                    db      000h, 000h, 02Ah
                    ; Red
                    db      02Ah, 000h, 000h
                    ; Magenta
                    db      02Ah, 000h, 02Ah
                    ; Green
                    db      000h, 02Ah, 000h
                    ; Cyan
                    db      000h, 02Ah, 02Ah
                    ; Yellow
                    db      02Ah, 02Ah, 000h
                    ; White
                    db      02Ah, 02Ah, 02Ah
                    ; Black
                    db      000h, 000h, 000h
                    ; Light blue
                    db      000h, 000h, 03Fh
                    ; Light red
                    db      03Fh, 000h, 000h
                    ; Light magenta
                    db      03Fh, 000h, 03Fh
                    ; Light green
                    db      000h, 03Fh, 000h
                    ; Light cyan
                    db      000h, 03Fh, 03Fh
                    ; Light yellow
                    db      03Fh, 03Fh, 000h
                    ; Light white
                    db      03Fh, 03Fh, 03Fh
DACRegistersBlack   label   byte
                    db      48 dup(0)

ZXPalleteTableA     label   byte
                    db      00100h dup(0)
ZXPalleteTableB     label   byte
                    db      000h, 011h, 022h, 033h, 044h, 055h, 066h, 077h
ZXScrnAdrTable      label   dword
                    dd      384 dup(0)

ColorShift1x        label   byte
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
                    db      000h, 0C0h, 0F0h, 0FCh, 000h, 0C0h, 0F0h, 0FCh
ColorShift2x        label   byte
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
                    db      000h, 080h, 0C0h, 0E0h, 0F0h, 0F8h, 0FCh, 0FEh
ColorShift4x        label   byte
                    db      000h, 000h, 080h, 080h, 0C0h, 0C0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F8h, 0F8h, 0FCh, 0FCh, 0FEh, 0FEh
                    db      000h, 000h, 080h, 080h, 0C0h, 0C0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F8h, 0F8h, 0FCh, 0FCh, 0FEh, 0FEh
                    db      000h, 000h, 080h, 080h, 0C0h, 0C0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F8h, 0F8h, 0FCh, 0FCh, 0FEh, 0FEh
                    db      000h, 000h, 080h, 080h, 0C0h, 0C0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F8h, 0F8h, 0FCh, 0FCh, 0FEh, 0FEh
ColorShift8x        label   byte
                    db      000h, 000h, 000h, 000h, 080h, 080h, 080h, 080h
                    db      0C0h, 0C0h, 0C0h, 0C0h, 0E0h, 0E0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F0h, 0F0h, 0F8h, 0F8h, 0F8h, 0F8h
                    db      0FCh, 0FCh, 0FCh, 0FCh, 0FEh, 0FEh, 0FEh, 0FEh
                    db      000h, 000h, 000h, 000h, 080h, 080h, 080h, 080h
                    db      0C0h, 0C0h, 0C0h, 0C0h, 0E0h, 0E0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F0h, 0F0h, 0F8h, 0F8h, 0F8h, 0F8h
                    db      0FCh, 0FCh, 0FCh, 0FCh, 0FEh, 0FEh, 0FEh, 0FEh
ColorShift16x       label   byte
                    db      000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
                    db      080h, 080h, 080h, 080h, 080h, 080h, 080h, 080h
                    db      0C0h, 0C0h, 0C0h, 0C0h, 0C0h, 0C0h, 0C0h, 0C0h
                    db      0E0h, 0E0h, 0E0h, 0E0h, 0E0h, 0E0h, 0E0h, 0E0h
                    db      0F0h, 0F0h, 0F0h, 0F0h, 0F0h, 0F0h, 0F0h, 0F0h
                    db      0F8h, 0F8h, 0F8h, 0F8h, 0F8h, 0F8h, 0F8h, 0F8h
                    db      0FCh, 0FCh, 0FCh, 0FCh, 0FCh, 0FCh, 0FCh, 0FCh
                    db      0FEh, 0FEh, 0FEh, 0FEh, 0FEh, 0FEh, 0FEh, 0FEh

VideoScheme         dd      0
VideoInstUpdRoutine dd      0
VideoDivider        dd      0
VideoShiftPoint     dd      0
VideoAreaBaseAdr    dd      0
VideoAreaBaseAdr2   dd      0
VideoGfxBaseAdr     dd      0
VideoAreaTotalSize  dd      0
VideoAreaWindowSize dd      0
ScreenXYCoord       dd      0
VideoGPause         dd      0
VideoLPause         label   byte
                    dd      0
VideoSTables        label   dword
                    dd      ColorShift1x,   004h    ; 8   4  Z80
                    dd      ColorShift2x,   008h    ; 8   8  Z80
                    dd      ColorShift2x,   00Ch    ; 8   12  Z80
                    dd      ColorShift4x,   010h    ; 8   16  Z80
                    dd      ColorShift4x,   014h    ; 8   20  Z80
                    dd      ColorShift4x,   018h    ; 8   24  Z80
                    dd      ColorShift4x,   01Ch    ; 8   28  Z80
                    dd      ColorShift8x,   020h    ; 8   32  Z80
                    dd      ColorShift8x,   024h    ; 8   36  Z80
                    dd      ColorShift8x,   028h    ; 8   40  Z80
                    dd      ColorShift8x,   02Ch    ; 8   44  Z80
                    dd      ColorShift8x,   030h    ; 8   48  Z80
                    dd      ColorShift8x,   034h    ; 8   52  Z80
                    dd      ColorShift8x,   038h    ; 8   56  Z80
                    dd      ColorShift8x,   03Ch    ; 8   60  Z80
                    dd      ColorShift16x,  040h    ; 8   64  Z80

VMInitPrograms      label   dword
                    dd      VGA_Mode_InitText40x
                    dd      VGA_Mode_InitText80x
                    dd      VGA_Mode_InitGraph320x
                    dd      VGA_Mode_InitGraph640x
                    dd      VGA_Mode_InitGraph800x
                    dd      VGA_Mode_InitGraph1024x
vmtbl_size=($-VMInitPrograms)/4

VGAMinTicksPerFrame dd      0
VGAMaxTicksPerFrame dd      0
VGAMinTPFMax        dd      0
VGAMaxTPFMin        dd      0
VGAMinTPFCounter    dd      0
VGAMaxTPFCounter    dd      0

VGATicksPerFrame    dd      0
VGANumOfTPFHits     dd      0
VGACurTicksPerFrame dd      0
VGACurNumOfTPFHits  dd      0
VGAMaxNumOfTPFHits  dd      0
VGARetraceDetectBuf label   dword
                    dd      64 dup(?)
vgardbuf_size=$-VGARetraceDetectBuf

;VGAStandartModes    label   word
;                    dw      000h, 001h, 002h, 003h, 004h, 005h, 006h, 007h, 008h, 009h, 00Ah, 00Bh, 00Ch, 00Dh, 00Eh, 00Fh
;                    dw      010h, 011h, 012h, 013h, 014h, 015h, 016h, 017h, 018h, 019h, 01Ah, 01Bh, 01Ch, 01Dh, 01Eh, 01Fh
;                    dw      0FFFFh
;vgasmtbl_size=$-VGAStandartModes

; CRTC info block
CRTCInfoBlock       label   byte
HorizontalTotal     dw      400
HorizontalSyncStart dw      320
HorizontalSyncEnd   dw      0
VerticalTotal       dw      252
VerticalSyncStart   dw      240
VerticalSyncEnd     dw      0
CRTCFlags           db      000h
PixelClock          dd      5040000
RefreshRate         dw      5000
CRTCReserved        db      40 dup(?)
crtcinfo_size=$-CRTCInfoBlock

; VESA/VBE - VBE info block
VBEInfoBlock        label   byte
VBESignature        db      "VESA"
VBEVersion          dw      00300h
OEMStringPtr        dd      ?
Capabilities        db      4 dup(?)
VideoModePtr        dd      ?
TotalMemory         dw      ?
OEMSoftwareRev      dw      ?
OEMVendorNamePtr    dd      ?
OEMProductNamePtr   dd      ?
OEMProductRevPtr    dd      ?
VBEInfoReserved     db      222 dup(?)
OEMData             db      256 dup(?)
vbeinfo_size=$-VBEInfoBlock

; VESA/VBE - mode info block
ModeInfoBlock       label   byte
; Mandatory information for all VBE revisions
ModeAttributes      dw      ?           ; mode attributes
WinAAttributes      db      ?           ; window A attributes
WinBAttributes      db      ?           ; window B attributes
WinGranularity      dw      ?           ; window granularity
WinSize             dw      ?           ; window size
WinASegment         dw      ?           ; window A start segment
WinBSegment         dw      ?           ; window B start segment
WinFuncPtr          dd      ?           ; pointer to windor function
BytesPerScanLine    dw      ?           ; bytes per scan line
; Mandatory information for VBE 1.2 and above
XResolution         dw      ?           ; horizontal resolution
YResolution         dw      ?           ; vertical resolution
XCharSize           db      ?           ; character cell width
YCharSize           db      ?           ; character cell height
NumberOfPlanes      db      ?           ; number of memory planes
BitsPerPixel        db      ?           ; bits per pixel
NumberOfBanks       db      ?           ; number of banks
MemoryModel         db      ?           ; memory model type
BankSize            db      ?           ; bank size in kb
NumberOfImagePages  db      ?           ; number of images
ModeInfoReserved    db      1           ; reserved for page function
; Direct Color fields (required for direct/6 and YUV/7 memory models)
RedMaskSize         db      ?           ; size of direct color red mask in bits
RedFieldPosition    db      ?           ; bit position of LSB of red mask
GreenMaskSize       db      ?           ; size of direct color green mask in bits
GreenFieldPosition  db      ?           ; bit position of LSB of green mask
BlueMaskSize        db      ?           ; size of direct color blue mask in bits
BlueFieldPosition   db      ?           ; bit position of LSB of blue mask
RsvdMaskSize        db      ?           ; size of direct color reserved mask in bits
DirectColorModeInfo db      ?           ; Direct Color mode attributes
; Mandatory information for VBE 2.0 and above
PhysBasePtr         dd      ?           ; physical address for flat memory frame buffer
                    dd      0           ; reserved - always set to 0
                    dw      0           ; reserved - always set to 0
; Mandatory information for VBE 3.0 and above
LinBytesPerScanLine dw      ?           ; bytes per scan line for linear modes
BnkNumOfImgPages    db      ?           ; number of images for banked modes
LinNumOfImgPages    db      ?           ; number of images for linear modes
LinRedMaskSize      db      ?           ; size of direct color red mask (linear modes)
LinRedFieldPos      db      ?           ; bit position of lsb of red mask (linear modes)
LinGreenMaskSize    db      ?           ; size of direct color green mask (linear modes)
LinGreenFieldPos    db      ?           ; bit position of lsb of green mask (linear modes)
LinBlueMaskSize     db      ?           ; size of direct color blue mask (linear modes)
LinBlueFieldPos     db      ?           ; bit position of lsb of blue mask (linear modes)
LinRsvdMaskSize     db      ?           ; size of direct color reserved mask (linear modes)
LinRsvdFieldPos     db      ?           ; bit position of lsb of reserved mask (linear modes)
MaxPixelClock       dd      ?           ; maximum pixel clock (in Hz) for graphics mode
Reserved            db      256-($-ModeInfoBlock) dup(?)
vmodeinfo_size=$-ModeInfoBlock


;****************************************************************************;



                end
