
include defines.inc
include macro.inc


;****************************************************************************;

.CODE

;----------------------------------------------------------------------------;

CODE_Begin:
                lea     edx, TxtFatalError
QEmuWithMessage:
                push    edx
                call    ResourceRestore
                pop     esi
                PSTR
                jmp     EXIT_Emulator
QEmuWithMsgAndCode:
                push    eax edx
                call    ResourceRestore
                pop     esi
                PSTR
                pop     eax
                PHEXD   eax
                jmp     EXIT_Emulator
QuitEmulator:
                call    ResourceRestore
                jmp     EXIT_Emulator

QEmuWithErrorCode:
                push    eax
                call    ResourceRestore
                PSTR    TxtDPMIErrCode
                pop     eax
                PHEXD   eax
                PENT
                call    Display_ZXState
EXIT_Emulator:
                mov     ax, 04C00h
                int     21h                     ; DOS/DPMI - TERMINATE
ABORT_ThereIsNoDPMI:
                PSTR    TxtDPMIAbsent
                jmp     EXIT_Emulator

;----------------------------------------------------------------------------;

; ---------------------------------------
; -=* S-T-A-R-T-I-N-G---V-Z-X---!-!-! *=-
; ---------------------------------------
Start_VirtualZX:
                mov     ax, 00400h              ; DPMI - GET VERSION
                int     31h
                jc      ABORT_ThereIsNoDPMI
                test    bl, 001h
                jz      ABORT_ThereIsNoDPMI
                cmp     cl, 3
                jc      ABORT_ThereIsNoDPMI
                mov     DOSPICState, dx
                mov     VirtualPICState, dx
                mov     DPMIVersion, ax
                or      InitResources, IR_DPMI

                mov     sel_Data, ds
                xor     eax, eax
                mov     sel_PSP, es             ; save PSP selector
                mov     InitResources, eax

                mov     ebx, datados_size / 010h
                xor     edx, edx
                mov     eax, 00100h
                int     31h
                jnc     Svzx_5
                ; not enough memory
                PSTR    TxtNotEnoughMem
                jmp     EXIT_Emulator
Svzx_5:
                mov     rmbase_DataDOS, ax
                and     eax, 0FFFFh
                shl     eax, 4
                mov     pmoffs_DataDOS, eax
                mov     sel_DataDOS, dx
                or      InitResources, IR_DOSMEM

; -----------------------------------------------
; -=* I-N-I-T-I-A-L-I-Z-E---R-E-S-O-U-R-C-E-S *=-
; -----------------------------------------------
InitializeAll:
                ; print copyright string
                PSTR    TxtCopyright

                call    Paths_Locate

                call    CFG_Load

                call    Cmd_Scan

                call    INI_Load

                call    Font_Load

                xor     al, al
                call    VGA_InitMode

                call    Kbd_Init

                call    INTs_Init

                call    TMR_Init

                call    VGA_DetectVR

                call    Mouse_Init

                call    SB_Init

                ;------------------------------------
                lea     ebx, ZXCore
                call    Synchro_Init

                lea     edx, ZX_Memory
                add     edx, 000003FFFh
                and     edx, 0FFFFC000h
                lea     esi, [ebx].Banks2AlignTable
                mov     ecx, banks2align_size / 4
Iall_1:
                mov     [esi], edx
                add     edx, 16384
                add     esi, 4
                loopd   Iall_1
                mov     eax, [ebx].RomBank_Menu128k
                mov     [ebx].RomPrimary, eax
                mov     eax, [ebx].RomBank_Monitor
                mov     [ebx].RomSecondary, eax

                mov     eax, [ebx].BlankBankAdr
                SRW16K0 eax
                SRW16K1 eax
                SRW16K2 eax
                SRW16K3 eax

                call    ROMs_Load

                ; reset ZX Spectrum or load snapshot
                cmp     CFG_ZXStartupMode, 0
                jnz     Iall_2
                call    Std48k_Init
                jmp     Iall_6
Iall_2:
                cmp     CFG_ZXStartupMode, 1
                jnz     Iall_3
                call    Ext48k_Init
                jmp     Iall_6
Iall_3:
                cmp     CFG_ZXStartupMode, 2
                jnz     Iall_4
                call    Std128k_Init
                jmp     Iall_6
Iall_4:
                cmp     CFG_ZXStartupMode, 3
                jnz     Iall_5
                call    Ext128k_Init
                jmp     Iall_6
Iall_5:
                call    Std128k_Init
Iall_6:
                xor     al, al
                call    LoadSaveSnapshot

                ; display debug info and wait for keypress
                PENT
                call    Display_VZXParameters
                PSTR    TxtPressAnyKey
                call    Kbd_WaitKeypress

                ; starting...
                xor     eax, eax
                mov     SyncEventsCnr, eax
                jmp     ZXCommandMonitor

;----------------------------------------------------------------------------;

Sync_ZX:
                mov     eax, Z80SynchroAdd
                add     Z80SynchroCount, eax
                inc     SyncEventsCnr

                test    InitResources, IR_DMA
                jz      Szx_5
                ; move current playing position pointer
                mov     eax, SBBlockSize
                add     eax, SBPlayingBlockOffs
                cmp     eax, SBTotalBufferEnd
                jc      Szx_4
                ; restarting DMA channel...
                push    ebx ecx edx
                test    SBMode, 010h
                jnz     Szx_0
                mov     dl, SBMode
                call    SB_StopDMA
Szx_0:
                mov     ebx, phys_DataDMA
                mov     ecx, CFG_SBBufferSize
                mov     al, CFG_SBDMAChannel
                mov     ah, 048h
                call    DMA_Start
                movzx   ecx, SBStartCmdString
                jecxz   Szx_2
                lea     ebx, [SBStartCmdString + 1]
Szx_1:
                mov     al, [ebx]
                call    SB_WriteByte
                inc     ebx
                loopd   Szx_1
Szx_2:
                pop     edx ecx ebx
Szx_3:
                xor     eax, eax
                xchg    eax, SoundSamplesCnr
                mov     LastSoundSamples, eax
                mov     eax, SBTotalBufferOffs
Szx_4:
                mov     SBPlayingBlockOffs, eax
Szx_5:

                ret

;----------------------------------------------------------------------------;

GetSet_VectorHandler:
                ;-------------------------------------------------------------------
                ; Get/Set Vector Handler
                ;-------------------------------------------------------------------
                ;  IN: AL - interrupt/exception/fault number
                ;      AH: D0...D3 - subfunction:
                ;             000h, 001h - get interrupt handler (in ECX:EDX);
                ;             002h - set interrupt handler (in EDX);
                ;             003h - set interrupt handler (in ECX:EDX);
                ;             004h, 005h - get exception handler (in ECX:EDX);
                ;             006h - set exception handler (in EDX);
                ;             007h - set exception handler (in ECX:EDX);
                ;             008h, 009h - get hardware interrupt handler (in ECX:EDX);
                ;             00Ah - set hardware interrupt handler (in EDX);
                ;             00Bh - set hardware interrupt handler (in ECX:EDX);
                ;          D7=1 - disable quit on error
                ;      (CX):EDX - address of new interrupt/exception/fault handler;
                ;-------------------------------------------------------------------
                ; OUT: CX:EDX - address of current interrupt/exception/fault
                ;                handler for subfunctions 0, 1, 4 or 5.
                ;-------------------------------------------------------------------
                push    ds
                mov     ds, cs:sel_Data
                or      CriticalRuntime, CR_GETSETVECTOR
                push    eax ebx esi edi ebp
                mov     ebx, eax
                and     bh, 00Fh
                cmp     bh, 8
                jnc     Gsvh_7
                cmp     bh, 4
                jnc     Gsvh_3
Gsvh_0:
                ; 0...3 - operating with interrupt handlers
                cmp     bh, 2
                jnc     Gsvh_1
                ; 0, 1 - get address
                mov     eax, 00204h
                int     31h
                jmp     Gsvh_9
Gsvh_1:
                ; 2, 3 - set address
                push    ecx
                cmp     bh, 3
                jz      Gsvh_2
                mov     ecx, cs
Gsvh_2:
                mov     ebp, ecx
                mov     eax, 00205h
                int     31h
                pop     ecx
                jc      Gsvh_9
                ; verify if vector installed or not
                push    ebp edx
                mov     eax, 0204h
                int     31h
                pop     edi esi
                cmp     edi, edx
                jnz     Gsvh_ErrB
                cmp     si, cx
                jnz     Gsvh_ErrB
                jmp     Gsvh_9
Gsvh_3:
                ; 4...7 - operating with exception/fault handlers
                mov     esi, 00202h ; DPMI v0.9
                mov     edi, 00203h
                cmp     DPMIVersionHigh, 1
                jc      Gsvh_4
                ; 4, 5 - get address
                mov     esi, 00210h ; DPMI v1.0+
                mov     edi, 00212h
Gsvh_4:
                cmp     bh, 6
                jnc     Gsvh_5
                ; get address
                mov     eax, esi
                int     31h
                jmp     Gsvh_9
Gsvh_5:
                ; 6, 7 - set address
                push    ecx
                cmp     bh, 7
                jz      Gsvh_6
                mov     ecx, cs
Gsvh_6:
                mov     ebp, ecx
                mov     eax, edi
                int     31h
                pop     ecx
                jc      Gsvh_9
                ; verify if vector installed or not
                push    ebp edx
                mov     eax, esi
                int     31h
                pop     edi esi
                cmp     edi, edx
                jnz     Gsvh_ErrB
                cmp     si, cx
                jnz     Gsvh_ErrB
                jmp     Gsvh_9
Gsvh_7:
                ; 8...11 - operating with hardware interrupt handlers
                and     bl, 00Fh
;                movzx   esi, bl
                mov     al, VPICStateMaster
                add     al, bl
                cmp     bl, 008h
                jc      Gsvh_8
                mov     al, bl
                and     al, 007h
                add     al, VPICStateSlave
Gsvh_8:
                mov     bl, al
                sub     bh, 8
                jmp     Gsvh_0
Gsvh_9:
                jnc     Gsvh_F
Gsvh_ErrA:
                ; display a DPMI error
                test    bh, 080h
                jnz     Gsvh_F
                and     CriticalRuntime, not(CR_GETSETVECTOR)
                lea     edx, TxtGSVHErrorA
                cmp     ax, 08021h
                jz      QEmuWithMessage
                lea     edx, TxtGSVHErrorB
                cmp     ax, 08022h
                jz      QEmuWithMessage
                lea     edx, TxtGSVHErrorC
                cmp     ax, 08010h
                jz      QEmuWithMessage
                jmp     QEmuWithErrorCode
Gsvh_ErrB:
                ; display vector handler not installed
                test    bh, 080h
                jnz     Gsvh_F
                and     CriticalRuntime, not(CR_GETSETVECTOR)
                lea     edx, TxtGSVHErrorD
                jmp     QEmuWithMessage
Gsvh_F:
                setc    al
                and     CriticalRuntime, not(CR_GETSETVECTOR)
                ror     al, 1
                pop     ebp edi esi ebx eax
                pop     ds
                ret

;----------------------------------------------------------------------------;
ReadWriteFile:
                ;-------------------------------------
                ; FUNCTION: Read/Write File
                ;-------------------------------------
                ;       IN: DS:EDX - filename
                ;           DS:EDI - data buffer
                ;           AL - 0=read, 1=write
                ;           ECX - file size
                ;-------------------------------------
                ;      OUT: CF - read/write file error
                ;           NC - read/write ok
                ;-------------------------------------
                pushad
                ; open file
                and     al, 001h
                xor     ebx, ebx
                ; simple check for filename validity
                cmp     byte ptr [edx], " "
                lea     edx, [edx + 1]
                jz      Rwf_B
                test    byte ptr [edx], 0FFh
                lea     edx, [edx - 1]
                jz      Rwf_B
                call    File_Open
                lea     esi, TxtErrorOpenF
                jc      Rwf_9
                push    edx
                mov     edx, edi
                test    al, al
                jnz     Rwf_2
                call    File_Read
                jmp     Rwf_5
Rwf_2:
                call    File_Write
Rwf_5:
                pop     edx
                lea     esi, TxtErrorRdWrF
                jc      Rwf_9
Rwf_8:
                ; close file
                call    File_Close
                xor     ebx, ebx
                lea     esi, TxtErrorCloseF
                jnc     Rwf_C
Rwf_9:
                ; display an error message
                test    InitResources, (IR_SCREEN1 + IR_SCREEN2)
                jz      Rwf_A
                ; if valid (text) mode not initialized, initialize it
                mov     eax, 3
                int     10h
Rwf_A:
                PSTR                            ; error description
                mov     esi, edx                ; filename
                PSTR
                PENT
                test    ebx, ebx
                jnz     Rwf_8
Rwf_B:
                stc
Rwf_C:
                popad
                ret

;----------------------------------------------------------------------------;

; ---------------------------------------
; -=* D-E-T-E-C-T---L-O-C-A-T-I-O-N-S *=-
; ---------------------------------------
Paths_Locate:
                pushad
                push    es

                mov     es, sel_PSP
                mov     es, es:[02Ch]           ; selector of DOS variables environment

                ; let's find full path to VZX
                xor     esi, esi
                mov     ecx, 0FFFFh
Dpaths_1:
                cmp     word ptr es:[esi], 0
                jz      Dpaths_2
                inc     esi
                loop    Dpaths_1
                jmp     Dpaths_8                ; terminate if end of environment cannot be located
Dpaths_2:
                ; get and save it
                lea     esi, [esi + 4]
                cmp     esi, 4
                jc      Dpaths_8                ; terminate if no any actual string present
                xor     ecx, ecx
Dpaths_3:
                test    byte ptr es:[esi], -1
                jz      Dpaths_4
                inc     esi
                inc     cx
                jz      Dpaths_8
                jmp     Dpaths_3
Dpaths_4:
                jcxz    Dpaths_7
                cmp     byte ptr es:[esi], "\"
                jz      Dpaths_5
                dec     esi
                dec     cx
                jz      Dpaths_7
                jmp     Dpaths_4
Dpaths_5:
                sub     esi, ecx
                lea     edi, VZXSystemDir
Dpaths_6:
                mov     al, es:[esi]
                mov     [edi], al
                inc     esi
                inc     edi
                loop    Dpaths_6
Dpaths_7:
                ; get and save current path
                lea     edx, DOSCurrentDir
                call    File_GetPath
;                ; - try to convert to canonical pathname with LFN
;                mov     al, 8
;                call    File_OperatePathName
                ; set VZX path
                lea     edx, VZXSystemDir
                ; - try to convert to canonical pathname with LFN
                mov     al, 8
                call    File_OperatePathName
                call    File_SetPath

                or      InitResources, IR_VZXDIR
                PSTR    TxtOldCurDir
                PSTR    DOSCurrentDir
                PENT
                PSTR    TxtVzxSysDir
                PSTR    VZXSystemDir
                PENT
Dpaths_8:
                pop     es
                popad
                ret

;----------------------------------------------------------------------------;

; -----------------------------------------
; -=* S-C-A-N---C-O-M-M-A-N-D---L-I-N-E *=-
; -----------------------------------------
Cmd_Scan:
                pushad

                mov     es, sel_PSP
                mov     edi, 080h
                lea     esi, TempBuffer_A
                movzx   cx, es:[edi]
                xor     ax, ax
                inc     di
Icline_1:
                mov     al, es:[edi]
                inc     di
                test    al, al
                jz      Icline_6
                cmp     al, 00Dh
                jz      Icline_6
                cmp     al, " "
                jz      Icline_2
                jnc     Icline_3
                PSTR    TxtErrorComLine
                jmp     Icline_6
Icline_2:
                test    ah, ah
                jz      Icline_5
                cmp     ch, "."
                jnz     Icline_3
                dec     dh
                jnz     Icline_3
                xor     ch, ch
                jmp     Icline_6
Icline_3:
                mov     [esi], al
                cmp     al, "."
                jnz     Icline_4
                mov     dh, 3
                mov     ch, al
Icline_4:
                inc     esi
                inc     ah
Icline_5:
                dec     cl
                jnz     Icline_1
                xor     al, al
Icline_6:
                test    ah, ah
                jz      Icline_8
                lea     esi, TempBuffer_A
                push    esi edi
                lea     edi, SnapshotToLoad
Icline_7:
                mov     dl, [esi]
                mov     [edi], dl
                inc     esi
                inc     edi
                dec     ah
                jnz     Icline_7
                mov     byte ptr [edi], 0
                pop     edi esi
Icline_8:
                test    al, al
                jz      Icline_9
                cmp     al, 00Dh
                jnz     Icline_1
Icline_9:
                cmp     byte ptr SnapshotToLoad, 0
                jz      Icline_A
                lea     edx, SnapshotToLoad
                mov     al, 8
                call    File_OperatePathName
                PSTR    TxtSnapshot
                PSTR    SnapshotToLoad
                PENT
Icline_A:
                popad
                ret

;----------------------------------------------------------------------------;

; -------------------------------------------------------------------
; -=* I-N-I-T-I-A-L-I-Z-E---I-N-T-E-R-R-U-P-T---S-U-B-S-Y-S-T-E-M *=-
; -------------------------------------------------------------------
INTs_Init:
                pushad

                ; read PICs mask
                in      al, 021h                ; blank read
                in      al, 021h                ; actual read
                mov     DOSIntMaskA, al
                in      al, 0A1h                ; blank read
                in      al, 0A1h                ; actual read
                mov     DOSIntMaskB, al
                mov     ax, DOSIntMasks
                and     al, 0FEh
                mov     VZXIntMasks, ax
                or      InitResources, IR_PIC

                ; try to reprogram PIC's base vectors
                cmp     DOSPICState, 00870h
                jnz     Iints_2
                mov     ax, 05070h
                mov     VirtualPICState, ax
                mov     ax, VZXIntMasks
                call    PICs_Init

                mov     ax, 00400h              ; DPMI - GET VERSION
                int     31h
                mov     ax, DOSPICState
                jc      Iints_1
                cmp     dx, VirtualPICState
                mov     ax, dx
                jz      Iints_2
Iints_1:
                mov     VirtualPICState, ax
                mov     ax, VZXIntMasks
                call    PICs_Init
Iints_2:
                ; save old interrupt handlers
                mov     ecx, 010h
                lea     esi, InterruptsOld
Iints_3:
                cmp     dword ptr [esi], 0
                jnz     Iints_4
                mov     eax, 00810h             ; get old interrupt handler address
                sub     al, cl
                push    ecx
                call    GetSet_VectorHandler
                mov     [esi], edx
                mov     [esi + 4], cx
                pop     ecx
                jnc     Iints_4
                mov     [esi + 4], word ptr 0
Iints_4:
                lea     esi, [esi + 6]
                loopd   Iints_3

                ; set new interrupt handlers
                mov     ecx, 010h
                lea     esi, InterruptsList
Iints_6:
                mov     eax, 00B10h
                sub     al, cl
                mov     edx, [esi]
                test    edx, edx
                jz      Iints_7
                push    ecx
                mov     ecx, cs
                call    GetSet_VectorHandler
                pop     ecx
Iints_7:
                lea     esi, [esi + 4]
                loopd   Iints_6

                popad
                ret

;----------------------------------------------------------------------------;

; -------------------------------
; -=* D-E-T-E-C-T---M-O-U-S-E *=-
; -------------------------------
Mouse_Init:
                pushad
                PSTR    TxtInitMouse
                ; enable mouse driver (if exists)
                mov     ax, 020h
                INTX    33h
                ; reset mouse software
                mov     ax, 021h
                INTX    33h
                cmp     ax, 0FFFFh
                jz      Imouse_1
                ; reset mouse software and hardware
                xor     eax, eax
                INTX    33h
                cmp     ax, 0FFFFh
                jz      Imouse_1
                PSTR    TxtNoMouseDrv
                xor     al, al
                jmp     Imouse_3
Imouse_1:
                mov     ax, 024h
                INTX    33h
                test    cl, cl
                jz      Imouse_2
                ; install mouse handler
                mov     MouseInterrupt, cl
                mov     ax, 0FFFEh
                rol     ax, cl
                and     ax, VZXIntMasks
                mov     VZXIntMasks, ax
                call    PICs_Init
                mov     al, cl
                mov     ah, 00Ah
                lea     edx, INTMouse
                call    GetSet_VectorHandler
Imouse_2:
;                mov     eax, 7
;                xor     ecx, ecx
;                mov     edx, 003FFh
;                int     33h
;                inc     al
;                int     33h
;                mov     ch, 1
;                mov     eax, ecx
;                mov     edx, eax
;                mov     eax, 4
;                int     33h
                mov     al, 0DFh
Imouse_3:
                mov     MousePresense, al
                PENT
                popad
                ret

;----------------------------------------------------------------------------;

; -----------------------------------------
; -=* L-O-A-D---M-E-S-S-A-G-E---F-O-N-T *=-
; -----------------------------------------
Font_Load:
                pushad

                PSTR    TxtLoadFont
                lea     edi, MSGSystemFont
                lea     edx, FNTFileName
                mov     ecx, 00800h
                xor     al, al
                call    ReadWriteFile
                jc      Fload_1
                PCHR    "."
Fload_1:
                PENT
                popad
                ret

;----------------------------------------------------------------------------;

; -----------------------------------
; -=* L-O-A-D---R-O-M---F-I-L-E-S *=-
; -----------------------------------
ROMs_Load:
                pushad

                PSTR    TxtLoadROM
                mov     edi, [ebx].RomBank_Menu128k
                lea     edx, ROM0FileName
                mov     ecx, 16384
                xor     al, al
                call    ReadWriteFile
                jc      Rload_1
                PCHR    "."
Rload_1:
                mov     edi, [ebx].RomBank_Basic48k
                lea     edx, ROM1FileName
                mov     ecx, 16384
                xor     al, al
                call    ReadWriteFile
                jc      Rload_2
                PCHR    "."
Rload_2:
                mov     edi, [ebx].RomBank_Monitor
                lea     edx, ROM2FileName
                mov     ecx, 16384
                xor     al, al
                call    ReadWriteFile
                jc      Rload_3
                PCHR    "."
Rload_3:
                mov     edi, [ebx].RomBank_TRDOS
                lea     edx, ROM3FileName
                mov     ecx, 16384
                xor     al, al
                call    ReadWriteFile
                jc      Rload_4
                PCHR    "."
Rload_4:
                PENT
                popad
                ret

;----------------------------------------------------------------------------;

; ---------------------------------------------------
; -=* I-N-I-T---S-Y-N-C-H-R-O---S-U-B-S-Y-S-T-E-M *=-
; ---------------------------------------------------
Synchro_Init:
                pushad

                PSTR    TxtInitSyncData

                ; calculating tacts per frame
                cmp     CFG_ULATactsInFrame, 0
                jnz     Isyncdata_01
                mov     eax, CFG_ULAResolutionX
                mov     ecx, CFG_ULAResolutionY
                mul     ecx
                shl     eax, 2
                mov     ecx, CFG_Z80ClockMul
                mul     ecx
                mov     CFG_ULATactsInFrame, eax
Isyncdata_01:
                ; initializing Z80 frequency data
                xor     edx, edx
                mov     eax, CFG_Z80ClockFreq
                shld    edx, eax, FREQ_RAISE
                shl     eax, FREQ_RAISE
                mov     ecx, CFG_Z80Speed
                test    ecx, ecx
                jnz     Isyncdata_02
                cmp     VRetraceFreqRaised, 0
                jz      Isyncdata_03
                mov     ecx, CFG_ULATactsInFrame
                mov     eax, VRetraceFreqRaised
                mul     ecx
                jmp     Isyncdata_03
Isyncdata_02:
                mov     edi, 1000
                div     edi
                xor     edx, edx
                mul     ecx
Isyncdata_03:
                mov     [ebx].Z80FreqRaisedL, eax
                mov     [ebx].Z80FreqRaisedH, edx
                shrd    eax, edx, FREQ_RAISE
                mov     [ebx].Z80Frequency, eax

                ; calculating synchro frequency data
                mov     eax, SynchroFreqRaised
                test    InitResources, IR_DMA
                jz      Isyncdata_04
                mov     eax, SBSampleRateRaised
                xor     edx, edx
                mov     ecx, SBBlockSize
                div     ecx
                mov     SynchroFreqRaised, eax
Isyncdata_04:
                mov     ecx, eax
                mov     eax, TMRRetracePeriod
                cmp     CFG_SynchroFreq, 0
                jz      Isyncdata_05
                mov     edx, TMRFrequencyHigh
                mov     eax, TMRFrequencyLow
                shld    edx, eax, FREQ_RAISE
                shl     eax, FREQ_RAISE
                div     ecx
Isyncdata_05:
                mov     TMRPeriodSyncLow, eax
                mov     ecx, eax
                mov     edx, TMRFrequencyHigh
                mov     eax, TMRFrequencyLow
                shld    edx, eax, FREQ_RAISE
                shl     eax, FREQ_RAISE
                div     ecx
                mov     SynchroFreqRaised, eax
                push    eax

                mov     eax, [ebx].Z80FreqRaisedL
                mov     edx, [ebx].Z80FreqRaisedH
                mov     ebp, eax
                mov     ecx, edx
                lea     esi, [ebx].Z80DivideTable
                lea     edi, [esi + z80divtbl_size - 4]
                shrd    ebp, ecx, FREQ_RAISE
                shr     ecx, FREQ_RAISE
Isyncdata_06:
                div     dword ptr [esp]

                mov     [esi], eax
                mov     dword ptr [esi + 4], 8
                sub     ebp, eax
                lea     esi, [esi + 8]
                sbb     ecx, 0
                jc      Isyncdata_07

                mov     eax, edx
                xor     edx, edx
                add     eax, [ebx].Z80FreqRaisedL
                adc     edx, [ebx].Z80FreqRaisedH
                cmp     esi, edi
                jc      Isyncdata_06
Isyncdata_07:
                pop     eax
                lea     eax, [ebx].Z80DivideTable + 8
                sub     eax, esi
                mov     [esi - 4], eax

                ; initializing Sound Generator frequency data
                mov     SoundGenTactsCnr, 0
                mov     eax, [ebx].Z80FreqRaisedL
                mov     edx, [ebx].Z80FreqRaisedH
                shld    edx, eax, 14
                shl     eax, 14
                mov     ecx, CFG_AYClockFreq
;                shr     ecx, 3
;                mov     AYGenClockFreq, ecx
                test    ecx, ecx
                jnz     Isyncdata_08
                mov     ecx, CFG_Z80ClockFreq
                shr     ecx, 1
                sub     ecx, 1
                adc     ecx, 1
                mov     CFG_AYClockFreq, ecx
;                shr     ecx, 3
Isyncdata_08:
                shr     ecx, 5;2
                sub     ecx, 1
                adc     ecx, 1
                div     ecx
                mov     ecx, CFG_AYGenClkDivider
                mul     ecx
                mov     SoundGenTactsConst, eax
                mov     eax, CFG_AYClockFreq
                shr     eax, 3
                xor     edx, edx
                div     ecx
                mov     AYGenClockFreq, eax

                ; initializing Sample Updater frequency data
                xor     eax, eax
                mov     SamplerTactsCnr, eax
                mov     SamplerTactsConst, eax

                mov     eax, [ebx].Z80FreqRaisedL
                mov     edx, [ebx].Z80FreqRaisedH
                shld    edx, eax, 14
                shl     eax, 14
                mov     ecx, SBSampleRateRaised
                shr     ecx, FREQ_RAISE+2
                adc     ecx, 0
                jz      Isyncdata_09
                mov     SamplerTactsCnr, ecx
                div     ecx
                mov     SamplerTactsConst, eax
Isyncdata_09:
                mov     eax, [ebx].Z80FreqRaisedL
                mov     edx, [ebx].Z80FreqRaisedH
                mov     ecx, AYGenClockFreq
                shl     ecx, FREQ_RAISE
                div     ecx
                cmp     eax, CFG_Z80TactsAsOne
                ja      Isyncdata_10
                mov     CFG_Z80TactsAsOne, eax
Isyncdata_10:
                mov     [ebx].Z80MinTactsAs1, eax
                mov     eax, CFG_Z80ClockMul
                mov     Z80SynchroAdd, eax
                popad
                ret

;----------------------------------------------------------------------------;

; -----------------------------------------------------
; -=* L-O-A-D---C-O-N-F-I-G-U-R-A-T-I-O-N---F-I-L-E *=-
; -----------------------------------------------------
CFG_Load:
                pushad

                push    ds
                pop     es
                ; read configuration file
                lea     edi, TempBuffer_A
                lea     edx, VZXConfigFilename
                mov     ecx, CfgTotal_size
                xor     al, al
                call    ReadWriteFile
                jnc     Lcfg_010
                ; if configuration file not present, create it
                mov     al, FILE_CREATE_READWRITE
                call    File_Open
                jc      Lcfg_070
                call    File_Close
                jc      Lcfg_070
                mov     ecx, CfgTotal_size
                lea     edi, CFG_Total
                mov     al, FILE_WRITEONLY
                call    ReadWriteFile
                jmp     Lcfg_070
Lcfg_010:
                ; verify configuration file signature
                lea     ebx, CFG_Signature
                mov     esi, 4
Lcfg_020:
                mov     al, [ebx]
                test    al, al
                jz      Lcfg_030
                inc     ebx
                cmp     al, [edi + esi]
                jz      Lcfg_030
                or      al, 020h
                cmp     al, [edi + esi]
                jnz     Lcfg_040
Lcfg_030:
                inc     esi
                mov     eax, [edi]
                sub     eax, esi
                cmp     eax, 4
                jc      Lcfg_040
                mov     eax, [edi + esi]
                cmp     eax, [edi]
                jc      Lcfg_050
Lcfg_040:
                ; inconsistent or invalid configuration file
                PSTR    TxtInvalidCfg
                stc
                jmp     Lcfg_070
Lcfg_050:
                add     esi, eax
                cmp     esi, [edi]
                jc      Lcfg_020
Lcfg_060:
                ; configuration is OK - copy
                lea     esi, TempBuffer_A
                lea     edi, CFG_Total
                mov     ecx, CfgTotal_size
                cld
                rep     movs byte ptr es:[edi], ds:[esi]

                clc
Lcfg_070:
                popad
                ret

;----------------------------------------------------------------------------;

Display_VZXParameters:
                pushad

                PSTR    TxtZ80Freq
                mov     edx, [ebx].Z80FreqRaisedH
                mov     eax, [ebx].Z80FreqRaisedL
                mov     esi, 1 shl FREQ_RAISE
                div     esi
                PDECD   eax
                mov     eax, edx
                xor     edx, edx
                mov     ecx, 10000
                mul     ecx
                div     esi
                PCHR    "."
                PDECD   eax, 4
                PENT

                PSTR    TxtVRFreq
                xor     edx, edx
                mov     eax, VRetraceFreqRaised
                mov     esi, 1 shl FREQ_RAISE
                div     esi
                PDECD   eax
                mov     eax, edx
                xor     edx, edx
                mov     ecx, 10000
                mul     ecx
                div     esi
                PCHR    "."
                PDECD   eax, 4
                PENT

                PSTR    TxtMainFreq
                xor     edx, edx
                mov     eax, SynchroFreqRaised
                mov     esi, 1 shl FREQ_RAISE
                div     esi
                PDECD   eax
                mov     eax, edx
                xor     edx, edx
                mov     ecx, 10000
                mul     ecx
                div     esi
                PCHR    "."
                PDECD   eax, 4
                PENT

                popad
                ret

;----------------------------------------------------------------------------;

Display_ZXState:
                PSTR    TxtZXRegsAF
                PHEXW   [ebx].rAF
                PSTR    TxtZXRegsAFx
                PHEXW   [ebx].rAFx
                PENT
                PSTR    TxtZXRegsBC
                PHEXW   [ebx].rBC
                PSTR    TxtZXRegsBCx
                PHEXW   [ebx].rBCx
                PENT
                PSTR    TxtZXRegsDE
                PHEXW   [ebx].rDE
                PSTR    TxtZXRegsDEx
                PHEXW   [ebx].rDEx
                PENT
                PSTR    TxtZXRegsHL
                PHEXW   [ebx].rHL
                PSTR    TxtZXRegsHLx
                PHEXW   [ebx].rHLx
                PENT
                PSTR    TxtZXRegsIX
                PHEXW   [ebx].rIX
                PSTR    TxtZXRegsSP
                PHEXW   [ebx].rSP
                PENT
                PSTR    TxtZXRegsIY
                PHEXW   [ebx].rIY
                PSTR    TxtZXRegsPC
                PHEXW   [ebx].rPC
                PENT
                PSTR    TxtZXRegsIR
                PHEXW   [ebx].rIR
                PSTR    TxtZXRegsMP
                PHEXW   [ebx].rMP
                PENT
                PSTR    TxtZXROM0
                PHEXD   [ebx].RomBank0Adr
                PENT
                PSTR    TxtZXROM1
                PHEXD   [ebx].RomBank1Adr
                PENT
                PSTR    TxtZXROM2
                PHEXD   [ebx].RomBank2Adr
                PENT
                PSTR    TxtZXROM3
                PHEXD   [ebx].RomBank3Adr
                PENT
                PSTR    TxtZXCurrentROM
                PHEXD   Read8kBank0Adr
                PENT
                ret
;----------------------------------------------------------------------------;
INTTempTimer:
                BEGININTM

                dec     TimerFunctionA
;                INTX    08h

                ENDOFINT
;----------------------------------------------------------------------------;
INTKeyboard:
                BEGININTM

                call    Kbd_Process

                ENDOFINT
;----------------------------------------------------------------------------;
INTNullTimer:
                BEGININTM

;                INTX    08h

                ENDOFINT
;----------------------------------------------------------------------------;
INTCascade:
                BEGININTM

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQ2:
                push    esi
                lea     esi, IRQTestBuffer
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQ2:
                ENDOFINT

INTTest_Common:
                add     word ptr [esi], 1
                sbb     word ptr [esi], 0
                cmp     IRQTestID, 1
                jz      SB_AcknowledgeIRQ
                ret
;----------------------------------------------------------------------------;
INTCom24:
                BEGININTM

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQ3:
                push    esi
                lea     esi, IRQTestBuffer + 002h
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQ3:
INTEnd4All:
                ENDOFINT
;----------------------------------------------------------------------------;
INTLpt2Sound:
                BEGININTM

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQ5:
                push    esi
                lea     esi, IRQTestBuffer + 004h
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQ5:
                ENDOFINT
;----------------------------------------------------------------------------;
INTSoundBlaster:
                BEGININTS
                ; acknowledge SB interrupt
                call    SB_AcknowledgeIRQ
                ENDOFINT
;----------------------------------------------------------------------------;
INTMouse:
                BEGININTS
                ; call interrupt handler in mouse driver
                movzx   ax, MouseInterrupt
                add     al, VPICStateMaster
                INTX    ax
                ENDOFINT
;----------------------------------------------------------------------------;
INTLpt1Printer:
                BEGININTM

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQ7:
                push    esi
                lea     esi, IRQTestBuffer + 006h
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQ7:
                ENDOFINT
;----------------------------------------------------------------------------;
INTReserved9:
                BEGININTS

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQ9:
                push    esi
                lea     esi, IRQTestBuffer + 008h
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQ9:
                ENDOFINT
;----------------------------------------------------------------------------;
INTVideoGraphicsArray:
                BEGININTS

INTvga_1:
                ENDOFINT
;----------------------------------------------------------------------------;
INTReservedA:
                BEGININTS

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQA:
                push    esi
                lea     esi, IRQTestBuffer + 00Ah
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQA:
                ENDOFINT
;----------------------------------------------------------------------------;
INTReservedB:
                BEGININTS

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQB:
                push    esi
                lea     esi, IRQTestBuffer + 00Ch
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQB:
                ENDOFINT
;----------------------------------------------------------------------------;
INTMousePS2:
                BEGININTS

                ENDOFINT
;----------------------------------------------------------------------------;
INTReservedF:
                BEGININTS

                cmp     IRQTestID, 0
                jz      INTEnd4All
INTTestSB_IRQF:
                push    esi
                lea     esi, IRQTestBuffer + 00Eh
                call    INTTest_Common
                pop     esi
INTTestVGA_IRQF:
                ENDOFINT



;****************************************************************************;

.DATA

sel_Data            dw      0
sel_DataDOS         dw      0
sel_PSP             dw      0
rmbase_DataDMA      dw      0
pmoffs_DataDMA      dd      0
phys_DataDMA        dd      0
bufid_DataDMA       dw      0
rmbase_DataDOS      dw      0
pmoffs_DataDOS      dd      0

DOSCurrentDir       label   byte
                    db      256 dup (0)
VZXCurrentDir       label   byte
                    db      256 dup (0)
VZXSystemDir        label   byte
                    db      256 dup (0)
VZXSnapsDir         label   byte
                    db      256 dup (0)
VZXTapesDir         label   byte
                    db      256 dup (0)
VZXDisksDir         label   byte
                    db      256 dup (0)
VZXMiscADir         label   byte
                    db      256 dup (0)
VZXMiscBDir         label   byte
                    db      256 dup (0)
VZXConfigFilename   label   byte
                    db      "vzx.cfg"
                    db      256-($-VZXConfigFilename) dup (0)

DPMIVersion         label   word
DPMIVersionLow      db      0
DPMIVersionHigh     db      0

DOSTimerCount   dw 0
DOSMemStrategy  dw 0
DOSMemLinkstate dw 0

TimerFunctionA      dd      0


; Frequency and synchro data
SynchroFreqRaised   dd      0
VRetraceFreqRaised  dd      0

DOSPICState         label   word
DPICStateSlave      db      0
DPICStateMaster     db      0
VirtualPICState     label   word
VPICStateSlave      db      0
VPICStateMaster     db      0


InitResources       dd      0           ; Initialized resources bits (see defines.inc, IR_*)
CriticalRuntime     dd      0           ; Critical routines running info (see defines.inc, CR_*)
MousePresense       db      0
MouseInterrupt      db      0

ADCState        db 0

SBFlagsIO       db 0

DOSIntMasks         label   word
DOSIntMaskA         db      0
DOSIntMaskB         db      0
VZXIntMasks         label   word
VZXIntMaskA         db      0BCh
VZXIntMaskB         db      0FFh

ScrollScreenY0  db 0
ScrollScreenY1  db 0
IRQTestID       db 0


;****************************************************************************
;* STANDART MESSAGES ********************************************************
;****************************************************************************

TxtDPMIAbsent       db      "There is no suitable DPMI support.", 00Dh, 00Ah, 0
TxtDPMIErrCode      db      "DPMI error code #", 0
TxtNotEnoughMem     db      "Not enough memory.", 00Dh, 00Ah, 0
TxtByeBye           db      00Dh, 00Ah, "Have a nice day & night !", 00Dh, 00Ah, 0
TxtFatalError       db      "An internal error was occured.", 00Dh, 00Ah, "Program terminated.", 00Dh, 00Ah, 0
TxtVideoError       db      "Could not detect video signal.", 00Dh, 00Ah, 0
TxtGSVHErrorA       db      "*** GSVH: Invalid value (BL not in range 00-1Fh).", 00Dh, 00Ah, 0
TxtGSVHErrorB       db      "*** GSVH: Invalid selector.", 00Dh, 00Ah, 0
TxtGSVHErrorC       db      "*** GSVH: Unrecognized error.", 00Dh, 00Ah, 0
TxtGSVHErrorD       db      "*** GSVH: Interrupt/exception handler not installed.", 00Dh, 00Ah, 0
TxtCopyright        db      00Dh, 00Ah, " -= Virtual ZX Spectrum =- "
                    db      00Dh, 00Ah, "   (c) 1997-2008 by ARTi   ", 00Dh, 00Ah, 00Dh, 00Ah, 0
TxtLoadROM          db      " Loading ROM files.", 0
TxtLoadFont         db      " Loading message font file...", 0
TxtInitSyncData     db      " Initializing synchronization data...", 00Dh, 00Ah, 0
TxtIntsRedirected   db      "Interrupt vectors redirected successfully.", 00Dh, 00Ah, 0
TxtInvalidCfg       db      "VZX configuration file may be corrupted, using default settings.", 00Dh, 00Ah, 0
TxtErrorRdWrF       db      "File access error: ", 0
TxtErrorOpenF       db      "Error opening file: ", 0
TxtErrorCloseF      db      "Warning: Couldn't close file: ", 0
TxtSnapshot         db      " Snapshot file to load: ", 0
TxtOldCurDir        db      " Current DOS directory: ", 0
TxtVzxSysDir        db      " VZX directory: ", 0
TxtErrorComLine     db      "Error in command line.", 00Dh, 00Ah, 0
TxtInitMouse        db      " Initializing mouse...", 0
TxtNoMouseDrv       db      "mouse not found or mouse driver not installed.", 0
TxtCantSynchronize  db      "Cannot synchronize. Sorry :)", 00Dh, 00Ah, 0
TxtPressAnyKey      db      00Dh, 00Ah, "Press any key to continue...", 00Dh, 00Ah, 0

;****************************************************************************
;* DEBUG MESSAGES AND STRINGS ***********************************************
;****************************************************************************

TxtZ80Freq          db      "Z80 Frequency  = ", 0
TxtVRFreq           db      "VR Frequency   = ", 0
TxtMainFreq         db      "Main Frequency = ", 0

TxtZXRegsAF         db      " AF = ", 0
TxtZXRegsAFx        db      ",   AF'= ", 0
TxtZXRegsBC         db      " BC = ", 0
TxtZXRegsBCx        db      ",   BC'= ", 0
TxtZXRegsDE         db      " DE = ", 0
TxtZXRegsDEx        db      ",   DE'= ", 0
TxtZXRegsHL         db      " HL = ", 0
TxtZXRegsHLx        db      ",   HL'= ", 0
TxtZXRegsIX         db      " IX = ", 0
TxtZXRegsSP         db      ",   SP = ", 0
TxtZXRegsIY         db      " IY = ", 0
TxtZXRegsPC         db      ",   PC = ", 0
TxtZXRegsIR         db      " IR = ", 0
TxtZXRegsMP         db      ",   MP = ", 0
TxtZXROM0           db      "ROM0 address: ", 0
TxtZXROM1           db      "ROM1 address: ", 0
TxtZXROM2           db      "ROM2 address: ", 0
TxtZXROM3           db      "ROM3 address: ", 0
TxtZXCurrentROM     db      "Current ROM address: ", 0


InterruptsList      label   dword
                    dd      0               ; Hardware Interrupt 000h
                    dd      0;INTKeyboard     ; * Hardware Interrupt 001h
                    dd      0               ; Hardware Interrupt 002h
                    dd      INTCom24        ; * Hardware Interrupt 003h
                    dd      0               ; Hardware Interrupt 004h (COM1, COM3)
                    dd      INTLpt2Sound    ; * Hardware Interrupt 005h
                    dd      0               ; Hardware Interrupt 006h (Floppy Disk)
                    dd      INTLpt1Printer  ; * Hardware Interrupt 007h
                    dd      0               ; Hardware Interrupt 008h (CMOS RTC)
                    dd      INTReserved9    ; * Hardware Interrupt 009h
                    dd      INTReservedA    ; * Hardware Interrupt 00Ah
                    dd      INTReservedB    ; * Hardware Interrupt 00Bh
                    dd      INTMousePS2     ; * Hardware Interrupt 00Ch
                    dd      0               ; Hardware Interrupt 00Dh (Math Coprocessor)
                    dd      0               ; Hardware Interrupt 00Eh (Hard Disk)
                    dd      INTReservedF    ; * Hardware Interrupt 00Fh

InterruptsOld       label   word
                    dw      16*3 dup (0)

FDCDiskFilesTable   label   dword
                    dd      offset DiskAFileName, offset CFG_FDCDriveAFlags
                    dd      offset DiskBFileName, offset CFG_FDCDriveBFlags
                    dd      offset DiskCFileName, offset CFG_FDCDriveCFlags
                    dd      offset DiskDFileName, offset CFG_FDCDriveDFlags

CFG_Signature       db      "VZXCFG", 0
;****************************************************************************
;* CONFIGURATION FILE AREA **************************************************
;****************************************************************************
CFG_Total           label
                    dd      CfgTotal_size
;-------------------------------------------------------------------
                    db      "V"
CFG_Common          label
                    dd      CfgCommon_size
CFG_SynchroFreq     label   dword
                    dd      1000
CFG_TimerMode       label   byte
                    db      1           ; D0-D1   - Timer Mode:
                                        ;   0=simple read counter twice (four bytes)
                                        ;   1=write latch command then read two bytes of counter
                                        ;   2=write read back command then read two bytes of counter
                                        ;   3=simple read two bytes of counter
                                        ; D2      - Use of TimeStamp Counter :
                                        ;   0=do not use RDTSC - delays and sync via PIT
                                        ;   1=use RDTSC - delays via PIT, sync via RDTSC
                                        ; D7      - CMOS Mode:
                                        ;   0=get seconds from sync (PIT or RDTSC)
                                        ;   1=get seconds from CMOS
CFG_PITChannel      db      0           ; PIT channel
                    dw      0
CfgCommon_size = $ - CFG_Common
;-------------------------------------------------------------------
                    db      "Z"
CFG_Audio           label
                    dd      CfgAudio_size
CFG_AudioProperties label   byte
                    dd      005h            ; Sound Devices properties
                                            ; D0-D1 - 0=single bit---\\
                                            ;         1=timer DAC-----}} PC Speaker
                                            ;         2,3 - reserved-//
                                            ; D2-D3 - 0=16-bit DMA mode-\\
                                            ;         1=8-bit DMA mode---\\
                                            ;                             }} Sound Blaster
                                            ;         2=fast DAC output--//
                                            ;         3=slow DAC output-//
                                            ; D4-D7 - reserved (win32 API)
CFG_SBAddress       label   word
                    dd      0               ; Autodetect IO Port
CFG_SBInterrupt     label   byte
                    dd      0               ; Autodetect interrupt line
CFG_SBDMAChannel    label   byte
                    dd      8               ; Autodetect DMA channel
CFG_SBDMA16Channel  label   byte
                    dd      8               ; Autodetect DMA16 channel
CFG_SBSampleRate    label   dword
                    dd      44100
CFG_SBMasterVolume  label   byte
                    dd      0
CFG_SBBufferSize    label   dword
                    dd      16384
CFG_SpMasterVolume  label   byte
                    dd      0
CFG_FilterSelect    label   byte
                    dd      1               ; D0-D1 - AY filter select, 0=none
                                            ; D2-D3 - Beeper filter select, 0=none
CfgAudio_size = $ - CFG_Audio
;-------------------------------------------------------------------
                    db      "X"
CFG_Video           label
                    dd      CfgVideo_size
CFG_VideoUBVisible  label   dword
                    dd      0               ; 0 = auto size
CFG_VideoLBVisible  label   byte
                    dd      0               ; 0 = auto size
CFG_VideoFrameSkip  label   byte
                    dd      1
CFG_VideoModePre    label   word
                    dd      0               ;       VGA-
CFG_VideoModeMain   label   word
                    dd      00110h          ; D0-D3   - main video mode
                                            ; D4      - 1=force VGA
                                            ; D5      - 1=buffered screen output
                                            ; D6      - 1=black and white pallette
                                            ; D7      - 1=do not use VESA/VBE
                                            ; D8      - 1=interlaced
CFG_VideoXTotalAdd  label   byte
                    dd      0               ;   ,    
                                            ;    [0...15], default=0
CFG_VideoXVisible   label   byte
                    dd      42              ;      
                                            ;  [32...48 step 2], default=48
CFG_VideoXAlignment label   byte
                    dd      0               ;    
                                            ;  [-7...7], default=0
CFG_VideoYTotalAdd  label   word
                    dd      0               ;   ,    
                                            ;  [0...31]
CFG_VideoYVisible   label   dword
                    dd      288             ; -  ,  
                                            ;  [192...320], default=280
CFG_VideoYAlignment label   word
                    dd      0               ;    
                                            ;  [-15...15]
CfgVideo_size = $ - CFG_Video
;-------------------------------------------------------------------
                    db      "C"
CFG_ZXCore          label
                    dd      CfgZXCore_size
CFG_ZXStartupMode   label   dword
                    dd      2
CFG_Z80Speed        label   dword
                    dd      0               ; Auto
CFG_Z80ClockMul     label   dword
                    dd      1               ; Z80 clock mul: 1x-16x
CFG_Z80ClockFreq    label   dword
                    dd      3546900
CFG_Z80TactsAsOne   label   dword
                    dd      100
CFG_AYClockFreq     label   dword
                    dd      0               ; Auto = CFG_Z80ClockFreq / 2
CFG_AYGenClkDivider label   dword
                    dd      4
CFG_ULAInterruptLen label   dword
                    dd      96
CFG_ULAResolutionX  label   dword
                    dd      56
CFG_ULAResolutionY  label   dword
                    dd      320
CFG_ULATactsInFrame label   dword
                    dd      0               ; 0 = auto
CFG_ULAFlashFreq    label   byte
                    dd      12
CFG_ULAUpperBorder  label   byte
                    dd      80
CFG_ULALeftBorder   label   byte
                    dd      17
CFG_FDCDriveAFlags  label   dword
                    dd      0
CFG_FDCDriveBFlags  label   dword
                    dd      0
CFG_FDCDriveCFlags  label   dword
                    dd      0
CFG_FDCDriveDFlags  label   dword
                    dd      0
CfgZXCore_size = $ - CFG_ZXCore
;-------------------------------------------------------------------
                    db      "F"
CFG_Mixer           label
                    dd      CfgMixer_size
CFG_MixBeeperVolume label   dword
                    dd      16368
CFG_MixTapeVolume   label   dword
                    dd      16368
CFG_MixAudioMapA    label   dword
                    dd      007h            ; Sound Devices redirect control
                                            ; D0 - 1=enable Beeper--------\\
                                            ; D1 - 1=enable AY-------------\\
                                            ; D2 - 1=enable Tape Out--------}} PC Speaker
                                            ; D3 - 1=enable General Sound--//
                                            ; D4-D31 - reserved-----------//
CFG_MixAudioMapB    label   dword
                    dd      0               ; same as CFG_MixAudioMapA but for Sound Blaster
CFG_MixAudioMapC    label   dword
                    dd      0
CFG_MixAudioMapD    label   dword
                    dd      0
CFG_MixAudioEnable  label   dword
                    dd      08000000Fh      ; Advanced I/O Control
                                            ; D0 - 1=global Beeper output enable
                                            ; D1 - 1=global AY enable
                                            ; D2 - 1=global Tape output enable
                                            ; D3 - 1=global General Sound enable
                                            ; D4-D30 - reserved
                                            ; D31 - 1=enable tape input from ADC
CFG_MixStereo       label   byte
                    dd      080h            ; D6 - 1=reverse stereo
                                            ; D7 - 1=enable stereo
CFG_MixStereoLeftA  label   byte
                    dd      0               ; Settings for special mixer on left
                                            ; D0 - Beeper enable
                                            ; D1 - AY channel A enable
                                            ; D2 - AY channel B enable
                                            ; D3 - AY channel C enable
                                            ; D4 - Tape enable
                                            ; D5-D31 - reserved
CFG_MixStereoLeftB  label   byte
                    dd      0               ; Settings for special mixer on left (continue)
CFG_MixStereoLeftC  label   byte
                    dd      0               ; Settings for special mixer on left (continue)
CFG_MixStereoLeftD  label   byte
                    dd      0               ; Settings for special mixer on left (continue)
CFG_MixStereoRightA label   byte
                    dd      0               ; Settings for special mixer on right,
                                            ; same as CFG_MixStereoLeftA
CFG_MixStereoRightB label   byte
                    dd      0               ; Settings for special mixer on right (continue)
CFG_MixStereoRightC label   byte
                    dd      0               ; Settings for special mixer on right (continue)
CFG_MixStereoRightD label   byte
                    dd      0               ; Settings for special mixer on right (continue)
CFG_MixStereoSepar  label   byte
                    dd      0FFh
CfgMixer_size = $ - CFG_Mixer
;-------------------------------------------------------------------
                    db      "G"
CFG_ZXDevices       label
                    dd      CfgZXDevs_size
CFG_ZXBaseDevices   label   byte
                    dd      0
CFG_ZXAuxDevices    label   dword
                    DEVDESC 128 dup(<>)
CfgZXDevs_size = $ - CFG_ZXDevices
;-------------------------------------------------------------------
                    db      " "
CFG_Files           label
                    dd      CfgFiles_size
ROM0FileName        label   byte
                    db      "128basic.rom"
                    db      256-($-ROM0FileName) dup(0)
ROM1FileName        label   byte
                    db      "48basic.rom"
                    db      256-($-ROM1FileName) dup(0)
ROM2FileName        label   byte
                    db      0
                    db      256-($-ROM2FileName) dup(0)
ROM3FileName        label   byte
                    db      "trdos.rom"
                    db      256-($-ROM3FileName) dup(0)
INIFileName         label   byte
                    db      "vzx.ini"
                    db      256-($-INIFileName) dup(0)
FNTFileName         label   byte
                    db      "vzx.fnt"
                    db      256-($-FNTFileName) dup(0)
SnapshotToSave      label   byte
                    db      "autosave.sna"
                    db      256-($-SnapshotToSave) dup(0)
SnapshotToLoad      label   byte
                    db      0
                    db      256-($-SnapshotToLoad) dup(0)
DiskAFileName       label   byte
                    db      "disk1.trd"
                    db      256-($-DiskAFileName) dup(0)
DiskBFileName       label   byte
                    db      "disk2.trd"
                    db      256-($-DiskBFileName) dup(0)
DiskCFileName       label   byte
                    db      "disk3.trd"
                    db      256-($-DiskCFileName) dup(0)
DiskDFileName       label   byte
                    db      "disk4.trd"
                    db      256-($-DiskDFileName) dup(0)
CfgFiles_size = $ - CFG_Files
;-------------------------------------------------------------------
CfgTotal_size = $ - CFG_Total


;****************************************************************************
;* STATIC BUFFERS ***********************************************************
;****************************************************************************

; Message font for messaging in graph mode
MSGSystemFont       label   byte
                    db      01000h dup(0)

TempBuffer_A        label   byte
                    db      04000h dup(0)
tmpbufa_size=$-TempBuffer_A
IRQTestBuffer       label   byte
                    dw      010h dup(0)


;****************************************************************************
;* ZX STATIC MEMORY *********************************************************
;****************************************************************************

ZX_Memory           label   byte
                    db      banks2align_size/4*16384 dup(0)
                    db      04000h dup(0)       ; blank for dummy writes (ROM, etc.)
                    db      04000h dup(0)       ; align

;****************************************************************************;

                end     Start_VirtualZX

