

;****************************************************************************;

; (E)DI - Z80 Program Counter
; EBP - Z80 Tacts Counter

;****************************************************************************;

;----------------------------------------------------------------------------
INCR            macro   z80reg
                and     dl, Z80F_C
                inc     z80reg
                lahf
                seto    [ebx].flagPV
                and     ah, Z80F_S + Z80F_Z + Z80F_H
                or      dl, ah
                mov     ah, z80reg
                and     ah, Z80F_5 + Z80F_3
                or      dl, ah
                endm
;----------------------------------------------------------------------------
DECR            macro   z80reg
                and     dl, Z80F_C
                dec     z80reg
                lahf
                seto    [ebx].flagPV
                and     ah, Z80F_S + Z80F_Z + Z80F_H
                or      dl, ah
                mov     ah, z80reg
                and     ah, Z80F_5 + Z80F_3
                or      dl, ah
                or      dl, Z80F_N
                endm
;----------------------------------------------------------------------------
ADSBR           macro   type, z80reg
                ifidni  <type>, <add>
                    add     dh, z80reg
                endif
                ifidni  <type>, <adc>
                    shr     dl, 1
                    adc     dh, z80reg
                endif
                ifidni  <type>, <sub>
                    sub     dh, z80reg
                endif
                ifidni  <type>, <sbc>
                    shr     dl, 1
                    sbb     dh, z80reg
                endif
                lahf
                seto    [ebx].flagPV
                mov     dl, dh
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_H + Z80F_C
                or      dl, ah
                ifidni  <type>, <sub>
                    or      dl, Z80F_N
                else
                    ifidni  <type>, <sbc>
                        or      dl, Z80F_N
                    endif
                endif
                endm
;----------------------------------------------------------------------------
CMPR            macro   z80reg
                mov     dl, z80reg
                cmp     dh, dl
                lahf
                seto    [ebx].flagPV
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_H + Z80F_C
                or      dl, ah
                or      dl, Z80F_N
                endm
;----------------------------------------------------------------------------
LOGR            macro   type, z80reg
                ifidni  <type>, <and>
                    and     dh, z80reg
                endif
                ifidni  <type>, <or>
                    or      dh, z80reg
                endif
                ifidni  <type>, <xor>
                    xor     dh, z80reg
                endif
                lahf
                setp    [ebx].flagPV
                mov     dl, dh
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z
                or      dl, ah
                ifidni  <type>, <and>
                    or      dl, Z80F_H
                endif
                endm
;----------------------------------------------------------------------------
ROTR            macro   direction, z80reg
                ifidni  <direction>, <l>
                    mov     dl, z80reg
                    rol     dl, 1
                endif
                ifidni  <direction>, <r>
                    mov     dl, z80reg
                    ror     dl, 1
                endif
                ifidni  <direction>, <lc>
                    shr     dl, 1
                    mov     dl, z80reg
                    rcl     dl, 1
                endif
                ifidni  <direction>, <rc>
                    shr     dl, 1
                    mov     dl, z80reg
                    rcr     dl, 1
                endif
                inc     dl
                dec     dl
                lahf
                setp    [ebx].flagPV
                mov     z80reg, dl
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_C
                or      dl, ah
                endm
;----------------------------------------------------------------------------
ROTIX           macro   direction, z80reg
                ifidni  <direction>, <lc>
                    shr     dl, 1
                    mov     dl, cl
                else
                    ifidni  <direction>, <rc>
                        shr     dl, 1
                        mov     dl, cl
                    endif
                endif
                ifidni  <direction>, <l>
                    rol     dl, 1
                endif
                ifidni  <direction>, <r>
                    ror     dl, 1
                endif
                ifidni  <direction>, <lc>
                    rcl     dl, 1
                endif
                ifidni  <direction>, <rc>
                    rcr     dl, 1
                endif
                inc     dl
                dec     dl
                lahf
                setp    [ebx].flagPV
                ifnb    <z80reg>
                    mov     z80reg, dl
                endif
                mov     al, dl
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_C
                or      dl, ah
                endm
;----------------------------------------------------------------------------
SHIFTR          macro   direction, z80reg
                ifidni  <direction>, <l>
                    shl     z80reg, 1
                endif
                ifidni  <direction>, <r>
                    sar     z80reg, 1
                endif
                ifidni  <direction>, <li>
                    shl     z80reg, 1
                    inc     z80reg
                endif
                ifidni  <direction>, <rl>
                    shr     z80reg, 1
                endif
                lahf
                setp    [ebx].flagPV
                mov     dl, z80reg
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_C
                or      dl, ah
                endm
;----------------------------------------------------------------------------
SHIFTIX         macro   direction, z80reg
                ifidni  <direction>, <l>
                    sal     dl, 1
                endif
                ifidni  <direction>, <r>
                    sar     dl, 1
                endif
                ifidni  <direction>, <li>
                    shl     dl, 1
                    inc     dl
                endif
                ifidni  <direction>, <rl>
                    shr     dl, 1
                endif
                lahf
                setp    [ebx].flagPV
                ifnb    <z80reg>
                    mov     z80reg, dl
                endif
                mov     al, dl
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_C
                or      dl, ah
                endm
;----------------------------------------------------------------------------
BITR            macro   BitNum, z80reg
                and     dl, 001h
                test    z80reg, byte ptr (1 shl BitNum)
                setz    al
                mov     [ebx].flagPV, al
                ifidni  <BitNum>, <7>
                    ror     al, 1
                    sar     al, 1
                    xor     al, 080h
                else
                    ror     al, 2
                endif
                or      dl, 010h
                or      dl, al
                ifidni  <z80reg>, <[esi]>
                    mov     al, [ebx].regMPh
                else
                    mov     al, z80reg
                endif
                and     al, 028h
                or      dl, al
                endm
;----------------------------------------------------------------------------
RESIX           macro   BitNum, z80reg
                ifdifi  <z80reg>, <dh>
                    and     cl, 0FFh - (1 shl BitNum)
                else
                    and     dh, 0FFh - (1 shl BitNum)
                endif
                ifnb    <z80reg>
                    ifdifi  <z80reg>, <dh>
                        mov     z80reg, cl
                    endif
                endif
                endm
;----------------------------------------------------------------------------
SETIX           macro   BitNum, z80reg
                ifdifi  <z80reg>, <dh>
                    or      cl, 1 shl BitNum
                else
                    or      dh, 1 shl BitNum
                endif
                ifnb    <z80reg>
                    ifdifi  <z80reg>, <dh>
                        mov     z80reg, cl
                    endif
                endif
                endm
;----------------------------------------------------------------------------

;****************************************************************************;

.CODE


;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;NOP - No OPeration, 4 tacts.
; . . . . . . . .
z80_nop:
                T_M1R   4               ; M1, OCR
                TOTAL   4
                ret

;HALT - no operation while interrupt, 4 tacts.
; . . . . . . . .
halt:
                T_M1R   4               ; M1, OCR
                or      [ebx].FlagsA, FA_HALT
                dec     edi
                TOTAL   4
                ret

;DI - Disable Interrupts, 4 tacts.
; . . . . . . . .
zx_di:
                T_M1R   4               ; M1, OCR
                or      [ebx].FlagsA, FA_IFF1_ + FA_IFF2_
                TOTAL   4
                ret

;EI - Enable Interrupts, 4 tacts.
; . . . . . . . .
zx_ei:
                T_M1R   4               ; M1, OCR
                and     [ebx].FlagsA, not(FA_IFF1_ + FA_IFF2_)
                or      [ebx].FlagsA, FA_ONESHOTDI
                TOTAL   4
                ret

;IM 0 - Interrupt Mode 0, 8 tacts.
; . . . . . . . .
im_0:
                T_M1R   4               ; M1, OCR
                and     [ebx].FlagsA, not(FA_INTMODE0 + FA_INTMODE1)
                TOTAL   4
                ret

;IM 1 - Interrupt Mode 1, 8 tacts.
; . . . . . . . .
im_1:
                T_M1R   4               ; M1, OCR
                and     [ebx].FlagsA, not(FA_INTMODE0 + FA_INTMODE1)
                or      [ebx].FlagsA, FA_INTMODE0
                TOTAL   4
                ret

;IM 2 - Interrupt Mode 2, 8 tacts.
; . . . . . . . .
im_2:
                T_M1R   4               ; M1, OCR
                and     [ebx].FlagsA, not(FA_INTMODE0 + FA_INTMODE1)
                or      [ebx].FlagsA, FA_INTMODE1
                TOTAL   4
                ret

;#CB - Prefix for bit operations.
; . . . . . . . .
prefix_cb:
                T_M1R   4               ; M1, CBR
                M1R
                xor     eax, eax
                inc     edi
                inc     [ebx].regR
                mov     al, [esi]
                or      [ebx].FlagsA, FA_CMDPREF
                TOTAL   4
                jmp     [eax*4 + offset Z80_CMDs_0xCB]

;#DD - Prefix for operations with IX register, 4 tacts.
; . . . . . . . .
prefix_ix:
                T_M1R   4               ; M1, CBR
                M1R
                xor     eax, eax
                inc     edi
                mov     al, [esi]
                or      [ebx].FlagsA, FA_CMDPREF
                mov     ecx, [ebx].rHLxHL
                lea     eax, [eax*4 + offset Z80_CMDs_Index]
                inc     [ebx].regR
                xchg    ecx, [ebx].rIYIX
                push    offset pix1
                mov     [ebx].rHLxHL, ecx
                TOTAL   4
                jmp     [eax]
pix1:
                mov     eax, [ebx].rHLxHL
                xchg    eax, [ebx].rIYIX
                mov     [ebx].rHLxHL, eax
                ret

;#ED - Prefix for extended commands.
; . . . . . . . .
prefix_ed:
                T_M1R   4               ; M1, CBR
                M1R
                xor     eax, eax
                inc     edi
                mov     al, [esi]
                or      [ebx].FlagsA, FA_CMDPREF
                inc     [ebx].regR
                TOTAL   4
                jmp     [eax*4 + offset Z80_CMDs_0xED]

;#FD - Prefix for operations with IY register, 4 tacts.
; . . . . . . . .
prefix_iy:
                T_M1R   4               ; M1, CBR
                M1R
                xor     eax, eax
                inc     edi
                mov     al, [esi]
                mov     ecx, [ebx].rHLxHL
                or      [ebx].FlagsA, FA_CMDPREF
                xchg    ecx, [ebx].rIYIX
                lea     eax, [eax*4 + offset Z80_CMDs_Index]
                inc     [ebx].regR
                ror     ecx, 16
                push    offset piy1
                mov     [ebx].rHLxHL, ecx
                TOTAL   4
                jmp     [eax]
piy1:
                mov     ecx, [ebx].rHLxHL
                ror     ecx, 16
                xchg    ecx, [ebx].rIYIX
                mov     [ebx].rHLxHL, ecx
                ret

;#DD/FD, #CB - 16-bit prefix for bit operations.
prefix_cbx:
                T_M1R   4               ; M1, CBR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   3               ; ORD
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     di
                inc     edi
                movzx   ecx, byte ptr [esi]
                T_MRD   5               ; OCR+IOP=5
                MRD     ax
                or      [ebx].FlagsA, FA_CMDPREF
                TOTAL   12
                jmp     [ecx*4 + offset Z80_CMDs_Index0xCB]

;#DD/FD, #DD/FD - drop previous prefix and restart current, 4 tacts.
prefix_ixx:
prefix_iyx:
prefix_nul:
                dec     edi
;                T_M1R   4               ; M1, OCR
                dec     [ebx].regR
                or      [ebx].FlagsA, FA_ONESHOTDI
;                TOTAL   4
                ret

;#DD/FD, #ED - ignore previous prefix.
prefix_edx:
                T_M1R   4               ; M1, CBR
                pop     eax
                call    eax
                M1R
                xor     eax, eax
                inc     edi
                mov     al, [esi]
                or      [ebx].FlagsA, FA_CMDPREF
                inc     [ebx].regR
                TOTAL   4
                jmp     [eax*4 + offset Z80_CMDs_0xED]

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;SCF - Set Carry Flag, 4 tacts.
; . . 5 0 3 . 0 1
scf:
                T_M1R   4               ; M1, OCR
                and     dl, Z80F_S + Z80F_Z
                mov     al, dh
                inc     dl
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;CCF - Complement Carry Flag, 4 tacts.
; . . 5 C 3 . 0 
ccf:
                T_M1R   4               ; M1, OCR
                mov     al, dl
                and     al, Z80F_C
                and     dl, Z80F_S + Z80F_Z + Z80F_C
                shl     al, 4
                xor     dl, Z80F_C
                or      dl, al
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;DAA - Decimal Adjust A, 4 tacts
;   5  3 P . 
zx_daa:
                T_M1R   4               ; M1, OCR
                ; Carry flag
                cmp     dh, 099h
                seta    al
                test    dl, Z80F_C
                setnz   ah
                and     dl, not(Z80F_C)
                or      al, ah
                or      dl, al
                ; Correction factor
                neg     al
                and     al, 060h
                mov     cl, al
                mov     al, dh
                and     al, 00Fh
                cmp     al, 00Ah
                setnc   al
                test    dl, Z80F_H
                setnz   ah
                or      al, ah
                neg     al
                and     al, 006h
                or      cl, al
                ; Decimal adjust...
                test    dl, Z80F_N
                jnz     zd1
                add     dh, cl
                jmp     zd2
zd1:
                sub     dh, cl
zd2:
                ; Sign, Zero, Half-carry and Parity flags
                lahf
                setp    [ebx].flagPV
                and     dl, Z80F_N + Z80F_C
                and     ah, Z80F_S + Z80F_Z + Z80F_H
                or      dl, ah
                ; F3 and F5 flags
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;CPL - CoMPlement A, 4 tacts.
; . . 5 1 3 . 1 .
cpl:
                T_M1R   4               ; M1, OCR
                and     dl, Z80F_S + Z80F_Z + Z80F_C
                not     dh
                or      dl, Z80F_H + Z80F_N
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;NEG - NEGative a, 8 tacts (with prefix #ED).
;   5  3 V 1 
z80_neg:
                T_M1R   4               ; M1, OCR
                neg     dh
                lahf
                seto    [ebx].flagPV
                mov     dl, dh
                and     dl, Z80F_5 + Z80F_3
                and     ah, Z80F_S + Z80F_Z + Z80F_H + Z80F_C
                or      dl, ah
                or      dl, Z80F_N
                TOTAL   4
                ret

;RLD - Rotate Left Digit, 18 tacts (with prefix #ED).
;   5 0 3 P 0 .
rld:
                T_M1R   4               ; M1, OCR
                and     dl, Z80F_C
                mov     cx, [ebx].rHL
                mov     ah, dh
                MRD     cx
                mov     al, [esi]
                T_MRD   7               ; MRD=3, IOP=4
                rol     ah, 4
                rol     ax, 4
                mov     dh, ah
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                and     ah, Z80F_5 + Z80F_3
                or      dl, ah
                or      dh, dh
                lahf
                setp    [ebx].flagPV
                and     ah, Z80F_S + Z80F_Z
                or      dl, ah
                mov     [ebx].rMP, cx
                inc     [ebx].rMP
                TOTAL   14
                ret

;RRD - Rotate Right Digit, 18 tacts (with prefix #ED).
;   5 0 3 P 0 .
rrd:
                T_M1R   4               ; M1, OCR
                and     dl, Z80F_C
                mov     cx, [ebx].rHL
                mov     ah, dh
                MRD     cx
                mov     al, [esi]
                T_MRD   7               ; MRD=3, IOP=4
                ror     ax, 4
                ror     ah, 4
                mov     dh, ah
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                and     ah, Z80F_5 + Z80F_3
                or      dl, ah
                or      dh, dh
                lahf
                setp    [ebx].flagPV
                and     ah, Z80F_S + Z80F_Z
                or      dl, ah
                mov     [ebx].rMP, cx
                inc     [ebx].rMP
                TOTAL   14
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;EX AF,AF' - EXchange AF and A'F', 4 tacts.
; . . . . . . . .
ex_af:
                T_M1R   4               ; M1, OCR
                ror     edx, 16
                ror     word ptr [ebx].flagPV, 8
                TOTAL   4
                ret

;EXX - EXchange with eXtended registers, 4 tacts.
; . . . . . . . .
exx:
                T_M1R   4               ; M1, OCR
                ror     [ebx].rBCxBC, 16
                ror     [ebx].rDExDE, 16
                test    [ebx].FlagsA, FA_CMDPREF
                jnz     exx_1
                ror     [ebx].rHLxHL, 16
                TOTAL   4
                ret
exx_1:
                ror     [ebx].rIYIX, 16
                TOTAL   4
                ret

;EX DE,HL - EXchange DE with HL, 4 tacts.
; . . . . . . . .
ex_de_hl:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rDE
                xchg    ax, [ebx].rHL
                mov     [ebx].rDE, ax
                TOTAL   4
                ret

;EX DE,HL - EXchange DE with HL, 8 tacts (with prefix #DD/#FD).
; . . . . . . . .
_ex_de_hl:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rDE
                xchg    ax, [ebx].rIX
                mov     [ebx].rDE, ax
                TOTAL   4
                ret

;EX (SP),HL - EXchange (SP) with HL, 19 tacts.
;EX (SP),IX - EXchange (SP) with IX, 23 tacts (with prefix #DD).
;EX (SP),IY - EXchange (SP) with IY, 23 tacts (with prefix #FD).
; . . . . . . . .
ex_sp_hl:
                T_M1R   4               ; M1, OCR
                xchg    di, [ebx].rSP
                MRD     di
                mov     al, [esi]
                T_MRD   3               ; SRL
                inc     edi
                MRD     di
                mov     ah, [esi]
                T_MRD   4               ; SRH+IOP=4
                mov     [ebx].rMP, ax
                xchg    ax, [ebx].rHL
                MWR     di
                mov     [esi], ah
                T_MWR   3               ; SWH
                dec     edi
                MWR     di
                mov     [esi], al
                T_MWR   5               ; SWL+IOP=5
                xchg    di, [ebx].rSP
                TOTAL   19
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;LD I,A - LoaD I with A, 9 tacts.
; . . . . . . . .
ld_i_a:
                T_M1R   5               ; M1, OCR
                mov     [ebx].regI, dh
                TOTAL   5
                ret

;LD R,A - LoaD R with A, 9 tacts.
; . . . . . . . .
ld_r_a:
                T_M1R   5               ; M1, OCR
                mov     [ebx].regR, dh
                test    dh, 080h
                setnz   [ebx].regRbit7
                TOTAL   5
                ret

;LD A,I - LoaD A with I, 9 tacts.
;   5 0 3 I 0 .
ld_a_i:
                T_M1R   5               ; M1, OCR
                test    [ebx].FlagsA, FA_IFF2_
                setz    [ebx].flagPV
                test    [ebx].FlagsA, FA_INT_
                setnz   al
                and     [ebx].flagPV, al
                mov     al, [ebx].regI
                and     dl, Z80F_C
                lahf
                mov     dh, al
                and     ax, ((Z80F_S + Z80F_Z) shl 8) + Z80F_5 + Z80F_3
                or      dl, al
                or      dl, ah
                TOTAL   5
                ret

;LD A,R - LoaD A from R, 9 tacts.
;   5 0 3 I 0 .
ld_a_r:
                T_M1R   5               ; M1, OCR
                test    [ebx].FlagsA, FA_IFF2_
                setz    [ebx].flagPV
                test    [ebx].FlagsA, FA_INT_
                setnz   al
                and     [ebx].flagPV, al
                mov     al, [ebx].regRbit7
                and     dl, Z80F_C
                ror     al, 1
                and     al, 080h
                mov     ah, [ebx].regR
                and     ah, 07Fh
                or      al, ah
                lahf
                mov     dh, al
                and     ax, ((Z80F_S + Z80F_Z) shl 8) + Z80F_5 + Z80F_3
                or      dl, al
                or      dl, ah
                TOTAL   5
                ret

;LD SP,HL - LoaD SP from HL, 6 tacts.
;LD SP,IX - LoaD SP from IX, 10 tacts (with prefix #DD).
;LD SP,IY - LoaD SP from IY, 10 tacts (with prefix #FD).
; . . . . . . . .
ld_sp_hl:
                T_M1R   6               ; M1, OCR
                mov     ax, [ebx].rHL
                mov     [ebx].rSP, ax
                TOTAL   6
                ret

;JP (HL) - JumP HL, 4 tacts.
;JP (IX) - JumP IX, 8 tacts (with prefix #DD).
;JP (IY) - JumP IY, 8 tacts (with prefix #FD).
; . . . . . . . .
jmp_hl:
                T_M1R   4               ; M1, OCR
                mov     di, [ebx].rHL
                TOTAL   4
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;LD B,C - LoaD B with C, 4 tacts.
; . . . . . . . .
ld_b_c:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regC
                mov     [ebx].regB, al
                TOTAL   4
                ret


;LD B,D - LoaD B with D, 4 tacts.
; . . . . . . . .
ld_b_d:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regD
                mov     [ebx].regB, al
                TOTAL   4
                ret

;LD B,E - LoaD B with E, 4 tacts.
; . . . . . . . .
ld_b_e:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regE
                mov     [ebx].regB, al
                TOTAL   4
                ret

;LD B,H - LoaD B with H, 4 tacts.
; . . . . . . . .
ld_b_h:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regH
                mov     [ebx].regB, al
                TOTAL   4
                ret

;LD B,L - LoaD B with L, 4 tacts.
; . . . . . . . .
ld_b_l:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regL
                mov     [ebx].regB, al
                TOTAL   4
                ret

;LD B,A - LoaD B with A, 4 tacts.
; . . . . . . . .
ld_b_a:
                T_M1R   4               ; M1, OCR
                mov     [ebx].regB, dh
                TOTAL   4
                ret

;LD C,B - LoaD C with B, 4 tacts.
; . . . . . . . .
ld_c_b:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regB
                mov     [ebx].regC, al
                TOTAL   4
                ret

;LD C,D - LoaD C with D, 4 tacts.
; . . . . . . . .
ld_c_d:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regD
                mov     [ebx].regC, al
                TOTAL   4
                ret

;LD C,E - LoaD C with E, 4 tacts.
; . . . . . . . .
ld_c_e:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regE
                mov     [ebx].regC, al
                TOTAL   4
                ret

;LD C,H - LoaD C with H, 4 tacts.
; . . . . . . . .
ld_c_h:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regH
                mov     [ebx].regC, al
                TOTAL   4
                ret

;LD C,L - LoaD C with L, 4 tacts.
; . . . . . . . .
ld_c_l:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regL
                mov     [ebx].regC, al
                TOTAL   4
                ret

;LD C,A - LoaD C with A, 4 tacts.
; . . . . . . . .
ld_c_a:
                T_M1R   4               ; M1, OCR
                mov     [ebx].regC, dh
                TOTAL   4
                ret

;LD D,B - LoaD D with B, 4 tacts.
; . . . . . . . .
ld_d_b:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regB
                mov     [ebx].regD, al
                TOTAL   4
                ret

;LD D,C - LoaD D with C, 4 tacts.
; . . . . . . . .
ld_d_c:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regC
                mov     [ebx].regD, al
                TOTAL   4
                ret

;LD D,E - LoaD D with E, 4 tacts.
; . . . . . . . .
ld_d_e:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regE
                mov     [ebx].regD, al
                TOTAL   4
                ret

;LD D,H - LoaD D with H, 4 tacts.
; . . . . . . . .
ld_d_h:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regH
                mov     [ebx].regD, al
                TOTAL   4
                ret

;LD D,L - LoaD D with L, 4 tacts.
; . . . . . . . .
ld_d_l:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regL
                mov     [ebx].regD, al
                TOTAL   4
                ret

;LD D,A - LoaD D with A, 4 tacts.
; . . . . . . . .
ld_d_a:
                T_M1R   4               ; M1, OCR
                mov     [ebx].regD, dh
                TOTAL   4
                ret

;LD E,B - LoaD E with B, 4 tacts.
; . . . . . . . .
ld_e_b:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regB
                mov     [ebx].regE, al
                TOTAL   4
                ret

;LD E,C - LoaD E with C, 4 tacts.
; . . . . . . . .
ld_e_c:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regC
                mov     [ebx].regE, al
                TOTAL   4
                ret

;LD E,D - LoaD E with D, 4 tacts.
; . . . . . . . .
ld_e_d:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regD
                mov     [ebx].regE, al
                TOTAL   4
                ret

;LD E,H - LoaD E with H, 4 tacts.
; . . . . . . . .
ld_e_h:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regH
                mov     [ebx].regE, al
                TOTAL   4
                ret

;LD E,L - LoaD E with L, 4 tacts.
; . . . . . . . .
ld_e_l:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regL
                mov     [ebx].regE, al
                TOTAL   4
                ret

;LD E,A - LoaD E with A, 4 tacts.
; . . . . . . . .
ld_e_a:
                T_M1R   4               ; M1, OCR
                mov     [ebx].regE, dh
                TOTAL   4
                ret

;LD H,B - LoaD H with B, 4 tacts.
; . . . . . . . .
ld_h_b:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regB
                mov     [ebx].regH, al
                TOTAL   4
                ret

;LD H,C - LoaD H with C, 4 tacts.
; . . . . . . . .
ld_h_c:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regC
                mov     [ebx].regH, al
                TOTAL   4
                ret

;LD H,D - LoaD H with D, 4 tacts.
; . . . . . . . .
ld_h_d:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regD
                mov     [ebx].regH, al
                TOTAL   4
                ret

;LD H,E - LoaD H with E, 4 tacts.
; . . . . . . . .
ld_h_e:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regE
                mov     [ebx].regH, al
                TOTAL   4
                ret

;LD H,L - LoaD H with L, 4 tacts.
; . . . . . . . .
ld_h_l:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regL
                mov     [ebx].regH, al
                TOTAL   4
                ret

;LD H,A - LoaD H with A, 4 tacts.
; . . . . . . . .
ld_h_a:
                T_M1R   4               ; M1, OCR
                mov     [ebx].regH, dh
                TOTAL   4
                ret

;LD L,B - LoaD L with B, 4 tacts.
; . . . . . . . .
ld_l_b:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regB
                mov     [ebx].regL, al
                TOTAL   4
                ret

;LD L,C - LoaD L with C, 4 tacts.
; . . . . . . . .
ld_l_c:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regC
                mov     [ebx].regL, al
                TOTAL   4
                ret

;LD L,D - LoaD L with D, 4 tacts.
; . . . . . . . .
ld_l_d:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regD
                mov     [ebx].regL, al
                TOTAL   4
                ret

;LD L,E - LoaD L with E, 4 tacts.
; . . . . . . . .
ld_l_e:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regE
                mov     [ebx].regL, al
                TOTAL   4
                ret

;LD L,H - LoaD L with H, 4 tacts.
; . . . . . . . .
ld_l_h:
                T_M1R   4               ; M1, OCR
                mov     al, [ebx].regH
                mov     [ebx].regL, al
                TOTAL   4
                ret

;LD L,A - LoaD L with A, 4 tacts.
; . . . . . . . .
ld_l_a:
                T_M1R   4               ; M1, OCR
                mov     [ebx].regL, dh
                TOTAL   4
                ret

;LD A,B - LoaD A with B, 4 tacts.
; . . . . . . . .
ld_a_b:
                T_M1R   4               ; M1, OCR
                mov     dh, [ebx].regB
                TOTAL   4
                ret

;LD A,C - LoaD A with C, 4 tacts.
; . . . . . . . .
ld_a_c:
                T_M1R   4               ; M1, OCR
                mov     dh, [ebx].regC
                TOTAL   4
                ret

;LD A,D - LoaD A with D, 4 tacts.
; . . . . . . . .
ld_a_d:
                T_M1R   4               ; M1, OCR
                mov     dh, [ebx].regD
                TOTAL   4
                ret

;LD A,E - LoaD A with E, 4 tacts.
; . . . . . . . .
ld_a_e:
                T_M1R   4               ; M1, OCR
                mov     dh, [ebx].regE
                TOTAL   4
                ret

;LD A,H - LoaD A with H, 4 tacts.
; . . . . . . . .
ld_a_h:
                T_M1R   4               ; M1, OCR
                mov     dh, [ebx].regH
                TOTAL   4
                ret

;LD A,L - LoaD A with L, 4 tacts.
; . . . . . . . .
ld_a_l:
                T_M1R   4               ; M1, OCR
                mov     dh, [ebx].regL
                TOTAL   4
                ret

;----------------------------------------------------------------------------;

Z80LRX          macro   reg
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                ifdifi  <reg>, <dh>
                    mov     al, [esi]
                else
                    mov     dh, [esi]
                endif
                T_MRD   3               ; ORD
                ifdifi  <reg>, <dh>
                    mov     reg, al
                endif
                TOTAL   7
                endm

;LD B,n - LoaD B with n, 7 tacts.
; . . . . . . . .
ld_b:
                Z80LRX  [ebx].regB
                ret

;LD C,n - LoaD C with n, 7 tacts.
; . . . . . . . .
ld_c:
                Z80LRX  [ebx].regC
                ret

;LD D,n - LoaD D with n, 7 tacts.
; . . . . . . . .
ld_d:
                Z80LRX  [ebx].regD
                ret

;LD E,n - LoaD E with n, 7 tacts.
; . . . . . . . .
ld_e:
                Z80LRX  [ebx].regE
                ret

;LD H,n - LoaD H with n, 7 tacts.
;LD XH,n - LoaD XH with n, 11 tacts (with prefix #DD).
;LD YH,n - LoaD YH with n, 11 tacts (with prefix #FD).
; . . . . . . . .
ld_h:
                Z80LRX  [ebx].regH
                ret

;LD L,n - LoaD L with n, 7 tacts.
;LD XL,n - LoaD XL with n, 11 tacts (with prefix #DD).
;LD YL,n - LoaD YL with n, 11 tacts (with prefix #FD).
; . . . . . . . .
ld_l:
                Z80LRX  [ebx].regL
                ret

;LD A,n - LoaD A with n, 7 tacts.
; . . . . . . . .
ld_a:
                Z80LRX  dh
                ret

;----------------------------------------------------------------------------;

;INC B - INCrement B, 4 tacts.
;   5  3 V 0 .
inc_b:
                T_M1R   4               ; M1, OCR
                INCR    [ebx].regB
                TOTAL   4
                ret

;INC C - INCrement C, 4 tacts.
;   5  3 V 0 .
inc_c:
                T_M1R   4               ; M1, OCR
                INCR    [ebx].regC
                TOTAL   4
                ret

;INC D - INCrement D, 4 tacts.
;   5  3 V 0 .
inc_d:
                T_M1R   4               ; M1, OCR
                INCR    [ebx].regD
                TOTAL   4
                ret

;INC E - INCrement E, 4 tacts.
;   5  3 V 0 .
inc_e:
                T_M1R   4               ; M1, OCR
                INCR    [ebx].regE
                TOTAL   4
                ret

;INC H - INCrement H, 4 tacts.
;INC XH - INCrement XH, 8 tacts (with prefix #DD).
;INC YH - INCrement XH, 8 tacts (with prefix #FD).
;   5  3 V 0 .
inc_h:
                T_M1R   4               ; M1, OCR
                INCR    [ebx].regH
                TOTAL   4
                ret

;INC L - INCrement L, 4 tacts.
;INC XL - INCrement XL, 8 tacts (with prefix #DD).
;INC YL - INCrement XL, 8 tacts (with prefix #FD).
;   5  3 V 0 .
inc_l:
                T_M1R   4               ; M1, OCR
                INCR    [ebx].regL
                TOTAL   4
                ret

;INC A - INCrement A, 4 tacts.
;   5  3 V 0 .
inc_a:
                T_M1R   4               ; M1, OCR
                INCR    dh
                TOTAL   4
                ret

;DEC B - DECrement B, 4 tacts.
;   5  3 V 1 .
dec_b:
                T_M1R   4               ; M1, OCR
                DECR    [ebx].regB
                TOTAL   4
                ret

;DEC C - DECrement C, 4 tacts.
;   5  3 V 1 .
dec_c:
                T_M1R   4               ; M1, OCR
                DECR    [ebx].regC
                TOTAL   4
                ret

;DEC D - DECrement D, 4 tacts.
;   5  3 V 1 .
dec_d:
                T_M1R   4               ; M1, OCR
                DECR    [ebx].regD
                TOTAL   4
                ret

;DEC E - DECrement E, 4 tacts.
;   5  3 V 1 .
dec_e:
                T_M1R   4               ; M1, OCR
                DECR    [ebx].regE
                TOTAL   4
                ret

;DEC H - DECrement H, 4 tacts.
;DEC XH - DECrement XH, 8 tacts (with prefix #DD).
;DEC YH - DECrement XH, 8 tacts (with prefix #FD).
;   5  3 V 1 .
dec_h:
                T_M1R   4               ; M1, OCR
                DECR    [ebx].regH
                TOTAL   4
                ret

;DEC L - DECrement L, 4 tacts.
;DEC XL - DECrement XL, 8 tacts (with prefix #DD).
;DEC YL - DECrement XL, 8 tacts (with prefix #FD).
;   5  3 V 1 .
dec_l:
                T_M1R   4               ; M1, OCR
                DECR    [ebx].regL
                TOTAL   4
                ret

;DEC A - DECrement A, 4 tacts.
;   5  3 V 1 .
dec_a:
                T_M1R   4               ; M1, OCR
                DECR    dh
                TOTAL   4
                ret

;----------------------------------------------------------------------------;

;RLCA - Rotate Left to Carry A, 4 tacts.
; . . 5 0 3 . 0 
rlca:
                T_M1R   4               ; M1, OCR
                and     dl, Z80F_S + Z80F_Z
                rol     dh, 1
                adc     dl, 0
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;RRCA - Rotate Right to Carry A, 4 tacts.
; . . 5 0 3 . 0 
rrca:
                T_M1R   4               ; M1, OCR
                and     dl, Z80F_S + Z80F_Z
                ror     dh, 1
                adc     dl, 0
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;RLA - Rotate Left through carry A, 4 tacts.
; . . 5 0 3 . 0 
rla:
                T_M1R   4               ; M1, OCR
                mov     ah, dl
                and     dl, Z80F_S + Z80F_Z
                sahf
                rcl     dh, 1
                adc     dl, 0
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;RRA - Rotate Right through carry A, 4 tacts.
; . . 5 0 3 . 0 
rra:
                T_M1R   4               ; M1, OCR
                mov     ah, dl
                and     dl, Z80F_S + Z80F_Z
                sahf
                rcr     dh, 1
                adc     dl, 0
                mov     al, dh
                and     al, Z80F_5 + Z80F_3
                or      dl, al
                TOTAL   4
                ret

;----------------------------------------------------------------------------;

;ADD A,B - ADD A with B, 4 tacts.
;   5  3 V 0 
add_b:
                T_M1R   4               ; M1, OCR
                ADSBR   add, [ebx].regB
                TOTAL   4
                ret

;ADD A,C - ADD A with C, 4 tacts.
;   5  3 V 0 
add_c:
                T_M1R   4               ; M1, OCR
                ADSBR   add, [ebx].regC
                TOTAL   4
                ret

;ADD A,D - ADD A with D, 4 tacts.
;   5  3 V 0 
add_d:
                T_M1R   4               ; M1, OCR
                ADSBR   add, [ebx].regD
                TOTAL   4
                ret

;ADD A,E - ADD A with E, 4 tacts.
;   5  3 V 0 
add_e:
                T_M1R   4               ; M1, OCR
                ADSBR   add, [ebx].regE
                TOTAL   4
                ret

;ADD A,H - ADD A with H, 4 tacts.
;ADD A,XH - ADD A with XH, 8 tacts (with prefix #DD).
;ADD A,YH - ADD A with YH, 8 tacts (with prefix #FD).
;   5  3 V 0 
add_h:
                T_M1R   4               ; M1, OCR
                ADSBR   add, [ebx].regH
                TOTAL   4
                ret

;ADD A,L - ADD A with L, 4 tacts.
;ADD A,XL - ADD A with XL, 8 tacts (with prefix #DD).
;ADD A,YL - ADD A with YL, 8 tacts (with prefix #FD).
;   5  3 V 0 
add_l:
                T_M1R   4               ; M1, OCR
                ADSBR   add, [ebx].regL
                TOTAL   4
                ret

;ADD A,A - ADD A with A, 4 tacts.
;   5  3 V 0 
add_a:
                T_M1R   4               ; M1, OCR
                ADSBR   add, dh
                TOTAL   4
                ret

;ADC A,B - ADd with Carry A and B, 4 tacts.
;   5  3 V 0 
adc_b:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, [ebx].regB
                TOTAL   4
                ret

;ADC A,C - ADd with Carry A and C, 4 tacts.
;   5  3 V 0 
adc_c:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, [ebx].regC
                TOTAL   4
                ret

;ADC A,D - ADd with Carry A and D, 4 tacts.
;   5  3 V 0 
adc_d:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, [ebx].regD
                TOTAL   4
                ret

;ADC A,E - ADd with Carry A and E, 4 tacts.
;   5  3 V 0 
adc_e:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, [ebx].regE
                TOTAL   4
                ret

;ADC A,H - ADd with Carry A and H, 4 tacts.
;ADC A,XH - ADd with Carry A and XH, 8 tacts (with prefix #DD).
;ADC A,YH - ADd with Carry A and YH, 8 tacts (with prefix #FD).
;   5  3 V 0 
adc_h:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, [ebx].regH
                TOTAL   4
                ret

;ADC A,L - ADd with Carry A and L, 4 tacts.
;ADC A,XL - ADd with Carry A and XL, 8 tacts (with prefix #DD).
;ADC A,YL - ADd with Carry A and YL, 8 tacts (with prefix #FD).
;   5  3 V 0 
adc_l:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, [ebx].regL
                TOTAL   4
                ret

;ADC A,A - ADd with Carry A and A, 4 tacts.
;   5  3 V 0 
adc_a:
                T_M1R   4               ; M1, OCR
                ADSBR   adc, dh
                TOTAL   4
                ret

;SUB B - SUBstruct B from a, 4 tacts.
;   5  3 V 1 
sub_b:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, [ebx].regB
                TOTAL   4
                ret

;SUB C - SUBstruct C from a, 4 tacts.
;   5  3 V 1 
sub_c:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, [ebx].regC
                TOTAL   4
                ret

;SUB D - SUBstruct D from a, 4 tacts.
;   5  3 V 1 
sub_d:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, [ebx].regD
                TOTAL   4
                ret

;SUB E - SUBstruct E from a, 4 tacts.
;   5  3 V 1 
sub_e:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, [ebx].regE
                TOTAL   4
                ret

;SUB H - SUBstruct H from a, 4 tacts.
;SUB XH - SUBstruct XH from a, 8 tacts (with prefix #DD).
;SUB YH - SUBstruct YH from a, 8 tacts (with prefix #FD).
;   5  3 V 1 
sub_h:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, [ebx].regH
                TOTAL   4
                ret

;SUB L - SUBstruct L from a, 4 tacts.
;SUB XL - SUBstruct XL from a, 8 tacts (with prefix #DD).
;SUB YL - SUBstruct YL from a, 8 tacts (with prefix #FD).
;   5  3 V 1 
sub_l:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, [ebx].regL
                TOTAL   4
                ret

;SUB A - SUBstruct A from a, 4 tacts.
;   5  3 V 1 
sub_a:
                T_M1R   4               ; M1, OCR
                ADSBR   sub, dh
                TOTAL   4
                ret

;SBC A,B - SuBstruct B from A with Carry, 4 tacts.
;   5  3 V 1 
sbc_b:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, [ebx].regB
                TOTAL   4
                ret

;SBC A,C - SuBstruct C from A with Carry, 4 tacts.
;   5  3 V 1 
sbc_c:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, [ebx].regC
                TOTAL   4
                ret

;SBC A,D - SuBstruct D from A with Carry, 4 tacts.
;   5  3 V 1 
sbc_d:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, [ebx].regD
                TOTAL   4
                ret

;SBC A,E - SuBstruct E from A with Carry, 4 tacts.
;   5  3 V 1 
sbc_e:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, [ebx].regE
                TOTAL   4
                ret

;SBC A,H - SuBstruct H from A with Carry, 4 tacts.
;SBC A,XH - SuBstruct XH from A with Carry, 8 tacts (with prefix #DD).
;SBC A,YH - SuBstruct YH from A with Carry, 8 tacts (with prefix #FD).
;   5  3 V 1 
sbc_h:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, [ebx].regH
                TOTAL   4
                ret

;SBC A,L - SuBstruct L from A with Carry, 4 tacts.
;SBC A,XL - SuBstruct XL from A with Carry, 8 tacts (with prefix #DD).
;SBC A,YL - SuBstruct YL from A with Carry, 8 tacts (with prefix #FD).
;   5  3 V 1 
sbc_l:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, [ebx].regL
                TOTAL   4
                ret

;SBC A,A - SuBstruct A from A with Carry, 4 tacts.
;   5  3 V 1 
sbc_a:
                T_M1R   4               ; M1, OCR
                ADSBR   sbc, dh
                TOTAL   4
                ret

;AND B - AND a with B, 4 tacts.
;   5 1 3 P 0 0
and_b:
                T_M1R   4               ; M1, OCR
                LOGR    and, [ebx].regB
                TOTAL   4
                ret

;AND C - AND a with C, 4 tacts.
;   5 1 3 P 0 0
and_c:
                T_M1R   4               ; M1, OCR
                LOGR    and, [ebx].regC
                TOTAL   4
                ret

;AND D - AND a with D, 4 tacts.
;   5 1 3 P 0 0
and_d:
                T_M1R   4               ; M1, OCR
                LOGR    and, [ebx].regD
                TOTAL   4
                ret

;AND E - AND a with E, 4 tacts.
;   5 1 3 P 0 0
and_e:
                T_M1R   4               ; M1, OCR
                LOGR    and, [ebx].regE
                TOTAL   4
                ret

;AND H - AND a with H, 4 tacts.
;AND XH - AND a with XH, 8 tacts (with prefix #DD).
;AND YH - AND a with YH, 8 tacts (with prefix #FD).
;   5 1 3 P 0 0
and_h:
                T_M1R   4               ; M1, OCR
                LOGR    and, [ebx].regH
                TOTAL   4
                ret

;AND L - AND a with L, 4 tacts.
;AND XL - AND a with XL, 8 tacts (with prefix #DD).
;AND YL - AND a with YL, 8 tacts (with prefix #FD).
;   5 1 3 P 0 0
and_l:
                T_M1R   4               ; M1, OCR
                LOGR    and, [ebx].regL
                TOTAL   4
                ret

;AND A - AND a with A, 4 tacts.
;   5 1 3 P 0 0
and_a:
                T_M1R   4               ; M1, OCR
                LOGR    and, dh
                TOTAL   4
                ret

;OR B - OR a with B, 4 tacts.
;   5 0 3 P 0 0
or_b:
                T_M1R   4               ; M1, OCR
                LOGR    or, [ebx].regB
                TOTAL   4
                ret

;OR C - OR a with C, 4 tacts.
;   5 0 3 P 0 0
or_c:
                T_M1R   4               ; M1, OCR
                LOGR    or, [ebx].regC
                TOTAL   4
                ret

;OR D - OR a with D, 4 tacts.
;   5 0 3 P 0 0
or_d:
                T_M1R   4               ; M1, OCR
                LOGR    or, [ebx].regD
                TOTAL   4
                ret

;OR E - OR a with E, 4 tacts.
;   5 0 3 P 0 0
or_e:
                T_M1R   4               ; M1, OCR
                LOGR    or, [ebx].regE
                TOTAL   4
                ret

;OR H - OR a with H, 4 tacts.
;OR XH - OR a with XH, 8 tacts (with prefix #DD).
;OR YH - OR a with YH, 8 tacts (with prefix #FD).
;   5 0 3 P 0 0
or_h:
                T_M1R   4               ; M1, OCR
                LOGR    or, [ebx].regH
                TOTAL   4
                ret

;OR L - OR a with L, 4 tacts.
;OR XL - OR a with XL, 8 tacts (with prefix #DD).
;OR YL - OR a with YL, 8 tacts (with prefix #FD).
;   5 0 3 P 0 0
or_l:
                T_M1R   4               ; M1, OCR
                LOGR    or, [ebx].regL
                TOTAL   4
                ret

;OR A - OR a with A, 4 tacts.
;   5 0 3 P 0 0
or_a:
                T_M1R   4               ; M1, OCR
                LOGR    or, dh
                TOTAL   4
                ret

;XOR B - XOR a with B, 4 tacts.
;   5 0 3 P 0 0
xor_b:
                T_M1R   4               ; M1, OCR
                LOGR    xor, [ebx].regB
                TOTAL   4
                ret

;XOR C - XOR a with C, 4 tacts.
;   5 0 3 P 0 0
xor_c:
                T_M1R   4               ; M1, OCR
                LOGR    xor, [ebx].regC
                TOTAL   4
                ret

;XOR D - XOR a with D, 4 tacts.
;   5 0 3 P 0 0
xor_d:
                T_M1R   4               ; M1, OCR
                LOGR    xor, [ebx].regD
                TOTAL   4
                ret

;XOR E - XOR a with E, 4 tacts.
;   5 0 3 P 0 0
xor_e:
                T_M1R   4               ; M1, OCR
                LOGR    xor, [ebx].regE
                TOTAL   4
                ret

;XOR H - XOR a with H, 4 tacts.
;XOR XH - XOR a with XH, 8 tacts (with prefix #DD).
;XOR YH - XOR a with YH, 8 tacts (with prefix #FD).
;   5 0 3 P 0 0
xor_h:
                T_M1R   4               ; M1, OCR
                LOGR    xor, [ebx].regH
                TOTAL   4
                ret

;XOR L - XOR a with L, 4 tacts.
;XOR XL - XOR a with XL, 8 tacts (with prefix #DD).
;XOR YL - XOR a with YL, 8 tacts (with prefix #FD).
;   5 0 3 P 0 0
xor_l:
                T_M1R   4               ; M1, OCR
                LOGR    xor, [ebx].regL
                TOTAL   4
                ret

;XOR A - XOR a with A, 4 tacts.
; 0 1 5 0 3 P 0 0
xor_a:
                T_M1R   4               ; M1, OCR
                LOGR    xor, dh
                TOTAL   4
                ret

;CP B - ComPare a with B, 4 tacts.
;   5  3 V 1 
cp_b:
                T_M1R   4               ; M1, OCR
                CMPR    [ebx].regB
                TOTAL   4
                ret

;CP C - ComPare a with C, 4 tacts.
;   5  3 V 1 
cp_c:
                T_M1R   4               ; M1, OCR
                CMPR    [ebx].regC
                TOTAL   4
                ret

;CP D - ComPare a with D, 4 tacts.
;   5  3 V 1 
cp_d:
                T_M1R   4               ; M1, OCR
                CMPR    [ebx].regD
                TOTAL   4
                ret

;CP E - ComPare a with E, 4 tacts.
;   5  3 V 1 
cp_e:
                T_M1R   4               ; M1, OCR
                CMPR    [ebx].regE
                TOTAL   4
                ret

;CP H - ComPare a with H, 4 tacts.
;CP XH - ComPare a with XH, 8 tacts (with prefix #DD).
;CP XH - ComPare a with XH, 8 tacts (with prefix #FD).
;   5  3 V 1 
cp_h:
                T_M1R   4               ; M1, OCR
                CMPR    [ebx].regH
                TOTAL   4
                ret

;CP L - ComPare a with L, 4 tacts.
;CP XL - ComPare a with XL, 8 tacts (with prefix #DD).
;CP XL - ComPare a with XL, 8 tacts (with prefix #FD).
;   5  3 V 1 
cp_l:
                T_M1R   4               ; M1, OCR
                CMPR    [ebx].regL
                TOTAL   4
                ret

;CP A - ComPare a with A, 4 tacts.
;   5  3 V 1 
cp_a:
                T_M1R   4               ; M1, OCR
                CMPR    dh
                TOTAL   4
                ret

;----------------------------------------------------------------------------;

;ADD A,n - ADD A with n, 7 tacts.
;   5  3 V 0 
add_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                ADSBR   add, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;ADC A,n - ADd with Carry A and n, 7 tacts.
;   5  3 V 0 
adc_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                ADSBR   adc, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;SUB n - SUBstruct n from a, 7 tacts.
;   5  3 V 1 
sub_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                ADSBR   sub, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;SBC A,n - SuBstruct n from A with Carry, 7 tacts.
;   5  3 V 1 
sbc_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                ADSBR   sbc, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;AND A,n - AND A with n, 7 tacts.
;   5 1 3 P 0 0
and_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                LOGR    and, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;XOR n - XOR a with n, 7 tacts.
;   5 0 3 P 0 0
xor_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                LOGR    xor, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;OR n - OR a with n, 7 tacts.
;   5 0 3 P 0 0
or_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                LOGR    or, [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;CP n - ComPare a with n, 7 tacts.
;   5  3 V 1 
cp_n:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                CMPR    [esi]
                T_MRD   3               ; ORD
                TOTAL   7
                ret

;----------------------------------------------------------------------------;

;BIT 0,B - test BIT 0 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0b:
                T_M1R   4               ; M1, OCR
                BITR    0, [ebx].regB
                TOTAL   4
                ret

;BIT 0,C - test BIT 0 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0c:
                T_M1R   4               ; M1, OCR
                BITR    0, [ebx].regC
                TOTAL   4
                ret

;BIT 0,D - test BIT 0 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0d:
                T_M1R   4               ; M1, OCR
                BITR    0, [ebx].regD
                TOTAL   4
                ret

;BIT 0,E - test BIT 0 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0e:
                T_M1R   4               ; M1, OCR
                BITR    0, [ebx].regE
                TOTAL   4
                ret

;BIT 0,H - test BIT 0 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0h:
                T_M1R   4               ; M1, OCR
                BITR    0, [ebx].regH
                TOTAL   4
                ret

;BIT 0,L - test BIT 0 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0l:
                T_M1R   4               ; M1, OCR
                BITR    0, [ebx].regL
                TOTAL   4
                ret

;BIT 0,A - test BIT 0 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0a:
                T_M1R   4               ; M1, OCR
                BITR    0, dh
                TOTAL   4
                ret

;BIT 1,B - test BIT 1 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1b:
                T_M1R   4               ; M1, OCR
                BITR    1, [ebx].regB
                TOTAL   4
                ret

;BIT 1,C - test BIT 1 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1c:
                T_M1R   4               ; M1, OCR
                BITR    1, [ebx].regC
                TOTAL   4
                ret

;BIT 1,D - test BIT 1 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1d:
                T_M1R   4               ; M1, OCR
                BITR    1, [ebx].regD
                TOTAL   4
                ret

;BIT 1,E - test BIT 1 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1e:
                T_M1R   4               ; M1, OCR
                BITR    1, [ebx].regE
                TOTAL   4
                ret

;BIT 1,H - test BIT 1 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1h:
                T_M1R   4               ; M1, OCR
                BITR    1, [ebx].regH
                TOTAL   4
                ret

;BIT 1,L - test BIT 1 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1l:
                T_M1R   4               ; M1, OCR
                BITR    1, [ebx].regL
                TOTAL   4
                ret

;BIT 1,A - test BIT 1 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1a:
                T_M1R   4               ; M1, OCR
                BITR    1, dh
                TOTAL   4
                ret

;BIT 2,B - test BIT 2 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2b:
                T_M1R   4               ; M1, OCR
                BITR    2, [ebx].regB
                TOTAL   4
                ret

;BIT 2,C - test BIT 2 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2c:
                T_M1R   4               ; M1, OCR
                BITR    2, [ebx].regC
                TOTAL   4
                ret

;BIT 2,D - test BIT 2 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2d:
                T_M1R   4               ; M1, OCR
                BITR    2, [ebx].regD
                TOTAL   4
                ret

;BIT 2,E - test BIT 2 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2e:
                T_M1R   4               ; M1, OCR
                BITR    2, [ebx].regE
                TOTAL   4
                ret

;BIT 2,H - test BIT 2 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2h:
                T_M1R   4               ; M1, OCR
                BITR    2, [ebx].regH
                TOTAL   4
                ret

;BIT 2,L - test BIT 2 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2l:
                T_M1R   4               ; M1, OCR
                BITR    2, [ebx].regL
                TOTAL   4
                ret

;BIT 2,A - test BIT 2 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2a:
                T_M1R   4               ; M1, OCR
                BITR    2, dh
                TOTAL   4
                ret

;BIT 3,B - test BIT 3 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3b:
                T_M1R   4               ; M1, OCR
                BITR    3, [ebx].regB
                TOTAL   4
                ret

;BIT 3,C - test BIT 3 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3c:
                T_M1R   4               ; M1, OCR
                BITR    3, [ebx].regC
                TOTAL   4
                ret

;BIT 3,D - test BIT 3 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3d:
                T_M1R   4               ; M1, OCR
                BITR    3, [ebx].regD
                TOTAL   4
                ret

;BIT 3,E - test BIT 3 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3e:
                T_M1R   4               ; M1, OCR
                BITR    3, [ebx].regE
                TOTAL   4
                ret

;BIT 3,H - test BIT 3 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3h:
                T_M1R   4               ; M1, OCR
                BITR    3, [ebx].regH
                TOTAL   4
                ret

;BIT 3,L - test BIT 3 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3l:
                T_M1R   4               ; M1, OCR
                BITR    3, [ebx].regL
                TOTAL   4
                ret

;BIT 3,A - test BIT 3 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3a:
                T_M1R   4               ; M1, OCR
                BITR    3, dh
                TOTAL   4
                ret

;BIT 4,B - test BIT 4 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4b:
                T_M1R   4               ; M1, OCR
                BITR    4, [ebx].regB
                TOTAL   4
                ret

;BIT 4,C - test BIT 4 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4c:
                T_M1R   4               ; M1, OCR
                BITR    4, [ebx].regC
                TOTAL   4
                ret

;BIT 4,D - test BIT 4 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4d:
                T_M1R   4               ; M1, OCR
                BITR    4, [ebx].regD
                TOTAL   4
                ret

;BIT 4,E - test BIT 4 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4e:
                T_M1R   4               ; M1, OCR
                BITR    4, [ebx].regE
                TOTAL   4
                ret

;BIT 4,H - test BIT 4 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4h:
                T_M1R   4               ; M1, OCR
                BITR    4, [ebx].regH
                TOTAL   4
                ret

;BIT 4,L - test BIT 4 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4l:
                T_M1R   4               ; M1, OCR
                BITR    4, [ebx].regL
                TOTAL   4
                ret

;BIT 4,A - test BIT 4 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4a:
                T_M1R   4               ; M1, OCR
                BITR    4, dh
                TOTAL   4
                ret

;BIT 5,B - test BIT 5 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5b:
                T_M1R   4               ; M1, OCR
                BITR    5, [ebx].regB
                TOTAL   4
                ret

;BIT 5,C - test BIT 5 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5c:
                T_M1R   4               ; M1, OCR
                BITR    5, [ebx].regC
                TOTAL   4
                ret

;BIT 5,D - test BIT 5 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5d:
                T_M1R   4               ; M1, OCR
                BITR    5, [ebx].regD
                TOTAL   4
                ret

;BIT 5,E - test BIT 5 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5e:
                T_M1R   4               ; M1, OCR
                BITR    5, [ebx].regE
                TOTAL   4
                ret

;BIT 5,H - test BIT 5 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5h:
                T_M1R   4               ; M1, OCR
                BITR    5, [ebx].regH
                TOTAL   4
                ret

;BIT 5,L - test BIT 5 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5l:
                T_M1R   4               ; M1, OCR
                BITR    5, [ebx].regL
                TOTAL   4
                ret

;BIT 5,A - test BIT 5 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5a:
                T_M1R   4               ; M1, OCR
                BITR    5, dh
                TOTAL   4
                ret

;BIT 6,B - test BIT 6 of B, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6b:
                T_M1R   4               ; M1, OCR
                BITR    6, [ebx].regB
                TOTAL   4
                ret

;BIT 6,C - test BIT 6 of C, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6c:
                T_M1R   4               ; M1, OCR
                BITR    6, [ebx].regC
                TOTAL   4
                ret

;BIT 6,D - test BIT 6 of D, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6d:
                T_M1R   4               ; M1, OCR
                BITR    6, [ebx].regD
                TOTAL   4
                ret

;BIT 6,E - test BIT 6 of E, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6e:
                T_M1R   4               ; M1, OCR
                BITR    6, [ebx].regE
                TOTAL   4
                ret

;BIT 6,H - test BIT 6 of H, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6h:
                T_M1R   4               ; M1, OCR
                BITR    6, [ebx].regH
                TOTAL   4
                ret

;BIT 6,L - test BIT 6 of L, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6l:
                T_M1R   4               ; M1, OCR
                BITR    6, [ebx].regL
                TOTAL   4
                ret

;BIT 6,A - test BIT 6 of A, 8 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6a:
                T_M1R   4               ; M1, OCR
                BITR    6, dh
                TOTAL   4
                ret

;BIT 7,B - test BIT 7 of B, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7b:
                T_M1R   4               ; M1, OCR
                BITR    7, [ebx].regB
                TOTAL   4
                ret

;BIT 7,C - test BIT 7 of C, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7c:
                T_M1R   4               ; M1, OCR
                BITR    7, [ebx].regC
                TOTAL   4
                ret

;BIT 7,D - test BIT 7 of D, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7d:
                T_M1R   4               ; M1, OCR
                BITR    7, [ebx].regD
                TOTAL   4
                ret

;BIT 7,E - test BIT 7 of E, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7e:
                T_M1R   4               ; M1, OCR
                BITR    7, [ebx].regE
                TOTAL   4
                ret

;BIT 7,H - test BIT 7 of H, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7h:
                T_M1R   4               ; M1, OCR
                BITR    7, [ebx].regH
                TOTAL   4
                ret

;BIT 7,L - test BIT 7 of L, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7l:
                T_M1R   4               ; M1, OCR
                BITR    7, [ebx].regL
                TOTAL   4
                ret

;BIT 7,A - test BIT 7 of A, 8 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7a:
                T_M1R   4               ; M1, OCR
                BITR    7, dh
                TOTAL   4
                ret

;RES 0,B - RESet bit 0 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0FEh
                TOTAL   4
                ret

;RES 0,C - RESet bit 0 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0FEh
                TOTAL   4
                ret

;RES 0,D - RESet bit 0 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0FEh
                TOTAL   4
                ret

;RES 0,E - RESet bit 0 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0FEh
                TOTAL   4
                ret

;RES 0,H - RESet bit 0 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0FEh
                TOTAL   4
                ret

;RES 0,L - RESet bit 0 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0FEh
                TOTAL   4
                ret

;RES 0,A - RESet bit 0 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_0a:
                T_M1R   4               ; M1, OCR
                and     dh, 0FEh
                TOTAL   4
                ret

;RES 1,B - RESet bit 1 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0FDh
                TOTAL   4
                ret

;RES 1,C - RESet bit 1 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0FDh
                TOTAL   4
                ret

;RES 1,D - RESet bit 1 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0FDh
                TOTAL   4
                ret

;RES 1,E - RESet bit 1 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0FDh
                TOTAL   4
                ret

;RES 1,H - RESet bit 1 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0FDh
                TOTAL   4
                ret

;RES 1,L - RESet bit 1 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0FDh
                TOTAL   4
                ret

;RES 1,A - RESet bit 1 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_1a:
                T_M1R   4               ; M1, OCR
                and     dh, 0FDh
                TOTAL   4
                ret

;RES 2,B - RESet bit 2 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0FBh
                TOTAL   4
                ret

;RES 2,C - RESet bit 2 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0FBh
                TOTAL   4
                ret

;RES 2,D - RESet bit 2 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0FBh
                TOTAL   4
                ret

;RES 2,E - RESet bit 2 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0FBh
                TOTAL   4
                ret

;RES 2,H - RESet bit 2 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0FBh
                TOTAL   4
                ret

;RES 2,L - RESet bit 2 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0FBh
                TOTAL   4
                ret

;RES 2,A - RESet bit 2 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_2a:
                T_M1R   4               ; M1, OCR
                and     dh, 0FBh
                TOTAL   4
                ret

;RES 3,B - RESet bit 3 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0F7h
                TOTAL   4
                ret

;RES 3,C - RESet bit 3 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0F7h
                TOTAL   4
                ret

;RES 3,D - RESet bit 3 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0F7h
                TOTAL   4
                ret

;RES 3,E - RESet bit 3 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0F7h
                TOTAL   4
                ret

;RES 3,H - RESet bit 3 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0F7h
                TOTAL   4
                ret

;RES 3,L - RESet bit 3 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0F7h
                TOTAL   4
                ret

;RES 3,A - RESet bit 3 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_3a:
                T_M1R   4               ; M1, OCR
                and     dh, 0F7h
                TOTAL   4
                ret

;RES 4,B - RESet bit 4 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0EFh
                TOTAL   4
                ret

;RES 4,C - RESet bit 4 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0EFh
                TOTAL   4
                ret

;RES 4,D - RESet bit 4 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0EFh
                TOTAL   4
                ret

;RES 4,E - RESet bit 4 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0EFh
                TOTAL   4
                ret

;RES 4,H - RESet bit 4 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0EFh
                TOTAL   4
                ret

;RES 4,L - RESet bit 4 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0EFh
                TOTAL   4
                ret

;RES 4,A - RESet bit 4 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_4a:
                T_M1R   4               ; M1, OCR
                and     dh, 0EFh
                TOTAL   4
                ret

;RES 5,B - RESet bit 5 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0DFh
                TOTAL   4
                ret

;RES 5,C - RESet bit 5 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0DFh
                TOTAL   4
                ret

;RES 5,D - RESet bit 5 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0DFh
                TOTAL   4
                ret

;RES 5,E - RESet bit 5 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0DFh
                TOTAL   4
                ret

;RES 5,H - RESet bit 5 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0DFh
                TOTAL   4
                ret

;RES 5,L - RESet bit 5 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0DFh
                TOTAL   4
                ret

;RES 5,A - RESet bit 5 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_5a:
                T_M1R   4               ; M1, OCR
                and     dh, 0DFh
                TOTAL   4
                ret

;RES 6,B - RESet bit 6 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 0BFh
                TOTAL   4
                ret

;RES 6,C - RESet bit 6 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 0BFh
                TOTAL   4
                ret

;RES 6,D - RESet bit 6 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 0BFh
                TOTAL   4
                ret

;RES 6,E - RESet bit 6 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 0BFh
                TOTAL   4
                ret

;RES 6,H - RESet bit 6 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 0BFh
                TOTAL   4
                ret

;RES 6,L - RESet bit 6 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 0BFh
                TOTAL   4
                ret

;RES 6,A - RESet bit 6 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_6a:
                T_M1R   4               ; M1, OCR
                and     dh, 0BFh
                TOTAL   4
                ret

;RES 7,B - RESet bit 7 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7b:
                T_M1R   4               ; M1, OCR
                and     [ebx].regB, 07Fh
                TOTAL   4
                ret

;RES 7,C - RESet bit 7 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7c:
                T_M1R   4               ; M1, OCR
                and     [ebx].regC, 07Fh
                TOTAL   4
                ret

;RES 7,D - RESet bit 7 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7d:
                T_M1R   4               ; M1, OCR
                and     [ebx].regD, 07Fh
                TOTAL   4
                ret

;RES 7,E - RESet bit 7 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7e:
                T_M1R   4               ; M1, OCR
                and     [ebx].regE, 07Fh
                TOTAL   4
                ret

;RES 7,H - RESet bit 7 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7h:
                T_M1R   4               ; M1, OCR
                and     [ebx].regH, 07Fh
                TOTAL   4
                ret

;RES 7,L - RESet bit 7 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7l:
                T_M1R   4               ; M1, OCR
                and     [ebx].regL, 07Fh
                TOTAL   4
                ret

;RES 7,A - RESet bit 7 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
res_7a:
                T_M1R   4               ; M1, OCR
                and     dh, 07Fh
                TOTAL   4
                ret

;SET 0,B - SET bit 0 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 001h
                TOTAL   4
                ret

;SET 0,C - SET bit 0 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 001h
                TOTAL   4
                ret

;SET 0,D - SET bit 0 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 001h
                TOTAL   4
                ret

;SET 0,E - SET bit 0 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 001h
                TOTAL   4
                ret

;SET 0,H - SET bit 0 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 001h
                TOTAL   4
                ret

;SET 0,L - SET bit 0 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 001h
                TOTAL   4
                ret

;SET 0,A - SET bit 0 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_0a:
                T_M1R   4               ; M1, OCR
                or      dh, 001h
                TOTAL   4
                ret

;SET 1,B - SET bit 1 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 002h
                TOTAL   4
                ret

;SET 1,C - SET bit 1 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 002h
                TOTAL   4
                ret

;SET 1,D - SET bit 1 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 002h
                TOTAL   4
                ret

;SET 1,E - SET bit 1 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 002h
                TOTAL   4
                ret

;SET 1,H - SET bit 1 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 002h
                TOTAL   4
                ret

;SET 1,L - SET bit 1 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 002h
                TOTAL   4
                ret

;SET 1,A - SET bit 1 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_1a:
                T_M1R   4               ; M1, OCR
                or      dh, 002h
                TOTAL   4
                ret

;SET 2,B - SET bit 2 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 004h
                TOTAL   4
                ret

;SET 2,C - SET bit 2 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 004h
                TOTAL   4
                ret

;SET 2,D - SET bit 2 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 004h
                TOTAL   4
                ret

;SET 2,E - SET bit 2 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 004h
                TOTAL   4
                ret

;SET 2,H - SET bit 2 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 004h
                TOTAL   4
                ret

;SET 2,L - SET bit 2 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 004h
                TOTAL   4
                ret

;SET 2,A - SET bit 2 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_2a:
                T_M1R   4               ; M1, OCR
                or      dh, 004h
                TOTAL   4
                ret

;SET 3,B - SET bit 3 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 008h
                TOTAL   4
                ret

;SET 3,C - SET bit 3 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 008h
                TOTAL   4
                ret

;SET 3,D - SET bit 3 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 008h
                TOTAL   4
                ret

;SET 3,E - SET bit 3 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 008h
                TOTAL   4
                ret

;SET 3,H - SET bit 3 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 008h
                TOTAL   4
                ret

;SET 3,L - SET bit 3 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 008h
                TOTAL   4
                ret

;SET 3,A - SET bit 3 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_3a:
                T_M1R   4               ; M1, OCR
                or      dh, 008h
                TOTAL   4
                ret

;SET 4,B - SET bit 4 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 010h
                TOTAL   4
                ret

;SET 4,C - SET bit 4 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 010h
                TOTAL   4
                ret

;SET 4,D - SET bit 4 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 010h
                TOTAL   4
                ret

;SET 4,E - SET bit 4 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 010h
                TOTAL   4
                ret

;SET 4,H - SET bit 4 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 010h
                TOTAL   4
                ret

;SET 4,L - SET bit 4 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 010h
                TOTAL   4
                ret

;SET 4,A - SET bit 4 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_4a:
                T_M1R   4               ; M1, OCR
                or      dh, 010h
                TOTAL   4
                ret

;SET 5,B - SET bit 5 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 020h
                TOTAL   4
                ret

;SET 5,C - SET bit 5 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 020h
                TOTAL   4
                ret

;SET 5,D - SET bit 5 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 020h
                TOTAL   4
                ret

;SET 5,E - SET bit 5 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 020h
                TOTAL   4
                ret

;SET 5,H - SET bit 5 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 020h
                TOTAL   4
                ret

;SET 5,L - SET bit 5 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 020h
                TOTAL   4
                ret

;SET 5,A - SET bit 5 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_5a:
                T_M1R   4               ; M1, OCR
                or      dh, 020h
                TOTAL   4
                ret

;SET 6,B - SET bit 6 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 040h
                TOTAL   4
                ret

;SET 6,C - SET bit 6 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 040h
                TOTAL   4
                ret

;SET 6,D - SET bit 6 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 040h
                TOTAL   4
                ret

;SET 6,E - SET bit 6 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 040h
                TOTAL   4
                ret

;SET 6,H - SET bit 6 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 040h
                TOTAL   4
                ret

;SET 6,L - SET bit 6 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 040h
                TOTAL   4
                ret

;SET 6,A - SET bit 6 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_6a:
                T_M1R   4               ; M1, OCR
                or      dh, 040h
                TOTAL   4
                ret

;SET 7,B - SET bit 7 of B, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7b:
                T_M1R   4               ; M1, OCR
                or      [ebx].regB, 080h
                TOTAL   4
                ret

;SET 7,C - SET bit 7 of C, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7c:
                T_M1R   4               ; M1, OCR
                or      [ebx].regC, 080h
                TOTAL   4
                ret

;SET 7,D - SET bit 7 of D, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7d:
                T_M1R   4               ; M1, OCR
                or      [ebx].regD, 080h
                TOTAL   4
                ret

;SET 7,E - SET bit 7 of E, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7e:
                T_M1R   4               ; M1, OCR
                or      [ebx].regE, 080h
                TOTAL   4
                ret

;SET 7,H - SET bit 7 of H, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7h:
                T_M1R   4               ; M1, OCR
                or      [ebx].regH, 080h
                TOTAL   4
                ret

;SET 7,L - SET bit 7 of L, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7l:
                T_M1R   4               ; M1, OCR
                or      [ebx].regL, 080h
                TOTAL   4
                ret

;SET 7,A - SET bit 7 of A, 8 tacts (with prefix #CB).
; . . . . . . . .
set_7a:
                T_M1R   4               ; M1, OCR
                or      dh, 080h
                TOTAL   4
                ret

;RLC B - Rotate Left Cyclic B, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_b:
                T_M1R   4               ; M1, OCR
                ROTR    l, [ebx].regB
                TOTAL   4
                ret


;RLC C - Rotate Left Cyclic C, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_c:
                T_M1R   4               ; M1, OCR
                ROTR    l, [ebx].regC
                TOTAL   4
                ret

;RLC D - Rotate Left Cyclic D, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_d:
                T_M1R   4               ; M1, OCR
                ROTR    l, [ebx].regD
                TOTAL   4
                ret

;RLC E - Rotate Left Cyclic E, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_e:
                T_M1R   4               ; M1, OCR
                ROTR    l, [ebx].regE
                TOTAL   4
                ret

;RLC H - Rotate Left Cyclic H, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_h:
                T_M1R   4               ; M1, OCR
                ROTR    l, [ebx].regH
                TOTAL   4
                ret

;RLC L - Rotate Left Cyclic L, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_l:
                T_M1R   4               ; M1, OCR
                ROTR    l, [ebx].regL
                TOTAL   4
                ret

;RLC A - Rotate Left Cyclic A, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_a:
                T_M1R   4               ; M1, OCR
                ROTR    l, dh
                TOTAL   4
                ret

;RRC B - Rotate Right Cyclic B, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_b:
                T_M1R   4               ; M1, OCR
                ROTR    r, [ebx].regB
                TOTAL   4
                ret

;RRC C - Rotate Right Cyclic C, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_c:
                T_M1R   4               ; M1, OCR
                ROTR    r, [ebx].regC
                TOTAL   4
                ret

;RRC D - Rotate Right Cyclic D, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_d:
                T_M1R   4               ; M1, OCR
                ROTR    r, [ebx].regD
                TOTAL   4
                ret

;RRC E - Rotate Right Cyclic E, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_e:
                T_M1R   4               ; M1, OCR
                ROTR    r, [ebx].regE
                TOTAL   4
                ret

;RRC H - Rotate Right Cyclic H, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_h:
                T_M1R   4               ; M1, OCR
                ROTR    r, [ebx].regH
                TOTAL   4
                ret

;RRC L - Rotate Right Cyclic L, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_l:
                T_M1R   4               ; M1, OCR
                ROTR    r, [ebx].regL
                TOTAL   4
                ret

;RRC A - Rotate Right Cyclic A, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_a:
                T_M1R   4               ; M1, OCR
                ROTR    r, dh
                TOTAL   4
                ret

;RL B - Rotate Left B through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_b:
                T_M1R   4               ; M1, OCR
                ROTR    lc, [ebx].regB
                TOTAL   4
                ret

;RL C - Rotate Left C through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_c:
                T_M1R   4               ; M1, OCR
                ROTR    lc, [ebx].regC
                TOTAL   4
                ret

;RL D - Rotate Left D through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_d:
                T_M1R   4               ; M1, OCR
                ROTR    lc, [ebx].regD
                TOTAL   4
                ret

;RL E - Rotate Left E through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_e:
                T_M1R   4               ; M1, OCR
                ROTR    lc, [ebx].regE
                TOTAL   4
                ret

;RL H - Rotate Left H through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_h:
                T_M1R   4               ; M1, OCR
                ROTR    lc, [ebx].regH
                TOTAL   4
                ret

;RL L - Rotate Left L through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_l:
                T_M1R   4               ; M1, OCR
                ROTR    lc, [ebx].regL
                TOTAL   4
                ret

;RL A - Rotate Left A through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_a:
                T_M1R   4               ; M1, OCR
                ROTR    lc, dh
                TOTAL   4
                ret

;RR B - Rotate Left B through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_b:
                T_M1R   4               ; M1, OCR
                ROTR    rc, [ebx].regB
                TOTAL   4
                ret

;RR C - Rotate Left C through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_c:
                T_M1R   4               ; M1, OCR
                ROTR    rc, [ebx].regC
                TOTAL   4
                ret

;RR D - Rotate Left D through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_d:
                T_M1R   4               ; M1, OCR
                ROTR    rc, [ebx].regD
                TOTAL   4
                ret

;RR E - Rotate Left E through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_e:
                T_M1R   4               ; M1, OCR
                ROTR    rc, [ebx].regE
                TOTAL   4
                ret

;RR H - Rotate Left H through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_h:
                T_M1R   4               ; M1, OCR
                ROTR    rc, [ebx].regH
                TOTAL   4
                ret

;RR L - Rotate Left L through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_l:
                T_M1R   4               ; M1, OCR
                ROTR    rc, [ebx].regL
                TOTAL   4
                ret

;RR A - Rotate Left A through carry, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_a:
                T_M1R   4               ; M1, OCR
                ROTR    rc, dh
                TOTAL   4
                ret

;SLA B - Shift Left Arithmetic B, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_b:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, [ebx].regB
                TOTAL   4
                ret

;SLA C - Shift Left Arithmetic C, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_c:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, [ebx].regC
                TOTAL   4
                ret

;SLA D - Shift Left Arithmetic D, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_d:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, [ebx].regD
                TOTAL   4
                ret

;SLA E - Shift Left Arithmetic E, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_e:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, [ebx].regE
                TOTAL   4
                ret

;SLA H - Shift Left Arithmetic H, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_h:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, [ebx].regH
                TOTAL   4
                ret

;SLA L - Shift Left Arithmetic L, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_l:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, [ebx].regL
                TOTAL   4
                ret

;SLA A - Shift Left Arithmetic A, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_a:
                T_M1R   4               ; M1, OCR
                SHIFTR  l, dh
                TOTAL   4
                ret

;SRA B - Shift Right Arithmetic B, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_b:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, [ebx].regB
                TOTAL   4
                ret

;SRA C - Shift Right Arithmetic C, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_c:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, [ebx].regC
                TOTAL   4
                ret

;SRA D - Shift Right Arithmetic D, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_d:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, [ebx].regD
                TOTAL   4
                ret

;SRA E - Shift Right Arithmetic E, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_e:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, [ebx].regE
                TOTAL   4
                ret

;SRA H - Shift Right Arithmetic H, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_h:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, [ebx].regH
                TOTAL   4
                ret

;SRA L - Shift Right Arithmetic L, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_l:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, [ebx].regL
                TOTAL   4
                ret

;SRA A - Shift Right Arithmetic A, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_a:
                T_M1R   4               ; M1, OCR
                SHIFTR  r, dh
                TOTAL   4
                ret

;SLI B - Shift Left and Increment B, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_b:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, [ebx].regB
                TOTAL   4
                ret

;SLI C - Shift Left and Increment C, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_c:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, [ebx].regC
                TOTAL   4
                ret

;SLI D - Shift Left and Increment D, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_d:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, [ebx].regD
                TOTAL   4
                ret

;SLI E - Shift Left and Increment E, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_e:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, [ebx].regE
                TOTAL   4
                ret

;SLI H - Shift Left and Increment H, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_h:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, [ebx].regH
                TOTAL   4
                ret

;SLI L - Shift Left and Increment L, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_l:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, [ebx].regL
                TOTAL   4
                ret

;SLI A - Shift Left and Increment A, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_a:
                T_M1R   4               ; M1, OCR
                SHIFTR  li, dh
                TOTAL   4
                ret

;SRL B - Shift Right Logical B, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_b:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, [ebx].regB
                TOTAL   4
                ret

;SRL C - Shift Right Logical C, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_c:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, [ebx].regC
                TOTAL   4
                ret

;SRL D - Shift Right Logical D, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_d:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, [ebx].regD
                TOTAL   4
                ret

;SRL E - Shift Right Logical E, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_e:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, [ebx].regE
                TOTAL   4
                ret

;SRL H - Shift Right Logical H, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_h:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, [ebx].regH
                TOTAL   4
                ret

;SRL L - Shift Right Logical L, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_l:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, [ebx].regL
                TOTAL   4
                ret

;SRL A - Shift Right Logical A, 8 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_a:
                T_M1R   4               ; M1, OCR
                SHIFTR  rl, dh
                TOTAL   4
                ret

;----------------------------------------------------------------------------;

;LD A,(nn) - LoaD A with (nn), 13 tacts.
; . . . . . . . .
ld_a_:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; ORH
                MRD     ax
                inc     eax
                mov     [ebx].rMP, ax
                mov     dh, [esi]
                T_MRD   3               ; MRD
                TOTAL   13
                ret

;LD A,(BC) - LoaD A from (BC), 7 tacts.
; . . . . . . . .
ld_a_bc:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rBC
                MRD     ax
                mov     dh, [esi]
                T_MRD   3               ; MRD
                inc     eax
                mov     [ebx].rMP, ax
                TOTAL   7
                ret

;LD A,(DE) - LoaD A with (DE), 7 tacts.
; . . . . . . . .
ld_a_de:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rDE
                MRD     ax
                mov     dh, [esi]
                T_MRD   3               ; MRD
                inc     eax
                mov     [ebx].rMP, ax
                TOTAL   7
                ret

;LD (nn),A - LoaD (nn) with A, 13 tacts.
; . . . . . . . .
ld__a:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; ORH
                MWR     ax
                mov     [esi], dh
                inc     al
                T_MWR   3               ; MWR
                mov     [ebx].regMPl, al
                mov     [ebx].regMPh, dh
                TOTAL   13
                ret

;LD (BC),A - LoaD (BC) with A, 7 tacts.
; . . . . . . . .
ld_bc_a:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rBC
                MWR     ax
                mov     [esi], dh
                T_MWR   3               ; MWR
                inc     al
                mov     [ebx].regMPl, al
                mov     [ebx].regMPh, dh
                TOTAL   7
                ret

;LD (DE),A - LoaD (DE) with A, 7 tacts.
; . . . . . . . .
ld_de_a:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rDE
                MWR     ax
                mov     [esi], dh
                T_MWR   3               ; MWR
                inc     al
                mov     [ebx].regMPl, al
                mov     [ebx].regMPh, dh
                TOTAL   7
                ret

;----------------------------------------------------------------------------;

;LD (HL),n - LoaD (HL) with n, 10 tacts.
; . . . . . . . .
ld__hl_:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORD
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   10
                ret

;INC (HL) - INCrement (HL), 11 tacts.
;   5  3 V 0 .
inc_hl_:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                INCR    al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;DEC (HL) - DECrement (HL), 11 tacts.
;   5  3 V 1 .
dec_hl_:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                DECR    al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;----------------------------------------------------------------------------;

;LD B,(HL) - LoaD B with (HL), 7 tacts.
; . . . . . . . .
ld_b_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regB, al
                TOTAL   7
                ret

;LD C,(HL) - LoaD C with (HL), 7 tacts.
; . . . . . . . .
ld_c_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regC, al
                TOTAL   7
                ret

;LD D,(HL) - LoaD D with (HL), 7 tacts.
; . . . . . . . .
ld_d_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regD, al
                TOTAL   7
                ret

;LD E,(HL) - LoaD E with (HL), 7 tacts.
; . . . . . . . .
ld_e_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regE, al
                TOTAL   7
                ret

;LD H,(HL) - LoaD H with (HL), 7 tacts.
; . . . . . . . .
ld_h_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regH, al
                TOTAL   7
                ret

;LD L,(HL) - LoaD L with (HL), 7 tacts.
; . . . . . . . .
ld_l_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regL, al
                TOTAL   7
                ret

;LD A,(HL) - LoaD A with (HL), 7 tacts.
; . . . . . . . .
ld_a_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     dh, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;LD (HL),B - LoaD (HL) with B, 7 tacts.
; . . . . . . . .
ld_hl_b:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     al, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;LD (HL),C - LoaD (HL) with C, 7 tacts.
; . . . . . . . .
ld_hl_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     al, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;LD (HL),D - LoaD (HL) with D, 7 tacts.
; . . . . . . . .
ld_hl_d:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     al, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;LD (HL),E - LoaD (HL) with E, 7 tacts.
; . . . . . . . .
ld_hl_e:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     al, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;LD (HL),H - LoaD (HL) with H, 7 tacts.
; . . . . . . . .
ld_hl_h:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     al, [ebx].regH
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;LD (HL),L - LoaD (HL) with L, 7 tacts.
; . . . . . . . .
ld_hl_l:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     al, [ebx].regL
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;LD (HL),A - LoaD (HL) with A, 7 tacts.
; . . . . . . . .
ld_hl_a:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MWR     cx
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;----------------------------------------------------------------------------;

;ADD A,(HL) - ADD A with (HL), 7 tacts.
;   5  3 V 0 
add_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                ADSBR   add, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;ADC A,(HL) - ADd with Carry A and (HL), 7 tacts.
;   5  3 V 0 
adc_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                ADSBR   adc, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;SUB (HL) - SUBstruct (HL) from a, 7 tacts.
;   5  3 V 1 
sub_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                ADSBR   sub, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;SBC A,(HL) - SuBstruct (HL) from A with Carry, 7 tacts.
;   5  3 V 1 
sbc_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                ADSBR   sbc, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;AND (HL) - AND a with (HL), 7 tacts.
;   5 1 3 P 0 0
and_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                LOGR    and, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;OR (HL) - OR a with (HL), 7 tacts.
;   5 0 3 P 0 0
or_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                LOGR    or, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;XOR (HL) - XOR a with (HL), 7 tacts.
;   5 0 3 P 0 0
xor_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                LOGR    xor, [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;CP (HL) - ComPare a with (HL), 7 tacts.
;   5  3 V 1 
cp_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                CMPR    [esi]
                T_MRD   3               ; MRD
                TOTAL   7
                ret

;----------------------------------------------------------------------------;

;BIT 0,(HL) - test BIT 0 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_0hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    0, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 1,(HL) - test BIT 1 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_1hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    1, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 2,(HL) - test BIT 2 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_2hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    2, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 3,(HL) - test BIT 3 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_3hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    3, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 4,(HL) - test BIT 4 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_4hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    4, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 5,(HL) - test BIT 5 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_5hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    5, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 6,(HL) - test BIT 6 of (HL), 12 tacts (with prefix #CB).
; 0  i 1 i Z 0 .
bit_6hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    6, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;BIT 7,(HL) - test BIT 7 of (HL), 12 tacts (with prefix #CB).
; S  i 1 i Z 0 .
bit_7hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                BITR    7, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   8
                ret

;RES 0,(HL) - RESet bit 0 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_0hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0FEh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 1,(HL) - RESet bit 1 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_1hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0FDh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 2,(HL) - RESet bit 2 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_2hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0FBh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 3,(HL) - RESet bit 3 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_3hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al,[esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0F7h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 4,(HL) - RESet bit 4 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_4hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0EFh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 5,(HL) - RESet bit 5 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_5hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0DFh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 6,(HL) - RESet bit 6 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_6hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 0BFh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RES 7,(HL) - RESet bit 7 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
res_7hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                and     al, 07Fh
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 0,(HL) - SET bit 0 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_0hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 001h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 1,(HL) - SET bit 1 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_1hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 002h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 2,(HL) - SET bit 2 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_2hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 004h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 3,(HL) - SET bit 3 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_3hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 008h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 4,(HL) - SET bit 4 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_4hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 010h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 5,(HL) - SET bit 5 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_5hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 020h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 6,(HL) - SET bit 6 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_6hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 040h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SET 7,(HL) - SET bit 7 of (HL), 15 tacts (with prefix #CB).
; . . . . . . . .
set_7hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                or      al, 080h
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RLC (HL) - Rotate Left Cyclic (HL), 15 tacts (with prefix #CB).
;   5 0 3 P 0 
rlc_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                ROTR    l, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RRC (HL) - Rotate Right Cyclic (HL), 15 tacts (with prefix #CB).
;   5 0 3 P 0 
rrc_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                ROTR    r, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RL (HL) - Rotate Left (HL) through carry, 15 tacts (with prefix #CB).
;   5 0 3 P 0 
rl_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                ROTR    lc, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;RR (HL) - Rotate Left (HL) through carry, 15 tacts (with prefix #CB).
;   5 0 3 P 0 
rr_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                ROTR    rc, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SLA (HL) - Shift Left Arithmetic (HL), 15 tacts (with prefix #CB).
;   5 0 3 P 0 
sla_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                SHIFTR  l, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SRA (HL) - Shift Right Arithmetic (HL), 15 tacts (with prefix #CB).
;   5 0 3 P 0 
sra_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                SHIFTR  r, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SLI (HL) - Shift Left and Increment (HL), 15 tacts (with prefix #CB).
;   5 0 3 P 0 
sli_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                SHIFTR  li, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;SRL (HL) - Shift Right Logical (HL), 15 tacts (with prefix #CB).
;   5 0 3 P 0 
srl_hl:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rHL
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                SHIFTR  rl, al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   11
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;LD (IX+d),n - LoaD (IX+d) with n, 19 tacts (with prefix #DD).
;LD (IY+d),n - LoaD (IY+d) with n, 19 tacts (with prefix #FD).
; . . . . . . . .
ld__ix_:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   3               ; ORD
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     di
                inc     edi
                mov     cl, [esi]
                T_MRD   5               ; ORD+IOP=5
                MWR     ax
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;INC (IX+d) - INCrement (IX+d), 23 tacts (with prefix #DD).
;INC (IY+d) - INCrement (IY+d), 23 tacts (with prefix #FD).
;   5  3 V 0 .
inc_ix_:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   ecx, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     cx, [ebx].rHL
                mov     [ebx].rMP, cx
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                INCR    al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   19
                ret

;DEC (IX+d) - DECrement (IX+d), 23 tacts (with prefix #DD).
;DEC (IY+d) - DECrement (IY+d), 23 tacts (with prefix #FD).
;   5  3 V 1 .
dec_ix_:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   ecx, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     cx, [ebx].rHL
                mov     [ebx].rMP, cx
                MRD     cx
                mov     al, [esi]
                T_MRD   4               ; MRD+IOP=4
                DECR    al
                MWR     cx
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   19
                ret

;LD B,(IX+d) - LoaD B with (IX+d), 19 tacts (with prefix #DD).
;LD B,(IY+d) - LoaD B with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_b_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regB, al
                TOTAL   15
                ret

;LD C,(IX+d) - LoaD C with (IX+d), 19 tacts (with prefix #DD).
;LD C,(IY+d) - LoaD C with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_c_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regC, al
                TOTAL   15
                ret

;LD D,(IX+d) - LoaD D with (IX+d), 19 tacts (with prefix #DD).
;LD D,(IY+d) - LoaD D with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_d_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regD, al
                TOTAL   15
                ret

;LD E,(IX+d) - LoaD E with (IX+d), 19 tacts (with prefix #DD).
;LD E,(IY+d) - LoaD E with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_e_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regE, al
                TOTAL   15
                ret

;LD H,(IX+d) - LoaD H with (IX+d), 19 tacts (with prefix #DD).
;LD H,(IY+d) - LoaD H with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_h_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regIXh, al
                TOTAL   15
                ret

;LD L,(IX+d) - LoaD L with (IX+d), 19 tacts (with prefix #DD).
;LD L,(IY+d) - LoaD L with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_l_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     al, [esi]
                T_MRD   3               ; MRD
                mov     [ebx].regIXl, al
                TOTAL   15
                ret

;LD A,(IX+d) - LoaD A with (IX+d), 19 tacts (with prefix #DD).
;LD A,(IY+d) - LoaD A with (IY+d), 19 tacts (with prefix #FD).
; . . . . . . . .
ld_a_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                mov     dh, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;LD (IX+d),B - LoaD (IX+d) with B, 19 tacts (with prefix #DD).
;LD (IY+d),B - LoaD (IY+d) with B, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_b:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     al, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;LD (IX+d),C - LoaD (IX+d) with C, 19 tacts (with prefix #DD).
;LD (IY+d),C - LoaD (IY+d) with C, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_c:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     al, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;LD (IX+d),D - LoaD (IX+d) with D, 19 tacts (with prefix #DD).
;LD (IY+d),D - LoaD (IY+d) with D, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_d:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     al, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;LD (IX+d),E - LoaD (IX+d) with E, 19 tacts (with prefix #DD).
;LD (IY+d),E - LoaD (IY+d) with E, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_e:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     al, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;LD (IX+d),H - LoaD (IX+d) with H, 19 tacts (with prefix #DD).
;LD (IY+d),H - LoaD (IY+d) with H, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_h:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     al, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;LD (IX+d),L - LoaD (IX+d) with L, 19 tacts (with prefix #DD).
;LD (IY+d),L - LoaD (IY+d) with L, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_l:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     al, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;LD (IX+d),A - LoaD (IX+d) with A, 19 tacts (with prefix #DD).
;LD (IY+d),A - LoaD (IY+d) with A, 19 tacts (with prefix #FD).
; . . . . . . . .
ld_ix_a:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MWR     ax
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   15
                ret

;ADD A,(IX+d) - ADD A with (IX+d), 19 tacts (with prefix #DD).
;ADD A,(IY+d) - ADD A with (IY+d), 19 tacts (with prefix #FD).
;   5  3 V 0 
add_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                ADSBR   add, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;ADC A,(IX+d) - ADd with Carry A and (IX+d), 19 tacts (with prefix #DD).
;ADC A,(IY+d) - ADd with Carry A and (IY+d), 19 tacts (with prefix #FD).
;   5  3 V 0 
adc_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                ADSBR   adc, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;SUB (IX+d) - SUBstruct (IX+d) from a, 19 tacts (with prefix #DD).
;SUB (IY+d) - SUBstruct (IY+d) from a, 19 tacts (with prefix #FD).
;   5  3 V 1 
sub_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                ADSBR   sub, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;SBC A,(IX+d) - SuBstruct (IX+d) from A with Carry, 19 tacts (with prefix #DD).
;SBC A,(IY+d) - SuBstruct (IY+d) from A with Carry, 19 tacts (with prefix #FD).
;   5  3 V 1 
sbc_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                ADSBR   sbc, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;AND (IX+d) - AND a with (IX+d), 19 tacts (with prefix #DD).
;AND (IY+d) - AND a with (IY+d), 19 tacts (with prefix #FD).
;   5 1 3 P 0 0
and_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                LOGR    and, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;OR (IX+d) - OR a with (IX+d), 19 tacts (with prefix #DD).
;OR (IY+d) - OR a with (IY+d), 19 tacts (with prefix #FD).
;   5 0 3 P 0 0
or_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                LOGR    or, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;XOR (IX+d) - XOR a with (IX+d), 19 tacts (with prefix #DD).
;XOR (IY+d) - XOR a with (IY+d), 19 tacts (with prefix #FD).
;   5 0 3 P 0 0
xor_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                LOGR    xor, [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;CP (IX+d) - ComPare a with (IX+d), 19 tacts (with prefix #DD).
;CP (IY+d) - ComPare a with (IY+d), 19 tacts (with prefix #FD).
;   5  3 V 1 
cp_ix:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                movsx   eax, byte ptr [esi]
                T_MRD   8               ; ORD=3, IOP=5
                add     ax, [ebx].rHL
                mov     [ebx].rMP, ax
                MRD     ax
                CMPR    [esi]
                T_MRD   3               ; MRD
                TOTAL   15
                ret

;----------------------------------------------------------------------------;

;BIT 0,(IX+d) - test BIT 0 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 0,(IY+d) - test BIT 0 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_0ix:
                BITR    0, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 1,(IX+d) - test BIT 1 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 1,(IY+d) - test BIT 1 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_1ix:
                BITR    1, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 2,(IX+d) - test BIT 2 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 2,(IY+d) - test BIT 2 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_2ix:
                BITR    2, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 3,(IX+d) - test BIT 3 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 3,(IY+d) - test BIT 3 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_3ix:
                BITR    3, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 4,(IX+d) - test BIT 4 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 4,(IY+d) - test BIT 4 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_4ix:
                BITR    4, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 5,(IX+d) - test BIT 5 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 5,(IY+d) - test BIT 5 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_5ix:
                BITR    5, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 6,(IX+d) - test BIT 6 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 6,(IY+d) - test BIT 6 of (IY+d), 20 tacts (with prefix #FD,#CB).
; 0  i 1 i Z 0 .
bit_6ix:
                BITR    6, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;BIT 7,(IX+d) - test BIT 7 of (IX+d), 20 tacts (with prefix #DD,#CB).
;BIT 7,(IY+d) - test BIT 7 of (IY+d), 20 tacts (with prefix #FD,#CB).
; S  i 1 i  0 .
bit_7ix:
                BITR    7, [esi]
                T_MRD   4               ; MRD+IOP=4
                TOTAL   4
                ret

;RES 0,(IX+d),B - RESet bit 0 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),B - RESet bit 0 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d),C - RESet bit 0 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),C - RESet bit 0 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d),D - RESet bit 0 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),D - RESet bit 0 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d),E - RESet bit 0 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),E - RESet bit 0 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d),H - RESet bit 0 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),H - RESet bit 0 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d),L - RESet bit 0 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),L - RESet bit 0 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d) - RESet bit 0 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d) - RESet bit 0 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 0,(IX+d),A - RESet bit 0 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 0,(IY+d),A - RESet bit 0 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_0aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   0, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),B - RESet bit 1 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),B - RESet bit 1 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),C - RESet bit 1 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),C - RESet bit 1 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),D - RESet bit 1 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),D - RESet bit 1 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),E - RESet bit 1 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),E - RESet bit 1 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),H - RESet bit 1 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),H - RESet bit 1 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),L - RESet bit 1 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),L - RESet bit 1 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d) - RESet bit 1 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d) - RESet bit 1 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 1,(IX+d),A - RESet bit 1 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 1,(IY+d),A - RESet bit 1 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_1aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   1, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),B - RESet bit 2 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),B - RESet bit 2 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),C - RESet bit 2 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),C - RESet bit 2 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),D - RESet bit 2 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),D - RESet bit 2 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),E - RESet bit 2 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),E - RESet bit 2 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),H - RESet bit 2 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),H - RESet bit 2 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),L - RESet bit 2 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),L - RESet bit 2 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d) - RESet bit 2 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d) - RESet bit 2 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 2,(IX+d),A - RESet bit 2 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 2,(IY+d),A - RESet bit 2 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_2aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   2, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),B - RESet bit 3 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),B - RESet bit 3 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),C - RESet bit 3 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),C - RESet bit 3 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),D - RESet bit 3 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),D - RESet bit 3 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),E - RESet bit 3 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),E - RESet bit 3 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),H - RESet bit 3 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),H - RESet bit 3 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),L - RESet bit 3 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),L - RESet bit 3 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d) - RESet bit 3 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d) - RESet bit 3 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 3,(IX+d),A - RESet bit 3 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 3,(IY+d),A - RESet bit 3 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_3aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   3, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),B - RESet bit 4 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),B - RESet bit 4 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),C - RESet bit 4 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),C - RESet bit 4 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),D - RESet bit 4 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),D - RESet bit 4 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),E - RESet bit 4 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),E - RESet bit 4 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),H - RESet bit 4 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),H - RESet bit 4 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),L - RESet bit 4 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),L - RESet bit 4 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d) - RESet bit 4 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d) - RESet bit 4 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 4,(IX+d),A - RESet bit 4 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 4,(IY+d),A - RESet bit 4 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_4aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   4, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),B - RESet bit 5 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),B - RESet bit 5 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),C - RESet bit 5 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),C - RESet bit 5 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),D - RESet bit 5 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),D - RESet bit 5 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),E - RESet bit 5 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),E - RESet bit 5 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),H - RESet bit 5 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),H - RESet bit 5 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),L - RESet bit 5 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),L - RESet bit 5 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d) - RESet bit 5 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d) - RESet bit 5 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 5,(IX+d),A - RESet bit 5 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 5,(IY+d),A - RESet bit 5 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_5aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   5, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),B - RESet bit 6 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),B - RESet bit 6 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),C - RESet bit 6 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),C - RESet bit 6 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),D - RESet bit 6 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),D - RESet bit 6 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),E - RESet bit 6 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),E - RESet bit 6 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),H - RESet bit 6 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),H - RESet bit 6 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),L - RESet bit 6 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),L - RESet bit 6 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d) - RESet bit 6 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d) - RESet bit 6 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 6,(IX+d),A - RESet bit 6 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 6,(IY+d),A - RESet bit 6 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_6aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   6, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),B - RESet bit 7 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),B - RESet bit 7 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   7, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),C - RESet bit 7 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),C - RESet bit 7 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   7, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),D - RESet bit 7 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),D - RESet bit 7 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   7, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),E - RESet bit 7 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),E - RESet bit 7 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   7, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),H - RESet bit 7 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),H - RESet bit 7 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                RESIX   7, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),L - RESet bit 7 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),L - RESet bit 7 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                RESIX   7, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d) - RESet bit 7 of (IX+d), 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d) - RESet bit 7 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                RESIX   7
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RES 7,(IX+d),A - RESet bit 7 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RES 7,(IY+d),A - RESet bit 7 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
res_7aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                RESIX   7, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),B - SET bit 0 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),B - SET bit 0 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),C - SET bit 0 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),C - SET bit 0 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),D - SET bit 0 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),D - SET bit 0 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),E - SET bit 0 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),E - SET bit 0 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),H - SET bit 0 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),H - SET bit 0 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),L - SET bit 0 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),L - SET bit 0 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d) - SET bit 0 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d) - SET bit 0 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 0,(IX+d),A - SET bit 0 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 0,(IY+d),A - SET bit 0 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_0aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   0, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),B - SET bit 1 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),B - SET bit 1 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),C - SET bit 1 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),C - SET bit 1 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),D - SET bit 1 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),D - SET bit 1 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),E - SET bit 1 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),E - SET bit 1 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),H - SET bit 1 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),H - SET bit 1 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),L - SET bit 1 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),L - SET bit 1 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d) - SET bit 1 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d) - SET bit 1 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 1,(IX+d),A - SET bit 1 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 1,(IY+d),A - SET bit 1 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_1aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   1, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),B - SET bit 2 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),B - SET bit 2 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),C - SET bit 2 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),C - SET bit 2 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),D - SET bit 2 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),D - SET bit 2 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),E - SET bit 2 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),E - SET bit 2 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),H - SET bit 2 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),H - SET bit 2 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),L - SET bit 2 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),L - SET bit 2 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d) - SET bit 2 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d) - SET bit 2 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 2,(IX+d),A - SET bit 2 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 2,(IY+d),A - SET bit 2 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_2aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   2, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),B - SET bit 3 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),B - SET bit 3 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),C - SET bit 3 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),C - SET bit 3 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),D - SET bit 3 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),D - SET bit 3 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),E - SET bit 3 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),E - SET bit 3 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),H - SET bit 3 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),H - SET bit 3 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),L - SET bit 3 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),L - SET bit 3 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d) - SET bit 3 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d) - SET bit 3 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 3,(IX+d),A - SET bit 3 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 3,(IY+d),A - SET bit 3 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_3aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   3, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),B - SET bit 4 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),B - SET bit 4 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),C - SET bit 4 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),C - SET bit 4 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),D - SET bit 4 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),D - SET bit 4 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),E - SET bit 4 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),E - SET bit 4 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),H - SET bit 4 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),H - SET bit 4 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),L - SET bit 4 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),L - SET bit 4 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d) - SET bit 4 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d) - SET bit 4 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 4,(IX+d),A - SET bit 4 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 4,(IY+d),A - SET bit 4 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_4aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   4, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),B - SET bit 5 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),B - SET bit 5 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),C - SET bit 5 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),C - SET bit 5 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),D - SET bit 5 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),D - SET bit 5 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),E - SET bit 5 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),E - SET bit 5 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),H - SET bit 5 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),H - SET bit 5 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),L - SET bit 5 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),L - SET bit 5 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d) - SET bit 5 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d) - SET bit 5 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 5,(IX+d),A - SET bit 5 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 5,(IY+d),A - SET bit 5 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_5aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   5, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),B - SET bit 6 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),B - SET bit 6 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),C - SET bit 6 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),C - SET bit 6 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),D - SET bit 6 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),D - SET bit 6 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),E - SET bit 6 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),E - SET bit 6 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),H - SET bit 6 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),H - SET bit 6 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),L - SET bit 6 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),L - SET bit 6 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d) - SET bit 6 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d) - SET bit 6 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 6,(IX+d),A - SET bit 6 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 6,(IY+d),A - SET bit 6 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_6aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   6, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),B - SET bit 7 of (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),B - SET bit 7 of (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, [ebx].regB
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),C - SET bit 7 of (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),C - SET bit 7 of (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, [ebx].regC
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),D - SET bit 7 of (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),D - SET bit 7 of (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, [ebx].regD
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),E - SET bit 7 of (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),E - SET bit 7 of (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, [ebx].regE
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),H - SET bit 7 of (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),H - SET bit 7 of (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, [ebx].regIXh
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),L - SET bit 7 of (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),L - SET bit 7 of (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, [ebx].regIXl
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d) - SET bit 7 of (IX+d), 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d) - SET bit 7 of (IY+d), 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7
                mov     [esi], cl
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SET 7,(IX+d),A - SET bit 7 of (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SET 7,(IY+d),A - SET bit 7 of (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
; . . . . . . . .
set_7aix:
                mov     dh, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SETIX   7, dh
                mov     [esi], dh
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),B - Rotate Left Cyclic (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),B - Rotate Left Cyclic (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_bix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),C - Rotate Left Cyclic (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),C - Rotate Left Cyclic (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_cix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),D - Rotate Left Cyclic (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),D - Rotate Left Cyclic (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_dix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),E - Rotate Left Cyclic (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),E - Rotate Left Cyclic (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_eix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),H - Rotate Left Cyclic (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),H - Rotate Left Cyclic (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_hix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),L - Rotate Left Cyclic (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),L - Rotate Left Cyclic (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_lix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d) - Rotate Left Cyclic (IX+d), 23 tacts (with prefix #DD,#CB).
;RLC (IY+d) - Rotate Left Cyclic (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_ix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RLC (IX+d),A - Rotate Left Cyclic (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RLC (IY+d),A - Rotate Left Cyclic (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rlc_aix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   l, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),B - Rotate Right Cyclic (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),B - Rotate Right Cyclic (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_bix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),C - Rotate Right Cyclic (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),C - Rotate Right Cyclic (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_cix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),D - Rotate Right Cyclic (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),D - Rotate Right Cyclic (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_dix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),E - Rotate Right Cyclic (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),E - Rotate Right Cyclic (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_eix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),H - Rotate Right Cyclic (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),H - Rotate Right Cyclic (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_hix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),L - Rotate Right Cyclic (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),L - Rotate Right Cyclic (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_lix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d) - Rotate Right Cyclic (IX+d), 23 tacts (with prefix #DD,#CB).
;RRC (IY+d) - Rotate Right Cyclic (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_ix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RRC (IX+d),A - Rotate Right Cyclic (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RRC (IY+d),A - Rotate Right Cyclic (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rrc_aix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   r, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),B - Rotate Left through carry (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),B - Rotate Left through carry (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),C - Rotate Left through carry (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),C - Rotate Left through carry (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),D - Rotate Left through carry (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),D - Rotate Left through carry (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),E - Rotate Left through carry (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),E - Rotate Left through carry (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),H - Rotate Left through carry (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),H - Rotate Left through carry (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),L - Rotate Left through carry (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),L - Rotate Left through carry (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d) - Rotate Left through carry (IX+d), 23 tacts (with prefix #DD,#CB).
;RL (IY+d) - Rotate Left through carry (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RL (IX+d),A - Rotate Left through carry (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RL (IY+d),A - Rotate Left through carry (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rl_aix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   lc, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),B - Rotate Right through carry (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),B - Rotate Right through carry (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_bix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),C - Rotate Right through carry (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),C - Rotate Right through carry (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_cix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),D - Rotate Right through carry (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),D - Rotate Right through carry (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_dix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),E - Rotate Right through carry (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),E - Rotate Right through carry (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_eix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),H - Rotate Right through carry (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),H - Rotate Right through carry (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_hix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),L - Rotate Right through carry (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),L - Rotate Right through carry (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_lix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d) - Rotate Right through carry (IX+d), 23 tacts (with prefix #DD,#CB).
;RR (IY+d) - Rotate Right through carry (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_ix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;RR (IX+d),A - Rotate Right through carry (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;RR (IY+d),A - Rotate Right through carry (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
rr_aix:
                mov     cl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                ROTIX   rc, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),B - Shift Left Arithmetic (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),B - Shift Left Arithmetic (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_bix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),C - Shift Left Arithmetic (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),C - Shift Left Arithmetic (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_cix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),D - Shift Left Arithmetic (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),D - Shift Left Arithmetic (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_dix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),E - Shift Left Arithmetic (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),E - Shift Left Arithmetic (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_eix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),H - Shift Left Arithmetic (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),H - Shift Left Arithmetic (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_hix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),L - Shift Left Arithmetic (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),L - Shift Left Arithmetic (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_lix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d) - Shift Left Arithmetic (IX+d), 23 tacts (with prefix #DD,#CB).
;SLA (IY+d) - Shift Left Arithmetic (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_ix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLA (IX+d),A - Shift Left Arithmetic (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SLA (IY+d),A - Shift Left Arithmetic (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sla_aix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX l, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),B - Shift Right Arithmetic (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),B - Shift Right Arithmetic (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_bix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),C - Shift Right Arithmetic (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),C - Shift Right Arithmetic (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_cix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),D - Shift Right Arithmetic (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),D - Shift Right Arithmetic (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_dix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),E - Shift Right Arithmetic (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),E - Shift Right Arithmetic (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_eix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),H - Shift Right Arithmetic (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),H - Shift Right Arithmetic (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_hix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),L - Shift Right Arithmetic (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),L - Shift Right Arithmetic (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_lix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d) - Shift Right Arithmetic (IX+d), 23 tacts (with prefix #DD,#CB).
;SRA (IY+d) - Shift Right Arithmetic (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_ix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRA (IX+d),A - Shift Right Arithmetic (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SRA (IY+d),A - Shift Right Arithmetic (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sra_aix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX r, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),B - Shift Left and Increment (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),B - Shift Left and Increment (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_bix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),C - Shift Left and Increment (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),C - Shift Left and Increment (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_cix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),D - Shift Left and Increment (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),D - Shift Left and Increment (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_dix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),E - Shift Left and Increment (IX+d) using E as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),E - Shift Left and Increment (IY+d) using E as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_eix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),H - Shift Left and Increment (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),H - Shift Left and Increment (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_hix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),L - Shift Left and Increment (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),L - Shift Left and Increment (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_lix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d) - Shift Left and Increment (IX+d), 23 tacts (with prefix #DD,#CB).
;SLI (IY+d) - Shift Left and Increment (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_ix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SLI (IX+d),A - Shift Left and Increment (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SLI (IY+d),A - Shift Left and Increment (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
sli_aix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX li, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),B - Shift Right Logical (IX+d) using B as temp register, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),B - Shift Right Logical (IY+d) using B as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_bix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, [ebx].regB
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),C - Shift Right Logical (IX+d) using C as temp register, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),C - Shift Right Logical (IY+d) using C as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_cix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, [ebx].regC
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),D - Shift Right Logical (IX+d) using D as temp register, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),D - Shift Right Logical (IY+d) using D as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_dix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, [ebx].regD
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),E - Shift Right Logical (IX+d) then copy to E, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),E - Shift Right Logical (IY+d) then copy to E, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_eix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, [ebx].regE
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),H - Shift Right Logical (IX+d) using H as temp register, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),H - Shift Right Logical (IY+d) using H as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_hix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, [ebx].regIXh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),L - Shift Right Logical (IX+d) using L as temp register, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),L - Shift Right Logical (IY+d) using L as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_lix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, [ebx].regIXl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d) - Shift Right Logical (IX+d), 23 tacts (with prefix #DD,#CB).
;SRL (IY+d) - Shift Right Logical (IY+d), 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_ix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;SRL (IX+d),A - Shift Right Logical (IX+d) using A as temp register, 23 tacts (with prefix #DD,#CB).
;SRL (IY+d),A - Shift Right Logical (IY+d) using A as temp register, 23 tacts (with prefix #FD,#CB).
;   5 0 3 P 0 
srl_aix:
                mov     dl, [esi]
                T_MRD   4               ; MRD+IOP=4
                MWR     ax
                SHIFTIX rl, dh
                mov     [esi], al
                T_MWR   3               ; MWR
                TOTAL   7
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

; Load registry Pair with X
Z80LPX          macro   regp
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; ORH
                mov     regp, ax
                TOTAL   10
                endm

;LD BC,nn - LoaD BC with nn, 10 tacts.
; . . . . . . . .
ld_bc:
                Z80LPX  [ebx].rBC
                ret

;LD DE,nn - LoaD DE with nn, 10 tacts.
; . . . . . . . .
ld_de:
                Z80LPX  [ebx].rDE
                ret

;LD HL,nn - LoaD HL with nn, 10 tacts.
;LD IX,nn - LoaD IX with nn, 14 tacts (with prefix #DD).
;LD IY,nn - LoaD IY with nn, 14 tacts (with prefix #FD).
; . . . . . . . .
ld_hl:
                Z80LPX  [ebx].rHL
                ret

;LD SP,nn - LoaD SP with nn, 10 tacts.
; . . . . . . . .
ld_sp:
                Z80LPX  [ebx].rSP
                ret

;----------------------------------------------------------------------------;

; Load registry Pair with (Memory)
Z80LPM          macro   regp
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; ORH
                MRD     ax
                inc     eax
                mov     [ebx].rMP, ax
                mov     cl, [esi]
                T_MRD   3               ; MRL
                MRD     ax
                mov     ch, [esi]
                T_MRD   3               ; MRH
                mov     regp, cx
                TOTAL   16
                endm

; Load (Memory) with registry Pair
Z80LMP          macro   regp
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; ORH
                MWR     ax
                inc     eax
                mov     cx, regp
                mov     [ebx].rMP, ax
                mov     [esi], cl
                T_MWR   3               ; MWL
                MWR     ax
                mov     [esi], ch
                T_MWR   3               ; MWH
                TOTAL   16
                endm


;LD BC,(nn) - LoaD BC with (nn), 20 tacts.
; . . . . . . . .
ld_bc_:
                Z80LPM  [ebx].rBC
                ret

;LD DE,(nn) - LoaD DE with (nn), 20 tacts.
; . . . . . . . .
ld_de_:
                Z80LPM  [ebx].rDE
                ret

;LD HL,(nn) - LoaD HL with (nn), 16 tacts.
;LD HL,(nn) - LoaD HL with (nn), 20 tacts (with prefix #ED).
;LD IX,(nn) - LoaD IX with (nn), 20 tacts (with prefix #DD).
;LD IY,(nn) - LoaD IY with (nn), 20 tacts (with prefix #FD).
; . . . . . . . .
ld_hl_:
                Z80LPM  [ebx].rHL
                ret

;LD SP,(nn) - LoaD SP with (nn), 20 tacts.
; . . . . . . . .
ld_sp_:
                Z80LPM  [ebx].rSP
                ret

;LD (nn),BC - LoaD (nn) with BC, 20 tacts.
; . . . . . . . .
ld__bc:
                Z80LMP  [ebx].rBC
                ret

;LD (nn),DE - LoaD (nn) with DE, 20 tacts.
; . . . . . . . .
ld__de:
                Z80LMP  [ebx].rDE
                ret

;LD (nn),HL - LoaD (nn) with HL, 16 tacts.
;LD (nn),HL - LoaD (nn) with HL, 20 tacts (with prefix #ED).
;LD (nn),IX - LoaD (nn) with IX, 20 tacts (with prefix #DD).
;LD (nn),IY - LoaD (nn) with IY, 20 tacts (with prefix #FD).
; . . . . . . . .
ld__hl:
                Z80LMP  [ebx].rHL
                ret

;LD (nn),SP - LoaD (nn) with SP, 20 tacts.
; . . . . . . . .
ld__sp:
                Z80LMP  [ebx].rSP
                ret

;----------------------------------------------------------------------------;

;INC BC - INCrement BC, 6 tacts.
; . . . . . . . .
inc_bc:
                T_M1R   6               ; M1, OCR
                inc     [ebx].rBC
                TOTAL   6
                ret

;INC DE - INCrement DE, 6 tacts.
; . . . . . . . .
inc_de:
                T_M1R   6               ; M1, OCR
                inc     [ebx].rDE
                TOTAL   6
                ret

;INC HL - INCrement HL, 6 tacts.
;INC IX - INCrement IX, 10 tacts (with prefix #DD).
;INC IY - INCrement IY, 10 tacts (with prefix #FD).
; . . . . . . . .
inc_hl:
                T_M1R   6               ; M1, OCR
                inc     [ebx].rHL
                TOTAL   6
                ret

;INC SP - INCrement SP, 6 tacts.
; . . . . . . . .
inc_sp:
                T_M1R   6               ; M1, OCR
                inc     [ebx].rSP
                TOTAL   6
                ret

;DEC BC - DECrement BC, 6 tacts.
; . . . . . . . .
dec_bc:
                T_M1R   6               ; M1, OCR
                dec     [ebx].rBC
                TOTAL   6
                ret

;DEC DE - DECrement DE, 6 tacts.
; . . . . . . . .
dec_de:
                T_M1R   6               ; M1, OCR
                dec     [ebx].rDE
                TOTAL   6
                ret

;DEC HL - DECrement HL, 6 tacts.
;DEC IX - DECrement IX, 10 tacts (with prefix #DD).
;DEC IY - DECrement IY, 10 tacts (with prefix #FD).
; . . . . . . . .
dec_hl:
                T_M1R   6               ; M1, OCR
                dec     [ebx].rHL
                TOTAL   6
                ret

;DEC SP - DECrement SP, 6 tacts.
; . . . . . . . .
dec_sp:
                T_M1R   6               ; M1, OCR
                dec     [ebx].rSP
                TOTAL   6
                ret

;----------------------------------------------------------------------------;

Z80PUSH         macro   regp
                T_M1R   5               ; M1, OCR
                ifdifi  <regp>, <dx>
                    mov     ax, regp
                endif
                xchg    di, [ebx].rSP
                dec     edi
                MWR     di
                ifdifi  <regp>, <dx>
                    mov     [esi], ah
                else
                    mov     [esi], dh
                endif
                T_MWR   3               ; SWH
                dec     edi
                MWR     di
                ifidni  <regp>, <dx>
                    mov     al, [ebx].flagPV
                    rol     al, 2
                    and     dl, not(Z80F_PV)
                    or      al, dl
                endif
                mov     [esi], al
                T_MWR   3               ; SWL
                xchg    di, [ebx].rSP
                TOTAL   11
                endm

Z80POP          macro   regp
                T_M1R   4               ; M1, OCR
                xchg    di, [ebx].rSP
                MRD     di
                inc     edi
                ifdifi  <regp>, <dx>
                    mov     al, [esi]
                else
                    mov     dl, [esi]
                    test    dl, 004h
                    setnz   [ebx].flagPV
                    and     dl, not(Z80F_PV)
                endif
                T_MRD   3               ; SRL
                MRD     di
                inc     edi
                ifdifi  <regp>, <dx>
                    mov     ah, [esi]
                else
                    mov     dh, [esi]
                endif
                T_MRD   3               ; SRH
                xchg    di, [ebx].rSP
                ifdifi  <regp>, <dx>
                    mov     regp, ax
                endif
                TOTAL   10
                endm


;PUSH BC - PUSH BC to stack, 11 tacts.
; . . . . . . . .
push_bc:
                Z80PUSH [ebx].rBC
                ret

;PUSH DE - PUSH DE to stack, 11 tacts.
; . . . . . . . .
push_de:
                Z80PUSH [ebx].rDE
                ret

;PUSH HL - PUSH HL to stack, 11 tacts.
; . . . . . . . .
push_hl:
                Z80PUSH [ebx].rHL
                ret

;PUSH AF - PUSH AF to stack, 11 tacts.
; . . . . . . . .
push_af:
                Z80PUSH dx
                ret

;POP BC - POP BC from stack, 10 tacts.
; . . . . . . . .
pop_bc:
                Z80POP  [ebx].rBC
                ret

;POP DE - POP DE from stack, 10 tacts.
; . . . . . . . .
pop_de:
                Z80POP  [ebx].rDE
                ret

;POP HL - POP HL from stack, 10 tacts.
; . . . . . . . .
pop_hl:
                Z80POP  [ebx].rHL
                ret

;POP AF - POP AF from stack, 10 tacts.
; . . . . . . . .
pop_af:
                Z80POP  dx
                ret

;----------------------------------------------------------------------------;

ADSBHL          macro   type, z80reg
                T_M1R   4, 11           ; M1, OCR=4; IOP=4; IOP=3
                mov     cx, [ebx].rHL
                mov     [ebx].rMP, cx
                inc     [ebx].rMP
                ifidni  <type>, <add>
                    and     dl, Z80F_S + Z80F_Z
                    add     cl, byte ptr z80reg
                    adc     ch, byte ptr z80reg + 1
                    lahf
                    and     ah, Z80F_H + Z80F_C
                    or      dl, ah
                    mov     al, ch
                    and     al, Z80F_5 + Z80F_3
                    or      dl, al
                else
                    shr     dl, 1
                    ifidni  <type>, <adc>
                        adc     cl, byte ptr z80reg
                        adc     ch, byte ptr z80reg + 1
                    endif
                    ifidni  <type>, <sbc>
                        sbb     cl, byte ptr z80reg
                        sbb     ch, byte ptr z80reg + 1
                    endif
                    lahf
                    seto    [ebx].flagPV
                    mov     dl, ch
                    and     dl, Z80F_5 + Z80F_3
                    and     ah, Z80F_S + Z80F_H + Z80F_C
                    or      dl, ah
                    or      cx, cx
                    setz    al
                    ror     al, 2
                    or      dl, al
                    ifidni  <type>, <sbc>
                        or      dl, Z80F_N
                    endif
                endif
                mov     [ebx].rHL, cx
                TOTAL   11
                endm

;ADD HL,BC - ADD HL with BC, 11 tacts.
;ADD IX,BC - ADD IX with BC, 15 tacts (with prefix #DD).
;ADD IY,BC - ADD IY with BC, 15 tacts (with prefix #FD).
; . . 5  3 . 0 
add_hl_bc:
                ADSBHL  add, [ebx].rBC
                ret

;ADD HL,DE - ADD HL with DE, 11 tacts.
;ADD IX,DE - ADD IX with DE, 15 tacts (with prefix #DD).
;ADD IY,DE - ADD IY with DE, 15 tacts (with prefix #FD).
; . . 5  3 . 0 
add_hl_de:
                ADSBHL  add, [ebx].rDE
                ret

;ADD HL,HL - ADD HL with HL, 11 tacts.
;ADD IX,IX - ADD IX with IX, 15 tacts (with prefix #DD).
;ADD IY,IY - ADD IY with IY, 15 tacts (with prefix #FD).
; . . 5  3 . 0 
add_hl_hl:
                ADSBHL  add, [ebx].rHL
                ret

;ADD HL,SP - ADD HL with SP, 11 tacts.
;ADD IX,SP - ADD IX with SP, 15 tacts (with prefix #DD).
;ADD IY,SP - ADD IY with SP, 15 tacts (with prefix #FD).
; . . 5  3 . 0 
add_hl_sp:
                ADSBHL  add, [ebx].rSP
                ret

;ADC HL,BC - ADd with Carry HL and BC, 15 tacts.
;   5  3 V 0 
adc_hl_bc:
                ADSBHL  adc, [ebx].rBC
                ret

;ADC HL,DE - ADd with Carry HL and DE, 15 tacts.
;   5  3 V 0 
adc_hl_de:
                ADSBHL  adc, [ebx].rDE
                ret

;ADC HL,HL - ADd with Carry HL and HL, 15 tacts.
;   5  3 V 0 
adc_hl_hl:
                ADSBHL  adc, [ebx].rHL
                ret

;ADC HL,SP - ADd with Carry SP and HL, 15 tacts.
;   5  3 V 0 
adc_hl_sp:
                ADSBHL  adc, [ebx].rSP
                ret

;SBC HL,BC - SuB with Carry BC from HL, 15 tacts.
;   5  3 V 1 
sbc_hl_bc:
                ADSBHL  sbc, [ebx].rBC
                ret

;SBC HL,DE - SuB with Carry DE from HL, 15 tacts.
;   5  3 V 1 
sbc_hl_de:
                ADSBHL  sbc, [ebx].rDE
                ret

;SBC HL,HL - SuB with Carry HL from HL, 15 tacts.
;   5  3 V 1 
sbc_hl_hl:
                ADSBHL  sbc, [ebx].rHL
                ret

;SBC HL,SP - SuB with Carry SP from HL, 15 tacts.
;   5  3 V 1 
sbc_hl_sp:
                ADSBHL  sbc, [ebx].rSP
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

Z80JR           macro   cc
                T_M1R   4               ; M1, OCR
                MRD     di
                movsx   eax, byte ptr [esi]
                ifb     <cc>
                    lea     edi, [edi + eax + 1]
                    T_MRD   8               ; ORD=3, IOP=5
                    mov     [ebx].rMP, di
                    TOTAL   12
                else
                    T_MRD   3               ; ORD
                    inc     edi
                    ifidni  <cc>, <nz>
                        test    dl, Z80F_Z
                        jz      jr_cc
                    endif
                    ifidni  <cc>, <z>
                        test    dl, Z80F_Z
                        jnz     jr_cc
                    endif
                    ifidni  <cc>, <nc>
                        test    dl, Z80F_C
                        jz      jr_cc
                    endif
                    ifidni  <cc>, <c>
                        test    dl, Z80F_C
                        jnz     jr_cc
                    endif
                    TOTAL   7
                endif
                endm

jr_cc:
                lea     edi, [edi + eax]
                T_IOP   5               ; IOP
                mov     [ebx].rMP, di
                TOTAL   12
                ret

;JR d - Jump Relative d, 12 tacts.
; . . . . . . . .
jr:
                Z80JR
                ret

;JR NZ,d - Jump Relative if Not Zero d, 12/7 tacts.
; . . . . . . . .
jr_nz:
                Z80JR   NZ
                ret
;JR Z,d - Jump Relative if Zero d, 12/7 tacts.
; . . . . . . . .
jr_z:
                Z80JR   Z
                ret

;JR NC,d - Jump Relative if Not Carry d, 12/7 tacts.
; . . . . . . . .
jr_nc:
                Z80JR   NC
                ret

;JR C,d - Jump Relative if Carry d, 12/7 tacts.
; . . . . . . . .
jr_c:
                Z80JR   C
                ret

;DJNZ d - Dec b and Jump relative if Not Zero, 13/8 tacts.
; . . . . . . . .
djnz:
                T_M1R   5               ; M1, OCR
                MRD     di
                movsx   eax, byte ptr [esi]
                T_MRD   3               ; ORD
                inc     edi
                dec     [ebx].regB
                jz      nodjnz
                lea     edi, [edi + eax]
                T_IOP   5               ; IOP
                mov     [ebx].rMP, di
                TOTAL   13
                ret
nodjnz:
                TOTAL   8
                ret

;----------------------------------------------------------------------------;

Z80JP           macro   cc
                local   nojp
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; ORL
                MRD     di
                mov     ah, [esi]
                T_MRD   3               ; ORH
                mov     [ebx].rMP, ax
                ifnb    <cc>
                    inc     edi
                    ifidni  <cc>, <nz>
                        test    dl, Z80F_Z
                        jnz     nojp
                    endif
                    ifidni  <cc>, <z>
                        test    dl, Z80F_Z
                        jz      nojp
                    endif
                    ifidni  <cc>, <nc>
                        test    dl, Z80F_C
                        jnz     nojp
                    endif
                    ifidni  <cc>, <c>
                        test    dl, Z80F_C
                        jz      nojp
                    endif
                    ifidni  <cc>, <po>
                        test    [ebx].flagPV, 001h
                        jnz     nojp
                    endif
                    ifidni  <cc>, <pe>
                        test    [ebx].flagPV, 001h
                        jz      nojp
                    endif
                    ifidni  <cc>, <p>
                        test    dl, Z80F_S
                        jnz     nojp
                    endif
                    ifidni  <cc>, <m>
                        test    dl, Z80F_S
                        jz      nojp
                    endif
                endif
                mov     edi, eax
nojp:
                TOTAL   10
                endm

;JP nn - JumP nn, 10 tacts.
; . . . . . . . .
jp:
                Z80JP
                ret

;JP NZ,nn - JumP if Not Zero nn, 10/10 tacts.
; . . . . . . . .
jp_nz:
                Z80JP   NZ
                ret

;JP Z,nn - JumP if Zero nn, 10/10 tacts.
; . . . . . . . .
jp_z:
                Z80JP   Z
                ret

;JP NC,nn - JumP if Not Carry nn, 10/10 tacts.
; . . . . . . . .
jp_nc:
                Z80JP   NC
                ret

;JP C,nn - JumP if Carry nn, 10/10 tacts.
; . . . . . . . .
jp_c:
                Z80JP   C
                ret

;JP PO,nn - JumP if not Parity nn, 10/10 tacts.
; . . . . . . . .
jp_po:
                Z80JP   PO
                ret

;JP PE,nn - JumP if Parity nn, 10/10 tacts.
; . . . . . . . .
jp_pe:
                Z80JP   PE
                ret

;JP P,nn - JumP if Plus nn, 10/10 tacts.
; . . . . . . . .
jp_p:
                Z80JP   P
                ret

;JP M,nn - JumP if Minus nn, 10/10 tacts.
; . . . . . . . .
jp_m:
                Z80JP   M
                ret

;----------------------------------------------------------------------------;

Z80CALL         macro   cc
                T_M1R   4               ; M1, OCR
                MRD     di
                mov     al, [esi]
                T_MRD   3               ; ORL
                inc     edi
                MRD     di
                mov     ah, [esi]
                ifb     <cc>
                    T_MRD   4               ; ORH+IOP=4
                    mov     [ebx].rMP, ax
                    mov     cx, [ebx].rSP
                    inc     edi
                    dec     cx
                    xchg    eax, edi
                    MWR     cx
                    mov     [esi], ah
                    dec     cx
                    T_MWR   3               ; SWH
                    MWR     cx
                    mov     [esi], al
                    mov     [ebx].rSP, cx
                    T_MWR   3               ; SWL
                    TOTAL   17
                else
                    T_MRD   3               ; ORH
                    inc     edi
                    mov     [ebx].rMP, ax
                    ifidni  <cc>, <nz>
                        test    dl, Z80F_Z
                        jz      call_cc
                    endif
                    ifidni  <cc>, <z>
                        test    dl, Z80F_Z
                        jnz     call_cc
                    endif
                    ifidni  <cc>, <nc>
                        test    dl, Z80F_C
                        jz      call_cc
                    endif
                    ifidni  <cc>, <c>
                        test    dl, Z80F_C
                        jnz     call_cc
                    endif
                    ifidni  <cc>, <po>
                        test    [ebx].flagPV, 001h
                        jz      call_cc
                    endif
                    ifidni  <cc>, <pe>
                        test    [ebx].flagPV, 001h
                        jnz     call_cc
                    endif
                    ifidni  <cc>, <p>
                        test    dl, Z80F_S
                        jz      call_cc
                    endif
                    ifidni  <cc>, <m>
                        test    dl, Z80F_S
                        jnz     call_cc
                    endif
                    TOTAL   10
                endif
                endm

;CALL nn - CALL nn, 17 tacts.
; . . . . . . . .
z80_call:
                Z80CALL
                ret

call_cc:
                mov     cx, [ebx].rSP
                T_IOP   1               ; +IOP
                dec     cx
                xchg    eax, edi
                MWR     cx
                mov     [esi], ah
                dec     cx
                T_MWR   3               ; SWH
                MWR     cx
                mov     [esi], al
                mov     [ebx].rSP, cx
                T_MWR   3               ; SWL
                TOTAL   17
                ret

;CALL NZ,nn - CALL if Not Zero nn, 17/10 tacts.
; . . . . . . . .
call_nz:
                Z80CALL NZ
                ret

;CALL Z,nn - CALL if Zero nn, 17/10 tacts.
; . . . . . . . .
call_z:
                Z80CALL Z
                ret

;CALL NC,nn - CALL if Not Carry nn, 17/10 tacts.
; . . . . . . . .
call_nc:
                Z80CALL NC
                ret

;CALL C,nn - CALL if Carry nn, 17/10 tacts.
; . . . . . . . .
call_c:
                Z80CALL C
                ret

;CALL PO,nn - CALL if not Parity nn, 17/10 tacts.
; . . . . . . . .
call_po:
                Z80CALL PO
                ret

;CALL PE,nn - CALL if Parity nn, 17/10 tacts.
; . . . . . . . .
call_pe:
                Z80CALL PE
                ret

;CALL P,nn - CALL if Plus nn, 17/10 tacts.
; . . . . . . . .
call_p:
                Z80CALL P
                ret

;CALL M,nn - CALL if Minus nn, 17/10 tacts.
; . . . . . . . .
call_m:
                Z80CALL M
                ret

;----------------------------------------------------------------------------;

Z80RST          macro   adr
                T_M1R   5               ; M1, OCR
                mov     eax, edi
                mov     di, [ebx].rSP
                dec     edi
                MWR     di
                mov     [esi], ah
                T_MWR   3               ; SWH
                dec     edi
                MWR     di
                mov     [esi], al
                T_MWR   3               ; SWL
                mov     [ebx].rSP, di
                if      adr eq 0
                    xor     edi, edi
                else
                    mov     edi, adr
                endif
                mov     [ebx].rMP, di
                TOTAL   11
                endm

;RST 0 - ReSTart 0, 11 tacts.
; . . . . . . . .
rst_0:
                Z80RST  0
                ret

;RST 8 - ReSTart 8, 11 tacts.
; . . . . . . . .
rst_8:
                Z80RST  8
                ret

;RST 16 - ReSTart 16, 11 tacts.
; . . . . . . . .
rst_16:
                Z80RST  16
                ret

;RST 24 - ReSTart 24, 11 tacts.
; . . . . . . . .
rst_24:
                Z80RST  24
                ret

;RST 32 - ReSTart 32, 11 tacts.
; . . . . . . . .
rst_32:
                Z80RST  32
                ret

;RST 40 - ReSTart 40, 11 tacts.
; . . . . . . . .
rst_40:
                Z80RST  40
                ret

;RST 48 - ReSTart 48, 11 tacts.
; . . . . . . . .
rst_48:
                Z80RST  48
                ret

;RST 56 - ReSTart 56, 11 tacts.
; . . . . . . . .
rst_56:
                Z80RST  56
                ret

;----------------------------------------------------------------------------;

;RET - RETurn, 10 tacts.
; . . . . . . . .
z80_ret:
                T_M1R   4               ; M1, OCR
                mov     di, [ebx].rSP
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; SRL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; SRH
                mov     [ebx].rSP, di
                mov     edi, eax
                mov     [ebx].rMP, ax
                TOTAL   10
                ret

;RET NZ - RETurn if Not Zero, 11/5 tacts.
; . . . . . . . .
ret_nz:
                T_M1R   5               ; M1, OCR
                test    dl, 040h
                jz      ret_cc
                TOTAL   5
                ret
ret_cc:
                mov     di, [ebx].rSP
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; SRL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; SRH
                mov     [ebx].rSP, di
                mov     edi, eax
                mov     [ebx].rMP, ax
                TOTAL   11
                ret

;RET Z - RETurn if Zero, 11/5 tacts.
; . . . . . . . .
ret_z:
                T_M1R   5               ; M1, OCR
                test    dl, 040h
                jnz     ret_cc
                TOTAL   5
                ret

;RET NC - RETurn if Not Carry, 11/5 tacts.
; . . . . . . . .
ret_nc:
                T_M1R   5               ; M1, OCR
                test    dl, 001h
                jz      ret_cc
                TOTAL   5
                ret

;RET C - RETurn if Carry, 11/5 tacts.
; . . . . . . . .
ret_c:
                T_M1R   5               ; M1, OCR
                test    dl, 001h
                jnz     ret_cc
                TOTAL   5
                ret

;RET PO - RETurn if not Parity, 11/5 tacts.
; . . . . . . . .
ret_po:
                T_M1R   5               ; M1, OCR
                test    [ebx].flagPV, 001h
                jz      ret_cc
                TOTAL   5
                ret

;RET PE - RETurn if Parity, 11/5 tacts.
; . . . . . . . .
ret_pe:
                T_M1R   5               ; M1, OCR
                test    [ebx].flagPV, 001h
                jnz     ret_cc
                TOTAL   5
                ret

;RET P - RETurn if Plus, 11/5 tacts.
; . . . . . . . .
ret_p:
                T_M1R   5               ; M1, OCR
                test    dl, 080h
                jz      ret_cc
                TOTAL   5
                ret

;RET M - RETurn if Minus, 11/5 tacts.
; . . . . . . . .
ret_m:
                T_M1R   5               ; M1, OCR
                test    dl, 080h
                jnz     ret_cc
                TOTAL   5
                ret

;----------------------------------------------------------------------------;

;RETN - ret from Non-maskable interrupt, 14 tacts (with prefix #ED).
; . . . . . . . .
z80_retn:
                T_M1R   4               ; M1, OCR
                mov     di, [ebx].rSP
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; SRL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; SRH
                mov     [ebx].rSP, di
                mov     edi, eax
                mov     [ebx].rMP, ax
                test    [ebx].FlagsA, FA_IFF2_
                setz    al
                and     [ebx].FlagsA, not(FA_IFF1_)
                dec     al
                and     al, FA_IFF1_
                or      [ebx].FlagsA, al
                TOTAL   10
                ret

;RETI - ret from maskable Interrupt, 14 tacts (with prefix #ED).
; . . . . . . . .
z80_reti:
                T_M1R   4               ; M1, OCR
                mov     di, [ebx].rSP
                MRD     di
                inc     edi
                mov     al, [esi]
                T_MRD   3               ; SRL
                MRD     di
                inc     edi
                mov     ah, [esi]
                T_MRD   3               ; SRH
                mov     [ebx].rSP, di
                mov     edi, eax
                mov     [ebx].rMP, ax
                test    [ebx].FlagsA, FA_IFF2_
                setz    al
                and     [ebx].FlagsA, not(FA_IFF1_)
                dec     al
                and     al, FA_IFF1_
                or      [ebx].FlagsA, al
                TOTAL   10
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;IN A,(n) - INput A from port n, 11 tacts.
; . . . . . . . .
in_a:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     cl, [esi]
                T_MRD   3               ; ORD
                lea     eax, [ecx + 1]
                mov     [ebx].regMPl, al
                mov     [ebx].regMPh, dh
                push    offset ia_1
                mov     ch, dh
                jmp     ZXPort_In
ia_1:
                mov     dh, ah
                T_PRD   4               ; PRD
                TOTAL   11
                ret

;OUT (n),A - OUTput A to port n, 11 tacts.
; . . . . . . . .
out_a:
                T_M1R   4               ; M1, OCR
                MRD     di
                inc     edi
                mov     cl, [esi]
                T_MRD   3               ; ORD
                mov     ch, dh
                lea     eax, [ecx + 1]
                mov     [ebx].regMPl, al
                mov     [ebx].regMPh, dh
                push    offset oa_1
                mov     ah, dh
                jmp     ZXPort_Out
oa_1:
                T_PWR   4               ; PWR
                TOTAL   11
                ret

;----------------------------------------------------------------------------;

;IN B,(C) - INput B from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_b_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ibc1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
ibc1:
                T_PRD   4               ; PRD
                mov     [ebx].regB, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN C,(C) - INput C from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_c_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset icc1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
icc1:
                T_PRD   4               ; PRD
                mov     [ebx].regC, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN D,(C) - INput D from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_d_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset idc1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
idc1:
                T_PRD   4               ; PRD
                mov     [ebx].regD, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN E,(C) - INput E from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_e_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset iec1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
iec1:
                T_PRD   4               ; PRD
                mov     [ebx].regE, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN H,(C) - INput H from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_h_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ihc1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
ihc1:
                T_PRD   4               ; PRD
                mov     [ebx].regH, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN L,(C) - INput L from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_l_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ilc1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
ilc1:
                T_PRD   4               ; PRD
                mov     [ebx].regL, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN F,(C) - INput F from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_f_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ifc1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
ifc1:
                T_PRD   4               ; PRD
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;IN A,(C) - INput A from port BC, 12 tacts (with prefix #ED).
;   x  x P 0 .
in_a_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset iac1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                jmp     ZXPort_In
iac1:
                T_PRD   4               ; PRD
                mov     dh, ah
                and     dl, 029h
                inc     ah
                dec     ah
                lahf
                setp    [ebx].flagPV
                and     ah, 0D0h
                or      dl, ah
                TOTAL   8
                ret

;OUT (C),B - save B to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_b:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ocb1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, [ebx].regB
                jmp     ZXPort_Out
ocb1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),C - save C to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_c:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset occ1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, [ebx].regC
                jmp     ZXPort_Out
occ1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),D - save D to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_d:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ocd1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, [ebx].regD
                jmp     ZXPort_Out
ocd1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),E - save E to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_e:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset oce1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, [ebx].regE
                jmp     ZXPort_Out
oce1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),H - save H to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_h:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset och1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, [ebx].regH
                jmp     ZXPort_Out
och1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),L - save L to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_l:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ocl1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, [ebx].regL
                jmp     ZXPort_Out
ocl1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),0 - OUTput 0 to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_f:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset ocf1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                xor     ah, ah
                jmp     ZXPort_Out
ocf1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;OUT (C),A - save A to port BC, 12 tacts (with prefix #ED).
; . . . . . . . .
out_c_a:
                T_M1R   4               ; M1, OCR
                mov     cx, [ebx].rBC
                push    offset oca1
                lea     eax, [ecx + 1]
                mov     [ebx].rMP, ax
                mov     ah, dh
                jmp     ZXPort_Out
oca1:
                T_PWR   4               ; PWR
                TOTAL   8
                ret

;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;
;----------------------------------------------------------------------------;

;LDI - LoaD (de) with (hl) and Increment, 16 tacts (with prefix #ED).
; . . L 0 L  0 .
ldi:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rHL
                and     dl, Z80F_S + Z80F_Z + Z80F_C
                MRD     ax
                inc     eax
                mov     [ebx].rHL, ax
                mov     cx, [ebx].rDE
                mov     al, [esi]
                T_MRD   3               ; MRD
                MWR     cx
                mov     [esi], al
                inc     ecx
                T_MWR   5               ; MWR+IOP=5
                mov     [ebx].rDE, cx
                add     al, dh
                mov     ah, al
                and     ax, 00208h
                shl     ah, 4
                or      dl, al
                or      dl, ah
                dec     [ebx].rBC
                setnz   [ebx].flagPV
                TOTAL   12
                ret

;LDD - LoaD (de) with (hl) and Decrement, 16 tacts (with prefix #ED).
; . . L 0 L  0 .
ldd:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rHL
                and     dl, Z80F_S + Z80F_Z + Z80F_C
                MRD     ax
                dec     eax
                mov     [ebx].rHL, ax
                mov     cx, [ebx].rDE
                mov     al, [esi]
                T_MRD   3               ; MRD
                MWR     cx
                mov     [esi], al
                dec     ecx
                T_MWR   5               ; MWR+IOP=5
                mov     [ebx].rDE, cx
                add     al, dh
                mov     ah, al
                and     eax, 00208h
                shl     ah, 4
                or      dl, al
                or      dl, ah
                dec     [ebx].rBC
                setnz   [ebx].flagPV
                TOTAL   12
                ret

;CPI - ComPare a with (hl) and Increment, 16 tacts (with prefix #ED).
;   L  L  1 .
cpi:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rHL
                and     dl, 001h
                MRD     ax
                inc     eax
                mov     [ebx].rHL, ax
                mov     al, dh
                mov     ah, [esi]
                T_MRD   8               ; MRD=3, IOP=5
                sub     al, ah
                lahf
                and     ah, 0D0h
                or      dl, ah
                dec     [ebx].rBC
                setnz   [ebx].flagPV
                inc     [ebx].rMP
                test    dl, 010h
                setnz   ah
                sub     al, ah
                mov     ah, al
                and     eax, 00208h
                shl     ah, 4
                or      dl, al
                or      dl, ah
                or      dl, 002h
                TOTAL   12
                ret

;CPD - ComPare a with (hl) and Decrement, 16 tacts (with prefix #ED).
;   L  L  1 .
cpd:
                T_M1R   4               ; M1, OCR
                mov     ax, [ebx].rHL
                and     dl, 001h
                MRD     ax
                dec     eax
                mov     [ebx].rHL, ax
                mov     al, dh
                mov     ah, [esi]
                T_MRD   8               ; MRD=3, IOP=5
                sub     al, ah
                lahf
                and     ah, 0D0h
                or      dl, ah
                dec     [ebx].rBC
                setnz   [ebx].flagPV
                dec     [ebx].rMP
                test    dl, 010h
                setnz   ah
                sub     al, ah
                mov     ah, al
                and     eax, 00208h
                shl     ah, 4
                or      dl, al
                or      dl, ah
                or      dl, 002h
                TOTAL   12
                ret

;INI - load (hl) with port (bc) and Increment b, 16 tacts (with prefix #ED).
;   5 x 3 x x x
ini:
                T_M1R   5               ; M1, OCR
                push    offset ini1
                mov     cx, [ebx].rBC
                jmp     ZXPort_In
ini1:
                mov     cx, [ebx].rHL
                T_PRD   4               ; PRD
                MWR     cx
                inc     ecx
                mov     [esi], ah
                mov     [ebx].rHL, cx
                T_MWR   3               ; MWR
                mov     al, [ebx].regC        ; carry flag undocumented behavior
                inc     al              ; ...
                add     al, ah          ; ...
                setc    dl              ; ...
                mov     al, dl          ; half-carry ==//==
                shl     al, 4           ; ...
                or      al, dl          ; ...
                test    ah, 080h        ; negative ==//==
                setnz   al              ; ...
                shl     al, 1           ; ...
                or      dl, al          ; ...
                ;movzx   ecx, [ebx].regC
                ;or      cl, 008h
                ;and     ecx, 0071Ch
                ;or      cl, ch
                ;xor     ch, ch
                ;mov     al, [ecx + Z80_Undoc_IOiPV]
                dec     [ebx].regB
                lahf
                mov     al, [ebx].regB        ; sign, F3 and F5 flags ==//==
                and     eax, 040A8h     ; ...
                or      dl, ah          ; ...
                or      dl, al          ; ...
                TOTAL   12
                ret

;IND - read (hl) with port (bc) and Decrement b, 16 tacts (with prefix #ED).
;   5 x 3 x x x
ind:
                T_M1R   5               ; M1, OCR
                push    offset ind1
                mov     cx, [ebx].rBC
                jmp     ZXPort_In
ind1:
                mov     cx, [ebx].rHL
                T_M1R   4               ; PRD
                MWR     cx
                dec     ecx
                mov     [esi], ah
                mov     [ebx].rHL, cx
                T_MWR   3               ; MWR
                mov     al, [ebx].regC        ; carry flag undocumented behavior
                dec     al              ; ...
                add     al, ah          ; ...
                setc    dl              ; ...
                mov     al, dl          ; half-carry ==//==
                shl     al, 4           ; ...
                or      al, dl          ; ...
                test    ah, 080h        ; negative ==//==
                setnz   al              ; ...
                shl     al, 1           ; ...
                or      dl, al          ; ...
                dec     [ebx].regB
                lahf
                mov     al, [ebx].regB        ; sign, F3 and F5 flags ==//==
                and     eax, 040A8h     ; ...
                or      dl, ah          ; ...
                or      dl, al          ; ...
                TOTAL   12
                ret

;OUTI - write (hl) to port (bc) and Increment b, 16 tacts (with prefix #ED).
;   5 x 3 x x x
outi:
                T_M1R   5               ; M1, OCR
                mov     ax, [ebx].rHL
                MRD     ax
                inc     eax
                mov     [ebx].rHL, ax
                mov     ah, [esi]
                T_MRD   3               ; MRD
                mov     al, [ebx].regC        ; carry flag undocumented behavior
                inc     al              ; ...
                add     al, ah          ; ...
                setc    dl              ; ...
                mov     al, dl          ; half-carry ==//==
                shl     al, 4           ; ...
                or      al, dl          ; ...
                test    ah, 080h        ; negative ==//==
                setnz   al              ; ...
                shl     al, 1           ; ...
                or      dl, al          ; ...
                push    offset outi1
                mov     cx, [ebx].rBC
                jmp     ZXPort_Out
outi1:
                T_PWR   4               ; PWR
                dec     [ebx].regB
                lahf
                mov     al, [ebx].regB        ; sign, F3 and F5 flags ==//==
                and     eax, 040A8h     ; ...
                or      dl, ah          ; ...
                or      dl, al          ; ...
                TOTAL   12
                ret

;OUTD - write (hl) to port (bc) and Decrement b, 16 tacts (with prefix #ED).
;   5 x 3 x x x
outd:
                T_M1R   5               ; M1, OCR
                mov     ax, [ebx].rHL
                MRD     ax
                dec     eax
                mov     [ebx].rHL, ax
                mov     ah, [esi]
                T_MRD   3               ; MRD
                mov     al, [ebx].regC        ; carry flag undocumented behavior
                dec     al              ; ...
                add     al, ah          ; ...
                setc    dl              ; ...
                mov     al, dl          ; half-carry ==//==
                shl     al, 4           ; ...
                or      al, dl          ; ...
                test    ah, 080h        ; negative ==//==
                setnz   al              ; ...
                shl     al, 1           ; ...
                or      dl, al          ; ...
                push    offset outd1
                mov     cx, [ebx].rBC
                jmp     ZXPort_Out
outd1:
                T_M1R   4               ; PWR
                dec     [ebx].regB
                lahf
                mov     al, [ebx].regB        ; sign, F3 and F5 flags ==//==
                and     eax, 040A8h     ; ...
                or      dl, ah          ; ...
                or      dl, al          ; ...
                TOTAL   12
                ret

;----------------------------------------------------------------------------;

;LDIR - LDI and Repeat until bc=0, 21/16 tacts (with prefix #ED).
; . . ? 0 ? 0 0 .
ldir:
                push    offset ldir1
                jmp     ldi
ldir1:
                test    [ebx].flagPV, 001h
                jz      cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                lea     eax, [edi + 1]
                mov     [ebx].rMP, ax
                TOTAL   5
                ret

;LDDR - LDD and Repeat until bc=0, 21/16 tacts (with prefix #ED).
; . . ? 0 ? 0 0 .
lddr:
                push    offset lddr1
                jmp     ldd
lddr1:
                test    [ebx].flagPV, 001h
                jz      cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                lea     eax, [edi + 1]
                mov     [ebx].rMP, ax
                TOTAL   5
cont:
                ret

;CPIR - CPI and Repeat until bc=0 or zero flag, 21/16 tacts (with prefix #ED).
; ? ? ? ? ? ? 1 .
cpir:
                push    offset cpir1
                jmp     cpi
cpir1:
                test    dl, 040h
                jnz     cont
                test    [ebx].flagPV, 001h
                jz      cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                lea     eax, [edi + 1]
                mov     [ebx].rMP, ax
                TOTAL   5
                ret

;CPDR - CPD and Repeat until bc=0 or zero flag, 21/16 tacts (with prefix #ED).
; ? ? ? ? ? ? 1 .
cpdr:
                push    offset cpdr1
                jmp     cpd
cpdr1:
                test    dl, 040h
                jnz     cont
                test    [ebx].flagPV, 001h
                jz      cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                lea     eax, [edi + 1]
                mov     [ebx].rMP, ax
                TOTAL   5
                ret

;INIR - INI and Repeat until b=0, 21/16 tacts (with prefix #ED).
; x x x x x x x x
inir:
                push    offset inir1
                jmp     ini
inir1:
                test    dl, 040h
                jnz     cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                TOTAL   5
                ret

;INDR - IND and Repeat until b=0, 21/16 tacts (with prefix #ED).
; x x x x x x x x
indr:
                push    offset indr1
                jmp     ind
indr1:
                test    dl, 040h
                jnz     cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                TOTAL   5
                ret

;OTIR - OuTI and Repeat until b=0, 21/16 tacts (with prefix #ED).
; x x x x x x x x
outir:
                push    offset outir1
                jmp     outi
outir1:
                test    dl, 040h
                jnz     cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                TOTAL   5
                ret

;OTDR - OuTD and Repeat until b=0, 21/16 tacts (with prefix #ED).
; x x x x x x x x
outdr:
                push    offset outdr1
                jmp     outd
outdr1:
                test    dl, 040h
                jnz     cont
                lea     edi, [edi - 2]
                T_IOP   5               ; IOP
                TOTAL   5
                ret



;****************************************************************************;

.DATA

Z80_Undoc_IOiPV     label   byte
                    db      0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0
Z80_Undoc_IOdPV     label   byte
                    db      0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1

Z80_CMDs_General    label   dword
                    dd      z80_nop,    ld_bc,      ld_bc_a,    inc_bc,     inc_b,      dec_b,      ld_b,       rlca
                    dd      ex_af,      add_hl_bc,  ld_a_bc,    dec_bc,     inc_c,      dec_c,      ld_c,       rrca
                    dd      djnz,       ld_de,      ld_de_a,    inc_de,     inc_d,      dec_d,      ld_d,       rla
                    dd      jr,         add_hl_de,  ld_a_de,    dec_de,     inc_e,      dec_e,      ld_e,       rra
                    dd      jr_nz,      ld_hl,      ld__hl,     inc_hl,     inc_h,      dec_h,      ld_h,       zx_daa
                    dd      jr_z,       add_hl_hl,  ld_hl_,     dec_hl,     inc_l,      dec_l,      ld_l,       cpl
                    dd      jr_nc,      ld_sp,      ld__a,      inc_sp,     inc_hl_,    dec_hl_,    ld__hl_,    scf
                    dd      jr_c,       add_hl_sp,  ld_a_,      dec_sp,     inc_a,      dec_a,      ld_a,       ccf
                    dd      z80_nop,    ld_b_c,     ld_b_d,     ld_b_e,     ld_b_h,     ld_b_l,     ld_b_hl,    ld_b_a
                    dd      ld_c_b,     z80_nop,    ld_c_d,     ld_c_e,     ld_c_h,     ld_c_l,     ld_c_hl,    ld_c_a
                    dd      ld_d_b,     ld_d_c,     z80_nop,    ld_d_e,     ld_d_h,     ld_d_l,     ld_d_hl,    ld_d_a
                    dd      ld_e_b,     ld_e_c,     ld_e_d,     z80_nop,    ld_e_h,     ld_e_l,     ld_e_hl,    ld_e_a
                    dd      ld_h_b,     ld_h_c,     ld_h_d,     ld_h_e,     z80_nop,    ld_h_l,     ld_h_hl,    ld_h_a
                    dd      ld_l_b,     ld_l_c,     ld_l_d,     ld_l_e,     ld_l_h,     z80_nop,    ld_l_hl,    ld_l_a
                    dd      ld_hl_b,    ld_hl_c,    ld_hl_d,    ld_hl_e,    ld_hl_h,    ld_hl_l,    halt,       ld_hl_a
                    dd      ld_a_b,     ld_a_c,     ld_a_d,     ld_a_e,     ld_a_h,     ld_a_l,     ld_a_hl,    z80_nop
                    dd      add_b,      add_c,      add_d,      add_e,      add_h,      add_l,      add_hl,     add_a
                    dd      adc_b,      adc_c,      adc_d,      adc_e,      adc_h,      adc_l,      adc_hl,     adc_a
                    dd      sub_b,      sub_c,      sub_d,      sub_e,      sub_h,      sub_l,      sub_hl,     sub_a
                    dd      sbc_b,      sbc_c,      sbc_d,      sbc_e,      sbc_h,      sbc_l,      sbc_hl,     sbc_a
                    dd      and_b,      and_c,      and_d,      and_e,      and_h,      and_l,      and_hl,     and_a
                    dd      xor_b,      xor_c,      xor_d,      xor_e,      xor_h,      xor_l,      xor_hl,     xor_a
                    dd      or_b,       or_c,       or_d,       or_e,       or_h,       or_l,       or_hl,      or_a
                    dd      cp_b,       cp_c,       cp_d,       cp_e,       cp_h,       cp_l,       cp_hl,      cp_a
                    dd      ret_nz,     pop_bc,     jp_nz,      jp,         call_nz,    push_bc,    add_n,      rst_0
                    dd      ret_z,      z80_ret,    jp_z,       prefix_cb,  call_z,     z80_call,   adc_n,      rst_8
                    dd      ret_nc,     pop_de,     jp_nc,      out_a,      call_nc,    push_de,    sub_n,      rst_16
                    dd      ret_c,      exx,        jp_c,       in_a,       call_c,     prefix_ix,  sbc_n,      rst_24
                    dd      ret_po,     pop_hl,     jp_po,      ex_sp_hl,   call_po,    push_hl,    and_n,      rst_32
                    dd      ret_pe,     jmp_hl,     jp_pe,      ex_de_hl,   call_pe,    prefix_ed,  xor_n,      rst_40
                    dd      ret_p,      pop_af,     jp_p,       zx_di,      call_p,     push_af,    or_n,       rst_48
                    dd      ret_m,      ld_sp_hl,   jp_m,       zx_ei,      call_m,     prefix_iy,  cp_n,       rst_56
Z80_CMDs_0xCB       label   dword
                    dd      rlc_b,      rlc_c,      rlc_d,      rlc_e,      rlc_h,      rlc_l,      rlc_hl,      rlc_a
                    dd      rrc_b,      rrc_c,      rrc_d,      rrc_e,      rrc_h,      rrc_l,      rrc_hl,      rrc_a
                    dd      rl_b,       rl_c,       rl_d,       rl_e,       rl_h,       rl_l,       rl_hl,       rl_a
                    dd      rr_b,       rr_c,       rr_d,       rr_e,       rr_h,       rr_l,       rr_hl,       rr_a
                    dd      sla_b,      sla_c,      sla_d,      sla_e,      sla_h,      sla_l,      sla_hl,      sla_a
                    dd      sra_b,      sra_c,      sra_d,      sra_e,      sra_h,      sra_l,      sra_hl,      sra_a
                    dd      sli_b,      sli_c,      sli_d,      sli_e,      sli_h,      sli_l,      sli_hl,      sli_a
                    dd      srl_b,      srl_c,      srl_d,      srl_e,      srl_h,      srl_l,      srl_hl,      srl_a
                    dd      bit_0b,     bit_0c,     bit_0d,     bit_0e,     bit_0h,     bit_0l,     bit_0hl,     bit_0a
                    dd      bit_1b,     bit_1c,     bit_1d,     bit_1e,     bit_1h,     bit_1l,     bit_1hl,     bit_1a
                    dd      bit_2b,     bit_2c,     bit_2d,     bit_2e,     bit_2h,     bit_2l,     bit_2hl,     bit_2a
                    dd      bit_3b,     bit_3c,     bit_3d,     bit_3e,     bit_3h,     bit_3l,     bit_3hl,     bit_3a
                    dd      bit_4b,     bit_4c,     bit_4d,     bit_4e,     bit_4h,     bit_4l,     bit_4hl,     bit_4a
                    dd      bit_5b,     bit_5c,     bit_5d,     bit_5e,     bit_5h,     bit_5l,     bit_5hl,     bit_5a
                    dd      bit_6b,     bit_6c,     bit_6d,     bit_6e,     bit_6h,     bit_6l,     bit_6hl,     bit_6a
                    dd      bit_7b,     bit_7c,     bit_7d,     bit_7e,     bit_7h,     bit_7l,     bit_7hl,     bit_7a
                    dd      res_0b,     res_0c,     res_0d,     res_0e,     res_0h,     res_0l,     res_0hl,     res_0a
                    dd      res_1b,     res_1c,     res_1d,     res_1e,     res_1h,     res_1l,     res_1hl,     res_1a
                    dd      res_2b,     res_2c,     res_2d,     res_2e,     res_2h,     res_2l,     res_2hl,     res_2a
                    dd      res_3b,     res_3c,     res_3d,     res_3e,     res_3h,     res_3l,     res_3hl,     res_3a
                    dd      res_4b,     res_4c,     res_4d,     res_4e,     res_4h,     res_4l,     res_4hl,     res_4a
                    dd      res_5b,     res_5c,     res_5d,     res_5e,     res_5h,     res_5l,     res_5hl,     res_5a
                    dd      res_6b,     res_6c,     res_6d,     res_6e,     res_6h,     res_6l,     res_6hl,     res_6a
                    dd      res_7b,     res_7c,     res_7d,     res_7e,     res_7h,     res_7l,     res_7hl,     res_7a
                    dd      set_0b,     set_0c,     set_0d,     set_0e,     set_0h,     set_0l,     set_0hl,     set_0a
                    dd      set_1b,     set_1c,     set_1d,     set_1e,     set_1h,     set_1l,     set_1hl,     set_1a
                    dd      set_2b,     set_2c,     set_2d,     set_2e,     set_2h,     set_2l,     set_2hl,     set_2a
                    dd      set_3b,     set_3c,     set_3d,     set_3e,     set_3h,     set_3l,     set_3hl,     set_3a
                    dd      set_4b,     set_4c,     set_4d,     set_4e,     set_4h,     set_4l,     set_4hl,     set_4a
                    dd      set_5b,     set_5c,     set_5d,     set_5e,     set_5h,     set_5l,     set_5hl,     set_5a
                    dd      set_6b,     set_6c,     set_6d,     set_6e,     set_6h,     set_6l,     set_6hl,     set_6a
                    dd      set_7b,     set_7c,     set_7d,     set_7e,     set_7h,     set_7l,     set_7hl,     set_7a
Z80_CMDs_0xED       label   dword
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      in_b_c,     out_c_b,    sbc_hl_bc,  ld__bc,     z80_neg,    z80_retn,   im_0,       ld_i_a
                    dd      in_c_c,     out_c_c,    adc_hl_bc,  ld_bc_,     z80_neg,    z80_reti,   im_0,       ld_r_a
                    dd      in_d_c,     out_c_d,    sbc_hl_de,  ld__de,     z80_neg,    z80_retn,   im_1,       ld_a_i
                    dd      in_e_c,     out_c_e,    adc_hl_de,  ld_de_,     z80_neg,    z80_reti,   im_2,       ld_a_r
                    dd      in_h_c,     out_c_h,    sbc_hl_hl,  ld__hl,     z80_neg,    z80_retn,   im_0,       rrd
                    dd      in_l_c,     out_c_l,    adc_hl_hl,  ld_hl_,     z80_neg,    z80_reti,   im_0,       rld
                    dd      in_f_c,     out_c_f,    sbc_hl_sp,  ld__sp,     z80_neg,    z80_retn,   im_1,       z80_nop
                    dd      in_a_c,     out_c_a,    adc_hl_sp,  ld_sp_,     z80_neg,    z80_reti,   im_2,       z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      ldi,        cpi,        ini,        outi,       z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      ldd,        cpd,        ind,        outd,       z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      ldir,       cpir,       inir,       outir,      z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      lddr,       cpdr,       indr,       outdr,      z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
                    dd      z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop,    z80_nop
Z80_CMDs_Index      label   dword
                    dd      z80_nop,    ld_bc,      ld_bc_a,    inc_bc,     inc_b,      dec_b,      ld_b,       rlca
                    dd      ex_af,      add_hl_bc,  ld_a_bc,    dec_bc,     inc_c,      dec_c,      ld_c,       rrca
                    dd      djnz,       ld_de,      ld_de_a,    inc_de,     inc_d,      dec_d,      ld_d,       rla
                    dd      jr,         add_hl_de,  ld_a_de,    dec_de,     inc_e,      dec_e,      ld_e,       rra
                    dd      jr_nz,      ld_hl,      ld__hl,     inc_hl,     inc_h,      dec_h,      ld_h,       zx_daa
                    dd      jr_z,       add_hl_hl,  ld_hl_,     dec_hl,     inc_l,      dec_l,      ld_l,       cpl
                    dd      jr_nc,      ld_sp,      ld__a,      inc_sp,     inc_ix_,    dec_ix_,    ld__ix_,    scf
                    dd      jr_c,       add_hl_sp,  ld_a_,      dec_sp,     inc_a,      dec_a,      ld_a,       ccf
                    dd      z80_nop,    ld_b_c,     ld_b_d,     ld_b_e,     ld_b_h,     ld_b_l,     ld_b_ix,    ld_b_a
                    dd      ld_c_b,     z80_nop,    ld_c_d,     ld_c_e,     ld_c_h,     ld_c_l,     ld_c_ix,    ld_c_a
                    dd      ld_d_b,     ld_d_c,     z80_nop,    ld_d_e,     ld_d_h,     ld_d_l,     ld_d_ix,    ld_d_a
                    dd      ld_e_b,     ld_e_c,     ld_e_d,     z80_nop,    ld_e_h,     ld_e_l,     ld_e_ix,    ld_e_a
                    dd      ld_h_b,     ld_h_c,     ld_h_d,     ld_h_e,     z80_nop,    ld_h_l,     ld_h_ix,    ld_h_a
                    dd      ld_l_b,     ld_l_c,     ld_l_d,     ld_l_e,     ld_l_h,     z80_nop,    ld_l_ix,    ld_l_a
                    dd      ld_ix_b,    ld_ix_c,    ld_ix_d,    ld_ix_e,    ld_ix_h,    ld_ix_l,    halt,       ld_ix_a
                    dd      ld_a_b,     ld_a_c,     ld_a_d,     ld_a_e,     ld_a_h,     ld_a_l,     ld_a_ix,    z80_nop
                    dd      add_b,      add_c,      add_d,      add_e,      add_h,      add_l,      add_ix,     add_a
                    dd      adc_b,      adc_c,      adc_d,      adc_e,      adc_h,      adc_l,      adc_ix,     adc_a
                    dd      sub_b,      sub_c,      sub_d,      sub_e,      sub_h,      sub_l,      sub_ix,     sub_a
                    dd      sbc_b,      sbc_c,      sbc_d,      sbc_e,      sbc_h,      sbc_l,      sbc_ix,     sbc_a
                    dd      and_b,      and_c,      and_d,      and_e,      and_h,      and_l,      and_ix,     and_a
                    dd      xor_b,      xor_c,      xor_d,      xor_e,      xor_h,      xor_l,      xor_ix,     xor_a
                    dd      or_b,       or_c,       or_d,       or_e,       or_h,       or_l,       or_ix,      or_a
                    dd      cp_b,       cp_c,       cp_d,       cp_e,       cp_h,       cp_l,       cp_ix,      cp_a
                    dd      ret_nz,     pop_bc,     jp_nz,      jp,         call_nz,    push_bc,    add_n,      rst_0
                    dd      ret_z,      z80_ret,    jp_z,       prefix_cbx, call_z,     z80_call,   adc_n,      rst_8
                    dd      ret_nc,     pop_de,     jp_nc,      out_a,      call_nc,    push_de,    sub_n,      rst_16
                    dd      ret_c,      exx,        jp_c,       in_a,       call_c,     prefix_ixx, sbc_n,      rst_24
                    dd      ret_po,     pop_hl,     jp_po,      ex_sp_hl,   call_po,    push_hl,    and_n,      rst_32
                    dd      ret_pe,     jmp_hl,     jp_pe,      _ex_de_hl,  call_pe,    prefix_edx, xor_n,      rst_40
                    dd      ret_p,      pop_af,     jp_p,       zx_di,      call_p,     push_af,    or_n,       rst_48
                    dd      ret_m,      ld_sp_hl,   jp_m,       zx_ei,      call_m,     prefix_iyx, cp_n,       rst_56
Z80_CMDs_Index0xCB  label   dword
                    dd      rlc_bix,    rlc_cix,    rlc_dix,    rlc_eix,    rlc_hix,    rlc_lix,    rlc_ix,     rlc_aix
                    dd      rrc_bix,    rrc_cix,    rrc_dix,    rrc_eix,    rrc_hix,    rrc_lix,    rrc_ix,     rrc_aix
                    dd      rl_bix,     rl_cix,     rl_dix,     rl_eix,     rl_hix,     rl_lix,     rl_ix,      rl_aix
                    dd      rr_bix,     rr_cix,     rr_dix,     rr_eix,     rr_hix,     rr_lix,     rr_ix,      rr_aix
                    dd      sla_bix,    sla_cix,    sla_dix,    sla_eix,    sla_hix,    sla_lix,    sla_ix,     sla_aix
                    dd      sra_bix,    sra_cix,    sra_dix,    sra_eix,    sra_hix,    sra_lix,    sra_ix,     sra_aix
                    dd      sli_bix,    sli_cix,    sli_dix,    sli_eix,    sli_hix,    sli_lix,    sli_ix,     sli_aix
                    dd      srl_bix,    srl_cix,    srl_dix,    srl_eix,    srl_hix,    srl_lix,    srl_ix,     srl_aix
                    dd      bit_0ix,    bit_0ix,    bit_0ix,    bit_0ix,    bit_0ix,    bit_0ix,    bit_0ix,    bit_0ix
                    dd      bit_1ix,    bit_1ix,    bit_1ix,    bit_1ix,    bit_1ix,    bit_1ix,    bit_1ix,    bit_1ix
                    dd      bit_2ix,    bit_2ix,    bit_2ix,    bit_2ix,    bit_2ix,    bit_2ix,    bit_2ix,    bit_2ix
                    dd      bit_3ix,    bit_3ix,    bit_3ix,    bit_3ix,    bit_3ix,    bit_3ix,    bit_3ix,    bit_3ix
                    dd      bit_4ix,    bit_4ix,    bit_4ix,    bit_4ix,    bit_4ix,    bit_4ix,    bit_4ix,    bit_4ix
                    dd      bit_5ix,    bit_5ix,    bit_5ix,    bit_5ix,    bit_5ix,    bit_5ix,    bit_5ix,    bit_5ix
                    dd      bit_6ix,    bit_6ix,    bit_6ix,    bit_6ix,    bit_6ix,    bit_6ix,    bit_6ix,    bit_6ix
                    dd      bit_7ix,    bit_7ix,    bit_7ix,    bit_7ix,    bit_7ix,    bit_7ix,    bit_7ix,    bit_7ix
                    dd      res_0bix,   res_0cix,   res_0dix,   res_0eix,   res_0hix,   res_0lix,   res_0ix,    res_0aix
                    dd      res_1bix,   res_1cix,   res_1dix,   res_1eix,   res_1hix,   res_1lix,   res_1ix,    res_1aix
                    dd      res_2bix,   res_2cix,   res_2dix,   res_2eix,   res_2hix,   res_2lix,   res_2ix,    res_2aix
                    dd      res_3bix,   res_3cix,   res_3dix,   res_3eix,   res_3hix,   res_3lix,   res_3ix,    res_3aix
                    dd      res_4bix,   res_4cix,   res_4dix,   res_4eix,   res_4hix,   res_4lix,   res_4ix,    res_4aix
                    dd      res_5bix,   res_5cix,   res_5dix,   res_5eix,   res_5hix,   res_5lix,   res_5ix,    res_5aix
                    dd      res_6bix,   res_6cix,   res_6dix,   res_6eix,   res_6hix,   res_6lix,   res_6ix,    res_6aix
                    dd      res_7bix,   res_7cix,   res_7dix,   res_7eix,   res_7hix,   res_7lix,   res_7ix,    res_7aix
                    dd      set_0bix,   set_0cix,   set_0dix,   set_0eix,   set_0hix,   set_0lix,   set_0ix,    set_0aix
                    dd      set_1bix,   set_1cix,   set_1dix,   set_1eix,   set_1hix,   set_1lix,   set_1ix,    set_1aix
                    dd      set_2bix,   set_2cix,   set_2dix,   set_2eix,   set_2hix,   set_2lix,   set_2ix,    set_2aix
                    dd      set_3bix,   set_3cix,   set_3dix,   set_3eix,   set_3hix,   set_3lix,   set_3ix,    set_3aix
                    dd      set_4bix,   set_4cix,   set_4dix,   set_4eix,   set_4hix,   set_4lix,   set_4ix,    set_4aix
                    dd      set_5bix,   set_5cix,   set_5dix,   set_5eix,   set_5hix,   set_5lix,   set_5ix,    set_5aix
                    dd      set_6bix,   set_6cix,   set_6dix,   set_6eix,   set_6hix,   set_6lix,   set_6ix,    set_6aix
                    dd      set_7bix,   set_7cix,   set_7dix,   set_7eix,   set_7hix,   set_7lix,   set_7ix,    set_7aix



;****************************************************************************;
